//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaBasePutOptions.java
//
// Defines      : class RomaBasePutOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
 * This class encapsulates the common options by which a Roma message
 * is retrieved.
 */
class RomaBasePutOptions
{
  private static final String copyrightNotice = 
    "Copyright Candle Corporation 1999,2000,2001";

  // selector mask
  final int SELECTOR = Kxc.POPT_USEMSGID | Kxc.POPT_USECORRELID;
  
  // theme allocation control mask
  final int THEME_USAGE = Kxc.POPT_NEWTHEME | Kxc.POPT_USETHEME;

  // theme retaining control mask
  final int THEME_CONTROL = Kxc.POPT_CLOSETHEME | Kxc.POPT_RETAINTHEME;

  int    m_status;
  long   m_incomingILSDataHandle = Kxc.NULL_ILS_DATA_HANDLE;
  long   m_outgoingILSDataHandle = Kxc.NULL_ILS_DATA_HANDLE;
  int    m_opts                  = Kxc.POPT_NONE;
  String m_apiProfileName        = null;
  String m_errorProfileName      = null;

  private RomaSyncpointDef m_sync = new RomaSyncpointDef( Roma.SYNC_PROFILE );
  
  /**
  * Non-public constructor.
  *
  * @param newTheme  if true, initialise the new RomaPutOptions
  *                  object to have the new theme option selected.
  */
  RomaBasePutOptions( boolean newTheme )
  {
    m_opts |= (newTheme? Kxc.POPT_NEWTHEME: Kxc.POPT_USETHEME);
  }
	
  /**
  * Sets the put selector.
  *
  * @param sel the new value of the put selector.
  *
  */
  public void setSelector( int sel )
  {
    m_opts &= ~SELECTOR;
    m_opts |= (sel & SELECTOR);
  }
  
  /**
  * Selects whether messages can be put under syncpoint
  * to a non-transactional queue.
  *
  * @param flag  one of :-
  * <ul>
  *  <li>true - disallow messages to be put under syncpoint to a
  *             non-transactional queue.
  *  <li>false - allow messages put under syncpoint from going
  *              to a non-transactional queue.
  * </ul>
  */
  public void setNoSyncAllowed( boolean flag )
  {
    if (flag)
      m_opts |= Kxc.POPT_ALLOWNOSYNC;
    else
      m_opts &= ~Kxc.POPT_ALLOWNOSYNC;
  }

  /**
  * Associates an API profile with the put operation.
  *
  * @param profile  the name of the API profile.
  */
  public void setApiProfileName( String profile )
  {
    m_apiProfileName = profile;
  }
    

  // Remaining Set functions are for use within package
  
  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options.
  *
  * @param flag  if true then new theme allocation is enabled,
  *   otherwise it is disabled. If disabled, the theme already
  *   associated with the RomaMessage will be used.
  */
  public void setNewTheme( boolean flag )
  {
    m_opts &= ~THEME_USAGE;
    m_opts |= flag ? Kxc.POPT_NEWTHEME : Kxc.POPT_USETHEME;
  }

  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options
  * 
  * @param flag  if true theme already associated with the 
  * RomaMessage will be used. If false the new theme 
  * allocation will be done.
  */
  public void setUseTheme(boolean b)
  {
    setNewTheme(!b);
  }

  /**
  * Selects whether or not the theme is to be closed or else
  * retained in the put operation.
  * 
  * @param flag  if true then new retain the theme. Otherwise
  *   close the theme.
  */
  public void setRetainTheme( boolean flag )
  {
    m_opts &= ~THEME_CONTROL;

    if (flag)
      m_opts |= Kxc.POPT_RETAINTHEME;
    else
      m_opts |= Kxc.POPT_CLOSETHEME;
  }

  /**
  * Selects whether of not the theme is to be closed or retained
  * in the put operation.
  * 
  * @param flag if true close the theme otherwise retain the theme
  */
  public void setCloseTheme(boolean flag)
  {
    if (flag) {
       m_opts |=  Kxc.POPT_CLOSETHEME;
       m_opts &= ~Kxc.POPT_RETAINTHEME;
    }
    else   {
       m_opts &= ~Kxc.POPT_CLOSETHEME;
       m_opts |=  Kxc.POPT_RETAINTHEME;
    }
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_sync = sync;
  }

  /**
  * Return sync point definitiion
  */
  RomaSyncpointDef syncpointDef( )
  {
    return m_sync;
  }

  /**
  * Return status of put operation
  * Package visibility
  *
  */
  int status()
  {
    return m_status;
  }
	
  /**
  * Called from a RomaContext instance when it prepares to do
  * one of the following actions :-
  * <ul>
  * <li> invoke an error analysis routine.
  * <li> get a message
  * <li> put a message
  * </li>
  * Receives a KxcContext object and performs a series of updates to
  * it using the current state of this instance. This is done prior
  * to the KxcContext object being used in a Roma activity
  *
  * @param ctx  instance of a Roma context abstraction that takes
  *             part in low level operations.
  */
  void prepare( KxcContext ctx )
  {
    ctx.putCtrl().setOpts( m_opts );
    ctx.putCtrl().setApiProfileName( m_apiProfileName );
    ctx.putCtrl().setErrorProfileName( m_errorProfileName );
    ctx.putCtrl().setILSDataHandle( m_outgoingILSDataHandle );
  }

  /**
  * Process message put operation results.
  *
  * @param ctx KxcContext
  */
  void processPut( KxcContext ctx )
  {
    KxcPutCtrl ctrl = ctx.putCtrl();
    m_incomingILSDataHandle =  ctrl.ilsDataHandle();
	m_status = ctrl.putgetStatus();
  }
}
