//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaBeResponse.java
//
// Defines      : class RomaBeResponse
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* This class encapsulates responses made by a Roma Business Element
* following a Roma Client request. It enables the application to
* set the options required when responding to a Roma Client request.
* <p>
* This class provides a framework for associating application
* specific data with a BusinessElement response.
* <p>
* It is intended that RomaBeResponse subclasses provide an
* implementation for the marshalling of application-specific data.
* The marshalResponse() method may be overridden to stream 
* application data into a message buffer.
*
* @see RomaBusinessElement
*/
public class RomaBeResponse 
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // RomaBeReponse has a RomaMessage  
  // Package visibility so other classes can access
  RomaMessage m_Message = new RomaMessage();
  private RomaPutOptions m_po = new RomaPutOptions(false);
  
  /**
  * Constructs a new RomaBeResponse object and associates it 
  * with the specified theme
  *
  * @param theme  the theme of the request that this response 
  *               is intended for.
  *
  * @see #setTheme
  */
  public RomaBeResponse( RomaTheme theme )
  {
    setTheme( theme );
  }
  
  /**
  * Constructs a new RomaBeResponse object, associates it 
  * with the specified theme and sets user message buffer.
  * 
  * @param theme  the theme of the request that this response 
  *               is intended for.
  * @param msgBuf byte array containing the message.
  * @param msgLen length of the message. 
  * 
  */
  public RomaBeResponse( RomaTheme  theme, 
                         byte       msgBuf[],
                         int        msgLen )
  {
    setTheme( theme );
    m_Message.setMsgBuffer( msgBuf, msgLen );  
  }
  
  /**
  * Constructs a new RomaBeResponse object, associates it 
  * with the specified theme and sets user message buffer.
  * 
  * @param theme  the theme of the request that this response 
  *               is intended for.
  * @param msgBuf byte array containing the message.
  * @param msgLen length of the message. 
  * @param tagBuf byte array containing the tag data.
  * @param tagLen length of the tag data. 
  */
  public RomaBeResponse( RomaTheme  theme, 
                         byte       msgBuf[],
                         int        msgLen,
                         byte       tagBuf[],
                         int        tagLen )
  {
    setTheme( theme );
    m_Message.setMsgBuffer( msgBuf, msgLen );  
    m_Message.setTagBuffer( tagBuf, tagLen );  
  }
  
  /**
  * Constructs a new RomaBeResponse object and initialises 
  * it with message id, correlation id and theme from supplied
  * RomaBeRequest.
  *
  * @param br a RomaBeRequest
  */
  public RomaBeResponse( RomaBeRequest br )
  {
    setMsgId( br.msgId() );
    setCorrelId(br.correlId());	
    setTheme( br.theme() );
  }
  
  /**
  * Marshalls response data immediately prior the reponse 
  * being sent. Invoked by the RomaBusinessElement.put() method.
  *
  * @see RomaBusinessElement#put( RomaBeResponse )
  * @exception IOException  if the marshalling operation fails
  *                         for some reason.
  */
  public void marshalResponse()
    throws IOException
  {
  }
  
  /**
  * Sets the put selector. This specifies which identifiers
  * contained in the RomaMessage should be used when the 
  * message is put.
  * @param sel the new value of the put selector.
  *
  * <p>
  * The expected return value will be one of the following fields :-
  * <ul>
  *   <li>USE_NONE
  *   <li>USE_MSGID
  *   <li>USE_CORRELID
  *   <li>USE_MSGID_AND_CORRELID
  * </ul>
  */
  public void setSelector( int sel )
  { 
    m_po.setSelector( sel ); 
  }
  
  /**
  * Controls whether an MSMQ message can be placed outside syncpoint
  * control. If the application attempts to put a message to MSMQ inside  
  * syncpoint control but the target queue does not support transactions,
  * the call is normally rejected with error code RC_NONXACTIONAL.
  * The application can call this method with true flag to force an MSMQ
  * message to be placed outside syncpoint control.
  * @param flag  one of :-
  * <ul>
  *  <li>true - allow MSMQ messages to be placed outside syncpoint control
  *  <li>false - don't allow MSMQ messages to be placed outside syncpoint
  *              control
  * </ul>
  */
  public void setNoSyncAllowed( boolean flag )
  { 
    m_po.setNoSyncAllowed( flag ); 
  }
  
  /**
  * Associates an API profile with the put operation.
  *
  * @param profile  the name of the API profile.
  */
  public void setApiProfileName( String profile )
  { 
    m_po.setApiProfileName( profile ); 
  }
  
  /**
  * Associates an error profile with the put operation.
  *
  * @param profile  the name of the error profile.
  */
  public void setErrorProfileName( String profile )
  { 
    m_po.setErrorProfileName( profile ); 
  }
  
  /**
  * Specify if error profiling is to be suppressed
  *   
  * @param flag true to suppress error profiling
  */
  public void setSuppressErrorProfile( boolean flag )
  {
	   m_po.setSuppressErrorProfile( flag );
  } 
  
  /**
  * Indicates that flags and options specified in the API profile
  * should be used for the Roma put operation.
  * <p>
  * If this option is chosen but the specified profile is not
  * available then default values are used for the put options.
  *
  * @param flag true to suppress error profiling
  *
  */
  public void setUseApiProfile( boolean flag )
  {
    m_po.useApiProfile( flag );
  }
  
  /**
  * Enables or disables the option to reset the retry count 
  * when sending the message.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> increment the retry
  *                count when sending the message.
  *    <li> false - reset the retry count when sending the message.
  * </ul>
  */
  public void setIncrementRetryCount(boolean select)
  { 
    m_po.incrementRetryCount( select ); 
  }
  
  /**
  * Enables or disables the option to send the message to an
  * exception message service if the put operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> perform a send to an
  *                exception message service if put fails.
  *    <li> false - allow a message to be sent to an exception
  *                 message service if the put fails.
  * </ul>
  */
  public void setNoEms( boolean select )
  { 
    m_po.setNoEms( select ); 
  }
  
  /**
  * Enables or disables the option to send a notification message
  * back to the sender if the message get operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true  - <strong>do not</strong> allow a notification
  *                 message to be sent on a get failure.
  *    <li> false - <strong>do </strong> allow a notification
  *                 message to be sent.
  * </ul>
  */
  public void setNoNotification( boolean select )
  { 
    m_po.setNoNotification( select ); 
  }
  
  /**
  * Enables or disables the option to send the message to a 
  * message transformation service if the next component in the
  * workflow cannot accept the message format.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> allow the message to
  *                sent to the transformation service.
  *    <li> false - allow the message to be routed via a
  *                 transformation service if necessary.
  * </ul>
  */
  public void setNoTransformation( boolean select )
  { 
    m_po.setNoTransformation( select ); 
  }
  
  
  /**
  * Specify whether an existing theme should be used.
  * <p>
  * @param flag if true then the existing theme is used.
  */
  public void setUseTheme(boolean flag)
  { 
    m_po.setUseTheme(flag); 
  }
  
  
  /**
  * Specify whether the theme should be retained for future operations.
  * <p>
  * @param flag  if true then the theme is retained.
  */
  public void setRetainTheme(boolean flag)
  { 
    m_po.setRetainTheme(flag); 
  }
  
  /**
  * Specify whether the theme should be closed automatically when the
  * operation completes.
  * <p>
  * @param flag  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean flag)
  { 
    m_po.setCloseTheme(flag); 
  }
  
  /**
  *  Set or clear notification flag
  * 
  *  @param flag   if true then Roma sends response immediately
  *  back to client, ignoring any further business components
  */
  public void setNotification( boolean flag )
  { 
    m_po.setNotification( flag ); 
  }
  
  /**
  * Specify if termination is supported
  *   
  * @param flag true to indicate termination is supported.
  */
  public void setTermSupported( boolean flag )
  {
    m_po.setTermSupported( flag );
  }
		
  //== END OF OPTIONS RELATED METHODS =================================
  
  //== RomaMesssage Access functions ==================================
  
  /**
  * Set the message priority.
  * @param priority  this can be one of the following :-
  * <ul>
  * <li>PRIORITY_DEFAULT - use the default priority
  *     from the transport.
  * <li>PRIORITY_PROFILE - use the priority as specified in the 
  *     associated Put API profile. If no such profile or attribute,
  *     use PRIORITY_DEFAULT.
  * <li>a specific integer meaningful to the underlying transport
  *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ 
  * </ul>
  */
  public void setPriority( int priority )
  {
    m_Message.setPriority(priority);
  }
  
  /**
  * Set a new value for the message perisitence.
  * @param  persistence  this can be one of the following :-
  *   <ul>
  *   <li>PERSISTENCE_YES
  *   <li>PERSISTENCE_NO
  *   <li>PERSISTENCE_PROFILE - use the persistence value as
  *       specified in the associated Put API profile. If no such
  *       profile or attribute, use PERSISTENCE_NO.
  *   </ul>
  */
  public void setPersistence( int persistence )
  {
    m_Message.setPersistence(persistence);
  }
  
  /**
  * Set a new message format value.
  * @param format  identifies the format of message. If the 
  *   value FORMAT_PROFILE is used then the message format value
  *   is retrieved from the associated Put API profile. If the
  *   profile is not available or else has no format attribute 
  *   value then the message format value FORMAT_BINARY is used.
  */
  public void setFormat( int format )
  {
    m_Message.setFormat(format);
  }
  
  /**
  * Sets a new message feedback code.
  * @param feedback  new message feedback value.
  */
  public void setFeedback( int feedback )
  {
    m_Message.setFeedback(feedback);
  }
  
  /**
  * Set a new message expiry time.
  * @param expiry  the new expiry time value
  *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
  *   If EXPIRY_PROFILE is specified then
  *   the expiry value is retrieved from the associated Put API
  *   profile. If the profile is not available or else has no expiry
  *   attribute set, the EXPIRY_NEVER constant is used.
  */
  public void setExpiry( int expiry )
  {
    m_Message.setExpiry(expiry);
  }
  
  /**
  * Set a new message codepage value. 
  * @param codepage  the new codepage value. If CODEPAGE_PROFILE is
  *   specified then the codepage will be retrieved from the 
  *   associated Put API profile. If the profile is not available or
  *   else has no codepage attribute set, the default value of 
  *   CODEPAGE_USER will be used.
  */
  public void setCodepage( int codepage )
  {
    m_Message.setCodepage(codepage);
  }
  
  /**
  * Set the message encoding.
  * @param  encoding  the new encoding indicator. If this is 
  *   ENCODING_PROFILE then the value is retrieved from the 
  *   associated Put API profile. If the profile is not available
  *   or else has no encoding attribute set, the default value
  *   ENCODING_DEFAULT is used.
  */
  public void setEncoding( int encoding )
  {
    m_Message.setEncoding(encoding);
  }
  
  /**
  * Set the message identifier.
  * @param  msgId  a byte array containing the new message id value.
  */
  public void setMsgId( byte[] msgId )
  {
    m_Message.setMsgId(msgId);
  }
  
  /**
  * Return the message identifier in a byte array.
  */
  public byte[] msgId()
  {
    return m_Message.msgId();
  }
  
  /**
  * Set the message correlation identifier.
  * @param  correlId  the new correlation id.
  */
  public void setCorrelId( byte[] correlId )
  {
    m_Message.setCorrelId(correlId);
  }
  
  /**
  * Return the message correlation identifier.
  */
  public byte[] correlId()
  {
    return m_Message.correlId();
  }
  
  /**
  * Sets the format name.
  *
  * @param formatName the new format name.
  */
  public void setFormatName( String formatName )
  {
    m_Message.setFormatName( formatName );
  }
  
  /**
  * Update the message theme.
  *
  * @param theme RomaTheme object.
  */
  public void setTheme( RomaTheme theme )
  {
    m_Message.setTheme( theme );
  }
  
  /**
  * Return the message theme. Theme can be set internally when
  * the message is being sent or received or else explicitly by
  * the user.
  */
  public RomaTheme theme()
  {
    return m_Message.theme();
  }
  
  /**
  * Set the name of this message
  * @param messageName name to be used
  */
  public void setMsgName( String messageName )
  {
    m_Message.setMessageName(messageName);
  }
  
  /**
  * Return message status
  */
  public int status( )
  {
    return m_Message.status();
  }
  
  /**
  * Set security definition 
  * 
  * @param secDef security definition to be sent with message    
  * 
  */
  public void setSecurityDef( RomaSecurityDef secDef )
  {
    m_Message.setSecurityDef( secDef );  
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_po.setSyncpointDef( sync );
  }
  
  /**
  * Set buffer containing the message data and the length of the 
  * message for a PUT request. To send an empty message set length 
  * to 0 and buffer to null
  * 
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message. 
  * 
  * @return true if successfull, 
  *         false if datalength < 0 or datalength = 0 and buffer != null  
  */
  public boolean setMsgBuffer( byte buffer[],
    int dataLength )
  {
    return m_Message.setMsgBuffer( buffer, dataLength );  
  }

  /**
  * Sets message data.
  *
  * @param data Message data
  */
  public void setMsgBuffer( byte[] data )
  {
    m_Message.setMsgBuffer( data, data.length );  
  }

  /**
  * Sets message data.
  *
  * @param data Message data
  */
  public void setMsgBuffer( String data )
  {
    setMsgBuffer( data.getBytes() );  
  }
  
  /**
  * Set buffer containing the tag data and the length of the 
  * tag for a PUT request.
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message. 
  * 
  * @return true if successfull,
  *         false if datalength < 0 or datalength = 0 and buffer != null  
  */
  public boolean setTagBuffer( byte buffer[], int	dataLength )
  {
    return m_Message.setTagBuffer( buffer, dataLength );  
  }

  /**
  * Sets tag data.
  *
  * @param data Tag data
  */
  public void setTagBuffer( byte[] data )
  {
    m_Message.setTagBuffer( data, data.length );  
  }

  /**
  * Sets tag data.
  *
  * @param data Tag data
  */
  public void setTagBuffer( String data )
  {
    setTagBuffer( data.getBytes() );  
  }
  
  /**
  * Set outgoing ILS data
  *
  * @param  ilsData The ILS data to be sent with message 
  */
  public void setILSData(RomaILSData ilsData)
  {							 
    m_po.m_outgoingILSDataHandle = ilsData.getILSDataHandle( );
  }
  
  /**
  * Get incoming ILS data
  *
  */
  public RomaILSData getILSData()
  {							 
    if (m_po.m_incomingILSDataHandle == Kxc.NULL_ILS_DATA_HANDLE)
      return null;
    else
      return new RomaILSData( m_po.m_incomingILSDataHandle );
  }
  
  // putOptions called by Business Element get operation which then
  // invokes 'prepare' to get the infomation in the correct format
  // for the C API.
  RomaPutOptions putOptions() 
  { 
    return m_po; 
  }
}
