//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaBusinessElement.java,v 1.1.1.1.8.2.2.4 2003/10/16 18:47:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaBusinessElement.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaBusinessElement.java
//
// Defines      : class RomaBusinessElement
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* This class allows an application to instantiate a server-side
* connection to the Roma network as a Roma Business Element.
* <p>A Business Element typically receives requests from a Client
* and issue Responses.
* <p>A single application is allowed to instantiate multiple
* objects of the Business Element class.
* <p>The Business Element class be extended by an application to
* providing a derived class which may override createRequest() for
* each type of request to be distinguished.
*
* @see RomaBeRequest
*/
public class RomaBusinessElement extends RomaContext
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /*
  * m_kxcSvr is an instance of com.candle.roma.kxc.raw.KxcServer
  * that implements the connection to the server.
  */
  private KxcServer m_kxcSvr;
  
  private boolean m_terminationIndicator;
  
  /**
  * Constructs a RomaBusinessElement object but does not
  * attach the Business Element to the Roma network.
  */
  public RomaBusinessElement()
  {
  }
  
  /**
  * Constructs and connects the new Business Element to the Roma network.
  * Uses default values for the application options (APP_NONE)
  * and security options (null) associated with Business Element.
  *
  * @param name name of the Business Element
  * @exception RomaError is thrown on connection failure.
  */
  public RomaBusinessElement( String name )
    throws RomaError
  {
    connect( name, Roma.APP_NONE, null, null );
  }
  
  /**
  * Constructs and connects a Business Element to the Roma network.
  * Takes application options associated with
  * Business Element. See RomaContext for information on valid
  * application options.
  *
  * @param name name of the Business Element.
  * @param opts application options
  * @exception RomaError is thrown on connection failure.
  * @see RomaContext
  */
  public RomaBusinessElement( String  name,
                              int     opts )
    throws RomaError
  {
    connect( name, opts, null, null);
  }
  
  /**
  * Constructs and connects a Business Element to the Roma network.
  * Takes application options and security options associated with
  * Business Element. See RomaContext for information on valid
  * application options.
  *
  * @see RomaContext
  * @param name name of the Business Element.
  * @param opts application options
  * @param sec security options
  * @exception RomaError is thrown on connection failure.
  */
  public RomaBusinessElement( String          name,
                              int             opts,
                              RomaSecurityDef sec )
    throws RomaError
  {
    connect( name, opts, sec, null );
  }
  
  /**
  * Constructs and connects a Business Element to the Roma network.
  * Takes application options, security options and model queue associated 
  * with Business Element. See RomaContext for information on valid
  * application options.
  *
  * @see RomaContext
  * @param name name of the Business Element.
  * @param opts application options.
  * @param sec security options.
  * @param modelQName model the queue on this when creating.
  * @exception RomaError is thrown on connection failure.
  */
  public RomaBusinessElement( String          name,
                              int             opts,
                              RomaSecurityDef sec,
                              String          modelQName)
    throws RomaError
  {
    connect( name, opts, sec, modelQName);
  }
  
  /**
  * Attaches a named BusinessElement to the Roma network.
  * Uses default values for the application options (APP_NONE)
  * and security options (null) associated with Business Element.
  *
  * @param name  name of the Business Element.
  *
  * @exception RomaError is thrown on operation failure.
  */
  public void connect( String name )
    throws RomaError
  {
    connect( name, Roma.APP_NONE, null, null );
  }
  
  /**
  * Attaches a named BusinessElement to the Roma network.
  * Takes application options and security options associated with
  * the Business Element.
  *
  * @param name name of the Business Element
  * @param opts application options
  * @exception RomaError is thrown on operation failure.
  */
  public void connect( String name,
                       int    opts )
    throws RomaError
  {
    connect( name, opts, null, null );
  }
  
  /**
  * Attaches a named BusinessElement to the Roma network.
  * Takes application options and security options associated with
  * the Business Element.
  *
  * @param name name of the Business Element
  * @param opts application options
  * @param sec security options
  * 
  * @exception RomaError is thrown on operation failure.
  */
  public void connect( String          name,
                       int             opts,
                       RomaSecurityDef sec )
    throws RomaError
  {
    connect( name, opts, sec, null );
  }
  
  /**
  * Attaches a named BusinessElement to the Roma network.
  * Takes application options and security options associated with
  * the Business Element.
  *
  * @param name name of the Business Element
  * @param opts application options
  * @param sec security options
  * @param modelQName model queue name
  * 
  * @exception RomaError is thrown on operation failure.
  */
  public void connect( String          name,
                       int             opts,
                       RomaSecurityDef sec,
                       String          modelQName)
    throws RomaError
  {
    checkDisconnected();

    //load the native library
    KxcConnection f = nativeConn();
    
    //initialise data for kxcServerInitialize call
    KxcServer svr = f.newServer( name, 
                                 opts, 
                                 sec == null? null: sec.getBuffer(),
                                 modelQName);

    //perform  kxcServerInitialize
    RomaContext.kxcOper( svr.init() );
    m_kxcSvr = svr;
    setConnected( name, opts );
  }
  
  /**
  * Issues a response to a Client request. This method calls the
  * RomaBeResponse class' marshalResponse() method before issuing
  * the response.
  * The RomaBeResponse instance is updated with the generated theme
  * and/or the msgId and correlation id if the user has not
  * specified them on the request.
  *
  * @see RomaBeResponse
  *
  * @param resp  Business Element response
  * @exception RomaError is thrown on operation failure.
  */
  public void put( RomaBeResponse resp )
    throws RomaError
  {
    checkConnected();
    
    // Creates PutAdapter object
    PutAdapter pa = new PutAdapter()
    {
      // overrides PutAdapter.put() implementation by invoking
      // KxcServer.put method.
      public void put( KxcContext ctx )
        throws RomaError
      {
        RomaContext.kxcOper( ((KxcServer)ctx).put() );
      }
    }; // end of anonymous inner class 
    
    try {
      resp.marshalResponse();
    }
    catch ( IOException iox ) {
      RomaError err = new RomaError("");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }

    // invokes base put message service
    // implementing context independent part of
    // the put functionality
    putMsg( pa, resp.m_Message, resp.putOptions() );
    
    // record term indicator for status for isTermIndicator()
    m_terminationIndicator = 
      (resp.putOptions().status() & Kxc.GOPT_STAT_TERMINATE) != 0;
  }
  
  /**
  * Gets a client request using default get options,
  * i.e. the first available request is retrieved and the theme
  * for the response is left open.
  *
  * If the wait interval has expired a 
  * null RomaBeRequest is returned.
  *
  * @exception RomaError is thrown on failure of the get call.
  *
  */
  public RomaBeRequest get()
    throws RomaError
  {
    RomaGetOptions opt = new RomaGetOptions();
    RomaInteger noLen = new RomaInteger(-1); 
    
    // lengths of -1 means auto-allocation will be used.
    return get( opt, null, noLen, null, noLen );
  }
  
  /**
  * Gets a client request according to the supplied get options.
  *
  * If the wait interval has expired a 
  * null RomaBeRequest is returned.
  *
  * @param opt  get options used to select the request.
  *
  * @exception RomaError is thrown on failure of the get call.
  */
  public RomaBeRequest get( RomaGetOptions opt )
    throws RomaError
  {
    RomaInteger noLen = new RomaInteger(-1); 
    
    // lengths of -1 means auto-allocation will be used.
    return get( opt, null, noLen, null, noLen );
  }	
  
  /**
  * Gets a client request according to the supplied get options.
  * The supplied message buffer will be used to receive data,
  * the tag buffer will be allocated automatically.
  *
  * If the wait interval has expired a 
  * null RomaBeRequest is returned.
  *
  * @param opt  get options used to select the request.
  * @param msgBuf  buffer to receive message
  * @param msgBufSize  size of message buffer.
	 *
   * @exception RomaError is thrown on failure of the get call.
  */
  public RomaBeRequest get( RomaGetOptions  opt, 
                            byte            msgBuf[],
                            RomaInteger     msgBufSize ) 
    throws RomaError
  {
    RomaInteger noLen = new RomaInteger(-1); 
    
    // lengths of -1 means auto-allocation will be used.
    return get( opt, msgBuf, msgBufSize, null, noLen );
  }	
  
  /**
  * Gets a client request according to the supplied get options.
  * The supplied message and tag buffers will be used to receive data.
  *
  * If the wait interval has expired a 
  * null RomaBeRequest is returned.
  *
  * @param opt  get options used to select the request.
  * @param msgBuf  buffer to receive message
  * @param msgBufSize  size of message buffer.
  * @param tagBuf  buffer to receive tag data 
  * @param tagBufSize  size of buffer to receive tag data
  *
  * @exception RomaError is thrown on failure of the get call.
  */
  public RomaBeRequest get( RomaGetOptions  opt,
                            byte            msgBuf[],
                            RomaInteger     msgBufSize, 
                            byte            tagBuf[],
                            RomaInteger     tagBufSize )
    throws RomaError
  {
    checkConnected();
    
    //----------------------------------------------------------------
    // Supply an overriding implementation for the GetAdapter.get()
    // method.
    //----------------------------------------------------------------
    GetAdapter ga = new GetAdapter()
    {
      public boolean get( KxcContext ctx )
        throws RomaError
      {
        RomaContext.kxcOper( ctx.get() );
        return true;
      }
    }; // end of anonymous inner class 
    
    RomaMessage msg = new RomaMessage();
    
    // copy the buffer sizes to protect
    RomaInteger msgLen = new RomaInteger( msgBufSize.intValue() );
    RomaInteger tagLen = new RomaInteger( tagBufSize.intValue() );
    
    // Use buffers supplied instead of Roma auto-allocating
    // length of -1, buffer of null will still auto-allocate
    msg.setMsgBuffer( msgBuf, msgLen );
    msg.setTagBuffer( tagBuf, tagLen );
    
    try {
      if ( getMsg( ga, msg, opt.getBaseOptions() ) )
      {
        // record term indicator for status for isTermIndicator()
        m_terminationIndicator = (opt.getBaseOptions().status() & Kxc.GOPT_STAT_TERMINATE) != 0;
        
        RomaBeRequest request = new RomaBeRequest( msg );
        return createRequest( request );
      }
      return null;
    }
    catch( RomaError apix ) {
      if ( apix.primaryInfo().returnCode() == Roma.RC_BUFFER_TOO_SMALL  ||
        apix.primaryInfo().returnCode() == Roma.RC_MESSAGE_SUPPRESSED )
      {
        // update buffer sizes to the required message lengths
        msgBufSize.setValue( msgLen.intValue() );
        tagBufSize.setValue( tagLen.intValue() );
      }
      throw apix;
    }
    
  }
  
  /**
  * @return true if termination indicator was set after put or get
  * operation
  */
  public boolean isTermIndicator()
  {
    return m_terminationIndicator;  
  }

  /**
   *  Returns information on the specified theme.
   *
   *  @param theme Source theme
   *  @exception RomaError is thrown if the operation fails
   */
  public RomaThemeInfo queryTheme(RomaTheme theme)
  throws RomaError
  {
     checkConnected();
     return jniQueryTheme(kxcCtx().getContextHandle(), theme.handle(), false);
  }
  
  /**
   *  Creates a new theme from the source theme and the theme details.
   *
   *  @param theme Source theme
   *  @param details Theme details
   *  @param options Options indicating what parts of the theme information to
   *         use for the theme build. It is THEMEINFO_ALL or a combination of
   * <br><ul>
   * <li><code>Roma.THEMEINFO_THEMEID</code> - use theme ID</li>
   * <li><code>Roma.THEMEINFO_RETRYCOUNT</code> - use retry count</li>
   * <li><code>Roma.THEMEINFO_WORKFLOW</code> - use workflow information</li>
   * <li><code>Roma.THEMEINFO_ETEINFO</code> - use ETE information</li>
   * <li><code>Roma.THEMEINFO_TRAILERS</code> - use persistent trailers</li>
   * <li><code>Roma.THEMEINFO_CONFIRMFLAGS</code> - use confirmation flags</li>
   * </ul>
   *  @param close If true, closes the source theme
   *  @exception RomaError is thrown if the operation fails
   */
  public RomaTheme buildTheme(RomaTheme theme, 
                              RomaThemeInfo info,
                              int options,
                              boolean close)
  throws RomaError
  {
    return buildTheme(theme, info, options, close, false);
  }

  /**
   *  Returns detailed information on the specified message.
   *
   *  @param msg Source message
   *  @param options Feature options, should be zero or one of
   * <br><ul>
   * <li><code>Roma.INTMSG_THEME</code> - extract the message theme. The application is responsible for the theme closing.</li>
   * </ul>
   *  @exception RomaError is thrown if the operation fails
   */
  public RomaMessageInfo interpretMessage(RomaBeRequest msg, int options)
  throws RomaError
  {
     return interpretMessage(msg.m_Message, options);
  }

  /**
  * Unmarshal a retrieved request. This method is expected to be
  * overridden in derived classes. It can be used to differentiate
  * between different types of response so that users can construct
  * subclasses of class RomaBeRequest that deal with the response
  * data as appropriate.
  * <p>
  * The RomaBusinessElement subclass' createRequest() method can
  * construct the appropriate type of RomaBeRequest using the
  * RomaBeRequest instance data. The createRequest() method is called
  * by the get() method once the response is retrieved.
  *
  * @param request  a retrieved RomaBeRequest.
  * when constructing the RomaBeRequest object.
  */
  protected  RomaBeRequest createRequest( RomaBeRequest request )
  {
    return new RomaBeRequest( request );
  }
  
  /**
  * Returns underlying object of KxcServer type which
  * implements connection with business element.
  */
  KxcContext kxcCtx()
  {
    return m_kxcSvr;
  }
}
