//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2002
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaDirectoryItem.java
//
// Defines      : class RomaDirectoryItem
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;
import java.lang.*;


/**
* This class encapsalates a Roma directory item that has been
* retrieved from the Roma directory using the RomaDirectoryAgent
* class.
* <p>
* The class is constructed by calling the item()method on
* RomaDirectoryAgent.
* @see RomaDirectoryAgent
*/

// Methods which use pipe macros do not generate exceptions,
// but because the JNI interface provides for exceptions they
// are absorbed within the method so the caller does not
// have to provide exception handlers.

public class RomaDirectoryItem
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2002";

  private byte[]  m_buffer    = null;        // item buffer
  private int     m_bufferLen = 0;           // item buffer length
  private int     m_storeop   = Roma.FORCE;  // store option

  public static final int BUF_HEADER_SIZE = 60;
  public static final int BUF_RES_SIZE = 20;

  /**
  * Constructor
  *
  * All new items must have an objectclass attribute, so this is
  * mandated in the constructor. The dn and all the added attributes
  * must be consistent with any LDAP scheme which is in force.
  *
  */
  public RomaDirectoryItem(String dn, String objectClass)
  {
    m_bufferLen = BUF_HEADER_SIZE + BUF_RES_SIZE + dn.length();
    m_buffer = new byte[m_bufferLen];
    m_buffer = KxcContext.dirSetDN(m_buffer, m_bufferLen, dn);
    addAttribute("objectclass", objectClass);
  }

  /**
  * Constructor
  * Package visibility
  *
  */
  RomaDirectoryItem()
  {
  }

  /**
  * Constructor
  * Package visibility
  *
  * @param buffer    buffer containing item
  * @param bufferlen length of buffer containing item
  */
  RomaDirectoryItem( byte[] buffer,
                     int    bufferLen )
  {
    m_buffer = new byte[bufferLen];

    // copy data into a full size buffer to allow appending
    System.arraycopy(buffer, 0, m_buffer, 0, bufferLen);
    m_bufferLen = bufferLen;
  }

  /**
  * Retrieves the distinguished name
  *
  * @return   String containing distinguished name
  */
  public String dn( )
  {
    return KxcContext.dirGetDN( m_buffer, m_bufferLen );
  }

  /**
  * Set the distinguished name
  *
  * @param    name   distiguished name.
  */
  public void setDN( String disname )
  {
    // If this item has attributes we have to preserve them
    // by building up a new item (with the new name) and copy
    // the attributes across.
    int tmpLen = BUF_HEADER_SIZE + BUF_RES_SIZE + disname.length();
    int nAttr = attributes();
    int i = 0;
    String value, name;

    for (; i < nAttr; i++ )
    {
        name = attributeName( i );
        value = attributeValue( i );
        tmpLen += BUF_RES_SIZE + name.length() +
            BUF_RES_SIZE + value.length();
    }

    byte[] tmpBuf = new byte[tmpLen];
    tmpBuf = KxcContext.dirSetDN(tmpBuf, tmpLen, disname);

    for (i = 0; i < nAttr; i++ )
    {
        name = attributeName( i );
        value = attributeValue( i );
        tmpBuf = KxcContext.dirAddAttr(tmpBuf, tmpLen,
                                 name, value );
    }

    m_buffer = tmpBuf;
    m_bufferLen = tmpLen;
  }

  /**
  * Add attribute to item.
  * setDN() must be called prior to addAttribute().
  *
  * @param    name    attribute name.
  * @param    value   attribute value.
  */
  public void addAttribute( String name,
                            String value )
  {
     int tmpLen = m_bufferLen + BUF_RES_SIZE + name.length()
        + BUF_RES_SIZE + value.length();
     byte[] tmpBuf = new byte[tmpLen];

     System.arraycopy(m_buffer, 0, tmpBuf, 0, m_bufferLen);
     m_buffer = KxcContext.dirAddAttr(tmpBuf,
                                      tmpLen,
                                      name,
                                      value );
     m_bufferLen = tmpLen;
  }

  /**
  * Returns the number of attributes
  *
  * @return number of attributes
  */
  public int attributes( )
  {
    return KxcContext.dirGetAttributes( m_buffer,
                                        m_bufferLen);
  }

  /**
  * Returns value of the first attribute
  *
  * @return value of attribute
  */
  public String attributeValue( )
  {
    return KxcContext.dirGetAttribAtValue( m_buffer,
                                           m_bufferLen,
                                           0 );
  }

  /**
  * Returns value of nth attribute. The attributes are number from 0
  *
  * @param attribute number of attribute
  * @return value of attribute
  */
  public String attributeValue( int attribute )
  {
    return KxcContext.dirGetAttribAtValue( m_buffer,
                                           m_bufferLen,
                                           attribute );
  }

  /**
  * Returns name of the first attribute
  *
  * @return value of attribute
  */
  public String attributeName( )
  {
    return KxcContext.dirGetAttribAt( m_buffer,
                                      m_bufferLen,
                                      0 );
  }

  /**
  * Returns name of nth attribute. The attributes are number from 0.
  *
  * @param attribute number of attribute
  * @return value of attribute
  */
  public String attributeName( int attribute )
  {
    return KxcContext.dirGetAttribAt( m_buffer,
                                      m_bufferLen,
                                      attribute );
  }

  /**
  * Returns value of first occurrence of named attribute
  *
  * @param name  name of attribute
  * @return value of attribute
  */
  public String attributeValue( String name )
  {
    return KxcContext.dirGetAttribValue( m_buffer,
                                         m_bufferLen,
                                         name,
                                         0 );
  }

  /**
  * Returns value of nth occurrence of named attribute
  *
  * @param name       name of attribute
  * @param attribute  number of occurrence attribute
  * @return value of attribute
  */
  public String attributeValue( String name,
                                int attribute )
  {
    return KxcContext.dirGetAttribValue( m_buffer,
                                         m_bufferLen,
                                         name,
                                         attribute );
  }

  /**
   * Specify if an item with the same DN must or must not exist at
   * the time that a DirectoryAgent attempts to store this item
   * By default, the update is forced through.
   *
   * @param f if true an item with the same DN must exist,
   *        if false an item with the same DN must not exist
  */
  public void setItemMustExist(boolean f)
  {
    m_storeop = (f) ? Roma.MUST_EXIST : Roma.MUST_NOT_EXIST;
  }

  /**
   * Specify if an item with the same DN must or must not exist at
   * the time that a DirectoryAgent attempts to store this item
   * By default, the update is forced through.
   *
   * @param f if true an item with the same DN must not exist,
   *        if false an item with the same DN must exist
   */
  public void setItemMustNotExist(boolean f)
  {
    setItemMustExist(!f);
  }

  /**
  * Store into the directory.
  *
  * @param ctx       context on which to issue store
  * @param id        security identity
  * @param password  security password
  * @exception  RomaError is thrown on operation failure.
  */
  void storeItem( RomaContext ctx,
                  String      id,
                  String      password )
    throws RomaError
  {
    RomaContext.kxcOper(
      ctx.kxcCtx().dirStoreItem( m_buffer,
                                 m_bufferLen,
                                 m_storeop,
                                 id,
                                 password ));
  }
}
