//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaExceptionOptions.java
//
// Defines      : class RomaExceptionOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates the options when putting a
* RomaExceptionMessage on an Exception Message Queue.
*
* @see RomaExceptionMessage
*/
public class RomaExceptionOptions
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  private String m_serviceName = null;
  private RomaPutOptions m_po = new RomaPutOptions(true); 
  private RomaTheme  m_theme = null;
  
  /**
  * Constructs a new RomaExceptionOptions object with default
  * options.
  */
  public RomaExceptionOptions()
  {
    setRetainTheme(true);
    m_serviceName = null;
  }
  
  /**
  * The name of the Business Service that defines the exception
  * message queue. If this is set to be null, the name of the 
  * service is retrieved from the subnet stanza entry in the
  * directory configuration.
  */
  public void setServiceName( String serviceName )
  {
    m_serviceName = serviceName;
  }
  
  /**
  * Returns the exception business service name.
  * Package visibilty
  */
  String serviceName()
  {
    return m_serviceName;
  }
  
  //== PUT OPTIONS ========================================================
  
  /**
  * Sets the put selector. This specifies which identifiers
  * contained in the message should be used when the 
  * message is put.
  * @param sel the new value of the put selector.
  * <p>
  * The value will be one of the following fields :-
  * <ul>
  *   <li>USE_NONE
  *   <li>USE_MSGID
  *   <li>USE_CORRELID
  *   <li>USE_MSGID_AND_CORRELID
  * </ul>
  */
  public void setSelector( int sel )
  { 
    m_po.setSelector( sel ); 
  }
  
  /**
  * Controls whether an MSMQ message can be placed outside syncpoint
  * control. If the application attempts to put a message to MSMQ inside  
  * syncpoint control but the target queue does not support transactions,
  * the call is normally rejected with error code RC_NONXACTIONAL.
  * The application can call this method with true flag to force an MSMQ
  * message to be placed outside syncpoint control.
  * @param flag  one of :-
  * <ul>
  *  <li>true - allow MSMQ messages to be placed outside syncpoint control
  *  <li>false - don't allow MSMQ messages to be placed outside syncpoint
  *              control
  * </ul>
  */
  public void setNoSyncAllowed( boolean flag )
  { 
    m_po.setNoSyncAllowed( flag ); 
  }
  
  /**
  * Associates an API profile with the put operation.
  *
  * @param profile  the name of the API profile.
  */
  public void setApiProfileName( String profile )
  { 
    m_po.setApiProfileName( profile ); 
  }
  
  /**
  * Associates an error profile with the put operation.
  *
  * @param profile  the name of the error profile.
  */
  public void setErrorProfileName( String profile )
  { 
    m_po.setErrorProfileName( profile ); 
  }
  
  /**
  * Indicates that flags and options specified in the API profile
  * should be used for the Roma put operation.
  * <p>
  * If this option is chosen but the specified profile is not
  * available then default values are used for the put options.
  */
  public void setUseApiProfile( boolean flag )
  { 
    m_po.useApiProfile( flag ); 
  }
  
  /**
  * Enables or disables the option to reset the retry count 
  * when sending the message.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> increment the retry
  *                count when sending the message.
  *    <li> false - reset the retry count when sending the message.
  * </ul>
  */
  public void setIncrementRetryCount(boolean select)
  { 
    m_po.incrementRetryCount( select ); 
  }
  
  
  /**
  * Enables or disables the option to send the message to an
  * exception message service if the put operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> perform a send to an
  *                exception message service if put fails.
  *    <li> false - allow a message to be sent to an exception
  *                 message service if the put fails.
  * </ul>
  */
  public void setNoEms( boolean select )
  { 
    m_po.setNoEms( select ); 
  }
  
  /**
  * Enables or disables the option to send a notification message
  * back to the sender if the message get operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true  - <strong>do </strong> allow a notification
  *                 message to be sent.
  *    <li> false - <strong>do not</strong> allow a notification
  *                 message to be sent on a get failure.
  * </ul>
  */
  public void setNoNotification( boolean select )
  { 
    m_po.setNoNotification( select ); 
  }
  
  /**
  * Enables or disables the option to send the message to a 
  * message transformation service if the next component in the
  * workflow cannot accept the message format.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> allow the message to
  *                sent to the transformation service.
  *    <li> false - allow the message to be routed via a
  *                 transformation service if necessary.
  * </ul>
  */
  public void setNoTransformation( boolean select )
  { 
    m_po.setNoTransformation( select ); 
  }
  
  /**
  * Specify if termination is supported
  *   
  * @param flag true to indicate termination is supported.
  */
  public void setTermSupported( boolean flag )
  { 
    m_po.setTermSupported( flag ); 
  } 
  
  /**
  * Specify whether a new theme should be allocated.
  * <p>
  * @param flag  if true then a new theme is allocated.
  */
  public void setNewTheme(boolean flag) 
  { 
    m_po.setNewTheme( flag ); 
  }
  
  /**
  * Specify whether the theme should be retained for future operations.
  * <p>
  * @param flag  if true then the theme is retained.
  */
  public void setRetainTheme(boolean flag)
  { 
    m_po.setRetainTheme( flag ); 
  }
  
  /**
  * Specify whether the theme should be closed automatically when the
  * operation completes.
  * <p>
  * @param flag  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean flag)
  { 
    m_po.setCloseTheme( flag ); 
  }
  
  /**
  * Specify whether an existing theme should be used.
  * <p>
  * @param flag  if true then the existing theme is used.
  */
  public void setUseTheme(boolean flag)
  { 
    m_po.setUseTheme(flag); 
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_po.setSyncpointDef( sync );
  }
  
  /**
  * Sets the theme.
  *
  * @param theme  the new theme.
  */
  public void setTheme( RomaTheme theme )
  {
    m_theme = theme;
  }
  
  //== END OF OPTIONS RELATED METHODS =================================
  
  RomaPutOptions putOptions() 
  { 
    return m_po; 
  }
  
  RomaTheme theme() 
  { 
    return m_theme; 
  }
}
