//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaGetOptions.java
//
// Defines      : class RomaGetOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* The RomaGetOptions class encapsulates the criteria by which a
* Business Element retrieves a Client request or a Client
* retrieves a Business Service response.
*/
public class RomaGetOptions 
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // get options has a BaseGetOptions with package visibility
  RomaBaseGetOptions m_go = new RomaBaseGetOptions();
  
  /**
  * Constructs a new RomaGetOptions object with default options.
  */
  public RomaGetOptions()
  {
  }
  
  /**
  * Constructs a new RomaGetOptions object initialising
  * the theme, message id and correlation id from the
  * supplied RomaClientRequest
  * 
  * @param RomaClientRequest
  */
  public RomaGetOptions(RomaClientRequest rc)
  {
    m_go.setMsgId   ( rc.msgId() );
    m_go.setCorrelId( rc.correlId() );
    m_go.setSelector( Roma.BY_THEME );	  
    m_go.setTheme   ( rc.theme() );
  }
  
  /**
  * Enables or disables the option to retrieve native (ie, non-Roma)
  * messages.
  * <p>
  * 
  * @param select  one of :-
  * <ul>
  *   <li>true - allow native messages.
  *   <li>false - disallow native messages.
  * </ul>
  */
  public void allowNative( boolean b )
  {
    if (b)
      m_go.m_opts |= Kxc.GOPT_ALLOWNATIVE;
    else
      m_go.m_opts &= ~Kxc.GOPT_ALLOWNATIVE;
  }
  
  /**
  * Specify if termination is supported
  *   
  * @param b true to indicate termination is supported.
  */
  public void setTermSupported( boolean b )
  {
    if (b)
      m_go.m_opts |= Kxc.GOPT_ALLOWTERMINATE;
    else
      m_go.m_opts &= ~Kxc.GOPT_ALLOWTERMINATE;
  } 
  
  /**
  * Specify if error profiling is to be suppressed
  *   
  * @param b true to suppress error profiling
  */
  public void setSuppressErrorProfile( boolean b )
  {
    if (b)
      m_go.m_opts |= Kxc.GOPT_SUPP_ERROR;
    else
      m_go.m_opts &= ~Kxc.GOPT_SUPP_ERROR;
  } 
  
  /**
  * Sets the security definition. Subsequent get()
  * operations will pass this to Roma inline services.
  *
  * @param sec new security definition. May be null.
  *
  * @see RomaSecurityDef
  */
  public void setSecurityDef( RomaSecurityDef secDef )
  {
    m_go.m_sec = secDef;
  }
  
  /**
  * Set outgoing ILS data
  *
  * @param  ilsData The ILS data to be sent with message 
  */
  public void setILSData(RomaILSData ilsData)
  {							 
    m_go.m_outgoingILSDataHandle = ilsData.getILSDataHandle( );
  }
  
  /**
  * Sets the theme.
  *
  * @param control  new theme control value, one of :-
  *                <ul>
  *                <li> THEME_RETAIN
  *                <li> THEME_CLOSE
  *                </ul>
  */
  public void setThemeControl( int control)
  {
    m_go.m_opts &= ~(Roma.THEME_RETAIN | Roma.THEME_CLOSE);
    m_go.m_opts |= control;
  }
  
  /**
  * Sets the subject style for any receiving publications
  *
  * @param style  the style to be used, one of :-
  *                <ul>
  *                <li> STYLE_ROMA
  *                <li> STYLE_MQSERIES
  *                <li> STYLE_TIBRENDEZVOUS
  *                </ul>
  */
  public void setSubjectStyle( int style)
  {
    m_go.m_subjectStyle = style;
  }
  
  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options
  * 
  * @param flag  if true theme already associated with the 
  * RomaMessage will be used. If false the new theme 
  * allocation will be done.
  */
  public void setUseTheme(boolean b)
  {
    m_go.setUseTheme( b );
  }
  
  /**
  * Selects whether not to get message by matching theme
  * when applying these options
  * 
  * @param flag  if true the theme will be ignored  
  *              if false the theme will be matched 
  *  
  */
  public void setAnyTheme(boolean b)
  {
    m_go.setUseTheme(!b);	
  }
  
  /**
  * Specify whether the theme should be closed automatically when the
  * operation completes.
  * <p>
  * @param b  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean b)
  {
    if (b) {
      m_go.m_opts |=  Kxc.GOPT_CLOSETHEME;
      m_go.m_opts &= ~Kxc.GOPT_RETAINTHEME;
    }
    else {
      m_go.m_opts &= ~Kxc.GOPT_CLOSETHEME;
      m_go.m_opts |=  Kxc.GOPT_RETAINTHEME;
    }
  }
  
  /**
  * Specify whether the theme should be retained for future operations.
  * <p>
  * @param b  if true then the theme is retained.
  */
  public void setRetainTheme(boolean b)
  {
    setCloseTheme(!b);
  }
  
  /**
  * Sets the API profile name.
  * 
  * @param name  the name of the API profile.
  */
  public void setApiProfileName( String name )
  {
    m_go.setApiProfileName( name );
  }
  
  /**
  * Indicates that flags and options specified in the API profile
  * should be used for the Roma put operation.
  * <p>
  * If this option is chosen but the specified profile is not
  * available then default values are used for the put options.
  */
  public void useApiProfile( boolean b )
  {
    m_go.useApiProfile( b );
  }
  
  /**
  * Sets the error profile name.
  * 
  * @param name  the name of the error profile.
  */
  public void setErrorProfileName( String name )
  {
    m_go.setErrorProfileName( name );
  }
  
  /**
  * Set only the receive options of the object.
  *
  * @param op  the new message receive options. It is expected that
  *            this will be one of the following values :-
  *            <ul>
  *            <li> GET_FIRST
  *            <li> PEEK_FIRST
  *            <li> PEEK_NEXT
  *            <li> PEEK_SAME
  *            <li> GET_PEEKED
  *            </ul>
  */
  public void setOperation( int op )
  {
    m_go.setOperation( op );
  }
  
  /**
  * Sets the message selection criteria.
  *
  * @param sel  the new message selection options.
  *             It is expected that
  *             this will be one of the following values :-
  *             <ul>
  *             <li> BY_NONE
  *             <li> BY_THEME
  *             <li> BY_MSGID
  *             <li> BY_CORRELID
  *             <li> BY_MSGID_AND_CORRELID
  *             <li> BY_MSGID_AND_THEME
  *             </ul>
  * 
  */
  public void setSelector( int sel )
  {
    m_go.setSelector( sel );
  }
  
  /**
  * Sets the theme.
  *
  * @param theme  the new theme.
  * 
  */
  public void setTheme( RomaTheme theme )
  {
    m_go.setTheme( theme );
  }
  
  /**
  * Sets the message identifier.
  *
  * @param msgId  the new message identifier value
  *
  */
  public void setMsgId(byte[] msgId)
  {
    m_go.setMsgId( msgId );
  }
  
  /**
  * Sets the correlation identifier.
  *
  * @param correlId  the new correlation identifier value.
  *
  */
  public void setCorrelId(byte[] correlId )
  {
    m_go.setCorrelId( correlId );  
  }
  
  /**
  * Sets the wait interval in milliseconds.
  *
  * @param waitInterval  the new wait interval value in milliseconds.
  *
  */
  public void setWaitInterval( int waitInterval )
  {
    m_go.setWaitInterval( waitInterval );
  }
  
  /**
  * Sets whether to wait without timing out
  *
  * @param flag  true for indefinite wait
  *
  */
  public void setWaitIndefinite( boolean flag )
  {
    m_go.setWaitIndefinite( flag );
  }
  
  /**
  * Specifies whether or not a get failure is to result in a 
  * message send to the Exception Message Queue. 
  * <p>
  * If an API profile is to be used ( selected by invoking the
  * useApiProfile() method and verifiable by querying the
  * isApiProfileUsed() method ) then this setting will be overridden
  * either by the profile or, if not found, the default for this 
  * option.
  *
  * @param b  true if no message should be sent to the Exception 
  *           Message Queue upon a get operation failure.
  */
  public void setNoEms( boolean b )
  {
    m_go.setNoEms( b );
  }
  
  /**
  * Specify if a notification message is to be sent to the original
  * sender if the get failed.
  * <p>
  * If an API profile is to be used ( selected by invoking the
  * useApiProfile() method and verifiable by querying the
  * isApiProfileUsed() method ) then this setting will be overridden
  * either by the profile or, if not found, the default for this 
  * option.
  *
  * @param b  true if no message should be sent to the sender 
  *           upon a get operation failure.
  */
  public void setNoNotification( boolean b )
  {
    m_go.setNoNotification( b );
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_go.setSyncpointDef( sync );
  }
  
  // end of base get options methoods	
  
  /**
  * get base options
  * 
  */
  RomaBaseGetOptions getBaseOptions()
  {
    return  m_go;	
  }
}
