//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// Module       : RomaLogInformation.java
//
// Defines      : class RomaLogInformation
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates the human-readable information that 
* can be logged to an error log or sent as an alert.
*/
public class RomaLogInformation
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constructs RomaLogInformation with a default message.
  */
  public RomaLogInformation()
  {
  }
  
  /**
  * Sets severity of the error.
  *
  * @param severity severity of the error. It is one of the
  * following constants:
  * <ul>
  * <li> SEV_INFO
  * <li> SEV_WARNING
  * <li> SEV_ERROR
  * <li> SEV_SEVERE
  * </ul>
  */
  public void setSeverity( int severity )
  {
    m_severity = severity;
  }
  
  /**
  * Sets format flags.
  *
  * @param format  new Roma format setting. Can be FORMAT_NONE or
  * any combination of the following constants :-
  * <ul>
  * <li>FORMAT_HEXDUMP 
  * <li>FORMAT_NOTIMESTAMP
  * <li>FORMAT_NOSEVERITY
  * </ul>
  */
  public void setFormat( int format )
  {
    m_format = format;
  }
  
  /**
  * Sets the user text, which will be included in the log or alert
  * message.
  * @param userText the user text
  */
  public void setUserText( String userText )
  {
    m_userText = userText;
  }
  
  /**
  * Sets the user data part of the log or alert message.
  * If format is not set to FORMAT_HEXDUMP, then the user data will
  * be included as a text string in the message. Otherwise it will
  * be included as hex dump.
  *
  * @param data  the new user data
  * @param length  the length of the new user data
  */
  public void setUserData( byte data[], 
                           int  length )
  {
    m_data        = data;
    m_dataLength  = length;
  }
  
  /**
  * Sets the user data part of the log or alert message.
  * If format is not set to FORMAT_HEXDUMP, then the user data will
  * be included as a text string in the message. Otherwise it will
  * be included as hex dump.
  *
  * @param String  the new user data
  */
  public void setUserData( String data )
  {
    m_data        = data.getBytes();
    m_dataLength  = data.length();
  }
  
  /**
  * Receives a KxcLogData object and sets it up using the state
  * of this object.
  *
  * @param peer KxcLogData instance - ie, an encapsulation
  *             of a KXCLOGDATA structure in the C level layer.
  * 
  * @see com.candle.roma.kxc.raw.KxcLogData
  */
  void setPeer( KxcLogData peer )
  {
    peer.setSeverity  ( m_severity );
    peer.setFormat    ( m_format );
    peer.setUserText  ( m_userText );
    peer.setData      ( m_data );
    peer.setDataLength( m_dataLength );
  }
  
  private int    m_severity   = Roma.SEV_INFO;
  private int    m_format     = Roma.FORMAT_HEXDUMP;
  private String m_userText   = null;
  private byte[] m_data       = null;
  private int    m_dataLength = 0;
  
}
