//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaPutOptions.java
//
// Defines      : class RomaPutOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates the options available when issuing
* a Roma message.
*/
class RomaPutOptions extends RomaBasePutOptions
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constructor.
  *
  * @param newTheme  if true, initialise the new RomaPutOptions
  *                  object to have the new theme option selected.
  */
  public RomaPutOptions( boolean newTheme )
  {
    super(newTheme);
  }
  
  /**
  * Associates an error profile with the put operation.
  *
  * @param profile  the name of the error profile.
  */
  public void setErrorProfileName( String profile )
  {
    m_errorProfileName = profile;
  }
  
  /**
  * Enables or disables the option to reset the retry count 
  * when sending the message.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> increment the retry
  *                count when sending the message.
  *    <li> false - reset the retry count when sending the message.
  * </ul>
  *
  */
  public void incrementRetryCount( boolean select )
  {
    if (select)
      m_opts |= Kxc.POPT_INCRETRYCOUNT;
    else
      m_opts &= ~Kxc.POPT_INCRETRYCOUNT;
  }
  
  /**
  * Enables or disables the option to send the message to an
  * exception message service if the put operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> perform a send to an
  *                exception message service if put fails.
  *    <li> false - allow a message to be sent to an exception
  *                 message service if the put fails.
  * </ul>
  */
  public void setNoEms( boolean select )
  {
    if (select)
      m_opts |= Kxc.POPT_NOEMS;
    else
      m_opts &= ~Kxc.POPT_NOEMS;
  }
  
  /**
  * Enables or disables the option to send a notification message
  * back to the sender if the message get operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true  - <strong>do not </strong> allow a notification
  *                 message to be sent.
  *    <li> false - <strong>do </strong> allow a notification
  *                 message to be sent on a get failure.
  * </ul>
  */
  public void setNoNotification( boolean select )
  {
    if (select)
      m_opts |= Kxc.POPT_NONOTIFY;
    else
      m_opts &= ~Kxc.POPT_NONOTIFY;
  }
  
  /**
  * Enables or disables the option to send the message to a 
  * message transformation service if the next component in the
  * workflow cannot accept the message format.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> allow the message to be
  *                sent to the transformation service.
  *    <li> false - allow the message to be routed via a
  *                 transformation service if necessary.
  * </ul>
  */
  public void setNoTransformation( boolean select )
  {
    if (select)
      m_opts |= Kxc.POPT_NOXFORM;
    else
      m_opts &= ~Kxc.POPT_NOXFORM;
  }
  
  /**
  * Specify if termination is supported
  *   
  * @param b true to indicate termination is supported.
  */
  public void setTermSupported( boolean b )
  {
    if (b)
      m_opts |= Kxc.POPT_ALLOWTERMINATE;
    else
      m_opts &= ~Kxc.POPT_ALLOWTERMINATE;
  } 
  
  /**
  * Specify if error profiling is to be suppressed
  *   
  * @param b true to suppress error profiling
  */
  public void setSuppressErrorProfile( boolean b )
  {
    if (b)
      m_opts |= Kxc.POPT_NOERRORPROF;
    else
      m_opts &= ~Kxc.POPT_NOERRORPROF;
  } 
  
  /**
  * Returns an indicator of whether the theme is retained
  *   
  */
  public boolean isThemeRetained() 
  {
    return (m_opts & Kxc.POPT_CLOSETHEME) == 0;
  }
  
  /**
  * Indicates that flags and options specified in the API profile
  * should be used for the Roma put operation.
  * <p>
  * If this option is chosen but the specified profile is not
  * available then default values are used for the put options.
  */
  public void useApiProfile( boolean b )
  {
    if (b)
      m_opts |= Kxc.POPT_USEDEFAULT;
    else
      m_opts &= ~Kxc.POPT_USEDEFAULT;
  }
  
  /**
  *  set or clear confirmation of delivery flag 
  */
  public void setCOD( boolean flag )
  {
    if ( flag )
      m_opts |= Kxc.POPT_COD;
    else
      m_opts &= ~Kxc.POPT_COD;
  }
  
  /**
  *  set or clear confirmation of arrival flag 
  */
  public void setCOA( boolean flag )
  {
    if ( flag )
      m_opts |= Kxc.POPT_COA;
    else
      m_opts &= ~Kxc.POPT_COA;
  }
  
  /**
  *  set or clear notification flag 
  */
  void setNotification( boolean flag )
  {
    if ( flag )
      m_opts |= Kxc.POPT_NOTIFY;
    else
      m_opts &= ~Kxc.POPT_NOTIFY;
  }
  
  /**
  *  set or clear datagram flag 
  */
  public void setDatagram( boolean flag )
  {
    if ( flag )
      m_opts |= Kxc.POPT_DATAGRAM;
    else
      m_opts &= ~Kxc.POPT_DATAGRAM;
  }
}
