//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaStorable.java
//
// Defines      : class RomaStorable
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;
import java.util.*;
import java.io.*;

/**
* This class extends the RomaMessage class to provide the ability
* to be used on storage queues. 
*/
public class RomaStorable 
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // RomaStorable has a RomaMessage
  // Package visibility so other classes can access
  RomaMessage m_Message = new RomaMessage();
  
  /**
  * Default constructor. New object has default attributes.
  */
  public RomaStorable()
  {
  }
  
  /**
  * Constructor which takes a RomaStorable object.
  * Package visibility
  */
  public RomaStorable( RomaStorable storable )
  {
    m_Message = storable.m_Message;
  }
  
  /**
  * Constructor which takes a RomaMessage object.
  * Package visibility
	 */
  RomaStorable( RomaMessage msg )
  {
    m_Message = msg;
  }
  
  /**
  * Constructor which takes a data buffer.
  *
  * @param msgBuffer - data buffer
  * @param msgLength - length of data in buffer
  */
  public RomaStorable ( byte[]  msgBuffer,
                        int     msgLength)
  {  
    setMsgBuffer( msgBuffer, msgLength );
  }
  
  /**
  * Called before the PUT operation this method
  * may be overriden in a derived class for
  * data marshalling.
  * 
  * @exception RomaError thrown if marshalling fails.
  */
  public void marshalForStorage()
    throws RomaError
  {
  }
  
  //== RomaMessage Access functions ==================================
  
  /**
  * Set the message priority.
  * @param priority  this can be one of the following :-
  * <ul>
  * <li>PRIORITY_DEFAULT - use the default priority
  *     from the transport.
  * <li>PRIORITY_PROFILE - use the priority as specified in the 
  *     associated Put API profile. If no such profile or attribute,
  *     use PRIORITY_DEFAULT.
  * <li>a specific integer meaningful to the underlying transport
  *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ 
  * </ul>
  */
  public void setPriority( int priority )
  {
    m_Message.setPriority(priority);
  }
  
  /**
  * Returns the priority assigned to the message.
  */
  public int priority()
  {
    return m_Message.priority();
  }
  
  /**
  * Set a new value for the message perisitence.
  * @param  persistence  this can be one of the following :-
  *   <ul>
  *   <li>PERSISTENCE_YES
  *   <li>PERSISTENCE_NO
  *   <li>PERSISTENCE_PROFILE - use the persistence value as
  *       specified in the associated Put API profile. If no such
  *       profile or attribute, use PERSISTENCE_NO.
  *   </ul>
  */
  public void setPersistence( int persistence )
  {
    m_Message.setPersistence(persistence);
  }
  
  /**
  * return the message perisitence value.
  */
  public int persistence()
  {
    return m_Message.persistence();
  }
  
  /**
  * Set a new message format value.
  * @param format  identifies the format of message. If the 
  *   value FORMAT_PROFILE is used then the message format value
  *   is retrieved from the associated Put API profile. If the
  *   profile is not available or else has no format attribute 
  *   value then the message format value FORMAT_BINARY is used.
  */
  public void setFormat( int format )
  {
    m_Message.setFormat(format);
  }
  
  /**
  * Return the message format.
  */
  public int format()
  {
    return m_Message.format();
  }
  
  /**
  * Sets a new message feedback code.
  * @param feedback  new message feedback value.
  */
  public void setFeedback( int feedback )
  {
    m_Message.setFeedback(feedback);
  }
  
  /**
  * Get the message feedback code.
  */
  public int feedback()
  {
    return m_Message.feedback();
  }
  
  /**
  * Set a new message expiry time in seconds.
  * @param expiry  the new expiry time value
  *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
  *   If EXPIRY_PROFILE is specified then
  *   the expiry value is retrieved from the associated Put API
  *   profile. If the profile is not available or else has no expiry
  *   attribute set, the EXPIRY_NEVER constant is used.
  */
  public void setExpiry( int expiry )
  {
    m_Message.setExpiry(expiry);
  }
  
  /**
  * Return the message expiry time in seconds.
  */
  public int expiry()
  {
    return m_Message.expiry();
  }
  
  /**
  * Set a new message codepage value. 
  * @param codepage  the new codepage value. If CODEPAGE_PROFILE is
  *   specified then the codepage will be retrieved from the 
  *   associated Put API profile. If the profile is not available or
  *   else has no codepage attribute set, the default value of 
  *   CODEPAGE_USER will be used.
  */
  public void setCodepage( int codepage )
  {
    m_Message.setCodepage(codepage);
  }
  
  /**
  * Return the message codepage.
  */
  public int codepage()
  {
    return m_Message.codepage();
  }
  
  /**
  * Set the message encoding.
  * @param  encoding  the new encoding indicator. If this is 
  *   ENCODING_PROFILE then the value is retrieved from the 
  *   associated Put API profile. If the profile is not available
  *   or else has no encoding attribute set, the default value
  *   ENCODING_DEFAULT is used.
  */
  public void setEncoding( int encoding )
  {
    m_Message.setEncoding(encoding);
  }
  
  /**
  * Get the message encoding.
  */
  public int encoding()
  {
    return m_Message.encoding();
  }
  
  /**
  * Set the message identifier.
  * @param  msgId  a byte array containing the new message id value.
  * The msg id must not exceed kxc.MSG_ID_LENGTH.
  */
  public void setMsgId( byte[] msgId )
  {
    m_Message.setMsgId(msgId);
  }
  
  /**
  * Return the message identifier in a byte array.
  */
  public byte[] msgId()
  {
    return m_Message.msgId();
  }
  
  /**
  * Set the message correlation identifier.
  * @param  correlId  the new correlation id.
  */
  public void setCorrelId( byte[] correlId )
  {
    m_Message.setCorrelId(correlId);
  }
  
  /**
  * Return the message correlation identifier.
  */
  public byte[] correlId()
  {
    return m_Message.correlId();
  }
  
  /**
  * Sets the format name.
  *
  * @param formatName the new format name.
  */
  public void setFormatName( String formatName )
  {
    m_Message.setFormatName( formatName );
  }
  
  /**
  * Return the message format name.
  */
  public String formatName()
  {
    return m_Message.formatName();
  }
  
  /**
  * Update the message theme.
  *
  * @param theme RomaTheme object.
  */
  public void setTheme( RomaTheme theme )
  {
    m_Message.setTheme( theme );
  }
  
  /**
  * Return the message theme. Theme can be set internally when
  * the message is being sent or received or else explicitly by
  * the user.
  */
  public RomaTheme theme()
  {
    return m_Message.theme();
  }
  
  /**
  * Return the backout count. This is the number 
  * of times that the message has been rolled back.
  */
  public int backoutCount()
  {
    return m_Message.backoutCount();
  }
  
  /**
  * Returns the time stamp which, for get operations, is the 
  * time at which the put operation occurred.
  */
  public Date timeStamp()
  {
    return m_Message.timeStamp();
  }
  
  /**
  * Set the name of this message
  * @param messageName name to be used
  */
  public void setMsgName( String messageName )
  {
    m_Message.setMessageName(messageName);
  }
  
  /**
  * Set buffer containing the message data and the length of the 
  * message for a PUT request. To send an empty message set length 
  * to 0 and buffer to null
  * 
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message. 
  * 
  * @return true if successfull,
  *         false if datalength < 0 or datalength = 0 and buffer != null  
  */
  public boolean setMsgBuffer( byte buffer[],
                               int  dataLength )
  {
    return m_Message.setMsgBuffer( buffer, dataLength );  
  }
  
  /**
  * Return reference to the message data buffer.
  */
  public byte[] msgBuffer()
  {
    return m_Message.msgBuffer();
  }
  
  /**
  * Return length of data in the message buffer.
  */
  public int msgDataLength()
  {
    return m_Message.msgDataLength();
  }
}
