//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaSyncpointDef.java
//
// Defines      : class RomaSyncpointDef
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates a Roma syncpoint definition.
* Syncpointing is used to guarantee that either all or no
* messages in a transaction are processed.
* <p>
* Roma supports the following types of syncpointing:-
* <ul>
*   <li>NONE - messages are not subject to syncpointing.
*   <li>LOCAL - Roma controls the syncpointing.
*   <li>GLOBAL - an application outside of Roma controls
*                syncpointing.
*   <li>PROFILE - Roma uses the default setting specified in
*                 the API profile.
* </ul>
* <p>
* If the API profile does not contain a value for this field, Roma
* uses the default (NONE).
*/
public class RomaSyncpointDef
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constructs a new RomaSyncpointDef object of the supplied
  * syncpoint type.
  * 
  * @param type  the type of syncpoint to use. This may be one
  *              of the following :-
  * <ul>
  *   <li> NONE
  *   <li> LOCAL
  *   <li> PROFILE
  * </ul>
  */
  public RomaSyncpointDef( int type )
  {
    m_type = type;
  }
  
  /**
  * Constructs a global syncpoint definition.
  *
  * @param  ctx  the context string
  *
  */
  RomaSyncpointDef( int type, int ctx )
  {
    m_type = Roma.SYNC_GLOBAL;
    m_ctx  = ctx;
  }
  
  /**
  * Sets the type of syncpoint.
  *
  * @param type  the new type of syncpointing to use.
  */
  public void setType( int type )
  {
    m_type = type;
  }
  
  /**
  * Sets the context if GLOBAL syncpointing is in effect. The
  * context is passed to the transport layer.
  *
  * @param ctx  the new context value.
  */
  public void setContext( byte[] ctx )
  {
    // TODO Need to work out how to pass a void pointer 
  }
  
  /**
  * Creates and returns syncpoint definition of type LOCAL.
  */
  static RomaSyncpointDef local()
  {
    return new RomaSyncpointDef( Roma.SYNC_LOCAL );
  }
  
  /**
  * Creates and returns syncpoint definition of type NONE.
  */
  static RomaSyncpointDef none()
  {
    return new RomaSyncpointDef( Roma.SYNC_NONE );
  }
  
  /**
  * Creates and returns syncpoint definition of type PROFILE.
  */
  static RomaSyncpointDef profile()
  {
    return new RomaSyncpointDef( Roma.SYNC_PROFILE );
  }
  
  /**
  * Creates and returns syncpoint definition of type GLOBAL.
  */
  static RomaSyncpointDef global( int ctx )
  {
    return new RomaSyncpointDef( Roma.SYNC_GLOBAL, ctx );
  }
  
  /**
  * Returns the type of syncpoint.
  */
  int type()
  {
    return m_type;
  }
  
  /**
  * Returns the context if GLOBAL syncpointing is in effect.
  */
  int context()
  {
    return m_ctx;
  }
  
  /**
  * Sets the context if GLOBAL syncpointing is in effect. The
  * context is passed to the transport layer.
  *
  * @param ctx  the new context value.
  */
  void setContext( int ctx )
  {
    m_ctx = ctx;
  }
  
  private int m_type = 0;
  private int m_ctx  = 0;
}
