//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcContext.java
//
// Defines      : class KxcContext
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;
import  com.candle.roma.kxc.RomaInteger;

import  java.util.Vector;

/**
* Raw Roma context abstraction.
* Provides base for client/server raw context abstractions.
* Maintains KxcOper object which carries out a particular
* operation.
* Implements common set of operations.
* Provides the user with refernces to KxcStruct derived
* object which is used to carry out operation params.
*
* @see KxcOper
*/

abstract public class KxcContext implements KxcNativeInterface
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  KxcContext( KxcOper oper )
  {
    m_oper        = oper;
    m_ip          = oper.m_ip;
    m_op          = oper.m_op;
    m_lp          = oper.m_lp;
    m_msg         = new KxcMsg( m_oper );
    m_getCtrl     = new KxcGetCtrl( m_oper );
    m_putCtrl     = new KxcPutCtrl( m_oper );
    m_reportCtrl  = new KxcReportCtrl( m_oper );
    m_exception   = new KxcException( m_oper );
    m_msgInfo     = new KxcMsgInfo( m_oper );
    m_logData     = new KxcLogData( m_oper );
  }
  
  // returns initilization operation object.
  // the operation details are setup during
  // object construction.
  public KxcOper init()
  {
    return m_oper;
  }
  
  // configures and returns abstract get operation
  // since its invocation semantics is common between
  // server and client
  abstract public KxcOper get();
  
  // configures and returns kxcCloseHandle operation.
  public KxcOper closeHandle( long handle )
  {
    m_lp[HOBJ]    = handle;
    m_ip[OP_CODE] = NIOP_CLOSE_HANDLE;
    return m_oper;
  }
  
  // configures and returns kxcCommit operation.
  public KxcOper commit( int trType, 
                         int trCtx )
  {
    m_ip[TR_TYPE] = trType;
    m_ip[TR_CTX]  = trCtx;
    m_ip[OP_CODE] = NIOP_COMMIT;
    return m_oper;
  }
  
  // configures and returns kxcRollback operation.
  public KxcOper rollback( int trType, 
                           int trCtx )
  {
    m_ip[TR_TYPE] = trType;
    m_ip[TR_CTX]  = trCtx;
    m_ip[OP_CODE] = NIOP_ROLLBACK;
    return m_oper;
  }
  
  // configures and returns kxcDeleteStorageQueue operation.
  public KxcOper deleteQueue( String qname )
  {
    m_op[NAME]    = qname;
    m_ip[OP_CODE] = NIOP_DELETEQ;
    return m_oper;
  }
  
  // configures and returns kxcOpenStorageQueue operation.
  // after the operation has been invoked, the user should
  // obtain the queue handle with hStore() method.
  public KxcOper openStorageQ( String qname, 
                               int opt, 
                               String modelQName )
  {
    m_ip[OPT]         = opt;
    m_op[NAME]        = qname;
    m_op[MODEL_QUEUE] = modelQName;
    m_ip[OP_CODE]     = NIOP_OPEN_STORAGEQ;
    return m_oper;
  }
  
  // returns handle of the storage queue requested
  // by openStorageQ() operation.
  public long hStore()
  {
    return m_lp[HQUEUE];
  }
  
  // configures and returns kxcGetFromStorageQueue operation
  public KxcOper getFromStorageQ( long hStorageQ )
  {
    m_lp[HQUEUE]  = hStorageQ;
    m_ip[OP_CODE] = NIOP_GET_STORAGEQ;
    return m_oper;
  }
  
  // configures and returns kxcPutToStorageQueue operation
  public KxcOper putToStorageQ( long hStorageQ )
  {
    m_lp[HQUEUE]  = hStorageQ;
    m_ip[OP_CODE] = NIOP_PUT_STORAGEQ;
    return m_oper;
  }
  
  // configures and returns kxcCloseStorageQueue operation
  public KxcOper closeStorageQ( long hStorageQ )
  {
    m_lp[HQUEUE]  = hStorageQ;
    m_ip[OP_CODE] = NIOP_CLOSE_STORAGEQ;
    return m_oper;
  }
  
  // configures and returns kxcTerminate operation
  public KxcOper disconnect()
  {
    m_ip[OP_CODE] = NIOP_TERMINATE;
    return m_oper;
  }
  
  // configures and returns kxcPutException operation
  public KxcOper putException( String service )
  {
    m_op[NAME]    = service;
    m_ip[OP_CODE] = NIOP_PUTEXCEPTION;
    return m_oper;
  }
  
  // configures and returns kxcReportError operation
  public KxcOper reportError( boolean reportMsg )
  {
    m_ip[REPORT_MSG]  = reportMsg ? 1: 0;
    m_ip[OP_CODE]     = NIOP_REPORTERROR;
    return m_oper;
  }
  
  // configures and returns kxcSendAlert operation
  public KxcOper sendAlert()
  {
    m_ip[OP_CODE] = NIOP_SENDALERT;
    return m_oper;
  }
  
  // returns get control object
  public KxcGetCtrl getCtrl()
  {
    return m_getCtrl;
  }
  
  // returns put control object
  public KxcPutCtrl putCtrl()
  {
    return m_putCtrl;
  }
  
  // returns message object
  public KxcMsg msg()
  {
    return m_msg;
  }
  
  // returns report control object
  public KxcReportCtrl reportCtrl()
  {
    return m_reportCtrl;
  }
  
  // returns exception object
  public KxcException exception()
  {
    return m_exception;
  }
  
  // returns message info object
  public KxcMsgInfo msgInfo()
  {
    return m_msgInfo;
  }
  
  // returns log data object
  public KxcLogData logData()
  {
    return m_logData;
  }
  
  // Directory access methods	
  
  public KxcOper dirLookUp( String  startingPoint,
                            String  searchName,	
                            int     searchType,
                            int     options )
  {
    m_op[DIR_START]     = startingPoint;
    m_op[DIR_NAME]      = searchName;
    m_ip[DIR_SRC_TYPE]  = searchType;
    m_ip[DIR_SRC_OPT]   = options;
    m_ip[OP_CODE]       = NIOP_DIRLOOKUP;
    return m_oper;
  }
  
  public int dirLookupReturn( RomaInteger nModCount )
  {
    nModCount.setValue( m_ip[DIR_SRC_COUNT] );

    return m_ip[DIR_SRC_MATCHES];
  }
  
  public KxcOper dirGetItem( int itemRequired ) 	
  {
    m_ip[DIR_ITEM_NBR]  = itemRequired;
    m_ip[OP_CODE]       = NIOP_DIRGETITEM;
    return m_oper;
  }
  
  public byte[] dirGetItemReturn( RomaInteger length ) 	
  {
    length.setValue(m_ip[DIR_ITEM_DETAIL_LENGTH]);
    return (byte[])m_op[DIR_ITEM_DETAIL];
  }
  
  public KxcOper dirModCount( ) 	
  {
    m_ip[OP_CODE] = NIOP_GETMODCNT_FROM_DIR;
    return m_oper;
  }
  
  public int dirModCountReturn( ) 	
  {
    return m_ip[DIR_SRC_COUNT];
  }
  
  public static String dirGetDN( byte[] buffer, 
                                 int    bufferLen ) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_ip[OP_CODE]                = NIOP_DIRGETDN;
    
    try {  
      oper.invoke();
      return (String)oper.m_op[DIR_DIST_NAME];
    }
    catch ( Exception x ) {
      return null;
    }
  }
  
  public static byte[] dirSetDN(byte[]  buffer, 
                                int     bufferLen, 
                                String  name ) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_op[DIR_DIST_NAME]          = name;
    oper.m_ip[OP_CODE]                = NIOP_DIRSETDN;
    
    try {  
      oper.invoke();
      return  (byte[])oper.m_op[DIR_ITEM_DETAIL];
    }
    catch ( Exception x ) {
      return null;
    }
  }
  
  public static int dirGetAttributes( byte[]  buffer, 
                                      int     bufferLen ) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_ip[OP_CODE]                = NIOP_DIRCNTATTRIB;
    
    try {  
      oper.invoke();
      return oper.m_ip[DIR_SRC_COUNT];
    }
    catch ( Exception x ) {
      return 0;
    }
  }
  
  public static String dirGetAttribValue( byte[]  buffer,
                                          int     bufferLen, 
                                          String  name, 
                                          int     nAttrib ) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_op[DIR_ATTR_NAME]          = name;
    oper.m_ip[DIR_ITEM_NBR]           = nAttrib;
    oper.m_ip[OP_CODE]                = NIOP_GETATTRIBVALUE;
    
    try {  
      oper.invoke();
      return (String)oper.m_op[DIR_ATTR_VALUE];
    }
    catch ( Exception x ) {
      return null;
    }
  }
  
  public static String dirGetAttribAt(byte[]  buffer,
                                      int     bufferLen, 
                                      int     nAttrib ) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_ip[DIR_ITEM_NBR]           = nAttrib;
    oper.m_ip[OP_CODE]                = NIOP_GETATTRIBAT;
    
    try {  
      oper.invoke();
      return (String)oper.m_op[DIR_ATTR_NAME];
    }
    catch ( Exception x ) {
      return null;
    }
  }
  
  public static String dirGetAttribAtValue( byte[] buffer, 
                                            int bufferLen,
                                            int nAttrib	) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_ip[DIR_ITEM_NBR]           = nAttrib;
    oper.m_ip[OP_CODE]                = NIOP_GETATTRIBAT;
    
    try {  
      oper.invoke();
      return (String)oper.m_op[DIR_ATTR_VALUE];
    }
    catch ( Exception x ) {
      return null;
    }
  }		
  
  public static byte[] dirAddAttr(byte[]  buffer, 
                                  int     bufferLen,
                                  String  name,
                                  String  value ) 	
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();
    
    oper.m_op[DIR_ITEM_DETAIL]        = buffer;
    oper.m_ip[DIR_ITEM_DETAIL_LENGTH] = bufferLen;
    oper.m_op[DIR_ATTR_NAME]          = name;
    oper.m_op[DIR_ATTR_VALUE]         = value;
    oper.m_ip[OP_CODE]                = NIOP_DIRADDATTR;
    
    try {  
      oper.invoke();
      return (byte[])oper.m_op[DIR_ITEM_DETAIL];
    }
    catch ( Exception x ) {
      return (byte[])oper.m_op[DIR_ITEM_DETAIL];
    }
  }
		
  public KxcOper dirStoreItem(byte[]  buffer, 
                              int     bufferLen,
                              int     options, 
                              String  id, 
                              String  password) 
  {
    m_op[DIR_ITEM_DETAIL]         = buffer;
    m_ip[DIR_ITEM_DETAIL_LENGTH]  = bufferLen;
    m_ip[DIR_SRC_OPT]             = options;
    m_op[DIR_IDENTITY]            = id;
    m_op[DIR_PASSWORD]            = password;	 
    m_ip[OP_CODE]                 = NIOP_DIRUPDATE;
    return m_oper;
  }
  
  public KxcOper dirDeleteItem( byte[]  buffer, 
                                int     bufferLen, 
                                int     options, 
                                String  id, 
                                String  password) 
  {
    m_op[DIR_ITEM_DETAIL]         = buffer;
    m_ip[DIR_ITEM_DETAIL_LENGTH]  = bufferLen;
    m_ip[DIR_SRC_OPT]             = options;
    m_op[DIR_IDENTITY]            = id;
    m_op[DIR_PASSWORD]            = password;	 
    m_ip[OP_CODE]                 = NIOP_DIRDELETE;
    return m_oper;
  }
  
  public static byte[] getILSBuffer( long ilsDataHandle,
                                     int  ilsId,
                                     int  type)
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();

    oper.m_ip[OP_CODE]          = NIOP_GETILSDATA_SPEC;
    oper.m_lp[ILS_DATA_HANDLE]  = ilsDataHandle;
    oper.m_ip[ILS_IDENTITY]     = ilsId;
    oper.m_ip[ILS_TYPE]         = type;
    
    try {  
      oper.invoke();
      return  (byte[])oper.m_op[ILS_BUFFER];		
    }
    catch ( Exception x ) {
      return null;
    }
  }
  
  public static byte[] getILSBuffer( long         ilsDataHandle,
                                     RomaInteger  ilsId,
                                     RomaInteger  type )
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();

    oper.m_ip[OP_CODE]          = NIOP_GETILSDATA_NEXT;
    oper.m_lp[ILS_DATA_HANDLE]  = ilsDataHandle;
    
    try {  
      oper.invoke();
      ilsId.setValue( oper.m_ip[ILS_IDENTITY] );
      type.setValue ( oper.m_ip[ILS_TYPE] );
      return  (byte[])oper.m_op[ILS_BUFFER];		
    }
    catch ( Exception x ) {
      return null;
    }
  }
  
  public static long setILSBuffer( long   ilsDataHandle,
                                   byte[] buffer,	
                                   int    ilsId,
                                   int    type,
                                   int    option)
  {
    KxcConnection f     = KxcConnection.nativeConnection();
    KxcOper       oper  = f.newOper();

    oper.m_ip[OP_CODE]          = NIOP_SETILSBUFFER;
    oper.m_lp[ILS_DATA_HANDLE]  = ilsDataHandle;
    oper.m_ip[ILS_IDENTITY]     = ilsId;
    oper.m_ip[ILS_TYPE]         = type;
    oper.m_ip[ILS_OPTION]       = option;
    oper.m_op[ILS_BUFFER]       = buffer;
    
    try {  
      oper.invoke();
      return  oper.m_lp[ILS_DATA_HANDLE];		
    }
    catch ( Exception x ) {
      return 0;
    }
  }
  
  public static KxcOper setSecurityBufferOper( int nItems )
  {
    KxcConnection f = KxcConnection.nativeConnection();
    // dynamically sized oper
    // 1 slot for opcode,
    // next nItems slots for length/data,
    // next nItems for ILS identities (only integer array is used).
    KxcOper oper = f.createNewOper( 1 + nItems * 2 );

    oper.m_op[0]        = null;
    oper.m_ip[OP_CODE]  = NIOP_SETSECURITYBUFFER;
    return oper;
  }
  
  public static void setSecurityBuffer( KxcOper oper, 
                                        int     nItems,
                                        int     nbrBuffer,
                                        int     nILS,
                                        byte[]  data, 
                                        int     length)
  {
    oper.m_op[nbrBuffer] = data;
    oper.m_ip[nbrBuffer] = length;
    oper.m_ip[nbrBuffer + nItems] = nILS;
  }
  
  public static byte[] getSecurityBuffer( KxcOper oper ) 
  {
    // buffer returned in first ILS data slot
    return  (byte[])oper.m_op[1];
  }
		
  public KxcOper queryContext( ) 	
  {
    m_ip[OP_CODE] = NIOP_QUERY_CONTEXT;
    return m_oper;
  }
  
  public boolean queryContextReturn( ) 	
  {
    return m_lp[ HCTX ] != 0;
  }

  public long getContextHandle()
  {
    return m_lp[HCTX];
  }
		
  KxcOper       m_oper;
  KxcMsg        m_msg;
  KxcGetCtrl    m_getCtrl;
  KxcPutCtrl    m_putCtrl;
  KxcReportCtrl m_reportCtrl;
  KxcException  m_exception;
  KxcMsgInfo    m_msgInfo;
  KxcLogData    m_logData;
  
  int           m_ip[];
  Object        m_op[];
  long          m_lp[];
}
