//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcjnia.cpp
//
// Description  : Java native library (JNI)
//
// Implements   : class RomaJxEnv, JByteArray, JString
//                This file contains classes which manage the transfer
//                of data between the Java and C layers.
//                This file contains the JNI calls to translate data
//                from java encoding to native encoding and back.
//
//--------------------------------------------------------------------

// KXC includes
#include <kxcenv.h>
#include <kxcutl.h>

// KXC JNI includes
#include "kxcjbnxa.hpp"

// Standard includes
#include <string.h>
#include <stdio.h>

TRACE_MODULE_VERSION($Revision: 1.1.1.1.8.2.2.4 $);

// Method entry trace macro (traces this pointer automatically).
#define TRACE_METHOD_ENTRY( method, args )     \
  TRACE_ENTRY( method, << TRACE( this ) args )

// Assertion method handling errors
// Note: Currently, it traces and prints error and invoke system exit call.
void jniAssertionFailed( const char*  msg, 
                         const char*  file, 
                         size_t       line )
{
  TRACE_ENTRY( jniAssertionFailed, << TRACE(msg) 
                                   << TRACE(file) 
                                   << TRACE(line) );
  printf( "%s(%i) : kxcjni assertion : '%s'", file, line, msg );
  exit( 0 );
  
  TRACE_VOID_RETURN;
}

// Assertion macro.
#define JNI_ASSERT( exp )                                               \
  if ( (exp) == 0 ) { jniAssertionFailed( #exp, __FILE__, __LINE__ ); } 
  
//--------------------------------------------------------------------
// Class : RomaJxEnv
// Encapsulates the data transferred between the Java and C layers.
// To transfer data between the Java and C layers, two arrays are
// maintained:
// m_jop an array of java objects
// m_jip an array of java integers
//
// Native code accesses Java VM features using JNI functions.
// JNI functions are available through an interface pointer.
// This pointer is stored in member variable m_env.
// m_ip is the native encoding of java integer array m_jip.
//--------------------------------------------------------------------

//=====================================================================
// Method name : RomaJxEnv::RomaJxEnv
// Description : Ctor - Initialise member variables
//---------------------------------------------------------------------
RomaJxEnv::RomaJxEnv() :
  m_env( 0 ), 
  m_jip( 0 ), 
  m_jop( 0 ), 
  m_ip( 0 ), 
  m_ipSize( 0 ),
  m_lp( 0 ), 
  m_lpSize( 0 )
{
}

//=====================================================================
// Method name : RomaJxEnv::~RomaJxEnv
// Description : Dtor - Copy any changes to m_ip back to m_jip.
//---------------------------------------------------------------------
RomaJxEnv::~RomaJxEnv()
{
  TRACE_METHOD_ENTRY( RomaJxEnv::~RomaJxEnv, << "" );
  if ( m_ipSize > 0 )
  {
    m_env->SetIntArrayRegion( m_jip, 0, m_ipSize, m_ip );
  }
  if ( m_lpSize > 0 )
  {
    m_env->SetLongArrayRegion( m_jlp, 0, m_lpSize, m_lp );
  }
  delete m_ip;
  delete m_lp;

  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : RomaJxEnv::intParam
// Description : Return the native integer at index i
// Return type : jint& 
//---------------------------------------------------------------------
jint& RomaJxEnv::intParam( jsize i )
{
  TRACE_METHOD_ENTRY( RomaJxEnv::intParam, << TRACE( i ) );
  // boundary check
  JNI_ASSERT( i >= 0 && i < m_ipSize && m_ip != 0 );

  TRACE_RETURN( m_ip[i] );
}

//=====================================================================
// Method name : RomaJxEnv::longParam
// Description : Return the native long index i
// Return type : jlong& 
//---------------------------------------------------------------------
jlong& RomaJxEnv::longParam( jsize i )
{
  TRACE_METHOD_ENTRY( RomaJxEnv::longParam, << TRACE( i ) );
  // boundary check
  JNI_ASSERT( i >= 0 && i < m_lpSize && m_lp != 0 );
  // TRACE_RETURN does not compile correctly on NT
  // due to fix applied to make it work on Digital Unix
  // We're only using longs for handles which can 
  // be traced in the pipe, so don't trace here.
  //TRACE_RETURN( m_lp[i] );
  
  return m_lp[i];
}

//=====================================================================
// Method name : RomaJxEnv::byteArrayParam
// Description : This method initializes instance of a JByteArray
//               The instance is initialized by retrieving the java 
//               object array element at index i, casting this java 
//               object to a java byte array, and retaining in the 
//               client's JByteArray instance
// Return type : void 
//---------------------------------------------------------------------
void RomaJxEnv::byteArrayParam(jsize        i, 
                               JByteArray&  jb)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::byteArrayParam, << TRACE(i) 
                                                 << TRACE_HEX(&jb) );
  jb.retain( m_env, (jbyteArray)objParam( i ) );
  
  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : RomaJxEnv::newByteArrayParam
// Description : This method constructs a byte array of size 'size'
//               The array is added to the maintained object array 
//               m_jobj at index i and retained in the client's 
//               JByteArray instance
// Return type : void 
//---------------------------------------------------------------------
void RomaJxEnv::newByteArrayParam(jsize       i, 
                                  jsize       size, 
                                  JByteArray& jb)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::newByteArrayParam, << TRACE( i ) 
                                                    << TRACE( size ) 
                                                    << TRACE_HEX( &jb ) );
  jbyteArray jarr = m_env->NewByteArray( size );
  TRACE_EVENT( << TRACE_HEX( jarr ) );
  setObjParam( i, jarr );
  jb.retain( m_env, jarr );
  
  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : RomaJxEnv::strParam
// Description : This method initializes a JString using the element 
//               at index i of the object array m_jobj.
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxEnv::strParam(jsize    i, 
                         JString& js)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::strParam, << TRACE( i ) 
                                           << TRACE_HEX( &js ) );
  js.retain( m_env, (jstring)objParam( i ));
  
  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : RomaJxEnv::setStrParam
// Description : This method constructs a java string using the input 
//               string. The java string is then added to this classes 
//               object array, m_jobj at index i.
// Return type : void 
//---------------------------------------------------------------------
void RomaJxEnv::setStrParam(jsize       i, 
                            const char* str)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::setStrParam, << TRACE( i ) 
                                              << TRACE_HEX( str ) );
  jstring jstr;
#ifndef __MVS__
  jstr = m_env->NewStringUTF( str );
#else
  NewStringPlatform( m_env, str, &jstr, NULL );
#endif
  TRACE_EVENT( << TRACE_HEX( jstr ) );
  setObjParam( i, jstr );
  
  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : RomaJxEnv::objParam
// Description : Returns the java object element at index i
// Return type : jobject 
//---------------------------------------------------------------------
jobject RomaJxEnv::objParam(jsize i)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::objParam, << TRACE( i ) );
  JNI_ASSERT( i < m_ipSize );
  jobject ret = m_env->GetObjectArrayElement( m_jop, i );
  TRACE_EVENT( << TRACE_HEX( ret ) );
  
  TRACE_RETURN( ret );
}

//=====================================================================
// Method name : RomaJxEnv::hasObjParam
// Description : Tests whether there is a valid (not null) java object 
//               element at index i
// Return type : int 
//---------------------------------------------------------------------
int RomaJxEnv::hasObjParam(jsize i)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::hasObjParam, << TRACE( i ) );
  JNI_ASSERT( i < m_ipSize );
  int ret = m_env->GetObjectArrayElement( m_jop, i ) != 0;
  
  TRACE_INT_RETURN( ret );
}

//=====================================================================
// Method name : RomaJxEnv::setObjParam
// Description : This method updates the java object array member 
//               variable. The element at index i is set to obj.
// Return type : void 
//---------------------------------------------------------------------
void RomaJxEnv::setObjParam(jsize   i, 
                            jobject obj)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::setObjParam, << TRACE( i )
                                              << TRACE_HEX( obj ) );
  JNI_ASSERT( i < m_ipSize );
  m_env->SetObjectArrayElement( m_jop, i, obj );
  
  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : RomaJxEnv::init
// Description : Initialise member variables
//               Additionallly set member variable m_ip to hold a 
//               native encoding copy of java integer array m_jip
// Return type : void 
//---------------------------------------------------------------------
void RomaJxEnv::init(JNIEnv      *env, 
                     jintArray    jip, 
                     jobjectArray jop , 
                     jlongArray   jlp)
{
  TRACE_METHOD_ENTRY( RomaJxEnv::init, << TRACE_HEX( env ) 
                                       << TRACE_HEX( jip ) 
                                       << TRACE_HEX( jop ) );
  JNI_ASSERT( m_env == 0 );
  m_env = env;
  m_jip = jip;
  m_jop = jop;
  m_jlp = jlp;
  m_ipSize = m_env->GetArrayLength( m_jip );
  m_lpSize = m_env->GetArrayLength( m_jlp );
  if ( m_ipSize > 0 )
  {
    m_ip = new jint[ m_ipSize ];
    m_env->GetIntArrayRegion( m_jip, 0, m_ipSize, m_ip );
  }
  if ( m_lpSize > 0 )
  {
    m_lp = new jlong[ m_lpSize ];
    m_env->GetLongArrayRegion( m_jlp, 0, m_lpSize, m_lp );
  }

  TRACE_VOID_RETURN;
}


//--------------------------------------------------------------------
// Class : JByteArray
// Encapsulates a Java byte array and its corresponding native encoding
// This constructor initialises member variables using input parms
// env and jarr.
// if member variable m_arr is already pointing to a native byte array
// then this memory is released before setting m_arr to the native
// encoding of m_jarr
//--------------------------------------------------------------------

// Macro tracing content of JByteArray instance
#define TRACE_JBYTE_ARRAY_CONTENT \
  TRACE_HEX( m_env)   <<          \
  TRACE_HEX( m_jarr ) <<          \
  TRACE_HEX( m_arr )  <<          \
  TRACE( m_arrSize )

//=====================================================================
// Method name : JByteArray::JByteArray
// Description : Ctor - Initialise all member variables
//---------------------------------------------------------------------
JByteArray::JByteArray() :
  m_env( 0 ),
  m_jarr( 0 ),
  m_arr( 0 ),
  m_arrSize( 0 )
{
}

//--------------------------------------------------------------------
// Inform the VM that access to native byte array m_arr is no longer
// needed. Also copy any changes to native array m_arr back to its
// java representation m_jarr and releases native cache buffer for
// java byte array.
//--------------------------------------------------------------------

//=====================================================================
// Method name : JByteArray::~JByteArray
// Description : Dtor - Inform the VM that access to native byte array
//               m_arr is no longer needed. Also copy any changes to 
//               native array m_arr back to its java representation 
//               m_jarr and releases native cache buffer for java byte 
//               array.
//---------------------------------------------------------------------
JByteArray::~JByteArray()
{
  release();
}

//=====================================================================
// Method name : JByteArray::retain
// Description : Retains content of Java byte array in the JByteArray 
//               instance. Initializes size of array.
// Return type : void 
//---------------------------------------------------------------------
void JByteArray::retain(JNIEnv*     env, 
                        jbyteArray  jarr)
{
  release();
  m_env = env;
  m_jarr = jarr;
  if ( m_jarr != 0 )
  {
    m_arrSize = m_env->GetArrayLength( m_jarr );
  }
}

//=====================================================================
// Method name : JByteArray::size
// Description : Returns the number of elements in the array
// Return type : jsize 
//---------------------------------------------------------------------
jsize JByteArray::size() const
{
  return m_arrSize;
}

//=====================================================================
// Method name : JByteArray::copyTo
// Description : Copies 'cnt' bytes of the java byte array, m_jarr to 
//               dest using GetByteArrayRegion JNI call.
// Return type : void 
//---------------------------------------------------------------------
void JByteArray::copyTo(void* dest, 
                        jsize cnt) const
{
  if ( m_jarr != 0 )
  {
    m_env->GetByteArrayRegion( m_jarr, 0, min( cnt, size() ), (jbyte*)dest );
  }
}

//=====================================================================
// Method name : JByteArray::copyFrom
// Description : Copies 'cnt' bytes of the input src to the java byte 
//               array, m_jarr using SetByteArrayRegion JNI call.
// Return type : void 
//---------------------------------------------------------------------
void JByteArray::copyFrom(const void* src, 
                          jsize       cnt)
{
  if ( cnt > 0 )
  {
    JNI_ASSERT( m_jarr != 0 && cnt <= size() );
    m_env->SetByteArrayRegion( m_jarr, 0, min( cnt, size() ), (jbyte*)src );
    // invalidate cached array
    delete m_arr;
    m_arr = 0;
  }
}

//=====================================================================
// Method name : JByteArray::mem
// Description : Returns pointer to cached native buffer which is copy
//               of java byte array object.
// Return type : jbyte* 
//---------------------------------------------------------------------
jbyte* JByteArray::mem()
{
  if ( m_jarr != 0 && m_arr == 0 )
  {
    m_arr = new jbyte[ size() ];
    m_env->GetByteArrayRegion( m_jarr, 0, size(), (jbyte*)m_arr );
  }

  return( m_arr );
}

//=====================================================================
// Method name : JByteArray::release
// Description : Inform the VM that access to the native byte array 
//               m_arr is no needed. m_arr is the native encoding of 
//               the java byte array m_jarr. Any changes to m_arr are 
//               copied back to m_jarr before m_arr is released. 
//               This allows the VM to freee the memory used by m_arr
// Return type : void 
//---------------------------------------------------------------------
void JByteArray::release()
{
  TRACE_METHOD_ENTRY( JByteArray::release, "" );
  delete m_arr;
  m_jarr = 0;
  m_arr = 0;
  m_arrSize = 0;

  TRACE_VOID_RETURN;
}


//--------------------------------------------------------------------
// Class JString
// Encapsulates a Java string and its native encoding equivalent.
//--------------------------------------------------------------------
#define TRACE_JSTRING_CONTENT \
  TRACE_HEX( m_env)   <<      \
  TRACE_HEX( m_jstr ) <<      \
  TRACE_HEX( m_str )

//=====================================================================
// Method name : JString::JString
// Description : Ctor - Initialises member variables
//---------------------------------------------------------------------
JString::JString() :
  m_env( 0 ),
  m_jstr( 0 ),
  m_str( 0 )
{
}

//=====================================================================
// Method name : JString::~JString
// Description : Dtor - Informs the VM that access to m_str is no 
//               longer needed so that any memory used by the string 
//               can be freed.
//---------------------------------------------------------------------
JString::~JString()
{
  TRACE_METHOD_ENTRY( JString::~JString, << TRACE_JSTRING_CONTENT );
  release();
  TRACE_EVENT( << TRACE_JSTRING_CONTENT );

  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : JString::retain
// Description : Assignment operator
//               Release any memory currently held by this instance.
//               set this instance to the java string contained within 
//               instance ano.
// Return type : void 
//---------------------------------------------------------------------
void JString::retain(JNIEnv*  env, 
                     jstring  jstr)
{
  TRACE_METHOD_ENTRY( JString::retain, << TRACE_HEX( env ) 
                                       << TRACE_HEX( jstr ) 
                                       << TRACE_JSTRING_CONTENT );
  release();
  m_env = env;
  m_jstr = jstr;
  if ( m_jstr != 0 )
  {
#ifndef __MVS__
    m_str = m_env->GetStringUTFChars( m_jstr, NULL );
#else
    jint outlen = 0;
    const char* encoding = NULL;
    GetStringPlatformLength( env, jstr, &outlen, encoding );
    if ( outlen > 0 )
    {
	m_str = new char[ outlen ];
    	GetStringPlatform( env, m_jstr, m_str, outlen, encoding );
    }
#endif
    TRACE_EVENT( << TRACE_HEX( m_str ) );
  }
  TRACE_EVENT( << TRACE_JSTRING_CONTENT );

  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : JString::release
// Description : If m_jstr is non-zero then inform the VM that access 
//               to m_str, the native encoding of m_jstr, is no longer 
//               needed.
//               The VM can then free the memory used by m_str.
// Return type : void 
//---------------------------------------------------------------------
void JString::release()
{
  TRACE_METHOD_ENTRY( JString::release, << "" );
#ifndef __MVS__
  if ( m_jstr != 0 )
  {
    m_env->ReleaseStringUTFChars( m_jstr, m_str );
  }
#else
  delete[] m_str;
#endif
  m_jstr = 0;
  m_str = 0;

  TRACE_VOID_RETURN;
}

//=====================================================================
// Method name : JString::length
// Description : Returns the length of the string
// Return type : jsize 
//---------------------------------------------------------------------
jsize JString::length() const
{
  return m_str != 0? strlen(m_str) : 0;
}

//=====================================================================
// Method name : JString::copy
// Description : Copies 'cnt' bytes to the destination
// Return type : void 
//---------------------------------------------------------------------
void JString::copy(char* dest, 
                   jsize cnt) const
{
  TRACE_METHOD_ENTRY( JString::copy, << TRACE_HEX( dest ) 
                                     << TRACE( cnt ) 
                                     << TRACE_JSTRING_CONTENT );
  if ( m_str != 0 )
  {
    strncpy( dest, m_str, cnt );
  }

  TRACE_VOID_RETURN;
}


//=====================================================================
// Method name : Java_com_candle_roma_kxc_raw_KxcNativeConnection_nativeOper
// Description : Entry point from Java code to native code.
//               This function is called from the Java code. It is 
//               responsible for converting the unicode Java 
//               parameters contained in the integer array jip and in 
//               the object array jop to native platform encoding and 
//               invoking the requested Pipe functions.
// Return type : jint
//---------------------------------------------------------------------
extern "C"
JNIEXPORT jint JNICALL Java_com_candle_roma_kxc_raw_KxcNativeConnection_nativeOper(
                                       JNIEnv*      env, 
                                       jclass       jcl, 
                                       jintArray    jip, 
                                       jobjectArray jop, 
                                       jlongArray   jlp)
{
  TRACE_ENTRY( Java_com_candle_roma_kxc_raw_KxcNativeConnection_nativeOper,
                                       << TRACE_HEX( env ) 
                                       << TRACE_HEX( jip ) 
                                       << TRACE_HEX( jop ) );
  RomaJxOper nx;
  nx.init( env, jip, jop, jlp );
  int rc = nx.doOper();

  TRACE_INT_RETURN( rc );
}


//=====================================================================
// Method name : Java_com_candle_roma_kxc_raw_KxcNativeConnection_nativeEncoding
// Description : This method returns the platform encoding
// Return type : jint 
//---------------------------------------------------------------------
extern "C"
JNIEXPORT jint JNICALL Java_com_candle_roma_kxc_raw_KxcNativeConnection_nativeEncoding(
                                       JNIEnv *env, 
                                       jobject)
{
  return RomaJxOper::encoding();
}
