//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// KXC JNI helper classes
//
//--------------------------------------------------------------------
// $Id: kxcjobj.cpp,v 1.1.2.7 2003/10/21 10:41:42 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/Attic/kxcjobj.cpp,v $
//--------------------------------------------------------------------

#ifdef __MVS__
#include <jni_convert.h>
#endif

#ifdef KXC_WINNT
#include <malloc.h>
#elif defined(KXC_AIX)
#include <stdlib.h>
#else
#include <alloca.h>
#endif

#include "kxcjobj.hpp"

#ifdef _VARARG_EXT_
#error _VARARG_EXT_ is defined
#endif


//--------------------------------------------------------------------
// KxcJEnv class
//--------------------------------------------------------------------

KxcJEnv::KxcJEnv(JNIEnv& env) : m_env(env)
{
}

KxcJEnv::~KxcJEnv()
{
}

void KxcJEnv::checkJavaError()
{
    jthrowable e = m_env.ExceptionOccurred();

    if (e)
    {
        throw KxcJavaException();
    }
}

jstring KxcJEnv::newStringUTF(const char* utf)
{
#ifdef __MVS__

    jstring js = NULL;
    m_env.NewStringPlatform(utf, &js, NULL);

#else

    jstring js = m_env.NewStringUTF(utf);

#endif

    if (js == NULL)
    {
        checkJavaError();
    }

    return js;
}

jstring KxcJEnv::newStringUTF(const unsigned char utf[])
{
    return newStringUTF(REINTERPRET_CAST(const char*, &utf[0]));
}

jbyteArray KxcJEnv::newByteArray(jsize sz)
{
    jbyteArray ba = m_env.NewByteArray(sz);

    if (ba == NULL)
    {
        checkJavaError();
    }

    return ba;
}

void KxcJEnv::setByteArrayRegion(jbyteArray array, jsize len, jbyte* buf)
{
    m_env.SetByteArrayRegion(array, 0, len, buf);
    checkJavaError();
}

jbyteArray KxcJEnv::createByteArray(jbyte* buf, jsize sz)
{
    jbyteArray ba = newByteArray(sz);
    setByteArrayRegion(ba, sz, buf);
    return ba;
}

jbyteArray KxcJEnv::createByteArray(unsigned char buf[], jsize sz)
{
    return createByteArray(REINTERPRET_CAST(jbyte*, &buf[0]), sz);
}

void KxcJEnv::reportOutOfMemory(size_t sz, const char* funcName)
{
    size_t len = ((funcName == NULL) ? 0 : strlen(funcName)) + 100;
    char* msg = (char*)alloca(len);

    if (msg != NULL)
    {
        sprintf(msg, "Roma JNI routine %s failed to allocate %lu bytes",
            (funcName == NULL ? "" : funcName), sz);
    }

    m_env.ThrowNew(m_env.FindClass(KxcJError::ms_className),
        (msg == NULL ? "Roma JNI is out of memory" : msg));

    throw KxcJavaException();
}

void KxcJEnv::reportUnexpected()
{
    m_env.ThrowNew(m_env.FindClass(KxcJError::ms_className), 
        "Caught unexpected exception in the Roma JNI library");
}



//--------------------------------------------------------------------
// KxcJString class
//--------------------------------------------------------------------

KxcJString::KxcJString(JNIEnv& env, jstring js)
    : KxcJEnv(env), m_js(js), m_data(NULL), m_length(0)
{
}

KxcJString::KxcJString(const KxcJString& js)
    : KxcJEnv(js.m_env), m_js(js.m_js), m_data(NULL), m_length(0)
{
}

KxcJString::~KxcJString()
{
    release();
}

bool KxcJString::isNull() const
{
    return (m_js == NULL);
}

KxcJString& KxcJString::operator=(const KxcJString& js)
{
    if (this != &js)
    {
        release();
        m_js = js.m_js;
    }

    return *this;
}

void KxcJString::release()
{

#ifdef __MVS__

    delete[] m_data;
    m_data = NULL;

#else

    if (m_data != NULL)
    {
        m_env.ReleaseStringUTFChars(m_js, m_data);
        m_data = NULL;
        checkJavaError();
    }

#endif
}

const char* KxcJString::getData()
{
    if (m_data == NULL && m_js != NULL)
    {

#ifdef __MVS__

        jsize sz = getLength();
        m_data = new char[sz];

        if (m_data == NULL)
        {
            reportOutOfMemeory(sz, "KxcJString::getData()");
        }

        if (m_env.GetStringPlatform(m_js, m_data, sz, NULL))
        {
            checkJavaError();
        }

#else

        m_data = m_env.GetStringUTFChars(m_js, NULL);

        if (m_data == NULL)
        {
            checkJavaError();
        }

#endif

    }

    return m_data;
}

jsize KxcJString::getLength()
{
    if (!m_length && m_js != NULL)
    {

#ifdef __MVS__
        
        jint len = 0;

        if (m_env.GetStringPlatformLength(m_js, &len, NULL))
        {
            checkJavaError();
        }
        else
        {
            m_length = len;
        }

#else

        m_length = m_env.GetStringUTFLength(m_js);
        checkJavaError();

#endif

    }

    return m_length;
}

void KxcJString::getData(char* dst, size_t len)
{
    if (m_js == NULL)
    {
        memset(dst, 0, len);
    }

    strncpy(dst, getData(), len);
    dst[len - 1] = 0;
}

void KxcJString::getData(unsigned char dst[], size_t len)
{
    getData(REINTERPRET_CAST(char*, &dst[0]), len);
}



//--------------------------------------------------------------------
// KxcJByteArray class
//--------------------------------------------------------------------

KxcJByteArray::KxcJByteArray(JNIEnv& env, jbyteArray jba)
    : KxcJEnv(env), m_jba(jba), m_data(NULL)
{
}

KxcJByteArray::~KxcJByteArray()
{
    release();
}

bool KxcJByteArray::isNull() const
{
    return (m_jba == NULL);
}

KxcJByteArray::KxcJByteArray(const KxcJByteArray& ba)
    : KxcJEnv(ba.m_env), m_jba(ba.m_jba), m_data(NULL)
{
}

KxcJByteArray& KxcJByteArray::operator=(const KxcJByteArray& ba)
{
    if (this != &ba)
    {
        release();
        m_jba = ba.m_jba;
    }

    return *this;
}

jbyte* KxcJByteArray::getData()
{    
    if (m_data == NULL && m_jba != NULL)
    {
        m_data = m_env.GetByteArrayElements(m_jba, NULL);

        if (m_data == NULL)
        {
            checkJavaError();
        }
    }

    return m_data;
}

jsize KxcJByteArray::getLength()
{
    if (m_jba == NULL)
    {
        return 0;
    }

    jsize sz = m_env.GetArrayLength(m_jba);
    checkJavaError();
    return sz;
}

void KxcJByteArray::release()
{
    if (m_data != NULL)
    {
        m_env.ReleaseByteArrayElements(m_jba, m_data, JNI_ABORT);
        m_data = NULL;
        checkJavaError();
    }
}

void KxcJByteArray::getData(void* dst, size_t len)
{
    memset(dst, 0, len);

    if (m_jba != NULL)
    {
        jbyte* src = getData();
        memcpy(dst, src, min(len, getLength()));
    }
}




//--------------------------------------------------------------------
// KxcJObjectArray class
//--------------------------------------------------------------------

KxcJObjectArray::KxcJObjectArray(JNIEnv& env, jobjectArray joa)
    : KxcJEnv(env), m_joa(joa)
{
}

KxcJObjectArray::~KxcJObjectArray()
{
}

bool KxcJObjectArray::isNull() const
{
    return (m_joa == NULL);
}

jobject KxcJObjectArray::getElement(jsize i)
{
    if (m_joa == NULL)
    {
        return NULL;
    }

    jobject res = m_env.GetObjectArrayElement(m_joa, i);

    if (res == NULL)
    {
        checkJavaError();
    }

    return res;
}

jobject KxcJObjectArray::operator[](jsize i)
{
    return getElement(i);
}

jsize KxcJObjectArray::getLength()
{
    if (m_joa == NULL)
    {
        return 0;
    }

    jsize sz = m_env.GetArrayLength(m_joa);
    checkJavaError();
    return sz;
}



//--------------------------------------------------------------------
// KxcJObject class
//--------------------------------------------------------------------

KxcJObject::KxcJObject(JNIEnv& env, const char* className)
    : KxcJEnv(env), m_handle(NULL)
{
    m_class = findClass(className);
}

KxcJObject::KxcJObject(JNIEnv& env, const char* className, jobject jo)
    : KxcJEnv(env), m_handle(jo)
{
    m_class = findClass(className);

    if (jo == NULL)
    {
        return;
    }

    if (isInstanceOf(m_handle, m_class) == JNI_FALSE)
    {
        char* buf = (char*)alloca(strlen(className) + 100);
        
        if (buf != NULL)
        {
            sprintf(buf, "Internal error: IsInstanceOf(%s) returned FALSE",
                className);
        }

        env.ThrowNew(m_env.FindClass(KxcJError::ms_className),
                (buf == NULL ? "Internal error: IsInstanceOf failed" : buf));
        throw KxcJavaException();
    }
}

KxcJObject::~KxcJObject()
{
}

bool KxcJObject::isNull() const
{
    return (m_handle == NULL);
}

jclass KxcJObject::getObjectClass()
{
    if (m_handle == NULL)
    {
        return NULL;
    }

    jclass jc = m_env.GetObjectClass(m_handle);

    if (jc == NULL)
    {
        checkJavaError();
    }

    return jc;
}

jobject KxcJObject::getHandle() const
{ 
    return m_handle; 
}

KxcJObject::operator jobject() const
{
    return m_handle; 
}

jclass KxcJObject::findClass(const char* className)
{
    jclass jc = m_env.FindClass(className);

    if (jc == NULL)
    {
        checkJavaError();
    }

    return jc;
}

jmethodID KxcJObject::getMethodID(const char* methodName, 
                                  const char* methodSpec)
{
    jmethodID mid = m_env.GetMethodID(m_class, methodName, methodSpec);

    if (mid == NULL)
    {
        checkJavaError();
    }

    return mid;
}

jobject KxcJObject::newObject(const char* ctorSpec, ...)
{
    va_list args;
    va_start(args, ctorSpec);
    jmethodID mid = getMethodID("<init>", ctorSpec);
    jobject obj = m_env.NewObjectV(m_class, mid, args);

    if (obj == NULL)
    {
        checkJavaError();
    }

    return obj;
}

jboolean KxcJObject::isInstanceOf(jobject jo, jclass jc)
{
    jboolean res = m_env.IsInstanceOf(jo, jc);
    checkJavaError();
    return res;
}

jboolean KxcJObject::isInstanceOf(const char* className)
{
    if (m_handle == NULL)
    {
        return JNI_FALSE;
    }

    return isInstanceOf(m_handle, findClass(className));
}

jint KxcJObject::callIntMethod(jmethodID mid, ...)
{
    if (m_handle == NULL)
    {
        return 0;
    }

    va_list args;
    va_start(args, mid);
    jint res = m_env.CallIntMethodV(m_handle, mid, args);
    checkJavaError();
    return res;
}

jobject KxcJObject::callObjectMethod(jmethodID mid, ...)
{
    if (m_handle == NULL)
    {
        return NULL;
    }

    va_list args;
    va_start(args, mid);
    jobject res = m_env.CallObjectMethodV(m_handle, mid, args);
    checkJavaError();
    return res;
}

jboolean KxcJObject::callBooleanMethod(jmethodID mid, ...)
{
    if (m_handle == NULL)
    {
        return JNI_FALSE;
    }

    va_list args;
    va_start(args, mid);
    jboolean res = m_env.CallBooleanMethodV(m_handle, mid, args);
    checkJavaError();
    return res;
}



//--------------------------------------------------------------------
// KxcJLinkedList class
//--------------------------------------------------------------------

const char* KxcJLinkedList::ms_className =
    "java/util/LinkedList";

KxcJLinkedList::KxcJLinkedList(JNIEnv& env)
    : KxcJObject(env, ms_className), m_mAdd(NULL)
{
    m_handle = newObject("()V");
}

KxcJLinkedList::~KxcJLinkedList()
{
}

jboolean KxcJLinkedList::add(jobject obj)
{
    if (m_mAdd == NULL)
    {
        m_mAdd = getMethodID("add", "(Ljava/lang/Object;)Z");
    }

    return callBooleanMethod(m_mAdd, obj);
}



//--------------------------------------------------------------------
// KxcJWorkflowService class
//--------------------------------------------------------------------

const char* KxcJWorkflowService::ms_className =
    "com/candle/roma/kxc/RomaWorkflowService";

KxcJWorkflowService::KxcJWorkflowService(JNIEnv& env, KXCWSERVICE& ws)
    : KxcJObject(env, ms_className),
      m_mGetService(NULL),
      m_mGetComponent(NULL),
      m_mGetAudit(NULL),
      m_mGetSourceService(NULL),
      m_mGetSourceFlow(NULL),
      m_mGetSourceOperation(NULL)
{
    m_handle = newObject(
        "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V",
        newStringUTF(ws.Service),
        newStringUTF(ws.CurComponent),
        newStringUTF(ws.AuditName),
        newStringUTF(ws.BusinessService),
        newStringUTF(ws.BusinessFlow),
        newStringUTF(ws.Operation));
}

KxcJWorkflowService::KxcJWorkflowService(JNIEnv& env, jobject jo)
    : KxcJObject(env, ms_className, jo),
      m_mGetService(NULL),
      m_mGetComponent(NULL),
      m_mGetAudit(NULL),
      m_mGetSourceService(NULL),
      m_mGetSourceFlow(NULL),
      m_mGetSourceOperation(NULL)
{
}

KxcJWorkflowService::~KxcJWorkflowService()
{
}

KxcJString KxcJWorkflowService::getService()
{
    if (m_mGetService == NULL)
    {
        m_mGetService = 
            getMethodID("getService", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetService));
}

KxcJString KxcJWorkflowService::getComponent()
{
    if (m_mGetComponent == NULL)
    {
        m_mGetComponent = 
            getMethodID("getComponent", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetComponent));
}

KxcJString KxcJWorkflowService::getAudit()
{
    if (m_mGetAudit == NULL)
    {
        m_mGetAudit = 
            getMethodID("getAudit", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetAudit));
}

KxcJString KxcJWorkflowService::getSourceService()
{
    if (m_mGetSourceService == NULL)
    {
        m_mGetSourceService = 
            getMethodID("getSourceService", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetSourceService));
}

KxcJString KxcJWorkflowService::getSourceFlow()
{
    if (m_mGetSourceFlow == NULL)
    {
        m_mGetSourceFlow = 
            getMethodID("getSourceFlow", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetSourceFlow));
}

KxcJString KxcJWorkflowService::getSourceOperation()
{
    if (m_mGetSourceOperation == NULL)
    {
        m_mGetSourceOperation = 
            getMethodID("getSourceOperation", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetSourceOperation));
}

void KxcJWorkflowService::getData(KXCWSERVICE& ws)
{
    ws.Options = 0;
    getService().getData(ws.Service, sizeof(ws.Service));
    getComponent().getData(ws.CurComponent, sizeof(ws.CurComponent));
    getAudit().getData(ws.AuditName, sizeof(ws.AuditName));
    getSourceService().getData(ws.BusinessService, sizeof(ws.BusinessService));
    getSourceFlow().getData(ws.BusinessFlow, sizeof(ws.BusinessFlow));
    getSourceOperation().getData(ws.Operation, sizeof(ws.Operation));
}



//--------------------------------------------------------------------
// KxcJRoute class
//--------------------------------------------------------------------

const char* KxcJRoute::ms_className =
    "com/candle/roma/kxc/RomaRoute";

KxcJRoute::KxcJRoute(JNIEnv& env, const KXCWROUTE& r)
    : KxcJObject(env, ms_className),
      m_mGetType(NULL),
      m_mGetFlags(NULL),
      m_mGetTargetQueue(NULL),
      m_mGetTargetElement(NULL),
      m_mGetTargetComponent(NULL),
      m_mGetTargetCodePage(NULL),
      m_mGetSourceService(NULL),
      m_mGetSourceFlow(NULL),
      m_mGetSourceOperation(NULL)
{
    m_handle = newObject(
        "(ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V",
        STATIC_CAST(jint, r.RouteType),
        newStringUTF(r.TargetQueue),
        newStringUTF(r.TargetElement),
        newStringUTF(r.TargetComponent),
        STATIC_CAST(jint, r.RouteFlags),
        newStringUTF(r.TargetCodePage),
        newStringUTF(r.BusinessService),
        newStringUTF(r.BusinessFlow),
        newStringUTF(r.Operation));
}

KxcJRoute::KxcJRoute(JNIEnv& env, jobject jo)
    : KxcJObject(env, ms_className, jo),
      m_mGetType(NULL),
      m_mGetFlags(NULL),
      m_mGetTargetQueue(NULL),
      m_mGetTargetElement(NULL),
      m_mGetTargetComponent(NULL),
      m_mGetTargetCodePage(NULL),
      m_mGetSourceService(NULL),
      m_mGetSourceFlow(NULL),
      m_mGetSourceOperation(NULL)
{
}

KxcJRoute::~KxcJRoute()
{
}

jint KxcJRoute::getType()
{
    if (m_mGetType == NULL)
    {
        m_mGetType = getMethodID("getType", "()I");
    }

    return callIntMethod(m_mGetType);
}

jint KxcJRoute::getFlags()
{
    if (m_mGetFlags == NULL)
    {
        m_mGetFlags = getMethodID("getFlags", "()I");
    }

    return callIntMethod(m_mGetFlags);
}

KxcJString KxcJRoute::getTargetQueue()
{
    if (m_mGetTargetQueue == NULL)
    {
        m_mGetTargetQueue = 
            getMethodID("getTargetQueue", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetTargetQueue));
}

KxcJString KxcJRoute::getTargetElement()
{
    if (m_mGetTargetElement == NULL)
    {
        m_mGetTargetElement = 
            getMethodID("getTargetElement", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetTargetElement));
}

KxcJString KxcJRoute::getTargetComponent()
{
    if (m_mGetTargetComponent == NULL)
    {
        m_mGetTargetComponent = 
            getMethodID("getTargetComponent", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetTargetComponent));
}

KxcJString KxcJRoute::getTargetCodePage()
{
    if (m_mGetTargetCodePage == NULL)
    {
        m_mGetTargetCodePage = 
            getMethodID("getTargetCodePage", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetTargetCodePage));
}

KxcJString KxcJRoute::getSourceService()
{
    if (m_mGetSourceService == NULL)
    {
        m_mGetSourceService = 
            getMethodID("getSourceService", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetSourceService));
}

KxcJString KxcJRoute::getSourceFlow()
{
    if (m_mGetSourceFlow == NULL)
    {
        m_mGetSourceFlow = 
            getMethodID("getSourceFlow", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetSourceFlow));
}

KxcJString KxcJRoute::getSourceOperation()
{
    if (m_mGetSourceOperation == NULL)
    {
        m_mGetSourceOperation = 
            getMethodID("getSourceOperation", "()Ljava/lang/String;");
    }

    return KxcJString(m_env, 
        (jstring)callObjectMethod(m_mGetSourceOperation));
}

void KxcJRoute::getData(KXCWROUTE& rt)
{
    rt.RouteType = getType();
    getTargetQueue().getData(rt.TargetQueue, sizeof(rt.TargetQueue));
    getTargetElement().getData(rt.TargetElement, sizeof(rt.TargetElement));
    getTargetComponent().getData(rt.TargetComponent, sizeof(rt.TargetComponent));
    rt.RouteFlags = getFlags();
    rt.NodeVersion = KXCNODEVER_CURR;
    getTargetCodePage().getData(rt.TargetCodePage, sizeof(rt.TargetCodePage));
    getSourceService().getData(rt.BusinessService, sizeof(rt.BusinessService));
    getSourceFlow().getData(rt.BusinessFlow, sizeof(rt.BusinessFlow));
    getSourceOperation().getData(rt.Operation, sizeof(rt.Operation));
}



//--------------------------------------------------------------------
// KxcJWorkflow class
//--------------------------------------------------------------------

const char* KxcJWorkflow::ms_className =
    "com/candle/roma/kxc/RomaWorkflow";

KxcJWorkflow::KxcJWorkflow(JNIEnv& env, KXCWFLOW& wf)
    : KxcJObject(env, ms_className),
      m_mGetWorkflowService(NULL),
      m_mGetRoutesArray(NULL)
{
    KxcJWorkflowService ws(env, wf.Service);
    KxcJLinkedList rl(env);

    for (KXCULONG i = 0; i < wf.NumRoutes; ++i)
    {
        KxcJRoute rt(env, wf.Routes[i]);
        rl.add(rt);
    }

    m_handle = newObject(
        "(Lcom/candle/roma/kxc/RomaWorkflowService;Ljava/util/LinkedList;)V",
        ws.getHandle(),
        rl.getHandle());
}

KxcJWorkflow::KxcJWorkflow(JNIEnv& env, jobject jo)
    : KxcJObject(env, ms_className, jo),
      m_mGetWorkflowService(NULL),
      m_mGetRoutesArray(NULL)
{
}

KxcJWorkflow::~KxcJWorkflow()
{
}

KxcJWorkflowService KxcJWorkflow::getWorkflowService()
{
    if (m_mGetWorkflowService == NULL)
    {
        m_mGetWorkflowService = 
            getMethodID("getWorkflowService",
                "()Lcom/candle/roma/kxc/RomaWorkflowService;");
    }

    return KxcJWorkflowService(m_env,
        callObjectMethod(m_mGetWorkflowService));
}

KxcJObjectArray KxcJWorkflow::getRoutesArray()
{
    if (m_mGetRoutesArray == NULL)
    {
        m_mGetRoutesArray = 
            getMethodID("getRoutesArray", "()[Ljava/lang/Object;");
    }

    return KxcJObjectArray(m_env,
        (jobjectArray)callObjectMethod(m_mGetRoutesArray));
}



//--------------------------------------------------------------------
// KxcJTrailer class
//--------------------------------------------------------------------

const char* KxcJTrailer::ms_className =
    "com/candle/roma/kxc/RomaTrailer";

KxcJTrailer::KxcJTrailer(JNIEnv& env, KXCTHEMEPTRLDATA& tr)
    : KxcJObject(env, ms_className),
      m_mGetType(NULL),
      m_mGetMethod(NULL),
      m_mGetData(NULL)
{
    PKXCTHEMEPTRLDATA pData = &tr;
    jbyteArray ba = newByteArray(tr.BufLen);

    setByteArrayRegion(ba, tr.BufLen,
        REINTERPRET_CAST(jbyte*, pData + 1));

    m_handle = newObject(
        "(II[B)V",
        STATIC_CAST(jint, tr.Type),
        STATIC_CAST(jint, tr.Method),
        ba);
}

KxcJTrailer::KxcJTrailer(JNIEnv& env, jobject jo)
    : KxcJObject(env, ms_className, jo),
      m_mGetType(NULL),
      m_mGetMethod(NULL),
      m_mGetData(NULL)
{
}

KxcJTrailer::~KxcJTrailer()
{
}

jint KxcJTrailer::getType()
{
    if (m_mGetType == NULL)
    {
        m_mGetType = getMethodID("getType", "()I");
    }

    return callIntMethod(m_mGetType);
}

jint KxcJTrailer::getMethod()
{
    if (m_mGetMethod == NULL)
    {
        m_mGetMethod = getMethodID("getMethod", "()I");
    }

    return callIntMethod(m_mGetMethod);
}

KxcJByteArray KxcJTrailer::getData()
{
    if (m_mGetData == NULL)
    {
        m_mGetData = getMethodID("getData", "()[B");
    }

    return KxcJByteArray(m_env, 
        (jbyteArray)callObjectMethod(m_mGetData));
}




//--------------------------------------------------------------------
// KxcJThemeDetails class
//--------------------------------------------------------------------

const char* KxcJThemeInfo::ms_className =
    "com/candle/roma/kxc/RomaThemeInfo";

KxcJThemeInfo::KxcJThemeInfo(JNIEnv& env, KXCBUFFER& buf, jboolean client)
    : KxcJObject(env, ms_className),
      m_mGetWorkflow(NULL),
      m_mGetRetryCount(NULL),
      m_mGetThemeId(NULL),
      m_mGetConfirmFlags(NULL),
      m_mGetTrailersArray(NULL),
      m_mGetEteData(NULL)
{
    jbyte* ptr;
    kxcQryLookUp(buf.pBuffer, KXCTID_THEMEID, &ptr);
    jbyteArray jbaThemeID = createByteArray(ptr, KXCMSG_ID_LENGTH);

    KXCULONG* lp;
    kxcQryLookUp(buf.pBuffer, KXCTID_RETRY, &lp);
    jint jiRetry = *lp;

    PKXCWFLOW pFlow;
    kxcQryLookUp(buf.pBuffer,
        (client == JNI_TRUE ? KXCTID_REQWFLOW : KXCTID_RSPWFLOW), &pFlow);
    KxcJWorkflow joWFlow(env, *pFlow);

    KXCULONG etelen = 0;
    kxcQryLookUpWithLen(buf.pBuffer, KXCTID_ETEINFO, &ptr, &etelen);
    jbyteArray jbaEte = createByteArray(ptr, etelen);

    PKXCTHEMEPTRLHDR thdr;
    kxcQryLookUp(buf.pBuffer, KXCTID_PTRAILER, &thdr);

    KxcJLinkedList joTrailers(env);

    if (thdr->Number)
    {
        PKXCTHEMEPTRLDATA pData = 
            REINTERPRET_CAST(PKXCTHEMEPTRLDATA, thdr + 1);

        for (KXCULONG i = 0; i < thdr->Number; ++i)
        {
            KxcJTrailer trailer(env, *pData);
            joTrailers.add(trailer);
            
            char* ptr = REINTERPRET_CAST(char*, pData + 1);
            pData = REINTERPRET_CAST(PKXCTHEMEPTRLDATA, ptr + pData->BufLen);
        }
    }

    KXCCONFIRMSTATUS* conf;
    kxcQryLookUp(buf.pBuffer, KXCTID_CONFIRM, &conf);
    jint jiFlags = (conf->CoA) ? KXCPOPT_COA : 0;
    if (conf->CoD) jiFlags |= KXCPOPT_COD;

    m_handle = newObject(
        "([BILcom/candle/roma/kxc/RomaWorkflow;ILjava/util/LinkedList;[B)V",
        jbaThemeID,
        jiRetry,
        joWFlow.getHandle(),
        jiFlags,
        joTrailers.getHandle(),
        jbaEte);
}

KxcJThemeInfo::KxcJThemeInfo(JNIEnv& env, jobject jo)
    : KxcJObject(env, ms_className, jo),
      m_mGetWorkflow(NULL),
      m_mGetRetryCount(NULL),
      m_mGetThemeId(NULL),
      m_mGetConfirmFlags(NULL),
      m_mGetTrailersArray(NULL),
      m_mGetEteData(NULL)
{
}

KxcJThemeInfo::~KxcJThemeInfo()
{
}

KxcJWorkflow KxcJThemeInfo::getWorkflow()
{
    if (m_mGetWorkflow == NULL)
    {
        m_mGetWorkflow = 
            getMethodID("getWorkflow", "()Lcom/candle/roma/kxc/RomaWorkflow;");
    }

    return KxcJWorkflow(m_env, callObjectMethod(m_mGetWorkflow));
}

jint KxcJThemeInfo::getRetryCount()
{
    if (m_mGetRetryCount == NULL)
    {
        m_mGetRetryCount = getMethodID("getRetryCount", "()I");
    }

    return callIntMethod(m_mGetRetryCount);
}

KxcJByteArray KxcJThemeInfo::getThemeId()
{
    if (m_mGetThemeId == NULL)
    {
        m_mGetThemeId = getMethodID("getThemeId", "()[B");
    }

    return KxcJByteArray(m_env, 
        (jbyteArray)callObjectMethod(m_mGetThemeId));
}

jint KxcJThemeInfo::getConfirmFlags()
{
    if (m_mGetConfirmFlags == NULL)
    {
        m_mGetConfirmFlags = getMethodID("getConfirmFlags", "()I");
    }

    return callIntMethod(m_mGetConfirmFlags);
}

KxcJObjectArray KxcJThemeInfo::getTrailersArray()
{
    if (m_mGetTrailersArray == NULL)
    {
        m_mGetTrailersArray = 
            getMethodID("getTrailersArray", "()[Ljava/lang/Object;");
    }

    return KxcJObjectArray(m_env,
        (jobjectArray)callObjectMethod(m_mGetTrailersArray));
}

KxcJByteArray KxcJThemeInfo::getEteData()
{
    if (m_mGetEteData == NULL)
    {
        m_mGetEteData = getMethodID("getEteData", "()[B");
    }

    return KxcJByteArray(m_env, 
        (jbyteArray)callObjectMethod(m_mGetEteData));
}



//--------------------------------------------------------------------
// KxcJError class
//--------------------------------------------------------------------

const char* KxcJError::ms_className =
    "com/candle/roma/kxc/RomaError";

KxcJError::KxcJError(JNIEnv& env, KXCERROR& e)
    : KxcJObject(env, ms_className)
{
    KXCSUBERROR& s1 = e.Result;
    KXCSUBERROR& s2 = e.SubResult;
    jstring ct1 = NULL, ft1 = NULL, ct2 = NULL, ft2 = NULL;
    char buf[4 * 1024] = { 0 };
    KXCULONG sz = sizeof(buf);

    if ((e.Status & KXCERRSTAT_MAIN) != 0)
    {
        kxcFormatError(&s1, KXCERROR_COMPACT, buf, &sz);
        ct1 = newStringUTF(buf);

        kxcFormatError(&s1, KXCERROR_FULL, buf, &sz);
        ft1 = newStringUTF(buf);
    }

    if ((e.Status & KXCERRSTAT_SUB) != 0)
    {
        kxcFormatError(&s2, KXCERROR_COMPACT, buf, &sz);
        ct2 = newStringUTF(buf);

        kxcFormatError(&s2, KXCERROR_FULL, buf, &sz);
        ft2 = newStringUTF(buf);
    }

    m_handle = newObject(
        "(IIIIIIILjava/lang/String;Ljava/lang/String;Ljava/lang/String;IIIIIILjava/lang/String;Ljava/lang/String;Ljava/lang/String;)V",
        STATIC_CAST(jint, e.Status),
        STATIC_CAST(jint, s1.LastCompCode),
        STATIC_CAST(jint, s1.LastReturnCode),
        STATIC_CAST(jint, s1.LastOpType),
        STATIC_CAST(jint, s1.LastReason),
        STATIC_CAST(jint, s1.LastPosition),
        STATIC_CAST(jint, s1.LastErrorType),
        newStringUTF(s1.LastLocation),
        ct1,
        ft1,
        STATIC_CAST(jint, s2.LastCompCode),
        STATIC_CAST(jint, s2.LastReturnCode),
        STATIC_CAST(jint, s2.LastOpType),
        STATIC_CAST(jint, s2.LastReason),
        STATIC_CAST(jint, s2.LastPosition),
        STATIC_CAST(jint, s2.LastErrorType),
        newStringUTF(s2.LastLocation),
        ct2,
        ft2);
}

KxcJError::~KxcJError()
{
}

jint KxcJError::throwToJava()
{
    return m_env.Throw(STATIC_CAST(jthrowable, m_handle));
}



//--------------------------------------------------------------------
// KxcJRomaConfig class
//--------------------------------------------------------------------

const char* KxcJRomaConfig::ms_className =
    "com/candle/roma/kxc/RomaConfig";

KxcJRomaConfig::KxcJRomaConfig(JNIEnv& env, KXCEXCROMACONF& rc)
    : KxcJObject(env, ms_className)
{
    m_handle = newObject(
        "(Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V",
        newStringUTF(rc.ElementName),
        STATIC_CAST(jint, rc.ElementType),
        newStringUTF(rc.QueueName),
        newStringUTF(rc.ComponentName),
        newStringUTF(rc.ServiceName),
        newStringUTF(rc.NodeName));
}

KxcJRomaConfig::~KxcJRomaConfig()
{
}



//--------------------------------------------------------------------
// KxcJSystemConfig class
//--------------------------------------------------------------------

const char* KxcJSystemConfig::ms_className =
    "com/candle/roma/kxc/RomaSystemConfig";

KxcJSystemConfig::KxcJSystemConfig(JNIEnv& env, KXCEXCSYSCONF& sc)
    : KxcJObject(env, ms_className)
{
    m_handle = newObject(
        "(ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;JJ)V",
        STATIC_CAST(jint, sc.EnvironmentType),
        newStringUTF(sc.DomainName),
        newStringUTF(sc.UserId),
        newStringUTF(sc.ProgramName),
        STATIC_CAST(jlong, sc.ProcessId),
        STATIC_CAST(jlong, sc.ThreadId));
}

KxcJSystemConfig::~KxcJSystemConfig()
{
}



//--------------------------------------------------------------------
// KxcJExcDetails class
//--------------------------------------------------------------------

const char* KxcJExcInfo::ms_className =
    "com/candle/roma/kxc/RomaExceptionInfo";

KxcJExcInfo::KxcJExcInfo(JNIEnv& env, KXCEXCEPTION& ex)
    : KxcJObject(env, ms_className)
{
    jlong sec = 0;
    jint msec = 0;
    jobject joErrorData = NULL, joRomaConfig = NULL, joSystemConfig = NULL;
    jbyteArray jbaUserData = NULL;

    if ((ex.ExceptionFlags & KXCEXC_ERRORDATA) != 0)
    {
        KxcJExcErrorData ed(env, ex.ErrorData);
        joErrorData = ed.getHandle();
    }

    if ((ex.ExceptionFlags & KXCEXC_TIMESTAMP) != 0)
    {
        sec = ex.TimeStamp.Seconds;
        msec = ex.TimeStamp.Millisecs;
    }

    if ((ex.ExceptionFlags & KXCEXC_ROMADATA) != 0)
    {
        KxcJRomaConfig rc(env, ex.RomaConfData);
        joRomaConfig = rc.getHandle();
    }

    if ((ex.ExceptionFlags & KXCEXC_SYSTEMDATA) != 0)
    {
        KxcJSystemConfig sc(env, ex.SystemConfData);
        joSystemConfig = sc.getHandle();
    }

    if ((ex.ExceptionFlags & KXCEXC_USERDATA) != 0)
    {
        jbaUserData = createByteArray(
            (jbyte*)ex.UserData.UserBuf.pBuffer, ex.UserData.UserBufLen);
    }

    m_handle = newObject(
        "(JILcom/candle/roma/kxc/RomaConfig;Lcom/candle/roma/kxc/RomaSystemConfig;Lcom/candle/roma/kxc/RomaExcErrorData;[B)V",
        sec,
        msec,
        joRomaConfig,
        joSystemConfig,
        joErrorData,
        jbaUserData);
}

KxcJExcInfo::~KxcJExcInfo()
{
}



//--------------------------------------------------------------------
// KxcJMessage class
//--------------------------------------------------------------------

const char* KxcJMessage::ms_className =
    "com/candle/roma/kxc/RomaMessage";

KxcJMessage::KxcJMessage(JNIEnv& env, KXCMSG& msg, HKXCTHEME theme)
    : KxcJObject(env, ms_className)
{
    jbyteArray jbaMsgID = createByteArray(msg.MsgId, KXCMSG_ID_LENGTH);
    jbyteArray jbaCorID = createByteArray(msg.CorrelId, KXCMSG_ID_LENGTH);

    jbyteArray jbaMsgData = createByteArray(
        (jbyte*)msg.MsgBuffer.pBuffer, msg.MsgLen);
    jbyteArray jbaTagData = createByteArray(
        (jbyte*)msg.TagBuffer.pBuffer, msg.TagLen);

    m_handle = newObject(
        "(IIIIIIII[B[BI[B[BIJILjava/lang/String;Ljava/lang/String;Ljava/lang/String;J)V",
        STATIC_CAST(jint, msg.MsgType),
        STATIC_CAST(jint, msg.Priority),
        STATIC_CAST(jint, msg.Persistence),
        STATIC_CAST(jint, msg.Format),
        STATIC_CAST(jint, msg.Feedback),
        STATIC_CAST(jint, msg.Expiry),
        STATIC_CAST(jint, msg.CodePage),
        STATIC_CAST(jint, msg.Encoding),
        jbaMsgID,
        jbaCorID,
        STATIC_CAST(jint, msg.Status),
        jbaMsgData,
        jbaTagData,
        STATIC_CAST(jint, msg.BackoutCount),
        STATIC_CAST(jlong, msg.TimeStamp.Seconds),
        STATIC_CAST(jint, msg.TimeStamp.Millisecs),
        newStringUTF(msg.FormatName),
        newStringUTF(msg.SubjectName),
        newStringUTF(msg.MessageName),
        STATIC_CAST(jlong, theme));
}

KxcJMessage::~KxcJMessage()
{
}



//--------------------------------------------------------------------
// KxcJMsgInfo class
//--------------------------------------------------------------------

const char* KxcJMsgInfo::ms_className =
    "com/candle/roma/kxc/RomaMessageInfo";

KxcJMsgInfo::KxcJMsgInfo(JNIEnv& env, KXCMSGINFO& mi)
    : KxcJObject(env, ms_className)
{
    jobject joMsg = NULL, joWFlow = NULL, joExc = NULL;
    jlong jlTheme = 0;

    if ((mi.Status & KXCMSGPART_MESSAGE) != 0)
    {
        KxcJMessage msg(env, mi.Message, NULL_HKXCTHEME);
        joMsg = msg.getHandle();
    }

    if ((mi.Status & KXCMSGPART_WORKFLOW) != 0)
    {
        PKXCWFLOW pFlow = (PKXCWFLOW)mi.WorkFlow.pBuffer;
        KxcJWorkflow wf(env, *pFlow);
        joWFlow = wf.getHandle();
    }

    if ((mi.Status & KXCMSGPART_EXCEPTION) != 0)
    {
        KxcJExcInfo ei(env, mi.Exception);
        joExc = ei.getHandle();
    }

    if ((mi.Status & KXCMSGPART_HTHEME) != 0)
    {
        jlTheme = mi.hTheme;
    }

    m_handle = newObject(
        "(Lcom/candle/roma/kxc/RomaWorkflow;JLcom/candle/roma/kxc/RomaMessage;Lcom/candle/roma/kxc/RomaExceptionInfo;)V",
        joWFlow,
        jlTheme,
        joMsg,
        joExc);
}

KxcJMsgInfo::~KxcJMsgInfo()
{
}



//--------------------------------------------------------------------
// KxcJExcErrorData class
//--------------------------------------------------------------------

const char* KxcJExcErrorData::ms_className =
    "com/candle/roma/kxc/RomaExcErrorData";

KxcJExcErrorData::KxcJExcErrorData(JNIEnv& env, KXCEXCERRDATA& ed)
    : KxcJObject(env, ms_className)
{
    KxcJError err(env, ed.Error);

    m_handle = newObject(
        "(Lcom/candle/roma/kxc/RomaError;ILjava/lang/String;)V",
        err.getHandle(),
        STATIC_CAST(jint, ed.CallType),
        newStringUTF(ed.QueueName));
}

KxcJExcErrorData::~KxcJExcErrorData()
{
}

