//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// KXC JNI helper classes
//
//--------------------------------------------------------------------
// $Id: kxcjobj.hpp,v 1.1.2.4 2003/10/17 14:04:37 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/Attic/kxcjobj.hpp,v $
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes.
//--------------------------------------------------------------------
#ifndef __KXCJOBJ_HPP
#define __KXCJOBJ_HPP

#include <stdarg.h>
#include <jni.h>

#include <kxc.h>
#include <kxcointa.hpp>


#ifdef KXC_AIX
#define REINTERPRET_CAST(type,var) (type)(var)
#define STATIC_CAST(type,var) type(var)
#else
#define REINTERPRET_CAST(type,var) reinterpret_cast<type>(var)
#define STATIC_CAST(type,var) static_cast<type>(var)
#endif


class KxcSimplePtr
{
public:
    explicit KxcSimplePtr(size_t s) : ptr(NULL) { ptr = new char[s]; }
    ~KxcSimplePtr() { delete[] ptr; }

    char* get() { return ptr; }

private:
    KxcSimplePtr& operator=(const KxcSimplePtr&);
    KxcSimplePtr(const KxcSimplePtr&);

    char* ptr;
};


class KxcJavaException
{
public:
    KxcJavaException() {}
    ~KxcJavaException() {}
};


class KxcJEnv
{
public:
    KxcJEnv(JNIEnv& env);
    ~KxcJEnv();

    jstring newStringUTF(const char*);
    jstring newStringUTF(const unsigned char utf[]);
    jbyteArray newByteArray(jsize sz);
    void setByteArrayRegion(jbyteArray array, jsize len, jbyte* buf);
    jbyteArray createByteArray(jbyte* buf, jsize sz);
    jbyteArray createByteArray(unsigned char buf[], jsize sz);

    void checkJavaError();

    void reportOutOfMemory(size_t sz, const char* funcName);
    void reportUnexpected();

private:
    // prohibit dynamic allocation
    void* operator new(size_t);

protected:
    JNIEnv& m_env;
};


class KxcJString : public KxcJEnv
{
public:
    KxcJString(JNIEnv& env, jstring js);
    ~KxcJString();

    KxcJString& operator=(const KxcJString&);
    KxcJString(const KxcJString&);

    const char* getData();
    void getData(char* dst, size_t len);
    void getData(unsigned char dst[], size_t len);
    jsize getLength();

    bool isNull() const;

private:
    void release();

    jstring m_js;
    jint m_length;

#ifndef __MVS__
    const
#endif
    char* m_data;
};


class KxcJByteArray : public KxcJEnv
{
public:
    KxcJByteArray(JNIEnv& env, jbyteArray jba);
    ~KxcJByteArray();

    KxcJByteArray& operator=(const KxcJByteArray&);
    KxcJByteArray(const KxcJByteArray&);

    jbyte* getData();
    void getData(void* dst, size_t len);
    jsize getLength();

    bool isNull() const;

private:

    void release();

    jbyteArray m_jba;
    jbyte* m_data;
};


class KxcJObjectArray : public KxcJEnv
{
public:
    KxcJObjectArray(JNIEnv& env, jobjectArray joa);
    ~KxcJObjectArray();

    jobject getElement(jsize i);
    jobject operator[](jsize i);
    jsize getLength();

    bool isNull() const;

private:
    jobjectArray m_joa;
};


class KxcJObject : public KxcJEnv
{
public:
    KxcJObject(JNIEnv& env, const char* className);
    KxcJObject(JNIEnv& env, const char* className, jobject jo);
    ~KxcJObject();

    jclass findClass(const char* className);
    jclass getObjectClass();
    jmethodID getMethodID(const char* methodName, const char* methodSpec);
    jobject newObject(const char* ctorSpec, ...);
    jboolean isInstanceOf(jobject, jclass);
    jboolean isInstanceOf(const char* className);

    jint callIntMethod(jmethodID, ...);
    jobject callObjectMethod(jmethodID, ...);
    jboolean callBooleanMethod(jmethodID, ...);

    jobject getHandle() const;
    operator jobject() const;

    bool isNull() const;

protected:
    jclass m_class;
    jobject m_handle;
};


class KxcJLinkedList : public KxcJObject
{
public:
    KxcJLinkedList(JNIEnv&);
    ~KxcJLinkedList();

    jboolean add(jobject);

private:
    static const char* ms_className;

    jmethodID m_mAdd;
};


class KxcJWorkflowService : public KxcJObject
{
public:
    KxcJWorkflowService(JNIEnv&, KXCWSERVICE& ws);
    KxcJWorkflowService(JNIEnv& env, jobject jo);
    ~KxcJWorkflowService();

    KxcJString getService();
    KxcJString getComponent();
    KxcJString getAudit();
    KxcJString getSourceService();
    KxcJString getSourceFlow();
    KxcJString getSourceOperation();

    void getData(KXCWSERVICE& ws);

private:
    static const char* ms_className;

    jmethodID m_mGetService;
    jmethodID m_mGetComponent;
    jmethodID m_mGetAudit;
    jmethodID m_mGetSourceService;
    jmethodID m_mGetSourceFlow;
    jmethodID m_mGetSourceOperation;
};


class KxcJRoute : public KxcJObject
{
public:
    KxcJRoute(JNIEnv&, const KXCWROUTE&);
    KxcJRoute(JNIEnv&, jobject);
    ~KxcJRoute();

    jint getType();
    jint getFlags();
    KxcJString getTargetQueue();
    KxcJString getTargetElement();
    KxcJString getTargetComponent();
    KxcJString getTargetCodePage();
    KxcJString getSourceService();
    KxcJString getSourceFlow();
    KxcJString getSourceOperation();

    void getData(KXCWROUTE&);

private:
    static const char* ms_className;

    jmethodID m_mGetType;
    jmethodID m_mGetFlags;
    jmethodID m_mGetTargetQueue;
    jmethodID m_mGetTargetElement;
    jmethodID m_mGetTargetComponent;
    jmethodID m_mGetTargetCodePage;
    jmethodID m_mGetSourceService;
    jmethodID m_mGetSourceFlow;
    jmethodID m_mGetSourceOperation;
};


class KxcJWorkflow : public KxcJObject
{
public:
    KxcJWorkflow(JNIEnv&, KXCWFLOW& wf);
    KxcJWorkflow(JNIEnv&, jobject jo);
    ~KxcJWorkflow();

    KxcJWorkflowService getWorkflowService();
    KxcJObjectArray getRoutesArray();

private:
    static const char* ms_className;

    jmethodID m_mGetWorkflowService;
    jmethodID m_mGetRoutesArray;
};


class KxcJTrailer : public KxcJObject
{
public:
    KxcJTrailer(JNIEnv&, KXCTHEMEPTRLDATA&);
    KxcJTrailer(JNIEnv&, jobject);
    ~KxcJTrailer();

    jint getType();
    jint getMethod();
    KxcJByteArray getData();

private:
    static const char* ms_className;

    jmethodID m_mGetType;
    jmethodID m_mGetMethod;
    jmethodID m_mGetData;
};


class KxcJThemeInfo : public KxcJObject
{
public:
    KxcJThemeInfo(JNIEnv& env, KXCBUFFER& buf, jboolean client);
    KxcJThemeInfo(JNIEnv& env, jobject jo);
    ~KxcJThemeInfo();

    KxcJWorkflow getWorkflow();
    jint getRetryCount();
    KxcJByteArray getThemeId();
    jint getConfirmFlags();
    KxcJObjectArray getTrailersArray();
    KxcJByteArray getEteData();

private:
    static const char* ms_className;

    jmethodID m_mGetWorkflow;
    jmethodID m_mGetRetryCount;
    jmethodID m_mGetThemeId;
    jmethodID m_mGetConfirmFlags;
    jmethodID m_mGetTrailersArray;
    jmethodID m_mGetEteData;
};


class KxcJError : public KxcJObject
{
public:
    KxcJError(JNIEnv& env, KXCERROR& e);
    ~KxcJError();

    jint throwToJava();

    static const char* ms_className;
};


class KxcJRomaConfig : public KxcJObject
{
public:
    KxcJRomaConfig(JNIEnv& env, KXCEXCROMACONF& rc);
    ~KxcJRomaConfig();

    static const char* ms_className;
};


class KxcJSystemConfig : public KxcJObject
{
public:
    KxcJSystemConfig(JNIEnv& env, KXCEXCSYSCONF& sc);
    ~KxcJSystemConfig();

    static const char* ms_className;
};


class KxcJExcInfo : public KxcJObject
{
public:
    KxcJExcInfo(JNIEnv& env, KXCEXCEPTION& ex);
    ~KxcJExcInfo();

    static const char* ms_className;
};


class KxcJMsgInfo : public KxcJObject
{
public:
    KxcJMsgInfo(JNIEnv& env, KXCMSGINFO& msg);
    ~KxcJMsgInfo();

    static const char* ms_className;
};


class KxcJMessage : public KxcJObject
{
public:
    KxcJMessage(JNIEnv& env, KXCMSG& msg, HKXCTHEME);
    ~KxcJMessage();

    static const char* ms_className;
};


class KxcJExcErrorData : public KxcJObject
{
public:
    KxcJExcErrorData(JNIEnv& env, KXCEXCERRDATA& e);
    ~KxcJExcErrorData();

    static const char* ms_className;
};


#endif

