/********************************************************************/
/*                                                                  */
/* Module      : Client11.C                                         */
/*                                                                  */
/* Description :                                                    */
/*               This client send a TRANSFER message to the server. */
/*               The server is unable to accept these messages and  */
/*               will handle the message as an error.               */
/*                                                                  */
/*               This example uses hard-coded literals as this      */
/*               makes it easier to see what is happening.          */
/*                                                                  */
/* Environment :                                                    */
/*               Server11.c contains code to handle the server      */
/*               side.                                              */
/*               Examples11.ldf contains the directory.             */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 2000                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include the Roma header file                                     */
/********************************************************************/
#include <kxc.h>


/********************************************************************/
/* Include the C standard header files                              */
/********************************************************************/
#include <string.h>
#include <stdio.h>

/********************************************************************/
/* Application specific structure - transfer                        */
/********************************************************************/

typedef struct {
  char         AccountName       [30];    
  char         FromAccountNumber [30];
  char         ToAccountNumber   [30];
  char         TransferAmount    [10];  
} ACCOUNT_TRANSFER;


/********************************************************************/
/*                                                                  */
/* Function:  HandleError                                           */
/*                                                                  */
/* Purpose:   This is the single point at which the application     */
/*            handles all errors.  Input is a string which says     */
/*            which operation didn't work.                          */
/*                                                                  */
/********************************************************************/

void HandleError (char *     OpType)

{
  KXCERROR     Error;      /* Used to get the details about a      */
                           /* failure to execute a Roma function   */
  KXCCHAR      ErrorString[256];
                           /* Error reason in human-readable format*/
  KXCULONG     ErrorStringLen; 
                           /* Length of the error string           */

  printf ("%s\n", OpType);
  kxcGetLastError (&Error);
  if (Error.Status & KXCERRSTAT_MAIN)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.Result,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
  if (Error.Status & KXCERRSTAT_SUB)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.SubResult,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
}


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Purpose:   As described above                                    */
/*                                                                  */
/********************************************************************/
void main ()

{
  KXCAPPDEF    ApplicationDef; 
                           /* Defines the client & options          */
  HKXC         XCi;        /* Used by Roma to associate operations  */
  KXCSERVICE   ServiceDef; /* Defines the service when sending a    */
                           /* request                               */
  KXCMSG       Message;    /* Defines the message being sent and    */
                           /* message control options               */
  KXCPUTOPT    PutControl; /* Defines the options when sending the  */
                           /* message                               */
  KXCBOOL      result;     /* Used to check each Roma operation     */

  ACCOUNT_TRANSFER
               transfer;   /* Defines the application information   */

  /******************************************************************/
  /* First build the Application Definition.                        */
  /******************************************************************/

  kxcSetAppDefDefault (&ApplicationDef);
  strcpy ((char *)ApplicationDef.ElementName, "TestClient11");

  /******************************************************************/
  /* Initialize the client                                          */
  /* Returns Success/Failure indicator.                             */
  /******************************************************************/
  
  result = kxcClientInitialize (&ApplicationDef, &XCi);
  if (!result)
  {
    HandleError ("Could not initialize the client");
    return;
  }

  /******************************************************************/
  /* Next, we have to define the service that we want to send the   */
  /* message to, the message itself and the options that we're      */
  /* using to send the message.                                     */
  /******************************************************************/

  kxcSetServiceDefault (&ServiceDef);
  strcpy ((char *)ServiceDef.ServiceName, "AccountQuery");

  kxcSetMsgDefault (&Message);
  strcpy ((char *)Message.FormatName,  "TRANSFER");
  Message.Format = KXCMSGFORMAT_USENAME;

  /******************************************************************/
  /* Populate the message information.                              */
  /******************************************************************/

  Message.MsgLen            = sizeof (transfer);
  Message.MsgBuffer.Status  = KXCBUFSTAT_USER;
  Message.MsgBuffer.BufLen  = sizeof (transfer);
  Message.MsgBuffer.pBuffer = (PKXCVOID) &transfer;

  /******************************************************************/
  /* Applications have to populate their specific information       */
  /* In this case, the information is hard coded.                   */
  /* The operation that we want to carry out is to transfer $10000  */
  /* between two of APerson's accounts.                             */
  /******************************************************************/

  strcpy (transfer.AccountName,       "APerson");
  strcpy (transfer.FromAccountNumber, "1234567890"); 
  strcpy (transfer.ToAccountNumber,   "1234567890Z"); 
  strcpy (transfer.TransferAmount,    "$10000");

  /******************************************************************/
  /* The Put Options                                                */
  /* The message is a datagram.                                     */
  /******************************************************************/

  kxcSetPutOptDefault (&PutControl);
  PutControl.RequestOptions   = KXCPOPT_DATAGRAM;

  /******************************************************************/
  /* Send the message to the server application.                    */
  /******************************************************************/
 
  result = kxcPutRequest 
                 (XCi,           /* Context as returned from        */
                                 /*   kxcClientInitialize           */
                  &ServiceDef,   /* Where to send the message       */
                  &Message,      /* Defines the message being sent  */
                  &PutControl,   /* Options pertaining to the put   */
                  NULL,          /* No syncpoint operation          */
                  NULL);         /* No theme needed                 */

  /******************************************************************/
  /* Simple error handling - abandon the application if an error    */
  /* happens.  Normal applications may want to take different       */
  /* action.                                                        */
  /******************************************************************/

  if (!result)
  {
    HandleError ("Could not send the message");
    return;
  }
  puts("Sent TRANSFER message");

  /******************************************************************/
  /* Normally, applications will initialize, send multiple messages */
  /* and then terminate.  This example, sends a single message and  */
  /* then stops.  This application does not check too see if the    */
  /* terminate worked.                                              */
  /******************************************************************/

  kxcTerminate (&XCi);
  return;
}
