/********************************************************************/
/*                                                                  */
/* Module      : Client20.C                                         */
/*                                                                  */
/* Description :                                                    */
/*               This contains a simple client that operates in     */
/*               conjunction with ILS20.c.                          */
/*               This client sends a message to the server and      */
/*               waits for a response.  It also sets an Id which it */
/*               passes down to the ILS.  ILS20 adds this Id to the */
/*               message so that it can be authenticated by the     */
/*               server.                                            */
/*                                                                  */
/*               This example uses hard-coded literals as this      */
/*               makes it easier to see what is happening.          */
/*                                                                  */
/* Environment :                                                    */
/*               Server20.c contains code to handle the server      */
/*               side.                                              */
/*               Examples20.ldf contains the directory.             */
/*               ILS20.c c contains the sample ILS that operates    */
/*               with this client.                                  */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 2000                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include the Roma header file                                     */
/* We also need kxcxit to define the ILS type                       */
/********************************************************************/
#include <kxc.h>
#include <kxcxit.h>

/********************************************************************/
/* Include the C standard header files                              */
/********************************************************************/
#include <string.h>
#include <stdio.h>

typedef struct {
  char         AccountName   [30];    /* Input */
  char         AccountNumber [30];    /* Input */
  char         CurrentAmount [10];    /* Returned */
} ACCOUNT_QUERY;


/********************************************************************/
/*                                                                  */
/* Function:  HandleError                                           */
/*                                                                  */
/* Purpose:   This is the single point at which the application     */
/*            handles all errors.  Input is a string which says     */
/*            which operation didn't work.                          */
/*                                                                  */
/********************************************************************/

void HandleError (char *     OpType)

{
  KXCERROR     Error;      /* Used to get the details about a       */
                           /* failure to execute a Roma function    */
  KXCCHAR      ErrorString[256];
                           /* Error reason in human-readable format */
  KXCULONG     ErrorStringLen; 
                           /* Length of the error string            */

  printf ("%s\n", OpType);
  kxcGetLastError (&Error);
  if (Error.Status & KXCERRSTAT_MAIN)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.Result,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
  if (Error.Status & KXCERRSTAT_SUB)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.SubResult,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
}


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Purpose:   As described above                                    */
/*                                                                  */
/********************************************************************/
void main ()

{
  KXCAPPDEF     ApplicationDef; /* Defines the client & options     */
  HKXC          XCi;            /* Used by Roma to associate        */
                                /* operations                       */
  KXCSERVICE    ServiceDef;     /* Defines the service when sending */
                                /* a request                        */
  KXCMSG        Message;        /* Defines the message and message  */
                                /* control options                  */
  KXCPUTOPT     PutControl;     /* Defines the options when sending */
                                /* the message                      */
  HKXCTHEME     Theme;          /* To tie response to the request   */
  KXCGETOPT     GetControl;     /* Defines the options when         */
                                /* receiving the response message   */

  KXCILSOPT     ILSOptions;     /* Data structure to pass           */
                                /* information to ILS               */
  HKXCILS       ILSHandle = 0;  /* Handle to the ILS buffer         */
  KXCCHAR       ID [] = {"Elizabeth"};
                                /* Id to forward                    */

  KXCBOOL       result;         /* Used to check each Roma operation*/
  ACCOUNT_QUERY query;          /* Defines the application          */
                                /* information that the client will */
                                /* send to the server.              */
  ACCOUNT_QUERY *pQuery;        /* The information returned by the  */
                                /* server.                          */


  /******************************************************************/
  /* Build the Application Definition.                              */
  /******************************************************************/

  kxcSetAppDefDefault (&ApplicationDef);
  strcpy ((char *)ApplicationDef.ElementName, "TestClient20");

  /******************************************************************/
  /* Initialize the client                                          */
  /******************************************************************/
  
  result = kxcClientInitialize (&ApplicationDef, &XCi);
  if (!result)
  {
    HandleError ("Could not initialize the client");
    return;
  }

  /******************************************************************/
  /* Define the service that we want to the message to, the message */
  /* itself and the options that we're using to send the message.   */
  /******************************************************************/

  kxcSetServiceDefault (&ServiceDef);
  strcpy ((char *)ServiceDef.ServiceName, "AccountQuery");

  kxcSetMsgDefault (&Message);
  Message.Format        = KXCMSGFORMAT_USENAME;
  Message.Expiry        = KXCMSGEXP_NEVER;
  strcpy ((char *)Message.FormatName,  "QUERY");
  strcpy ((char *)Message.MessageName, "MessageName");

  /******************************************************************/
  /* Populate the query information.                                */
  /******************************************************************/

  Message.MsgLen            = sizeof (query);
  Message.MsgBuffer.Status  = KXCBUFSTAT_USER;
  Message.MsgBuffer.BufLen  = sizeof (query);
  Message.MsgBuffer.pBuffer = (PKXCVOID) &query;

  /******************************************************************/
  /* Applications specific information.                             */
  /******************************************************************/

  strcpy ((char *)query.AccountName,   "APerson");
  strcpy ((char *)query.AccountNumber, "1234567890"); 

  /******************************************************************/
  /* Build the data structure that passes information from the      */
  /* application to the ILS.                                        */
  /* We are targeting the data at ILS 20.  Note that we have to     */
  /* know the number of the ILS.  We also set the type of service   */
  /* to user - KXCXIT_TYPE_SERVICE.                                 */
  /******************************************************************/

  kxcSetILSDefault (&ILSOptions);
  ILSOptions.Type    = KXCXIT_TYPE_SERVICE;
  ILSOptions.Method  = 20;
  ILSOptions.Options = KXCIOPT_REPLACE;  /* New or replace existing */
  ILSOptions.DataBuf.pBuffer = (PKXCVOID) ID;
  ILSOptions.DataBuf.BufLen  = sizeof (ID);
  ILSOptions.DataBuf.Status  = KXCBUFSTAT_USER;
  ILSOptions.DataLen         = sizeof (ID);

  /******************************************************************/
  /* Now tell Roma about this buffer exchange                       */
  /* Note that we should record the ILSHandle and reuse it, for     */
  /* setting buffers for other ILSs or sending new buffers to this  */
  /* ILS, replace option. When done we pass it to kxcCloseHandle    */
  /******************************************************************/

  result = kxcSetILSData (&ILSOptions, &ILSHandle);
  if (!result)
  {
    HandleError ("Could not create ILS Handle");
    return;
  }

  /******************************************************************/
  /* Set the ILS Handle in the Put Options                          */
  /******************************************************************/

  kxcSetPutOptDefault (&PutControl);
  PutControl.RequestOptions   = KXCPOPT_NONE;           
  PutControl.ILSData.hToILS = ILSHandle;

  /******************************************************************/
  /* Send the message to the server application.                    */
  /******************************************************************/
 
  result = kxcPutRequest 
                 (XCi,           /* Context as returned from        */
                                 /*   kxcClientInitialize           */
                  &ServiceDef,   /* Where to send the message       */
                  &Message,      /* Defines the message being sent  */
                  &PutControl,   /* Options pertaining to the put   */
                  NULL,          /* No syncpoint operation          */
                  &Theme);       /* Theme for response              */

  if (!result)
  {
    HandleError ("Could not send the message");
    return;
  }

  /******************************************************************/
  /* Wait for the response.                                         */
  /******************************************************************/

  kxcSetMsgDefault (&Message);

  kxcSetGetOptDefault (&GetControl);
  GetControl.ReceiveOptions   = KXCGOPT_AUTO_ALLOC | 
                                KXCGOPT_CLOSETHEME | 
                                KXCGOPT_WAIT;
  GetControl.WaitLen          = KXCGOWAIT_INDEFINITE;

  /******************************************************************/
  /* Now read the response.                                         */
  /******************************************************************/

  result = kxcGetResponse 
                 (XCi,           /* Context as returned from        */
                                 /*   kxcClientInitialize           */
                  &Message,      /* The message read will be        */
                                 /*  returned here                  */
                  &GetControl,   /* Options selected when reading   */
                                 /*  the queue                      */
                  NULL,          /* No syncpoint operation          */
                  &Theme);       /* Theme from request              */
  if (!result)
  {
    HandleError ("Could not get the response");
    return;
  }

  /******************************************************************/
  /* Check to see if this is a real response or an error            */
  /* notification.  For real responses, print out the answer to the */
  /* query.  For errors, print out the tag which contains the error */
  /* message.                                                       */
  /******************************************************************/

  if (!(Message.MsgType & KXCMSGTYPE_NOTIFY))
  {
    pQuery = (ACCOUNT_QUERY *) Message.MsgBuffer.pBuffer;
    printf ("QUERY: Name %s; Account %s; Balance %s\n",
                   pQuery->AccountName,
                   pQuery->AccountNumber,
                   pQuery->CurrentAmount);
    /*****************************************************************/
    /* Free inbound message buffer                                   */
    /*****************************************************************/
    kxcFreeBuffer (XCi, &(Message.MsgBuffer));
  }
  else
  {
    printf ("Error %s\n", (char *) Message.TagBuffer.pBuffer);
    /*****************************************************************/
    /* Free inbound tag message buffer                               */
    /*****************************************************************/
    kxcFreeBuffer (XCi, &(Message.TagBuffer));
  }

  /******************************************************************/
  /* Close ILS Data handle and Terminate                            */
  /******************************************************************/
  if (0 != ILSHandle)
    kxcCloseHandle (XCi, &ILSHandle);

  kxcTerminate (&XCi);

  return;
}
