/********************************************************************/
/*                                                                  */
/* Module      : ILS20.C                                            */
/*                                                                  */
/* Description :                                                    */
/*               This contains a simple inline service, providing   */
/*               simple authentication.  This demonstrates  the     */
/*               following ILS features:                            */
/*               * how to use ILS trailers.                         */
/*               * how to exchange information between ILSs and     */
/*                 applications.                                    */
/*                                                                  */
/*               This example uses hard-coded literals as this      */
/*               makes it easier to see what is happening.          */
/*                                                                  */
/* Environment :                                                    */
/*               Client20.c contains code to handle the client      */
/*               side.                                              */
/*               Server20.c contains code to handle the server      */
/*               side.                                              */
/*               Examples20.ldf contains the directory.             */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 2000                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include the eBP header file                                      */
/* Also include kxcxit.h as this defines the ILS interface.         */
/********************************************************************/
#include <kxc.h>
#include <kxcxit.h>
#include <kxcmac.h>


/********************************************************************/
/* Include the C standard header files                              */
/********************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/********************************************************************/
/* Function definitions                                             */
/********************************************************************/
KXCSERVINIT ILS20Initialize;
KXCSERVTERM ILS20Terminate;
KXCSERVPUT  ILS20OutBound;
KXCSERVGET  ILS20InBound;

/********************************************************************/
/* This table defines the functions that make up this ILS.          */
/********************************************************************/
KXCSERVFUNC UserILSFuncs = { ILS20Initialize
                           , ILS20Terminate
                           , ILS20OutBound
                           , ILS20InBound
                           };

/********************************************************************/
/* This structure defines the inline service to eBP.                */
/* See ILS18 for details.                                           */
/********************************************************************/
KXCXIT ILS20ExitDef = { KXCSIG_KXCXIT
                      , KXCXIT_VERSION_1
                      , KXCXIT_TYPE_SERVICE
                      , KXCXIT_CURR_IFACE_LEVEL
                      , KXCXIT_FUNC_LEVEL_1
                      , 20
                      , KXCXIT_CALLTYPE_TRAILER
                      , KXCXIT_SERV_NUMFUNCS
                      , &UserILSFuncs
                      };

/********************************************************************/
/* Define the Entry point for the in-line service                   */
/********************************************************************/
KXCXIT_DEFINEENTRY(ILS20ExitDef)


/********************************************************************/
/* Local type definitions.  This type defines a area of memory that */
/* this inline service creates for each context.  The inline        */
/* service keeps a buffer that it reuses each time.  If the buffer  */
/* is not large enough, it frees the existing buffer and creates a  */
/* new one.  This memory area records the address and size of the   */
/* current buffer.                                                  */
/********************************************************************/

typedef struct {
  KXCULONG            BufSize;
  PKXCVOID            pBuffer;
} BUFFERDEF;


/********************************************************************/
/* This is the function that eBP calls on initialization.           */
/* This function creates a buffer definition and hooks it onto      */
/* the pHandleContext.  This means that each separate context will  */
/* have its own structure - this ensures that the inline service is */
/* thread safe and gives the ILS the ability to reuse a buffer on   */
/* each message exchange rather than allocate a new one each time.  */
/********************************************************************/
KXCBOOL ILS20Initialize(PKXCXCTL   pXCtl,
                        PKXCCONFIG pConfig,
                        KXCULONG   InitType)
{
  KXCCBACK   *pCBack;          /* Pointer to call back functions    */
  KXCBOOL    bSuccess;         /* Result of this ILS - if we return */
                               /* failure, eBP will fail the        */
                               /* initialize call.                  */
  BUFFERDEF  *pBufferDef;      /* Pointer to the structure created  */
                               /* to record buffer pointer and size */

  /******************************************************************/
  /* Create an area to record the buffer information.               */
  /******************************************************************/
  pCBack   = (KXCCBACK *)pXCtl->pCallBacks;
  bSuccess = (pCBack->Alloc)(pXCtl->hKxc,
                             KXCALLOC_CONTEXT,
                             sizeof(BUFFERDEF),
                 (PKXCVOID *)&pBufferDef);

  if (bSuccess)
  {
    /****************************************************************/
    /* Initialize the buffer definition.                            */
    /****************************************************************/
    pBufferDef->BufSize = 0;
    pBufferDef->pBuffer = 0;

    /****************************************************************/
    /* And assign the handle context to the buffer.                 */
    /****************************************************************/
    pXCtl->pHandleContext = (PKXCVOID) pBufferDef;
  }

  return (bSuccess);
}


/********************************************************************/
/* This is the function that eBP calls on termination.              */
/********************************************************************/
KXCBOOL  ILS20Terminate(PKXCXCTL pXCtl,
                        KXCULONG TermType)
{
  KXCCBACK   *pCBack;          /* Pointer to call back functions    */
  BUFFERDEF  *pBufferDef;      /* Pointer to the structure created  */
                               /* to record buffer pointer and size */


  if ( 0 !=  pXCtl->pHandleContext )
  { 
    /******************************************************************/
    /* Clean up the area to record the buffer information.            */
    /******************************************************************/
    pCBack   = (KXCCBACK *)pXCtl->pCallBacks;
    pBufferDef = (BUFFERDEF  *)pXCtl->pHandleContext;

    if ( 0 != pBufferDef->pBuffer )
      free(pBufferDef->pBuffer);

    pCBack->Free(pXCtl->hKxc, pBufferDef);
    pXCtl->pHandleContext = 0;
  }

  return (TRUE);
}


/********************************************************************/
/* eBP calls this function when it sends a message.                 */
/********************************************************************/
KXCBOOL ILS20OutBound(PKXCXCTL       pXCtl,
                      PKXCSERVPUTDTL pPutDtl,
                      PKXCSERVACT    pAction)
{

  /******************************************************************/
  /* Action is only needed if this is the client.  The server does  */
  /* not need to add a identity buffer as the client has requested  */
  /* the message. Check we can modify message.                      */
  /******************************************************************/

  if ( (pPutDtl->pService != NULL) &&
       (pPutDtl->Status & KXCSERVMSG_READWRITE) )
  {
    /****************************************************************/
    /* The application is going to pass down the ID for the         */
    /* enquiry.  This is the buffer that the ILS will add as a      */
    /* trailer.                                                     */
    /****************************************************************/
    if (pPutDtl->ILSBuff.pBuffer)
    {
      pAction->TrlBuffer.pBuffer = pPutDtl->ILSBuff.pBuffer;
      pAction->TrlBuffer.BufLen  = pPutDtl->ILSBuff.BufLen;
      pAction->TrlBuffer.Status  = pPutDtl->ILSBuff.Status;
      pAction->Actions = KXCSERVACT_NEWTRAILER;
    }
    else
    {
      pAction->Actions = KXCSERVACT_NONE;
    }
  }

  /******************************************************************/
  /* Otherwise there is no action to take.                          */
  /******************************************************************/
  else
  {
    pAction->Actions = KXCSERVACT_NONE;
  }

  return (TRUE);
}

/********************************************************************/
/* The inbound function.  This checks to see if there is a trailer. */
/* If there is, it sets the message to authenticated and makes the  */
/* buffer available to the application. Note that eBP will take a   */
/* copy of the buffer and hand that to the application. eBP will    */
/* then release our buffer if it is a KXC buffer. So that the life  */
/* time of our buffer is in our control we use a user buffer which  */
/* can be reused each message                                       */
/********************************************************************/
KXCBOOL ILS20InBound(PKXCXCTL       pXCtl,
                     PKXCSERVGETDTL pGetDtl,
                     PKXCSERVACT    pAction)
{
  KXCCBACK   *pCBack;          /* Pointer to call back functions    */
  KXCBOOL    bSuccess = FALSE;
  BUFFERDEF  *pBufferDef;      /* Pointer to the structure created  */
                               /* to record buffer pointer and size */


  if ( 0 !=  pXCtl->pHandleContext )
  { 
    pCBack   = (KXCCBACK *)pXCtl->pCallBacks;
    pBufferDef = (BUFFERDEF  *)pXCtl->pHandleContext;
  } else {
    kxcSetError(pXCtl, KXCRC_BUFFER_ERROR, KXCOP_USER, 
                       KXCRC_BUFFER_ERROR, KXCPOS_1, 
                       bSuccess);
    return (bSuccess);
  }


  /******************************************************************/
  /* Check to see if there is a trailer.                            */
  /******************************************************************/
  if ((pGetDtl->pUsrTrl) && (pGetDtl->UTrlLen > 0))
  {

    /****************************************************************/
    /* Set the action to (i) authenticated and (ii) pass the ILS    */
    /* buffer to the application.                                   */
    /****************************************************************/
    pAction->Actions = KXCSERVACT_VALIDATED | 
                       KXCSERVACT_NEWILSBUFF;

    /****************************************************************/
    /* Allocate or reuse a user buffer for the ID                   */
    /****************************************************************/
    if ( pBufferDef->BufSize < pGetDtl->UTrlLen ) {
      if ( 0 != pBufferDef->pBuffer )
        free(pBufferDef->pBuffer);
      pBufferDef->pBuffer = 0;
      pBufferDef->BufSize  = 0;
    }
    if ( 0 == pBufferDef->pBuffer ) {
      pBufferDef->pBuffer  = malloc(pGetDtl->UTrlLen );
      pBufferDef->BufSize  = pGetDtl->UTrlLen ;
    }
    /****************************************************************/
    /* Now make the ID available to the application.                */
    /****************************************************************/
    if ( 0 != pBufferDef->pBuffer )
    {
      strncpy(pBufferDef->pBuffer, pGetDtl->pUsrTrl, pGetDtl->UTrlLen);
      pAction->ILSBuff.pBuffer = (PKXCVOID) pBufferDef->pBuffer;
      pAction->ILSBuff.BufLen  = pGetDtl->UTrlLen;
      pAction->ILSBuff.Status  = KXCBUFSTAT_USER;
      bSuccess = TRUE;
    }
    else
    {
      kxcSetError(pXCtl, KXCRC_STORAGE_FAILURE, KXCOP_USER, 
                         KXCRC_STORAGE_FAILURE, KXCPOS_1, 
                         bSuccess);
    }
  }

  /******************************************************************/
  /* Otherwise there is no action to take.                          */
  /******************************************************************/
  else
  {
    pAction->Actions = KXCSERVACT_NONE;
    bSuccess = TRUE;
  }

  return (bSuccess);
}
