/********************************************************************/
/*                                                                  */
/* Module      : Server11.C                                         */
/*                                                                  */
/* Description :                                                    */
/*               Thos server handles unexpected messages - that is, */
/*               any message that is not a QUERY or a DEPOSIT.      */
/*               Client11.c sends a TRANSFER message, which this    */
/*               server handles as an error.  It logs a message,    */
/*               sends a message to the alert system and sends the  */
/*               message to the EMS, and sends a notification       */
/*               message back to the client (but only if the        */
/*               message that it receives is not a datagram).  This */
/*               server does all of these operations explicitly.    */  
/*               Server12 uses Roma's error profiling to take       */
/*               action implicitly.                                 */
/*                                                                  */
/*               This example uses hard-coded literals as this      */
/*               makes it easier to see what is happening.          */
/*                                                                  */
/* Environment :                                                    */
/*               Client11 sends a bad message.                      */
/*               Examples11.ldf contains the directory.             */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 2000                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include the Roma header file                                     */
/********************************************************************/
#include <kxc.h>


/********************************************************************/
/* Include the C standard header files                              */
/********************************************************************/
#include <string.h>
#include <stdio.h>

/********************************************************************/
/* Applications will typically define a structure in a global file. */
/* The structure is defined here to make this a complete example.   */
/* In this example, the server assumes that the message always      */
/* contains information regarding either a deposit into an account  */
/* or a balance enquiry.                                            */
/********************************************************************/

typedef struct {
  char         AccountName   [30];
  char         AccountNumber [30];
  char         DepositAmount [10];
} DEPOSIT_IN_ACCOUNT;

typedef struct {
  char         AccountName   [30];    /* Input */
  char         AccountNumber [30];    /* Input */
  char         CurrentAmount [10];    /* Returned */
} ACCOUNT_QUERY;


/********************************************************************/
/*                                                                  */
/* Function:  HandleError                                           */
/*                                                                  */
/* Purpose:   This is the single point at which the application     */
/*            handles all errors.  Input is a string which says     */
/*            which operation didn't work.                          */
/*                                                                  */
/********************************************************************/

void HandleError (char *     OpType)

{
  KXCERROR     Error;      /* Used to get the details about a      */
                           /* failure to execute a Roma function   */
  KXCCHAR      ErrorString[256];
                           /* Error reason in human-readable format*/
  KXCULONG     ErrorStringLen; 
                           /* Length of the error string           */

  printf ("%s\n", OpType);
  kxcGetLastError (&Error);
  if (Error.Status & KXCERRSTAT_MAIN)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.Result,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
  if (Error.Status & KXCERRSTAT_SUB)
  {
   ErrorStringLen = sizeof(ErrorString);
   kxcFormatError (&Error.SubResult,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
   printf ("%s\n", ErrorString);
  }
}


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Purpose:   As described above                                    */
/*                                                                  */
/********************************************************************/
void main ()

{
  KXCAPPDEF    ApplicationDef; 
                          /* Defines the server & options          */
  HKXC         XCi;       /* Used by Roma to associate operations  */
  KXCMSG       ReqMsg;    /* Defines the message and message       */
                          /* control options for inbound messages  */
  KXCMSG       RespMsg;   /* Defines the message and message       */
                          /* control options for outbound messages */
  KXCGETOPT    GetControl;/* Defines the options when reading the  */
                          /* message                               */
  KXCBOOL      result;    /* Used to check each Roma operation     */
  KXCPUTOPT    PutControl;/* Defines options when sending messages */
  HKXCTHEME    Theme;     /* Theme returned from Roma to associate */
                          /* response with request                 */

  KXCLOGDATA   LogInformation; 
                          /* Defines the data which will be        */
                          /* written to the Roma log               */
  KXCSERVICE   EMService; /* Defines the Exception Message Service */
  KXCEXCEPTION Exception; /* Exception options                     */
  KXCMSG       ExcMsg;    /* Defines the message and message       */
                          /* control options for sending exception */
                          /* messages                              */
  DEPOSIT_IN_ACCOUNT
               *pDeposit; /* Defines the application information   */
  ACCOUNT_QUERY 
               *pQuery;   /* Defines the application information   */
  char         ErrorDesc[240]; 
                          /* Holds error string                    */


  /******************************************************************/
  /* First build the Application Definition.                        */
  /******************************************************************/

  kxcSetAppDefDefault (&ApplicationDef);
  strcpy ((char *)ApplicationDef.ElementName, "AccountManagerElem01");

  /******************************************************************/
  /* Initialize the server                                          */
  /* Returns Success/Failure indicator.                             */
  /******************************************************************/
  
  result = kxcServerInitialize (&ApplicationDef, &XCi);
  if (!result)
  {
    HandleError ("Could not initialize the server");
    return;
  }

  while (TRUE)
  {
    /****************************************************************/
    /* Set up options to retrieve the next message.  These options  */
    /* are to wait for a response and to automatically allocate the */
    /* buffer.                                                      */
    /****************************************************************/

    kxcSetGetOptDefault (&GetControl);
    GetControl.ReceiveOptions   = KXCGOPT_AUTO_ALLOC | 
                                  KXCGOPT_WAIT;
    GetControl.WaitLen          = KXCGOWAIT_INDEFINITE;

    /****************************************************************/
    /* Set up the buffer ready to read the next message.            */
    /****************************************************************/

    kxcSetMsgDefault (&ReqMsg);
    result = kxcGetRequest 
                 (XCi,           /* Context as returned from        */
                                 /*  kxcServerInitialize            */
                  &ReqMsg,       /* The message read will be        */
                                 /*  returned here                  */
                  &GetControl,   /* Options selected when reading   */
                                 /*  the message                    */
                  NULL,          /* No syncpoint operation          */
                  &Theme);       /* Theme required for the response */

    if (result)
    {
      /**************************************************************/
      /* First, determine what the message is.  This will tell us   */
      /* what we want to do next.  The choices are QUERY and        */
      /* DEPOSIT.  We assume that we won't get anything else for    */
      /* this server.  The Format will tell us what we've got....   */
      /**************************************************************/

      if (strcmp ((char *)ReqMsg.FormatName, "DEPOSIT") == 0)
      {
        /************************************************************/
        /* We got a deposit advice.  The application would          */
        /* typically do a database update here.  We're going to     */
        /* print out the advise to demonstrate that the message was */
        /* received correctly.                                      */
        /************************************************************/

        pDeposit = (DEPOSIT_IN_ACCOUNT *) ReqMsg.MsgBuffer.pBuffer;
        printf ("DEPOSIT: Name %s; Account %s; Deposit %s\n",
                 pDeposit->AccountName,
                 pDeposit->AccountNumber,
                 pDeposit->DepositAmount);

        /************************************************************/
        /* Roma automatically closes the theme when the received    */
        /* message is a datagram.                                   */
        /* But just in case we got a REQUEST message and not a      */
        /* DATAGRAM, we'll close the theme.                         */
        /************************************************************/

        if (ReqMsg.MsgType & KXCMSGTYPE_REQUEST)
        {
          result = kxcCloseHandle (XCi, &Theme);
          if (!result)
            HandleError ("Could not close the theme");
        }
      }

      else if (strcmp ((char *)ReqMsg.FormatName, "QUERY") == 0)
      {
        /************************************************************/
        /* We print out the query information to demonstrate that   */
        /* we received the information correctly.  Normally, we'd   */
        /* do a database lookup to get the amount in the account.   */          
        /* In this case, we'll make it up, and return the           */
        /* information back to the client.                          */
        /************************************************************/

        pQuery = (ACCOUNT_QUERY *) ReqMsg.MsgBuffer.pBuffer;
        printf ("QUERY: Name %s; Account %s\n",
                 pQuery->AccountName,
                 pQuery->AccountNumber);

        strcpy (pQuery->CurrentAmount, "$1000000");

        /************************************************************/
        /* Now prepare to send the response.  This means building a */
        /* KXCMSG structure and a KXCPUTOPT structure.              */
        /************************************************************/

        /************************************************************/
        /* Firstly, the message structure.  This application uses   */
        /* a different message buffer.  Applications can use the    */
        /* same message buffer if they choose to.  Roma uses the    */
        /* THEME to associated the response and the request.  So,   */
        /* as long as we specify the same THEME, it doesn't matter  */
        /* which buffer we use.                                     */
        /************************************************************/

        kxcSetMsgDefault (&RespMsg);
        RespMsg.Format            = KXCMSGFORMAT_USENAME;
        RespMsg.Expiry            = KXCMSGEXP_NEVER;
        strcpy ((char *)RespMsg.FormatName,  "QUERY");

        RespMsg.MsgLen            = sizeof (*pQuery);
        RespMsg.MsgBuffer.Status  = KXCBUFSTAT_USER;
        RespMsg.MsgBuffer.BufLen  = sizeof (*pQuery);
        RespMsg.MsgBuffer.pBuffer = (PKXCVOID) pQuery;
          
        /************************************************************/
        /* Next set up the Put options.  No specific options are    */
        /* needed for this.  The default operation is to use the    */
        /* given theme and close it after the send.  This is        */
        /* exactly what we want to do.                              */
        /************************************************************/

        kxcSetPutOptDefault (&PutControl);

        /************************************************************/
        /* And send the response                                    */
        /************************************************************/

        result = kxcPutResponse 
                  (XCi,           /* Context from ServerInit       */
                  &RespMsg,       /* The response message and      */
                                  /*  message control              */
                  &PutControl,    /* Put options                   */
                  NULL,           /* no syncpointing               */
                  &Theme);        /* theme which Roma will close   */

        if (!result)
          HandleError ("Could not send the response");
      }
      else /* this is an unrecognised message - report errors */
      {
        printf ("Received unknown Message Format %s will reject.\n", 
                ReqMsg.FormatName);
        /************************************************************/
        /* First, log an error to disk. Build the information to    */
        /* log.  This consists of:                                  */
        /* i)    Severity of the error - information, warning,      */
        /*       error or critical.                                 */
        /* ii)   Format.  Whether to include timestamp, severity or */
        /*       a hex dump.                                        */
        /* iii)  User Data.  Pointer to a null terminated string.   */
        /* iv)   Data buffer for the hex dump.                      */
        /* v)    Data buffer length.                                */
        /************************************************************/

        kxcSetLogDefault (&LogInformation);
        LogInformation.Severity        = KXCLOGSEVERITY_ERROR; 
        LogInformation.Format          = KXCLOGFORMAT_HEXDUMP;
        sprintf (ErrorDesc, "Unknown Message Format %s", ReqMsg.FormatName);
        LogInformation.pUserText       =  (PKXCCHAR)ErrorDesc;
        LogInformation.DataBuf.pBuffer = ReqMsg.MsgBuffer.pBuffer;
        LogInformation.DataBuf.Status  = ReqMsg.MsgBuffer.Status;
        LogInformation.DataBuf.BufLen  = ReqMsg.MsgBuffer.BufLen;
        LogInformation.DataLen         = ReqMsg.MsgLen;

        /************************************************************/
        /* Now do the log.                                          */
        /* Unless told otherwise by environment variables or the    */
        /* directory, this will create an entry in file             */
        /* "kxcuser.log" in the runtime directory.                  */
        /************************************************************/

        result = kxcWriteLog (&LogInformation);

        if (!result)
          HandleError ("Could not write to the log");

        /************************************************************/
        /* Next, send an alert.  The log information structure is   */
        /* also used for this call.  Note that the message is only  */
        /* alerted if there is a system management inline service   */
        /* loaded for the application in the directory.             */
        /************************************************************/
        
        result = kxcSendAlert (XCi, 
                               &LogInformation);

        if (!result)
          HandleError ("Could not raise the alert");

        /************************************************************/
        /* Next, send the message back to the client.  This is only */
        /* possible if the client sent the message as a request -   */
        /* not a datagram.  In our case, the message was sent as a  */
        /* datagram, so this path won't happen.                     */
        /************************************************************/

        if (ReqMsg.Status & KXCMSGSTAT_OK_TO_NOTIFY)
        {
          /**********************************************************/
          /* When sending notifications, we have to make sure that  */
          /* client will be able to understand why we are sending   */
          /* it.  Errors is only one potential use.  In this        */
          /* example, we put the reason for the notification in the */
          /* tag buffer.                                            */
          /**********************************************************/

          kxcSetMsgDefault (&RespMsg);
          strcpy ((char *)RespMsg.FormatName,  
                  (char *)ReqMsg.FormatName);
          strcpy ((char *)RespMsg.MessageName, 
                  (char *)ReqMsg.MessageName);

          RespMsg.TagLen            = strlen(ErrorDesc);
          RespMsg.TagBuffer.Status  = KXCBUFSTAT_USER;
          RespMsg.TagBuffer.BufLen  = strlen(ErrorDesc);
          RespMsg.TagBuffer.pBuffer = ErrorDesc;

          /**********************************************************/
          /* Next set up the Put options.                           */
          /**********************************************************/

          kxcSetPutOptDefault (&PutControl);
          PutControl.RequestOptions   = KXCPOPT_NOTIFY | 
                                        KXCPOPT_RETAINTHEME;

          /**********************************************************/
          /* And send the notification                              */
          /**********************************************************/

          result = kxcPutResponse 
                    (XCi,         /* Context from ServerInit       */
                    &RespMsg,     /* The response message and      */
                                  /*  message control              */
                    &PutControl,  /* Put options                   */
                    NULL,         /* no syncpointing               */
                    &Theme);      /* theme which Roma will retain  */

          if (!result)
            HandleError ("Could not send the notification");
        } /* If notify is allowed */

        /************************************************************/
        /* Finally send an exception message to a specified         */
        /* Exception Message Service.  This uses PutException,      */
        /* whose parameters are:                                    */
        /* i)   The context from Initialize                         */
        /* ii)  The Exception Message Service which has to be       */
        /*      defined in the configuration.                       */
        /* iii) The message being reported.  This will form part    */
        /*      of the exception message.                           */
        /* iv)  The exception structure that defines what goes in   */
        /*      the exception message.                              */
        /* v)   Put control options as for PutRequest etc           */
        /* vi)  Syncpoint control as for PutRequest etc             */
        /* vii) The theme, which is why we didn't close the theme   */
        /*      when we sent the notification.                      */
        /************************************************************/

        /************************************************************/
        /* First build the service.  The service name must be in    */
        /* the directory.                                           */
        /************************************************************/

        kxcSetServiceDefault (&EMService);
        strcpy ((char *)EMService.ServiceName, "Exception");

        /************************************************************/
        /* We're not going to build a message - we're going to      */
        /* report the message that we got.                          */
        /************************************************************/

        /************************************************************/
        /* Now build the exception structure.                       */
        /* This contains:                                           */
        /* i)   Flag to tell Roma what operation the message was    */
        /*      rejected from - so that it can be replayed          */
        /* ii)  Flags to tell Roma what to put in the exception     */
        /*      message.                                            */
        /* iii) Timestamp.  Roma puts this in automatically         */
        /* iv)  Roma configuration data.  Roma puts this in         */
        /*      automatically                                       */
        /* v)   System configuration data.  Roma puts this in       */
        /*      automatically                                       */
        /* vi)  Error data. The application has to supply this.     */
        /*      In this case, the application is arbitrarily        */
        /*      setting these fields.  In a live environment, these */
        /*      error codes will be defined globally.               */
        /* vii) User data. The application has to supply this.      */
        /************************************************************/
        kxcSetExceptionDefault (&Exception);

        /************************************************************/
        /* This is the Call Type. This enables Roma to know how     */
        /* what routing information to gather so that the message   */
        /* can be replayed from the EMS via an EMS server eg kxcems */
        /* Note if the message is not to be replayed set Application*/
        /* Possible values are: KXCCALLTYPE_APPLICATION             */
        /*  KXCCALLTYPE_PUTREQUEST KXCCALLTYPE_GETREQUEST           */
        /* KXCCALLTYPE_PUTRESPONSE KXCCALLTYPE_GETRESPONSE          */
        /************************************************************/
        Exception.ErrorData.CallType     = KXCCALLTYPE_GETREQUEST;

        /************************************************************/
        /* This is the error data.                                  */
        /************************************************************/
        Exception.ErrorData.Error.Status = KXCERRSTAT_MAIN;
        Exception.ErrorData.Error.Result.LastCompCode   = KXCCC_FAILED;
        Exception.ErrorData.Error.Result.LastReturnCode = 100;
        Exception.ErrorData.Error.Result.LastOpType     = 0;
        Exception.ErrorData.Error.Result.LastReason     = 100;
        Exception.ErrorData.Error.Result.LastPosition   = 1;
        Exception.ErrorData.Error.Result.LastErrorType  = KXCETYPE_ERROR;
        strcpy ((char *)Exception.ErrorData.Error.Result.LastLocation,
                 ErrorDesc);

        /************************************************************/
        /* And this is the user data - the application specific     */
        /* information.                                             */
        /************************************************************/

        Exception.UserData.UserBuf.Status  = KXCBUFSTAT_USER;
        Exception.UserData.UserBuf.BufLen  = strlen(ErrorDesc);
        Exception.UserData.UserBuf.pBuffer = ErrorDesc;
        Exception.UserData.UserBufLen      = strlen(ErrorDesc);

        /************************************************************/
        /* Next set up the Put options.                             */
        /* The put options also define what is going to go into the */
        /* exception message - in this case everything - Roma       */
        /* configuration information, system configuration          */
        /* information, the time, the error details and application */
        /* specific information.  Note that if the theme is valid,  */
        /* we include this theme in the message; otherwise we have  */
        /* to create a new theme.  Using the same theme ensures     */
        /* that the Exception Message Service can replay the        */
        /* message if needed.                                       */
        /************************************************************/

        kxcSetPutOptDefault (&PutControl);
        PutControl.RequestOptions   = KXCPOPT_CLOSETHEME |
                                      KXCPOPT_EXCROMADATA |
                                      KXCPOPT_EXCSYSDATA |
                                      KXCPOPT_EXCERRDATA |
                                      KXCPOPT_EXCUSERDATA |
                                      KXCPOPT_EXCTIMESTAMP;
        if (ReqMsg.Status & KXCMSGSTAT_OK_TO_NOTIFY)
          PutControl.RequestOptions  |= KXCPOPT_USETHEME;
        else
          PutControl.RequestOptions  |= KXCPOPT_NEWTHEME;

        kxcSetMsgDefault (&ExcMsg);
        ExcMsg.Format             = KXCMSGFORMAT_USENAME;
        ExcMsg.Expiry             = KXCMSGEXP_NEVER;
        strcpy ((char *)ExcMsg.FormatName,  
                (char *)ReqMsg.FormatName);
        strcpy ((char *)ExcMsg.MessageName, 
                (char *)ReqMsg.MessageName);

        /************************************************************/
        /* Copy the invalid message.  Applications don't have to do */
        /* this - they can choose to pass anything in the message   */
        /* buffer, or nothing at all.                               */
        /************************************************************/

        ExcMsg.MsgLen             = ReqMsg.MsgLen;
        ExcMsg.MsgBuffer.Status   = KXCBUFSTAT_USER;
        ExcMsg.MsgBuffer.BufLen   = ReqMsg.MsgLen;
        ExcMsg.MsgBuffer.pBuffer  = ReqMsg.MsgBuffer.pBuffer;
          
        /************************************************************/
        /* Now send the exception.                                  */
        /************************************************************/

        result = kxcPutException 
                    (XCi,         /* Context from ServerInit       */
                     &EMService,  /* defines the exception service */
                     &ExcMsg,     /* the exception message         */
                     &Exception,  /* the exception details         */
                     &PutControl, /* controls for the operation    */
                     NULL,        /* no syncpointing               */
                     &Theme);      /* no theme required             */
                          
        if (!result)
          HandleError ("Could not send the exception message");

      }
    } /* If the GetRequest worked */

    else 
    {
      HandleError ("Could not read the message");
    }
  }
}
