/********************************************************************/
/*                                                                  */
/* Module      : Server12.C                                         */
/*                                                                  */
/* Description :                                                    */
/*               This server is similar to Server11.c.  In this     */
/*               case, rather than handle each error condition      */
/*               explicitly, this server calls the Roma             */
/*               kxcReportError function which in turn invokes the  */
/*               Error Analysis Routine which will then take        */
/*               appropriate action - log a message to disk,        */
/*               raise an alert, send a notification back to the    */
/*               client, send a message to the Exception Message    */
/*               Service.                                           */
/*                                                                  */
/*               This example uses hard-coded literals as this      */
/*               makes it easier to see what is happening.          */
/*                                                                  */
/* Environment :                                                    */
/*               Client11.c sends a message that this application   */
/*               can't process.                                     */
/*               Examples12.ldf contains the directory.             */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 2000                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include the Roma header file                                     */
/********************************************************************/
#include <kxc.h>


/********************************************************************/
/* Include the C standard header files                              */
/********************************************************************/
#include <string.h>
#include <stdio.h>

/********************************************************************/
/* Applications will typically define a structure in a global file. */
/* The structure is defined here to make this a complete example.   */
/* In this example, the server assumes that the message always      */
/* contains information regarding an account update.                */
/********************************************************************/

typedef struct {
  char         AccountName   [30];
  char         AccountNumber [30];
  char         DepositAmount [10];
} DEPOSIT_IN_ACCOUNT;

typedef struct {
  char         AccountName   [30];    /* Input */
  char         AccountNumber [30];    /* Input */
  char         CurrentAmount [10];    /* Returned */
} ACCOUNT_QUERY;


/********************************************************************/
/*                                                                  */
/* Function:  HandleError                                           */
/*                                                                  */
/* Purpose:   This is the single point at which the application     */
/*            handles all errors.  Input is a string which says     */
/*            which operation didn't work.                          */
/*                                                                  */
/********************************************************************/

void HandleError (char *     OpType)

{
  KXCERROR     Error;      /* Used to get the details about a      */
                           /* failure to execute a Roma function   */
  KXCCHAR      ErrorString[256];
                           /* Error reason in human-readable format*/
  KXCULONG     ErrorStringLen; 
                           /* Length of the error string           */


  printf ("%s\n", OpType);
  kxcGetLastError (&Error);
  if (Error.Status & KXCERRSTAT_MAIN)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.Result,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
  if (Error.Status & KXCERRSTAT_SUB)
  {
   ErrorStringLen = sizeof(ErrorString);
   kxcFormatError (&Error.SubResult,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
   printf ("%s\n", ErrorString);
  }
}


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Purpose:   As described above                                    */
/*                                                                  */
/********************************************************************/
void main ()

{
  KXCAPPDEF    ApplicationDef; 
                          /* Defines the server & options          */
  HKXC         XCi;       /* Used by Roma to associate operations  */
  KXCMSG       ReqMsg;    /* Defines the message and message       */
                          /* control options for inbound messages  */
  KXCMSG       RespMsg;   /* Defines the message and message       */
                          /* control options for outbound messages */
  KXCGETOPT    GetControl;/* Defines the options when reading the  */
                          /* message                               */
  KXCBOOL      result;    /* Used to check each Roma operation     */
  KXCPUTOPT    PutControl;/* Defines options when sending messages */
  HKXCTHEME    Theme;     /* Theme returned from Roma to associate */
                          /* response with request                 */

  KXCEPOPT     ErrorOpt;  /* Options to use when reporting an error*/
  KXCEXCEPTION Exception; /* Exception information                 */

  DEPOSIT_IN_ACCOUNT
               *pDeposit; /* Defines the application information   */
  ACCOUNT_QUERY 
               *pQuery;   /* Defines the application information   */
  char         ErrorDesc[240]; 
                          /* Holds error string                    */
  

  /******************************************************************/
  /* First build the Application Definition.                        */
  /******************************************************************/

  kxcSetAppDefDefault (&ApplicationDef);
  strcpy ((char *)ApplicationDef.ElementName, "AccountManagerElem01");

  /******************************************************************/
  /* Initialize the server                                          */
  /* Returns Success/Failure indicator.                             */
  /******************************************************************/
  
  result = kxcServerInitialize (&ApplicationDef, &XCi);
  if (!result)
  {
    HandleError ("Could not initialize the server");
    return;
  }

  while (TRUE)
  {
    /****************************************************************/
    /* Set up options to retrieve the next message.  These options  */
    /* are to wait for a response and to automatically allocate the */
    /* buffer.                                                      */
    /****************************************************************/

    kxcSetGetOptDefault (&GetControl);
    GetControl.ReceiveOptions   = KXCGOPT_AUTO_ALLOC | 
                                  KXCGOPT_WAIT;
    GetControl.WaitLen          = KXCGOWAIT_INDEFINITE;

    /****************************************************************/
    /* Set up the buffer ready to read the next message.            */
    /****************************************************************/

    kxcSetMsgDefault (&ReqMsg);
    result = kxcGetRequest 
                 (XCi,           /* Context as returned from        */
                                 /*  kxcServerInitialize            */
                  &ReqMsg,       /* The message read will be        */
                                 /*  returned here                  */
                  &GetControl,   /* Options selected when reading   */
                                 /*  the message                    */
                  NULL,          /* No syncpoint operation          */
                  &Theme);       /* Theme required for the response */

    if (result)
    {
      /**************************************************************/
      /* First, determine what the message is.  This will tell us   */
      /* what we want to do next.  The choices are QUERY and        */
      /* DEPOSIT.  We assume that we won't get anything else for    */
      /* this server.  The Format will tell us what we've got....   */
      /**************************************************************/

      if (strcmp ((char *)ReqMsg.FormatName, "DEPOSIT") == 0)
      {
        /************************************************************/
        /* We got a deposit advice.  The application would          */
        /* typically do a database update here.  We're going to     */
        /* print out the advise to demonstrate that the message was */
        /* received correctly.                                      */
        /************************************************************/

        pDeposit = (DEPOSIT_IN_ACCOUNT *) ReqMsg.MsgBuffer.pBuffer;
        printf ("DEPOSIT: Name %s; Account %s; Deposit %s\n",
                 pDeposit->AccountName,
                 pDeposit->AccountNumber,
                 pDeposit->DepositAmount);

        /************************************************************/
        /* Roma automatically closes the theme when the received    */
        /* message is a datagram.                                   */
        /* But just in case we got a REQUEST message and not a      */
        /* DATAGRAM, we'll close the theme.                         */
        /************************************************************/

        if (ReqMsg.MsgType & KXCMSGTYPE_REQUEST)
        {
          result = kxcCloseHandle (XCi, &Theme);
          if (!result)
            HandleError ("Could not close the theme");
        }
      }

      else if (strcmp ((char *)ReqMsg.FormatName, "QUERY") == 0)
      {
        /************************************************************/
        /* We print out the query information to demonstrate that   */
        /* we received the information correctly.  Normally, we'd   */
        /* do a database lookup to get the amount in the account.   */          
        /* In this case, we'll make it up, and return the           */
        /* information back to the client.                          */
        /************************************************************/

        pQuery = (ACCOUNT_QUERY *) ReqMsg.MsgBuffer.pBuffer;
        printf ("QUERY: Name %s; Account %s\n",
                 pQuery->AccountName,
                 pQuery->AccountNumber);

        strcpy (pQuery->CurrentAmount, "$1000000");

        /************************************************************/
        /* Now prepare to send the response.  This means building a */
        /* KXCMSG structure and a KXCPUTOPT structure.              */
        /************************************************************/

        /************************************************************/
        /* Firstly, the message structure.  This application uses   */
        /* a different message buffer.  Applications can use the    */
        /* same message buffer if they choose to.  Roma uses the    */
        /* THEME to associated the response and the request.  So,   */
        /* as long as we specify the same THEME, it doesn't matter  */
        /* which buffer we use.                                     */
        /************************************************************/

        kxcSetMsgDefault (&RespMsg);
        RespMsg.Format            = KXCMSGFORMAT_USENAME;
        RespMsg.Expiry            = KXCMSGEXP_NEVER;
        strcpy ((char *)RespMsg.FormatName,  "QUERY");

        RespMsg.MsgLen            = sizeof (*pQuery);
        RespMsg.MsgBuffer.Status  = KXCBUFSTAT_USER;
        RespMsg.MsgBuffer.BufLen  = sizeof (*pQuery);
        RespMsg.MsgBuffer.pBuffer = (PKXCVOID) pQuery;
          
        /************************************************************/
        /* Next set up the Put options.  No specific options are    */
        /* needed for this.  The default operation is to use the    */
        /* given theme and close it after the send.  This is        */
        /* exactly what we want to do.                              */
        /************************************************************/

        kxcSetPutOptDefault (&PutControl);

        /************************************************************/
        /* And send the response                                    */
        /************************************************************/

        result = kxcPutResponse 
                  (XCi,           /* Context from ServerInit       */
                  &RespMsg,       /* The response message and      */
                                  /*  message control              */
                  &PutControl,    /* Put options                   */
                  NULL,           /* no syncpointing               */
                  &Theme);        /* theme which Roma will close   */

        if (!result)
          HandleError ("Could not send the response");
      }
      else /* this is an unrecognised message - report errors */
      {
        printf("Received unknown Message Format %s, report Error.\n", 
                ReqMsg.FormatName);

        /************************************************************/
        /* First build the error options.  This contains:           */
        /* i)   Options.  Use the listed default actions.           */
        /* ii)  Error Profile to use.  If not specified, Roma uses  */
        /*      the default profile for the business element, or    */
        /*      the subnet.                                         */
        /* iii) Default actions.  This says what we would like the  */
        /*      Error Analysis Routine to do.  It can choose to do  */
        /*      something else...                                   */
        /* iv)  Error information - returned if the ReportError     */
        /*      fails.                                              */
        /* v)   A list of what the ReportError actually did.  Roma  */
        /*      returns this after calling kxcReportError.          */
        /************************************************************/

        kxcSetErrOptDefault (&ErrorOpt);
        ErrorOpt.ErrorOptions          = KXCEPOPT_USEDEFAULT |
                                         KXCEPOPT_USEEPNAME;
        strcpy ((char *)ErrorOpt.ErrorProfile, "ErrProfile");
        ErrorOpt.DefaultAction         = KXCEP_ACTION_NOTIFY |
                                         KXCEP_ACTION_EMS |
                                         KXCEP_ACTION_LOG |
                                         KXCEP_ACTION_ALERT;

        /************************************************************/
        /* Now build the exception structure.                       */
        /* This contains:                                           */
        /* i)   Flag to tell Roma what operation the message was    */
        /*      rejected from - so that it can be replayed          */
        /* ii)  Flags to tell Roma what to put in the exception     */
        /*      message.  This is not used when calling ReportError */
        /* iii) Timestamp.  Roma puts this in automatically         */
        /* iv)  Roma configuration data.  Roma puts this in         */
        /*      automatically                                       */
        /* v)   System configuration data.  Roma puts this in       */
        /*      automatically                                       */
        /* vi)  Error data. The application has to supply this.     */
        /*      In this case, the application is arbitrarily        */
        /*      setting these fields.  In a live environment, these */
        /*      error codes will be defined globally.               */
        /* vii) User data. The application has to supply this.      */
        /************************************************************/
        kxcSetExceptionDefault (&Exception);

        /************************************************************/
        /* This is the Call Type. This enables Roma to know how     */
        /* what routing information to gather so that the message   */
        /* can be replayed from the EMS via an EMS server eg kxcems */
        /* Note if the message is not to be replayed set Application*/
        /* Possible values are: KXCCALLTYPE_APPLICATION             */
        /*  KXCCALLTYPE_PUTREQUEST KXCCALLTYPE_GETREQUEST           */
        /* KXCCALLTYPE_PUTRESPONSE KXCCALLTYPE_GETRESPONSE          */
        /************************************************************/
        Exception.ErrorData.CallType     = KXCCALLTYPE_GETREQUEST;

        /************************************************************/
        /* This is the error data.                                  */
        /************************************************************/
        Exception.ErrorData.Error.Status = KXCERRSTAT_MAIN;
        Exception.ErrorData.Error.Result.LastCompCode   = KXCCC_FAILED;
        Exception.ErrorData.Error.Result.LastReturnCode = 100;
        Exception.ErrorData.Error.Result.LastOpType     = 0;
        Exception.ErrorData.Error.Result.LastReason     = 100;
        Exception.ErrorData.Error.Result.LastPosition   = 1;
        Exception.ErrorData.Error.Result.LastErrorType  = KXCETYPE_ERROR;
        strcpy ((char *)Exception.ErrorData.Error.Result.LastLocation,
                 ErrorDesc);

        sprintf (ErrorDesc, "Unknown Message Format %s", ReqMsg.FormatName);
        Exception.UserData.Version = KXCVER_EXCUSRDATA_CURR;
        Exception.UserData.UserBuf.Status  = KXCBUFSTAT_USER;
        Exception.UserData.UserBuf.BufLen  = strlen(ErrorDesc);
        Exception.UserData.UserBuf.pBuffer = ErrorDesc;
        Exception.UserData.UserBufLen      = strlen(ErrorDesc);

        /************************************************************/
        /* Now report the error.                                    */
        /************************************************************/

        result = kxcReportError
                  (XCi,           /* Context from ServerInit       */
                   &ErrorOpt,     /* Options when reporting errors */
                   &ReqMsg,       /* The actual message recieved   */
                   &Exception,    /* Defines the error condition   */
                   NULL,          /* no syncpointing               */
                   &Theme);       
                          
        if (!result)
          HandleError ("Could send the exception message");

      }
    } /* If the GetRequest worked */

    else 
    {
      HandleError ("Could not read the message");
    }
  }
}
