/********************************************************************/
/*                                                                  */
/* Module      : Server20.C                                         */
/*                                                                  */
/* Description :                                                    */
/*               This contains a simple server.  It is a "long-     */
/*               running application", which initializes, and then  */
/*               continues to receive and process messages.  It     */
/*               never terminates itself.                           */
/*               The  application gets information from the inline  */
/*               service, and checks that the ID supplied is valid. */
/*               If it is invalid, it returns an error message in   */
/*               the Tag.                                           */
/*                                                                  */
/*               This example uses hard-coded literals as this      */
/*               makes it easier to see what is happening.          */
/*                                                                  */
/* Environment :                                                    */
/*               Client20.c contains code to handle the client      */
/*               side.                                              */
/*               Examples20.ldf contains the directory.             */
/*               ILS20.c contains the sample ILS that operates with */
/*               this client.                                       */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 2000                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include the Roma header file                                     */
/* We also need kxcxit to define the ILS type                       */
/********************************************************************/
#include <kxc.h>
#include <kxcxit.h>


/********************************************************************/
/* Include the C standard header files                              */
/********************************************************************/
#include <string.h>
#include <stdio.h>

/********************************************************************/
/* Applications will typically define a structure in a global file. */
/* The structure is defined here to make this a complete example.   */
/* In this example, the server assumes that the message always      */
/* contains information regarding an account update.                */
/********************************************************************/

typedef struct {
  char         AccountName   [30];    /* Input */
  char         AccountNumber [30];    /* Input */
  char         CurrentAmount [10];    /* Returned */
} ACCOUNT_QUERY;


/********************************************************************/
/*                                                                  */
/* Function:  HandleError                                           */
/*                                                                  */
/* Purpose:   This is the single point at which the application     */
/*            handles all errors.  Input is a string which says     */
/*            which operation didn't work.                          */
/*            See Server01 for a description of the operation.      */
/*                                                                  */
/********************************************************************/

void HandleError (char *     OpType)

{
  KXCERROR     Error;      /* Used to get the details about a       */
                           /* failure to execute a Roma function    */
  KXCCHAR      ErrorString[256];
                           /* Error reason in human-readable format */
  KXCULONG     ErrorStringLen; 
                           /* Length of the error string            */

  printf ("%s\n", OpType);
  kxcGetLastError (&Error);
  if (Error.Status & KXCERRSTAT_MAIN)
  {
    ErrorStringLen = sizeof(ErrorString);
    kxcFormatError (&Error.Result,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
    printf ("%s\n", ErrorString);
  }
  if (Error.Status & KXCERRSTAT_SUB)
  {
   ErrorStringLen = sizeof(ErrorString);
   kxcFormatError (&Error.SubResult,
                    KXCERROR_FULL,
                    (char *)ErrorString,
                    &ErrorStringLen);
   printf ("%s\n", ErrorString);
  }
}


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Purpose:   As described above                                    */
/*                                                                  */
/********************************************************************/
void main ()

{
  KXCAPPDEF     ApplicationDef; /* Defines the server & options     */
  HKXC          XCi;            /* Used by Roma to associate        */
                                /* operations                       */
  KXCMSG        ReqMsg;         /* Defines the message and message  */
                                /* control options for inbound      */
                                /* messages                         */
  KXCGETOPT     GetControl;     /* Defines the options when reading */
                                /* the message                      */
  HKXCTHEME     Theme;          /* Theme returned from Roma         */
  KXCMSG        RspMsg;         /* Defines the message and message  */
                                /* control options for outbound     */
                                /* messages                         */
  KXCPUTOPT     PutControl;     /* Defines options when sending     */
                                /* messages                         */
  KXCILSOPT     ILSOptions;     /* Controls obtaining information   */
                                /* from ILS                         */


  KXCBOOL       CorrectId;      /* Flag to indicate if we should    */
                                /* respond with requested data      */
  KXCBOOL       result;         /* Used to check each Roma operation*/
  ACCOUNT_QUERY *pQuery;        /* The information returned by the  */
                                /* server.                          */

  /******************************************************************/
  /* Build the Application Definition.                              */
  /******************************************************************/

  kxcSetAppDefDefault (&ApplicationDef);
  strcpy ((char *)ApplicationDef.ElementName, "AccountManagerElem01");

  /******************************************************************/
  /* Initialize the server                                          */
  /******************************************************************/
  
  result = kxcServerInitialize (&ApplicationDef, &XCi);
  if (!result)
  {
    HandleError ("Could not initialize the server");
    return;
  }

  while (TRUE)
  {
    kxcSetGetOptDefault (&GetControl);
    GetControl.ReceiveOptions   = KXCGOPT_AUTO_ALLOC | 
                                  KXCGOPT_WAIT;
    GetControl.WaitLen          = KXCGOWAIT_INDEFINITE;

    /****************************************************************/
    /* Set up the buffer ready to read the next message.            */
    /****************************************************************/

    kxcSetMsgDefault (&ReqMsg);

    result = kxcGetRequest 
                 (XCi,           /* Context as returned from        */
                                 /*   kxcServerInitialize           */
                  &ReqMsg,       /* The message read will be        */
                                 /*  returned here                  */
                  &GetControl,   /* Options selected when reading   */
                                 /*  the queue                      */
                  NULL,          /* No syncpoint operation          */
                  &Theme);       /* Theme for response              */

    /****************************************************************/
    /* If we got a message and the message was authenticated by the */
    /* inline service then attempt to get the ID.                   */
    /****************************************************************/

    CorrectId = FALSE;
    if (result  &&  (ReqMsg.Status & KXCMSGSTAT_AUTHENTICATED))
    {
      /**************************************************************/
      /* Get the ID from the inline service.                        */
      /* Note that the application needs to know that the ILS that  */
      /* gets the ID is number 20.                                  */
      /**************************************************************/

      kxcSetILSDefault (&ILSOptions);
      ILSOptions.Type    = KXCXIT_TYPE_SERVICE;
      ILSOptions.Method  = 20;

      result = kxcGetILSData 
                 (&ILSOptions,   /* Options when obtaining an ILS   */ 
                                 /*  buffer                         */
                  GetControl.ILSData.hFromILS);
                                 /* The handle returned from the    */
                                 /*  GetRequest                     */

      /**************************************************************/
      /* If there is an ID then check that it is correct            */
      /**************************************************************/
      if (result)
      {
        if (strncmp ((char *) ILSOptions.DataBuf.pBuffer,
                     (char *) "Elizabeth",
                     ILSOptions.DataLen) == 0)
          CorrectId = TRUE;
        /************************************************************/
        /* eBP hands the application a new copy of the ILS data so  */
        /* that the application manages the life time of it         */
        /* We have used it so release it and the handle             */
        /************************************************************/
        if (ILSOptions.DataBuf.Status == KXCBUFSTAT_KXC)
          kxcFreeBuffer (XCi, &(ILSOptions.DataBuf));
        kxcCloseHandle (XCi, &(GetControl.ILSData.hFromILS));
      }

      /**************************************************************/
      /* If the Correct ID is received then return the answer       */
      /**************************************************************/
      if (CorrectId)
      {
        pQuery = (ACCOUNT_QUERY *) ReqMsg.MsgBuffer.pBuffer;
        printf ("QUERY: Name %s; Account %s\n",
                 pQuery->AccountName,
                 pQuery->AccountNumber);

        strcpy ((char *)pQuery->CurrentAmount, "$1000000");

        /************************************************************/
        /* Now prepare to send the response.                        */
        /************************************************************/

        kxcSetMsgDefault (&RspMsg);
        RspMsg.Format            = KXCMSGFORMAT_USENAME;
        strcpy ((char *)RspMsg.FormatName,  "QUERY");
        strcpy ((char *)RspMsg.MessageName, "MessageName");

        RspMsg.MsgLen            = sizeof (*pQuery);
        RspMsg.MsgBuffer.Status  = KXCBUFSTAT_USER;
        RspMsg.MsgBuffer.BufLen  = sizeof (*pQuery);
        RspMsg.MsgBuffer.pBuffer = (PKXCVOID) pQuery;
          
        /************************************************************/
        /* Next set up the Put options.                             */
        /************************************************************/

        kxcSetPutOptDefault (&PutControl);
        PutControl.RequestOptions = KXCPOPT_FREEBUFFER;

        /************************************************************/
        /* And send the response                                    */
        /************************************************************/

        result = kxcPutResponse (XCi,            /* Context from ServerInit */
                                 &RspMsg,      /* The response message and control */
                                 &PutControl,    /* Put options */
                                 NULL,           /* no syncpointing */
                                 &Theme);        /* theme which Roma will close */

        if (!result)
        {      
          HandleError ("Could not send the response");
        }
      } /* If the ID was right */

      /**************************************************************/
      /* If the Correct ID is not received then return an error     */
      /* message in the tag buffer                                  */
      /**************************************************************/

      else 
      {
        /************************************************************/
        /* Now prepare to send the response.                        */
        /************************************************************/

        kxcSetMsgDefault (&RspMsg);
        RspMsg.Format            = KXCMSGFORMAT_USENAME;
        strcpy ((char *)RspMsg.FormatName,  "QUERY");
        strcpy ((char *)RspMsg.MessageName, "MessageName");

        RspMsg.TagBuffer.pBuffer = (PKXCVOID) "ID Invalid";
        RspMsg.TagBuffer.Status  = KXCBUFSTAT_USER;
        RspMsg.TagBuffer.BufLen  = strlen ((char *) RspMsg.TagBuffer.pBuffer);
        RspMsg.TagLen            = strlen ((char *) RspMsg.TagBuffer.pBuffer);
          
        /************************************************************/
        /* Next set up the Put options.                             */
        /************************************************************/

        kxcSetPutOptDefault (&PutControl);
        PutControl.RequestOptions   = KXCPOPT_NOTIFY;

        /************************************************************/
        /* And send the response                                    */
        /************************************************************/

        result = kxcPutResponse 
                 (XCi,           /* Context as returned from        */
                                 /*   kxcServerInitialize           */
                  &RspMsg,       /* This defines the response       */
                                 /*  message                        */
                  &PutControl,   /* Put options                     */
                  NULL,          /* no syncpointing                 */
                  &Theme);       /* theme from GetRequest           */

        if (!result)
          HandleError ("Could not send the response");

        /************************************************************/
        /* Free the inbound message buffer                          */
        /************************************************************/

        kxcFreeBuffer (XCi, &(ReqMsg.MsgBuffer));
      }
    } /* If the GetRequest worked */

    else 
    {
      HandleError ("Could not read the message");
    }
  }
}
