//=========================================================================
//
// Module      : RomaEmsServerSample.java
//
// Description : Sample Roma EMS Server program
//
//=========================================================================
//
// Copyright Candle Corporation 2003. All rights reserved.
//
//=========================================================================
//
// Disclaimer:  This sample program is provided for
//              information purposes only.  Candle neither makes
//              any warranties or representations regarding this
//              software, or its use.  This sample program is
//              provided "as is" and Candle expressly disclaims the
//              warranties of merchantability and fitness for a
//              particular purpose.
//
//=========================================================================
// $Id: RomaEmsServerSample.java,v 1.1.2.9 2003/10/24 14:51:01 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/Attic/RomaEmsServerSample.java,v $
//=========================================================================
//
// Description:
//    Sample Roma EMS Server Program.
//
//    This program shows a very simple Roma EMS server in action.
//    It is intended to show representative usage of the Roma
//    Java EMS API.
//
// Details:
//    This program reads EMS messages from its input queue, and
//    processes them according to command-line options.
//    It can browse and display the contents of the EMS messages,
//    or it can modify the user-message data and return the message
//    back onto the queue on which the reported operation failed.
//
//    So, for example, if a Roma server application fails to GET
//    a message from its input queue (for whatever reason), and
//    Error Profiling is configured to cause an EMS message to be
//    sent to this EMS server, then the EMS server can modify the
//    message data and return the message back to the Roma server's
//    input queue to allow it to re-attempt the GET operation.
//
//    If a PUT operation to a specific queue fails, and an EMS
//    message is set to this EMS server, then it will again replace
//    the message data and attempt to PUT the message onto the
//    original target queue again.
//
// Dependencies:
//    Requires the relevant services to be set up in the directory
//    and the underlying transport queues to be configured in
//    support of the routes defined for the directory entry.
//    Error Profiling needs to be configured to have EMS as the
//    required action, and the Busines Service representing this
//    EMS server to be the EMSServer that is used.
//
// Invocation Syntax:
// java RomaEmsServerSample [ -e ElementName ] [ -r ] [ -v ] [ -? ]
//
// Input Parameters (all optional):
//    -e ElementName:
//       Specifies the Roma Business Element name that this EMS
//       Server will initialise as.  Default is "TestElementEMS".
//
//    -r:
//       By default the server peeks messages from the EMS queue
//       and prints detailed information on them. With this option,
//       the server will get messages from the EMS queue, modify
//       their data and put back onto the queue around which the
//       original error occurred.
//
//    -v:
//       Turns on "verbose" mode.  Default is off.
//
//=========================================================================


import java.util.*;
import java.text.DateFormat;
import com.candle.roma.kxc.*;


public class RomaEmsServerSample
{
    public static void main(String[] args)
    {
        new RomaEmsServerSample(args);
    }

    public RomaEmsServerSample(String[] args)
    {
        try
        {
            //=============================================================
            // Parse command line.
            //=============================================================

            parseCmdLine(args);

		    if (m_help)
		    {
                //=========================================================
                // Print usage information.
                //=========================================================

			    System.out.println(
                "\nUsage: java RomaEmsServerSample [-e ElementName] " +
                "[-r] [-v]\n" +

                "Options:" +

                "\t-e ElementName" +
                "\t\tSpecifies the Roma Business Element name that this EMS" +
                "\t\tServer will initialise as. Default is TestElementEMS." +

                "\t-r" +
                "\t\tBy default the server peeks messages from the EMS queue" +
                "\t\tand prints detailed information on them. With this option," +
                "\t\tthe server will get messages from the EMS queue, modify" +
                "\t\ttheir data and put back onto the queue around which the" +
                "\t\toriginal error occurred." +

                "\t-v" +
                "\t\tTurns on verbose mode. Default is off.");

			    return;
		    }

            //=============================================================
            // Start server loop.
            //=============================================================

            runServer();

            //=============================================================
            // Disconnect and exit.
            //=============================================================

            System.out.println("EMS Server ending");

            if (m_elem != null && m_elem.isConnected())
            {
                m_elem.disconnect();
            }
        }
        catch (RomaError e)
        {
            //=============================================================
            // Handle Roma error.
            //=============================================================

            System.out.println("EMS Server failed");

            if (e.status() == Roma.ERROR_STATUS_NONE)
            {
                System.err.println(e.getMessage());
            }
            else
            {
                System.err.println(
                    e.errorText(Roma.PRIMARY_INFO, 
                        Roma.ERROR_TEXT_FULL));

                if (e.status() == Roma.ERROR_STATUS_BOTH)
                {
                    System.err.println(
                        e.errorText(Roma.SECONDARY_INFO, 
                            Roma.ERROR_TEXT_FULL));
                }
            }

            System.err.println("");
            e.printStackTrace();
        }
        catch (Throwable t)
        {
            //=============================================================
            // Handle other error.
            //=============================================================

            System.out.println("EMS Server failed");
            System.err.println(t.getMessage());
            System.err.println("");
            t.printStackTrace();
        }
    }


    //=====================================================================
    // Message retrieving and processing routine.
    //=====================================================================

    private void runServer()
    throws RomaError, Exception
    {
        //=================================================================
        // Setup GET options.
        //=================================================================

        RomaGetOptions opt = new RomaGetOptions();

        // Accept native messages.
        opt.allowNative(true);

        // Prohibit EAR intervention.
        opt.setNoEms(true);

        // Close the recieved message theme.
        opt.setCloseTheme(true);

        if (m_display)
        {
            // Leave messages on the queue.
            opt.setOperation(Roma.PEEK_FIRST);
            // Don't wait for a new message.
            opt.setWaitInterval(0);
        }

        if (m_verbose)
        {
            System.out.println(
                "\nInitialising EMS Server as " + m_name +
                "\nAction is " + (m_display ? "DISPLAY" : "RETURN"));
        }

        //=================================================================
        // Create and connect the server.
        //=================================================================

        m_elem = new RomaBusinessElement(m_name);

        //=================================================================
        // Message processing loop.
        //=================================================================

        while (true)
        {
            //=============================================================
            // Receive a message.
            //=============================================================

            RomaBeRequest msg = m_elem.get(opt);

            if (msg == null)
            {
                if (m_verbose)
                {
                    System.out.println("No more EMS messages");
                }

                break;
            }

            if (m_verbose)
            {
                System.out.println("Message received. Processing...");
            }

            //=============================================================
            // Extract detailed message information.
            //=============================================================

            RomaMessageInfo info = 
                m_elem.interpretMessage(msg, Roma.INTMSG_THEME);

            if (m_display)
            {
                //=========================================================
                // Print message information and proceed with the next one.
                //=========================================================

                printMsgInfo(info, msg);

                if (info.hasTheme())
                {
                    // It is application responsibility to close
                    // the extracted theme.

                    m_elem.closeTheme(info.getTheme());
                }

                System.out.println("");
                opt.setOperation(Roma.PEEK_NEXT);
                continue;
            }

            //=============================================================
            // Modify message data and put the message onto the queue
            // around which the original error occurred.
            //=============================================================

            if (m_verbose)
            {
                System.out.println("Returning EMS message ...");
            }

            returnMsg(info, msg);
        }
    }


    //=====================================================================
    // Helper routines.
    //=====================================================================


    //=====================================================================
    // Updates the theme workflow and returns the message to the sender.
    //=====================================================================

    private void returnMsg(RomaMessageInfo info, RomaBeRequest msg)
    throws RomaError, Exception
    {
        if (!info.hasTheme())
        {
            throw new Exception("No theme in the message information");            
        }

        //=================================================================
        // Extract theme details.
        //=================================================================

        RomaTheme theme = info.getTheme();
        RomaThemeInfo ti = m_elem.queryTheme(theme);

        //=================================================================
        // Check the retry count to avoid message bouncing.
        //=================================================================

        if (ti.getRetryCount() >= RETRY_MAX)
        {
            throw new Exception("Retry count exceeded maximum");
        }

        //=================================================================
        // Check workflow route steps.
        //=================================================================

        RomaWorkflow wf = ti.getWorkflow();
        LinkedList routes = wf.getRoutes();

        if (routes.size() == 0)
        {
            throw new Exception("No routes in workflow");
        }

        boolean rebuild = false;
        RomaRoute wr = (RomaRoute)routes.getFirst();

        //=================================================================
        // Remove bridges from the route. The message will be sent to
        // the first non-bridge application.
        //=================================================================

        if (wr.getType() == Roma.APPTYPE_BRIDGE)
        {
            rebuild = true;
            routes.removeFirst();
                
            while (routes.size() != 0)
            {
                wr = (RomaRoute)routes.getFirst();

                if (wr.getType() != Roma.APPTYPE_BRIDGE)
                {
                    break;
                }

                routes.removeFirst();
            }
        }

        //=================================================================
        // Create the response message that will be put back onto the
        // queue around which the original error occurred.
        //=================================================================

        if (info.hasException())
        {
            RomaExceptionInfo ei = info.getException();

            if (ei.hasErrorData() && info.hasWorkflow())
            {
                RomaExcErrorData ed = ei.getErrorData();
                int ct = ed.getCallType();

                if (ct == Roma.CALLTYPE_GETREQUEST ||
                    ct == Roma.CALLTYPE_GETRESPONSE)
                {
                    //=====================================================
                    // Rebuild the workflow to put the message back
                    // to the applications input queue.
                    //=====================================================

                    RomaConfig rc = ei.getRomaConfig();
                    wf.getWorkflowService().setComponent(
                        rc.getComponentName());

                    wr = new RomaRoute();
                    wr.setType(rc.getElementType());
                    wr.setTargetQueue(rc.getQueueName());
                    wr.setTargetElement(rc.getElementName());
                    wr.setTargetComponent(rc.getComponentName());
                    routes.addFirst(wr);
                    rebuild = true;
                }
            }
        }

        //=================================================================
        // If the workflow route is modified, we need to build a new
        // theme that will be used for the message return. The old
        // theme should be closed.
        // We also set business flow information.
        //=================================================================

        if (rebuild)
        {
            if (m_verbose)
            {
                System.out.println("Rebuilding the theme");
            }

            theme = m_elem.buildTheme(theme, ti,
                Roma.THEMEINFO_WORKFLOW, true);
        }

        if (m_verbose)
        {
            wr = (RomaRoute)routes.getFirst();
            System.out.println("Returning message to the application " +
                wr.getTargetElement() + " on the queue " +
                wr.getTargetQueue());
        }

        //=================================================================
        // Create the response message that will be put back onto the
        // queue around which the original error occurred.
        //=================================================================

        // The constructor will set the response with the
        // message ID, correlation ID and theme of the original message.
        RomaBeResponse rsp = new RomaBeResponse(
            info.hasMessage() ? info.getBeRequest() : msg);

        // Set the new theme.
        rsp.setTheme(theme);
        rsp.setUseTheme(true);

        // Use existing message and correlation IDs
        rsp.setSelector(Roma.USE_MSGID_AND_CORRELID);

        // Set the new message data.
        rsp.setMsgBuffer("MESSAGE-MODIFIED-BY-EMS");

        // Enable the retry counter.
        rsp.setIncrementRetryCount(true);

        // Close the theme when the message is sent.
        rsp.setCloseTheme(true);

        // Don't invoke error analysis services.
        rsp.setNoEms(true);

        // Don't put in a transactional queue.
        rsp.setNoSyncAllowed(true);

        // The server doesn't need a notfication.
        rsp.setNoNotification(true);

        // Prohibit a message transformation.
        rsp.setNoTransformation(true);

        //=================================================================
        // Send the modified message.
        //=================================================================

        m_elem.put(rsp);
    }


    //=====================================================================
    // Prints message information.
    //=====================================================================

    private void printMsgInfo(RomaMessageInfo info, RomaBeRequest msg)
    throws RomaError
    {
        System.out.println("Roma Exception Message Display");

        //=================================================================
        // If there is exception information, print it.
        //=================================================================

        if (info.hasException())
        {
            printExcData(info.getException());
        }
        else
        {
            System.out.println("\n No exception information");
        }

        //=================================================================
        // If there is workflow information, print it.
        //=================================================================

        if (info.hasWorkflow())
        {
            printWorkflow(info.getWorkflow());
        }
        else
        {
            System.out.println("\n No Workflow Information");
        }

        //=================================================================
        // If there is message data, print it.
        // Otherwise, print the received message.
        //=================================================================

        printMsg(info.hasMessage() ? info.getBeRequest() : msg);

        //=================================================================
        // If there is a theme, print its details.
        //=================================================================

        if (info.hasTheme())
        {
            RomaTheme theme = info.getTheme();

            if (theme.isNull())
            {
                System.out.println("\n Theme is NULL");
            }
            else
            {
                printTheme(m_elem.queryTheme(theme));

                // It is application's responsibility to close the
                // extracted theme.

                m_elem.closeTheme(theme);
            }
        }
        else
        {
            System.out.println("\n No Theme Information");
        }
    }


    //=====================================================================
    // Prints theme details.
    //=====================================================================

    private static void printTheme(RomaThemeInfo ti)
    {
        System.out.print(
            "\n Theme Information:" +
            "\n  Retry Count : " + ti.getRetryCount() +
            "\n  Theme ID ...: ");
        printAsHex(ti.getThemeId());

        //=================================================================
        // Print confirmation flags.
        //=================================================================

        if ((ti.getConfirmFlags() & Roma.CONFIRM_DELIVERY) != 0)
        {
            System.out.println("  Confirmation of Delivery is enabled");
        }

        if ((ti.getConfirmFlags() & Roma.CONFIRM_ARRIVAL) != 0)
        {
            System.out.println("  Confirmation of Arrival is enabled");
        }

        //=================================================================
        // Print persistent trailers.
        //=================================================================

        LinkedList trailers = ti.getTrailers();
        if (trailers.isEmpty()) return;

        System.out.println("  Persistent Trailers :");

        int i = 1;
        ListIterator iter = trailers.listIterator(0);

        do         
        {
            RomaTrailer rt = (RomaTrailer)iter.next();

            System.out.print(
                "   Trailer #" + i + " :" +
                "\n    Type ..: " + trailerType2Str(rt.getType()) +
                "\n    Method : " + rt.getMethod() +
                "\n    Data ..:");
            printAsHex(rt.getData());
            ++i;
        }          
        while (iter.hasNext());
    }


    //=====================================================================
    // Prints message data.
    //=====================================================================

    private static void printMsg(RomaBeRequest msg)
    {
        System.out.println(
            "\n Message Information :" +
            "\n  Format .......: " + msg.format() +
            "\n  Type .........: " + msg.msgType() +
            "\n  Persistence ..: " + msg.persistence() +
            "\n  Priority .....: " + msg.priority() +
            "\n  Feedback .....: " + msg.feedback() +
            "\n  Expiry .......: " + msg.expiry() +
            "\n  Code Page ....: " + msg.codepage() +
            "\n  Encoding .....: " + msg.encoding() +
            "\n  Status .......: " + msg.status() +
            "\n  Backout Count : " + msg.backoutCount() +
            "\n  Format Name ..: " + msg.formatName() +
            "\n  Message Name .: " + msg.msgName() +
            "\n  Timestamp ....: " + 
                DateFormat.getDateTimeInstance().format(msg.timeStamp()));

        System.out.print("  Message ID ...: ");
        printAsHex(msg.msgId());
        System.out.print("  Correl ID ....: ");
        printAsHex(msg.correlId());
        System.out.print("  Tag Data .....: ");
        printAsHex(msg.tagBuffer());
        System.out.print("  Message Data .: ");
        printAsHex(msg.msgBuffer());
    }


    //=====================================================================
    // Prints workflow information.
    //=====================================================================

    private static void printWorkflow(RomaWorkflow wf)
    {
        //=================================================================
        // Print workflow service data.
        //=================================================================

        RomaWorkflowService ws = wf.getWorkflowService();

        System.out.println(
            "\n Workflow Information :" +
            "\n  WorkFlow Service information:" +
            "\n   Current Service .....: " + ws.getService() +
            "\n   Current Component ...: " + ws.getComponent() +
            "\n   Current Audit .......: " + ws.getAudit() +
            "\n   Source Service ......: " + ws.getSourceService() +
            "\n   Source Business Flow : " + ws.getSourceFlow() +
            "\n   Source Operation ....: " + ws.getSourceOperation());

        //=================================================================
        // Print workflow route steps.
        //=================================================================

        LinkedList routes = wf.getRoutes();
        if (routes.isEmpty())
        {
            System.out.println("  No Route Information");
            return;
        }

        System.out.println("  Route Information :");

        int i = 1;
        ListIterator iter = routes.listIterator(0);

        do
        {
            RomaRoute wr = (RomaRoute)iter.next();

            System.out.println(
                "   Route #" + i + " :" +
                "\n    Type : " + appType2Str(wr.getType()) +
                "\n    Flags : " + wr.getFlags() +
                "\n    Target Queue : " + wr.getTargetQueue() +
                "\n    Target Element : " + wr.getTargetElement() +
                "\n    Target Component : " + wr.getTargetComponent() +
                "\n    Target Code Page : " + wr.getTargetCodePage() +
                "\n    Source Service : " + wr.getSourceService() +
                "\n    Source Flow : " + wr.getSourceFlow() +
                "\n    Source Operation : " + wr.getSourceOperation());
            ++i;
        }
        while (iter.hasNext());
    }

    //=====================================================================
    // Prints exception data.
    //=====================================================================

    private static void printExcData(RomaExceptionInfo ei)
    {
        System.out.println("\n Exception Information:");

        //=================================================================
        // If there is Roma configuration data, print it.
        //=================================================================

        if (ei.hasRomaConfig())
        {
            RomaConfig rc = ei.getRomaConfig();

            System.out.println(
                "  Roma Configuration Data :" +
                "\n   Application Type .............: " +
                    appType2Str(rc.getElementType()) +
                "\n   Application Name .............: " +
                    rc.getElementName() +
                "\n   Input Queue Name .............: " +
                    rc.getQueueName() +
                "\n   Associated Business Component : " +
                    rc.getComponentName() +
                "\n   Associated Business Service ..: " +
                    rc.getServiceName() +
                "\n   Node Name ....................: " +
                    rc.getNodeName());
        }
        else
        {
            System.out.println("  No Roma configuration information");
        }

        //=================================================================
        // If there is system configuration data, print it.
        //=================================================================

        if (ei.hasSystemConfig())
        {
            RomaSystemConfig sc = ei.getSystemConfig();

            System.out.println(
                "  System Configuration Data :" +
                "\n   Environment Type : " + envType2Str(sc.getEnvType()) +
                "\n   Environment OS ..: " + envOs2Str(sc.getEnvOS()) +
                "\n   Domain Name .....: " + sc.getDomainName() +
                "\n   User Id .........: " + sc.getUserId() +
                "\n   Program Name ....: " + sc.getProgramName() +
                "\n   Process Id ......: " + sc.getProcessId() +
                "\n   Thread Id .......: " + sc.getThreadId());
        }
        else
        {
            System.out.println("  No system configuration information");
        }

        //=================================================================
        // If there is a timestamp, print it.
        //=================================================================

        if (ei.hasTimestamp())
        {
            System.out.println("  Timestamp : " +
                DateFormat.getDateTimeInstance().format(ei.getTimestamp()));
        }
        else
        {
            System.out.println("  No timestamp information");
        }

        //=================================================================
        // If there is error information, print it.
        //=================================================================

        if (ei.hasErrorData())
        {
            RomaExcErrorData ed = ei.getErrorData();
            RomaError error = ed.getError();

            System.out.println(
                "  Error Data :" +
                "\n   Primary ..: " + 
                    error.errorText(Roma.PRIMARY_INFO,
                        Roma.ERROR_TEXT_FULL) +
                "\n   Secondary : " + 
                    error.errorText(Roma.SECONDARY_INFO,
                        Roma.ERROR_TEXT_FULL) +
                "\n   Call Type : " + callType2Str(ed.getCallType()) +
                "\n   QueueName : " + ed.getQueueName());
        }
        else
        {
            System.out.println("  No error information");
        }

        //=================================================================
        // If there is user data, print it.
        //=================================================================

        if (ei.hasUserData())
        {
            System.out.print("  User Data: ");
            printAsHex(ei.getUserData());
        }
        else
        {
            System.out.println("  No user data");
        }
    }

    //=====================================================================
    // Returns string representation of the trailer type.
    //=====================================================================

    private static String trailerType2Str(int type)
    {
        switch (type)
        {
            case Roma.ILS_TYPE_XPORT:
                return "XPORT";
            case Roma.ILS_TYPE_USER:
                return "USER";
            case Roma.ILS_TYPE_ROUTER:
                return "ROUTER";
            case Roma.ILS_TYPE_SYSMGMT:
                return "SYSMGMT";
            case Roma.ILS_TYPE_ERROR:
                return "ERROR";
            case Roma.ILS_TYPE_WEB:
                return "WEB";
            case Roma.ILS_TYPE_XPTDATA:
                return "XPTDATA";
        }

        return "Unknown";
    }

    //=====================================================================
    // Returns string representation of the application type.
    //=====================================================================

    private static String appType2Str(int type)
    {
        switch (type)
        {
            case Roma.APPTYPE_CLIENT:
                return "Client";
            case Roma.APPTYPE_SERVER:
                return "Server";
            case Roma.APPTYPE_BRIDGE:
                return "Bridge";
        }

        return "Unknown";
    }

    //=====================================================================
    // Returns string representation of the environment type.
    //=====================================================================

    private static String envType2Str(int type)
    {
        switch (type)
        {
            case Roma.ENVTYPE_INTEL:
                return "Intel";
            case Roma.ENVTYPE_MVS:
                return "MVS";
            case Roma.ENVTYPE_UNIX:
                return "Unix";
            case Roma.ENVTYPE_OS400:
                return "OS 400";
            case Roma.ENVTYPE_VMS:
                return "VMS";
        }

        return "Unknown";
    }

    //=====================================================================
    // Returns string representation of the OS type.
    //=====================================================================

    private static String envOs2Str(int type)
    {
        switch (type)
        {
            case Roma.ENVOS_AIX:
                return "AIX";
            case Roma.ENVOS_HPUX102:
                return "HP-UX 10";
            case Roma.ENVOS_HPUX11:
                return "HP-UX 11";
            case Roma.ENVOS_DUX:
                return "Digital Unix";
            case Roma.ENVOS_MVSBATCH:
                return "MVS Batch";
            case Roma.ENVOS_MVSCICS:
                return "CICS";
            case Roma.ENVOS_SUN:
                return "Solaris";
            case Roma.ENVOS_WIN95:
                return "Windows 95";
            case Roma.ENVOS_WINNT:
                return "Windows NT";
            case Roma.ENVOS_OS400:
                return "OS 400";
        }

        return "Unknown";
    }

    //=====================================================================
    // Returns string representation of the call type.
    //=====================================================================

    private static String callType2Str(int type)
    {
        switch (type)
        {
            case Roma.CALLTYPE_APPLICATION:
                return "APPLICATION";
            case Roma.CALLTYPE_PUTREQUEST:
                return "PUTREQUEST";
            case Roma.CALLTYPE_GETREQUEST:
                return "GETREQUEST";
            case Roma.CALLTYPE_PUTRESPONSE:
                return "PUTRESPONSE";
            case Roma.CALLTYPE_GETRESPONSE:
                return "GETRESPONSE";
            case Roma.CALLTYPE_BRIDGEPUT:
                return "BRIDGEPUT";
            case Roma.CALLTYPE_BRIDGEGET:
                return "BRIDGEGET";
            case Roma.CALLTYPE_PUTEXCEPTION:
                return "PUTEXCEPTION";
            case Roma.CALLTYPE_INITIALIZE:
                return "INITIALIZE";
            case Roma.CALLTYPE_TERMINATE:
                return "TERMINATE";
            case Roma.CALLTYPE_COMMIT:
                return "COMMIT";
            case Roma.CALLTYPE_ROLLBACK:
                return "ROLLBACK";
            case Roma.CALLTYPE_FREEBUFFER:
                return "FREEBUFFER";
            case Roma.CALLTYPE_CLOSEHANDLE:
                return "CLOSEHANDLE";
            case Roma.CALLTYPE_INTERPRETMSG:
                return "INTERPRETMSG";
            case Roma.CALLTYPE_WRITELOG:
                return "WRITELOG";
            case Roma.CALLTYPE_SENDALERT:
                return "SENDALERT";
            case Roma.CALLTYPE_OPENSTORAGEQ:
                return "OPENSTORAGEQ";
            case Roma.CALLTYPE_CLOSESTORAGEQ:
                return "CLOSESTORAGEQ";
            case Roma.CALLTYPE_DELETESTORAGEQ:
                return "DELETESTORAGEQ";
            case Roma.CALLTYPE_PUTTOSTORAGEQ:
                return "PUTTOSTORAGEQ";
            case Roma.CALLTYPE_GETFROMSTORAGEQ:
                return "GETFROMSTORAGEQ";
            case Roma.CALLTYPE_QUERYTHEME:
                return "QUERYTHEME";
            case Roma.CALLTYPE_BUILDTHEME:
                return "BUILDTHEME";
            case Roma.CALLTYPE_PURGEQTOEMS:
                return "PURGEQTOEMS";
            case Roma.CALLTYPE_REPORTERROR:
                return "REPORTERROR";
            case Roma.CALLTYPE_QUERYCONTEXT:
                return "QUERYCONTEXT";
            case Roma.CALLTYPE_SETLDAPSERVICE:
                return "SETLDAPSERVICE";
            case Roma.CALLTYPE_SETILSDATA:
                return "SETILSDATA";
            case Roma.CALLTYPE_GETILSDATA:
                return "GETILSDATA";
            case Roma.CALLTYPE_DIRLOOKUP:
                return "DIRLOOKUP";
            case Roma.CALLTYPE_DIRGETDN:
                return "DIRGETDN";
            case Roma.CALLTYPE_DIRUPDATEITEM:
                return "DIRUPDATEITEM";
            case Roma.CALLTYPE_DIRDELETEITEM:
                return "DIRDELETEITEM";
            case Roma.CALLTYPE_DIRGETMODCOUNT:
                return "DIRGETMODCOUNT";
            case Roma.CALLTYPE_XPORTDATA:
                return "XPORTDATA";
        }

        return "Unknown";
    }

    //=====================================================================
    // Prints a hexadecimal value of each byte of the data.
    //=====================================================================

    private static void printAsHex(byte[] data)
    {
        if (data == null)
        {
            System.out.println("");
            return;
        }

		StringBuffer sb = new StringBuffer(data.length * 2);

        for (int i = 0; i < data.length; ++i)
        {
			sb.append(Character.forDigit((data[i] >> 4) & 0x0F, 16));
			sb.append(Character.forDigit(data[i] & 0x0F, 16));
        }

		System.out.println(sb.toString().toUpperCase());
    }

    //=====================================================================
    // Parses command line options.
    //=====================================================================

    private void parseCmdLine(String args[])
    throws Exception
    {
        for (int i = 0; i < args.length; ++i)
        {
            String s = args[i];

            if (s.length() < 2 || s.charAt(0) != '-')
            {
                throw new Exception("Unknown option " + s);
            }

            switch (s.charAt(1))
            {
                case 'e':
                    if (s.length() == 2)
                    {
                        if (i == args.length)
                        {
                            throw new Exception(
                                "No element name after -e option");
                        }

                        m_name = args[++i];
                    }
                    else
                    {
                        m_name = s.substring(1);
                    }

                    break;

                case '?':
                    m_help = true;
                    break;

                case 'v':
                    m_verbose = true;
                    break;

                case 'r':
                    m_display = false;
                    break;
                                        
                default:
                    throw new Exception("Unknown option " + s);
            }
        }
    }

    // Element name.
    private String m_name = "TestElementEMS";

    // Indicates that the server should just peek messages
    // and print thier deatils.
    private boolean m_display = true;

    // Verbose mode indicator.
    private boolean m_verbose = false;

    // Indicates that usage information should be printed.
    private boolean m_help = false;

    // Business Element object.
    private RomaBusinessElement m_elem = null;

    // Maximal number of attempts to put back a message.
    private static final int RETRY_MAX = 5;
}

