/********************************************************************
 *                                                                    
 * Module      : kxmufgrp.c
 *                                                                    
 * Description : User Defined Functions C API Sample
 *                                                                    
 * This source is example of the map conversion library,
 * which provides four custom conversion functions:
 * "makeGroup"
 * "addForEach"
 * "findGroup"
 * "glueElems"
 * "getColors"
 * "getShifts"
 * "reverseElems"
 * "negateBoolean"
 *                                                                    
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 * 
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 * 
 ********************************************************************
 * $Id: kxmufgrp.c,v 1.17.2.1 2001/09/07 12:19:38 e123479 Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufgrp.c,v $
 * $Revision: 1.17.2.1 $                                                   
 ********************************************************************/

#include <math.h>
#include "kxmufful.h"

/********************************************************************
 *  Input is an array of strings. The ith element contains field name 
 *  and i+1 element contains value for it.
 * 
 *  The function creates a group by assigning 
 *  each field name with a corresponding field value.
 * 
 *  This sample user defined function shows an example of processing
 *  an array and setting a field based on a value retrieved.
 *  Note that the function name does not have to match the name
 *  of the function exposed to the conversion engine maps.
 ********************************************************************/
static KXMBOOL makeGroupFun( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    HKXMFIELD array;
    KXMULONG arrayLen;
    KXMULONG arrayIndex;
     
    array = evalArgs->Args[0];
    /* get length of an input array*/
    if( !kxmGetArrayLength( array, NULL, &arrayLen, NULL, NULL ) )
    {
        /* error, for example input is not array */
        SET_ERROR("*** ERROR: makeGroup: Unable to get Array length");
        return FALSE;
    }    

    /* return XML */           
    if( !kxmCreateMessage( "XmlDoc", NULL, NULL, &result->Value ) )
    {
        SET_ERROR("*** ERROR: makeGroup: Unable to create XML message");
        return FALSE;
    }            

    /* walk through array and assign group fields */
    for ( arrayIndex = 0; arrayIndex < arrayLen; arrayIndex += 2 ) 
    {
        HKXMFIELD elem;
        const char* fieldName;
        const char* fieldValue;

        /* get HKXMFIELD for the i-th array element */        
        if( !kxmGetField( array, NULL, arrayIndex, NULL, NULL, &elem ) ) 
        {
            SET_ERROR("*** ERROR: makeGroup: Unable to get array element field");
            return FALSE;
        }    
 
        /* get value of the i-th array element */
        if( !kxmGetString( elem, NULL, &fieldName, NULL, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: makeGroup: Unable to get array element value");
            return FALSE;
        }    

        /* get HKXMFIELD for the i+1 array element */
        if( !kxmGetField( array, NULL, arrayIndex + 1, NULL, NULL, &elem ) )
        {
            SET_ERROR("*** ERROR: makeGroup: Unable to get array element field");
            return FALSE;
        }    
        
        /* get value of the i+1 array element */
        if( !kxmGetString( elem, NULL, &fieldValue, NULL, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: makeGroup: Unable to get array element value");
            return FALSE;
        }    

        /* set group field */
        if( !kxmSetString( result->Value, fieldName, fieldValue, -1, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: makeGroup: Unable to set group field");
            return FALSE;
        }    
    }

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for makeGroup */
static KXMFUNARG makeGroupArgs[] =
{
    { KXMFUNARG_SIGVER, "array", "Array of values", KXMVALTYPE_ARRAY }
};

/* Function Definition for makeGroup */
KXMFUN makeGroupFunc =
{
    KXMFUN_SIGVER,
    "makeGroup", "Makes group from array",
    makeGroupFun,
    1, 1,
    sizeof(makeGroupArgs) / sizeof(*makeGroupArgs),
    makeGroupArgs,
    KXMVALTYPE_XMLELEM
};            

/********************************************************************
 * Adds a number to each element of an array
 *
 * This sample user defined function shows an example of processing
 * of an array.
 ********************************************************************/
static KXMBOOL addForEach( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    HKXMFIELD array;
    KXMULONG length;
    KXMULONG index;
    double addValue;
    double* values;
    KXMARRAYVALUE arVal = { KXMARRAYVALUE_DEFAULT };

    /* get value to add */
    if( !kxmGetDouble( evalArgs->Args[1], NULL, &addValue, NULL, NULL ) )
    {
        /* error, for example input is not a number */
        SET_ERROR("*** ERROR: addForEach: Unable to get Add value");
        return FALSE;
    }    

    /* get array */
    array = evalArgs->Args[0];
    /* get length of an input array */
    if( !kxmGetArrayLength( array, NULL, &length, NULL, NULL ) )
    {
        /* error, for example input is not an array */
        SET_ERROR("*** ERROR: addForEach: Unable to get array length");
        return FALSE;
    }    

    /* Malloc new space for the array of values */
    values = malloc( sizeof(double) * length );
    KXM_SetDoubleArrayVal( &arVal, values, length );

    /* get all array values at once */
    if ( !kxmGetArrayValue( array, NULL, NULL, &arVal, NULL ) )
    {
        SET_ERROR("*** ERROR: addForEach: Unable to get array values");
        return FALSE;
    }

    /* walk through array and add the specified value
       to each element of an array */
    for ( index = 0; index < length; index++ ) 
    {
        values[index] += addValue;
    }

    /* return updated array */
    if ( !kxmSetArrayValue( result->Value, NULL, NULL, &arVal, NULL ) )
    {
        free( values );
        SET_ERROR("*** ERROR: addForEach: Unable to set array value");
        return FALSE;
    }

    free( values );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for addForEach */
static KXMFUNARG addForEachArgs[] =
{
    { KXMFUNARG_SIGVER, "array", "Array of numbers",
        KXMVALTYPE_ARRAY | KXMVALTYPE_NUMBER },
    { KXMFUNARG_SIGVER, "value", "Number to add",
        KXMVALTYPE_NUMBER }
};

/* Function Definition for addForEach */
KXMFUN addForEachFunc =
{
    KXMFUN_SIGVER,
    "addForEach",
    "Adds specified number to each value of an array",
    addForEach,
    2, 2,
    sizeof(addForEachArgs) / sizeof(*addForEachArgs),
    addForEachArgs,
    KXMVALTYPE_ARRAY | KXMVALTYPE_DOUBLE
};            

/********************************************************************
 *  Searches an array of groups for a group that
 *  has the specified field containing the specified value.
 *  Returns an index of this group in teh array,
 *  or -1 if not found 
 ********************************************************************/
static KXMBOOL findGroup( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    HKXMFIELD array;
    KXMULONG size, index = 0, retval = (KXMULONG)-1;
    const char* fldName;
    const char* fldValue;

    array = evalArgs->Args[0];
    /* get array length of array of groups */
    if( !kxmGetArrayLength( array, NULL, &size, NULL, NULL ) ||
        /* get field name */
        !kxmGetString( evalArgs->Args[1], NULL, &fldName, NULL, NULL, NULL ) ||
        /* get field value */
        !kxmGetString( evalArgs->Args[2], NULL, &fldValue, NULL, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: findGroup: Unable to get input data");
        return FALSE;
    }    

    /* walk through array of groups and compare field values 
       of specified field */
    for (; index < size; index++ ) 
    {
        HKXMFIELD elem;
        const char* value;

        /* get HKXMFIELD for the i-th array element */        
        if( !kxmGetField( array, NULL, index, NULL, NULL, &elem ) ||
            /* get value of this field */        
            !kxmGetString( elem, fldName, &value, NULL, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: findGroup: Unable to compare array element");
            return FALSE;
        }    

        /* compare values */
        if ( !strcmp( fldValue, value ) )
        {
            /* group found, return its index in an array */
            retval = index;
            break;
        }
    }

    /* return result */
    if ( !kxmSetULong( result->Value, NULL, retval, NULL, NULL ) ) 
    {
        SET_ERROR("*** ERROR: findGroup: Unable to set return value");
        return FALSE;
    }    

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for findGroup */
static KXMFUNARG findGroupArgs[] =
{
    { KXMFUNARG_SIGVER, "array", "Array of groups",
        KXMVALTYPE_ARRAY | KXMVALTYPE_GROUP },
    { KXMFUNARG_SIGVER, "name", "Field name",
        KXMVALTYPE_STRING },
    { KXMFUNARG_SIGVER, "value", "Field value",
        KXMVALTYPE_STRING }
};

/* Function Definition for findGroup */
KXMFUN findGroupFunc =
{
    KXMFUN_SIGVER,
    "findGroup",
    "Searches array of groups",
    findGroup,
    3, 3,
    sizeof(findGroupArgs) / sizeof(*findGroupArgs),
    findGroupArgs,
    KXMVALTYPE_INT
};            

/********************************************************************
 *  Creates string from an array of XML elements optionally
 *  delimited by specified string. 
 ********************************************************************/
static KXMBOOL glueElems( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    HKXMFIELD array;
    KXMULONG arrayLength, delimLength = 0, strLength = 0, index = 0;
    const char* delim = NULL;
    char* str = NULL;
    char** values = NULL;
    KXMARRAYVALUE arVal = { KXMARRAYVALUE_DEFAULT };

    if ( evalArgs->NumOfArgs > 1 )
    {
        /* get a delimiter */
        if( !kxmGetString( evalArgs->Args[1], NULL, &delim, &delimLength, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: glueElems: Unable to get delimiter input data");
            return FALSE;
        }    
    }

    array = evalArgs->Args[0];
    /* get length of the input array*/
    if( !kxmGetArrayLength( array, NULL, &arrayLength, NULL, NULL ) )
    {
        /* error, for example input is not array */
        SET_ERROR("*** ERROR: glueElems: Unable to get array length");
        return FALSE;
    }    

    if ( !arrayLength )
    {
        /* if input array is empty, then result is empty string */
        return kxmSetString( result->Value, NULL, "", 0, NULL, NULL );
    }

    values = malloc( sizeof(char*) * arrayLength );
    KXM_SetStringArrayVal( &arVal, (const char**)values, arrayLength );

    /* get all array values at once */
    if ( !kxmGetArrayValue( array, NULL, NULL, &arVal, NULL ) )
    {
        free( values );
        SET_ERROR("*** ERROR: glueElems: Unable to get array values");
        return FALSE;
    }

    /* walk throuh an array and add each element
       of an array to the string */
    for (; index < arrayLength; index++ ) 
    {
        size_t len = strlen( values[index] );
        if ( strLength && delimLength )
        {
            /* add delimiter */
            strLength += delimLength;
            str = realloc( str, strLength + 1 );
            strncpy( str + strLength - delimLength, delim, delimLength );
        }

        strLength += len;
        str = realloc( str, strLength + 1 );
        strncpy( str + strLength - len, values[index], len );
    }

    str[strLength] = '\0';
    free( values );

    /* set result with string */
    if ( !kxmSetString( result->Value, NULL, str, strLength, NULL, NULL ) )
    {
        free( str );
        SET_ERROR("*** ERROR: glueElems: Unable to set return value");
        return FALSE;
    }

    free( str );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for glueElems */
static KXMFUNARG glueElemsArgs[] =
{
    { KXMFUNARG_SIGVER, "array", "Array of XML elements",
        KXMVALTYPE_ARRAY | KXMVALTYPE_XMLELEM },
    { KXMFUNARG_SIGVER, "delimiter", "Delimiter between elements",
        KXMVALTYPE_STRING }
};

/* Function Definition for glueElems */
KXMFUN glueElemsFunc =
{
    KXMFUN_SIGVER,
    "glueElems",
    "Concatenates XML elements delimited by specified delimiter",
    glueElems,
    1, 2,
    sizeof(glueElemsArgs) / sizeof(*glueElemsArgs),
    glueElemsArgs,
    KXMVALTYPE_STRING
};

/********************************************************************
 *  Returns an array of groups containing colors properties
 *
 *  This function takes no arguments.  It simply creates a new message
 *  and returns an array of groups.
 ********************************************************************/
static KXMBOOL getColors( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    struct { char* name; KXMULONG value; } colors[] =
    { /* colors data we provide */
        { "White", 0xFFFFFF },
        { "Red",   0xFF0000 },
        { "Green", 0x00FF00 },
        { "Blue",  0x0000FF },
        { "Black", 0x000000 }
    };

    KXMMSGOPT opt = { KXMMSGOPT_ARRAY };
    KXMULONG index = 0;
    HKXMFIELD array;

    /* create message of "Colors" format */
    if( !kxmCreateMessage( "Colors", NULL, NULL, &result->Value ) )
    {
        SET_ERROR("*** ERROR: getColors: Unable to create message of 'Colors' format");
        return FALSE;
    }
                
    /* get "array" field of the created message */
    if ( !kxmGetField( result->Value, "array", -1, NULL, NULL, &array ) )
    {
        SET_ERROR("*** ERROR: getColors: Unable to get 'array' field");
        return FALSE;
    }            

    /* walk through the array and assign group fields */
    for (; index < sizeof(colors) / sizeof(*colors); index++ ) 
    {
        HKXMFIELD elem;
        /* get HKXMFIELD for the array element by index */        
        if( !kxmGetField( array, NULL, index, NULL, NULL, &elem ) ||
            /* set 'name' field value */
            !kxmSetString( elem, "name", colors[index].name, 
                strlen( colors[index].name ), NULL, NULL ) ||
            /* set 'value' field value */
            !kxmSetULong( elem, "value", colors[index].value, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: getColors: Unable to get set return values");
            return FALSE;
        }     
    }

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Note: There are no typed argument definitions for getColors. */
/*       So the structure does not need to be provided.         */
/* Function Definition for glueElems                            */
KXMFUN getColorsFunc =
{
    KXMFUN_SIGVER,
    "getColors",
    "Returns array of colors",
    getColors,
    0, 0, 0,
    NULL,
    KXMVALTYPE_GROUP
};

/********************************************************************
 *  Returns an array of numbers containing values of 1
 *  shifted to the left by a specified range
 ********************************************************************/
static KXMBOOL getShifts( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    KXMMSGOPT opt = { KXMMSGOPT_ARRAY };
    KXMULONG shifts;
    KXMULONG* values;
    KXMARRAYVALUE arVal = { KXMARRAYVALUE_DEFAULT };
    KXMULONG index = 0;

    /* get number of shifts */
    if( !kxmGetULong( evalArgs->Args[0], NULL, &shifts, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: getShifts: Unable to get input value");
        return FALSE;
    }

    /* number of shifts shouldn't be greater then 
       number of bits in KXMULONG */
    if ( shifts > sizeof(KXMULONG) * 8 )
    {
        SET_ERROR( "*** ERROR: getShifts: 'shifts' value out of range" );
        return FALSE;
    }

    values = malloc( sizeof(KXMULONG) * shifts );
    for (; index < shifts; index++ ) 
    {
        values[index] = 1 << index;
    }

    KXM_SetULongArrayVal( &arVal, values, shifts );
    if ( !kxmSetArrayValue( result->Value, NULL, NULL, &arVal, NULL ) )
    {
        free( values );
        SET_ERROR( "*** ERROR: getShifts: Unable to set return array value");
        return FALSE;
    }            

    free( values );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for getShifts */
static KXMFUNARG getShiftsArgs[] =
{
    { KXMFUNARG_SIGVER, "shifts", "Number of shifts", KXMVALTYPE_UINT }
};

/* Function Definition for getShifts */
KXMFUN getShiftsFunc =
{
    KXMFUN_SIGVER,
    "getShifts",
    "Returns array of left shifted 1",
    getShifts,
    1, 1, 1,
    getShiftsArgs,
    KXMVALTYPE_ARRAY | KXMVALTYPE_UINT
};

/********************************************************************
 *  Reorders characters of XML elements to backward
 ********************************************************************/
static KXMBOOL reverseElems( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    KXMULONG arrayLength, index = 0;
    char** values = NULL;
    char** changed = NULL;
    KXMARRAYVALUE arVal = { KXMARRAYVALUE_DEFAULT };
    KXMBOOL ret;
    KXMMSGOPT opt = { KXMMSGOPT_DEFAULT };

    /* get length of an input array*/
    if( !kxmGetArrayLength( evalArgs->Args[0], NULL, &arrayLength, NULL, NULL ) )
    {
        /* error, for example input is not array */
        SET_ERROR("*** ERROR: reverseElems: Unable to get input array length");
        return FALSE;
    }    

    /* allocate memory to store original data of the array */
    values = malloc( sizeof(char*) * arrayLength );
    /* setup KXMARRAYVALUE structure fields */
    KXM_SetStringArrayVal( &arVal, (const char**)values, arrayLength );

    /* get all array values at once */
    if ( !kxmGetArrayValue( evalArgs->Args[0], NULL, NULL, &arVal, NULL ) )
    {
        free( values );
        SET_ERROR("*** ERROR: reverseElems: Unable to get input array values");
        return FALSE;
    }

    /* allocate memory to store changed data of the array */
    changed = malloc( sizeof(char*) * arrayLength );
    /* process array data */
    for (; index < arrayLength; index++ ) 
    {
        char* source = values[index];
        size_t x = 0, len = strlen( source );
        char* target = malloc( len + 1 );

        changed[index] = target;
        for (; x < len; x++ )
        {
            target[x] = source[len - x - 1];
        }
        target[len] = '\0';
    }

    free( values );
    /* re-setup KXMARRAYVALUE structure fields with other array */
    KXM_SetStringArrayVal( &arVal, (const char**)changed, arrayLength );

    /* set message options */
    opt.Options = KXMMSGOPT_STANDALONE | KXMMSGOPT_ARRAY;
    /* create message */
    ret = kxmCreateMessage( "XmlDoc", &opt, NULL, &result->Value );
    if ( ret )
    {
        for ( index = 0; index < arrayLength; index++ ) 
        {
            HKXMFIELD elem;
            /* get array element and set string to it */
            if ( !kxmGetField( result->Value, NULL, index, NULL, NULL, &elem ) ||
                 !kxmSetString( elem, NULL, changed[index], -1, NULL, NULL ) )
            {
                ret = FALSE;
                SET_ERROR("*** ERROR: reverseElems: failed to set return values");
                break;
            }
        }
    }

    /* clean up */
    for ( index = 0; index < arrayLength; index++ ) 
    {
        free( changed[index] );
    }

    free( changed );

    /* Indicate if the function failed or was successful in running */
    return ret;
}

/* Typed Arguement Definitions for reverseElems */
static KXMFUNARG reverseElemsArgs[] =
{
    { KXMFUNARG_SIGVER, "array", "Array of XML elements", 
        KXMVALTYPE_ARRAY | KXMVALTYPE_XMLELEM }
};

/* Function Definition for reverseElems */
KXMFUN reverseElemsFunc =
{
    KXMFUN_SIGVER,
    "reverseElems",
    "Reverses characters of XML elements",
    reverseElems,
    1, 1, 1,
    reverseElemsArgs,
    KXMVALTYPE_ARRAY | KXMVALTYPE_XMLELEM
};

/********************************************************************
 *  Negates the values of an array of booleans
 *
 *  This function checks the type of input being either an array
 *  or a scalar boolean value.
 ********************************************************************/
static KXMBOOL negateBoolean( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    KXMULONG length, idx = 0;
    KXMBOOL* values = NULL;
    KXMBOOL* changed = NULL;
    KXMARRAYVALUE arVal = { KXMARRAYVALUE_DEFAULT };

    /* get length of the input array */
    if( !kxmGetArrayLength( evalArgs->Args[0], NULL, &length, NULL, NULL ) )
    {
        /* seems it is not array */
        /* Attempt to get the value as a scalar (single) boolean value */
        KXMBOOL val;
        /* attempt to get argument value and set updated value to result */
        if ( !kxmGetBool( evalArgs->Args[0], NULL, &val, NULL, NULL ) ) 
        {
            SET_ERROR("*** ERROR: negateBoolean: Unable to get scalar input value");
            return FALSE;
        }

        if ( !kxmSetBool( result->Value, NULL, !val, NULL, NULL ) ) 
        {
            SET_ERROR("*** ERROR: negateBoolean: Unable to set scalar return value");
	        return FALSE;
        }

        return TRUE;
    }    

    /* allocate memory for the array */
    values = malloc( sizeof(KXMBOOL) * length );
    /* setup KXMARRAYVALUE structure fields */
    KXM_SetBoolArrayVal( &arVal, values, length );

    /* get values of the array */
    if ( !kxmGetArrayValue( evalArgs->Args[0], NULL, NULL, &arVal, NULL ) )
    {
        free( values );
        SET_ERROR("*** ERROR: negateBoolean: Unable to set array input value");
        return FALSE;
    }

    /* change array values */
    for (; idx < length; idx++ )
    {
        values[idx] = !values[idx];
    }

    /* set updated values to the array */
    if ( !kxmSetArrayValue( result->Value, NULL, NULL, &arVal, NULL ) )
    {
        free( values );
        SET_ERROR("*** ERROR: negateBoolean: Unable to set array return value");
        return FALSE;
    }

    free( values );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for negateBoolean */
static KXMFUNARG negateBooleanArgs[] =
{
    { KXMFUNARG_SIGVER, "bools", "Array of boolean values or single value", 
        KXMVALTYPE_BOOL | KXMVALTYPE_SCALAR | KXMVALTYPE_ARRAY }
};

/* Function Definition for negateBoolean */
KXMFUN negateBooleanFunc =
{
    KXMFUN_SIGVER,
    "negateBoolean",
    "Negates values of an array of booleans",
    negateBoolean,
    1, 1, 1,
    negateBooleanArgs,
    KXMVALTYPE_BOOL | KXMVALTYPE_SCALAR | KXMVALTYPE_ARRAY
};

