/********************************************************************
 *                                                                    
 * Module      : kxmufmth.c
 *                                                                    
 * Description : Full User Defined Functions C API Sample math functions
 *                                                                    
 * This source is example of the map conversion library,
 * which provides four custom conversion functions:
 * "pow"
 * "sum"
 *
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 * 
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 * 
 ********************************************************************
 * $Id: kxmufmth.c,v 1.5.2.1 2001/09/07 12:19:38 e123479 Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufmth.c,v $
 * $Revision: 1.5.2.1 $                                                   
 ********************************************************************/

#include <math.h>
#include "kxmufful.h"

/********************************************************************
 * pow( x, y )
 *
 * Calculates first argument raised to the power of the second argument.
 ********************************************************************/
static KXMBOOL powFun( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    double x, y, xpowy;

    /* get x and y */
    if( !kxmGetDouble( evalArgs->Args[0], NULL, &x, NULL, NULL ) ||
        !kxmGetDouble( evalArgs->Args[1], NULL, &y, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: pow: Unable to get input values");
        return FALSE;     
    }

    xpowy = pow( x, y );

    /* set result */
    if( !kxmSetDouble( result->Value, NULL, xpowy, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: pow: Unable to set return value");
        return FALSE;
    }    

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for pow */
static KXMFUNARG powArgs[] = 
{
    { KXMFUNARG_SIGVER, "x", NULL, KXMVALTYPE_NUMBER },
    { KXMFUNARG_SIGVER, "y", NULL, KXMVALTYPE_NUMBER }
};            

/* Function Definition for pow */
KXMFUN powFunc =
{
    KXMFUN_SIGVER,
    "pow", "Returns x raised to power of y",
    powFun,
    2, 2,
    sizeof(powArgs) / sizeof(powArgs[0]),
    powArgs,
    KXMVALTYPE_DOUBLE
};            

/********************************************************************
 * sum( arg1,: )
 * 
 * Accepts any number of arguments.
 * Argument can be double (or convertable to it) or
 * array of doubles (or covertable to doubles) values.
 * 
 * Returns sum of all doubles.
 *
 * Note that the function name does not have to match the name
 * of the function exposed to the conversion engine maps.
 ********************************************************************/
static KXMBOOL sumFun( const KXMFUNARGS* evalArgs, KXMFUNRES* result )
{
    KXMUINT32 argIndex;
    double sum=0;
    KXMFIELDINFO fieldInfo = { KXMFIELDINFO_DEFAULT };

    /* iterate over arguments */
    for( argIndex=0; argIndex<evalArgs->NumOfArgs; argIndex++ )
    {
        HKXMFIELD field = evalArgs->Args[argIndex];
        
        /* get field info */
        if( !kxmGetFieldInfo( field, NULL, -1, NULL, &fieldInfo ) )
        {
            SET_ERROR("*** ERROR: sum: Unable to get input field information");
            return FALSE;
        }

        /* field is array */
        if( fieldInfo.Flags & KXMFIELDFLAG_IS_ARRAY )
        {
            KXMULONG arrayLen;
            KXMULONG arrayIndex;
    
            /* get array length */        
            if( !kxmGetArrayLength( field, NULL, &arrayLen, NULL, NULL ) )
            {
                SET_ERROR("*** ERROR: sum: Unable to get input array length");
                return FALSE;
            }    

            /* walk through array and sum all elements */
            for( arrayIndex=0; arrayIndex<arrayLen; arrayIndex++ )
            {
                HKXMFIELD elem;
                double val;

                if( !kxmGetField( field, NULL, arrayIndex, NULL, NULL, &elem ) ||
                    !kxmGetDouble( elem, NULL, &val, NULL, NULL ) )
                {
                    SET_ERROR("*** ERROR: sum: Unable to get array element value");
                    return FALSE;
                }    
                sum += val;
            }            
        }   
        /* field is scalar, get its value and sum */
        else
        {
            double val;

            if( !kxmGetDouble( field, NULL, &val, NULL, NULL ) )
            {
                SET_ERROR("*** ERROR: sum: Unable to get scalar input value");
                return FALSE;
            }    
            sum += val;
        }                     
    }

    /* return result */
    if( !kxmSetDouble( result->Value, NULL, sum, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: sum: Unable to set return value");
        return FALSE;
    }    
    
    /* Indicate the function completed successfully */
    return TRUE;            
}    

/* Note: There are no typed argument definitions for sumFun.    */
/*       So the structure does not need to be provided.         */
/* Function Definition for sumFun                               */
KXMFUN sumFunc =
{
    KXMFUN_SIGVER,
    "sum", "Returns sum of all parameters",
    sumFun,
    0, KXMFUN_UNLIMITED, 0,
    NULL,
    KXMVALTYPE_DOUBLE
};            

