/********************************************************************
 *                                                                    
 * Module      : kxmufops.c
 *                                                                    
 * Description : Sample arithmetic operators library
 *                   
 * This source is part of the arithmetic operators library example (kxmufops).
 * This source shows how to implement custom arithmetic operators.
 *
 * This file contains the following user defined function in it:
 * "unary+ operator"
 * "unary- operator"
 * "~ operator"
 * "* operator"
 * "/ operator"
 * "% operator"
 * "| operator"
 * "& operator"
 * "^ operator"
 * "+ operator"
 * "- operator"
 *
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 *                                                                    
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 *                                                                    
 ********************************************************************
 * $Id: kxmufops.c,v 1.6.2.1 2001/09/07 12:19:38 e123479 Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufops.c,v $
 * $Revision: 1.6.2.1 $                                                   
 ********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <kxmcfun.h>

/* macro returning greater value */
#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b)) 
#endif

/* macro returning lesser value */
#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b)) 
#endif

/* composed integral type */
#define KXMRESTYPE_INTCAST ( KXMVALTYPE_INT\
                            | KXMVALTYPE_INTEGER\
                            | KXMVALTYPE_UINT )

/* container for signed or unsigned integral value */
typedef struct
{
    KXMBOOL sign;
    union { KXMINT64 s; KXMUINT64 u; } value;
} KxmIntegralValue;

/* Typed Arguement Definitions for unary plus, unary minus,
 * multiplication, division, addition and subtraction operators */
static KXMFUNARG numArgs[] = 
{
    { KXMFUNARG_SIGVER, "left", "Left argument", KXMVALTYPE_NUMBER },
    { KXMFUNARG_SIGVER, "right", "Right argument", KXMVALTYPE_NUMBER }
};

/* Typed Arguement Definitions for bitwise NOT and remainder operators */
static KXMFUNARG intArgs[] = 
{
    { KXMFUNARG_SIGVER, "left", "Left argument", KXMRESTYPE_INTCAST },
    { KXMFUNARG_SIGVER, "right", "Right argument", KXMRESTYPE_INTCAST }
};

/* Typed Arguement Definitions for bitwise AND, bitwise inclusive OR
 * and bitwise exclusive OR operators */
static KXMFUNARG intbinArgs[] = 
{
    { KXMFUNARG_SIGVER, "left", "Left argument", KXMVALTYPE_ANY },
    { KXMFUNARG_SIGVER, "right", "Right argument", KXMVALTYPE_ANY }
};

/* Utility function to return error message */
static void setError( const char* errorMsg, const char* file, int line )
{
    KXMERROR error = { KXMERROR_DEFAULT };
    strcpy( error.Primary.ErrorMessage, errorMsg );
    sprintf( error.Primary.Location, "File \'%.64s' line %d.", file, line );
    kxmSetLastError( &error ); 
}

#define SET_ERROR( errorMsg )\
    setError( errorMsg, __FILE__, __LINE__ )

/* Helper function used to get signed or unsigned integral value */
static KXMBOOL getIntegralValue( HKXMFIELD field, KxmIntegralValue* ival )
{
    /* should not be NULL */
    if ( !ival )
    {
        return FALSE;
    }

    /* attempt to get signed integer */
    if ( kxmGetInt64( field, NULL, &ival->value.s, NULL, NULL ) )
    {
        ival->sign = TRUE;
        return TRUE;
    }
    /* attempt to get unsigned integer */
    else if ( kxmGetUInt64( field, NULL, &ival->value.u, NULL, NULL ) )
    {
        ival->sign = FALSE;
        return TRUE;
    }

    /* unsuccessful */
    return FALSE;
}

/*
 *  Unary plus operator implementation
 */
static KXMBOOL unaryPlus( const KXMFUNARGS* args, KXMFUNRES* result )
{
    double value;

    /* get right value */
    if ( !kxmGetDouble( *args->Args, NULL, &value, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: unaryPlus: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value */
    if ( !kxmSetDouble( result->Value, NULL, value, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: unaryPlus: Unable to set resulting value");
        return FALSE;
    }

    /* success */
    return TRUE;
}

/* Function Definition for unary plus operator */
static KXMFUN plusFunc =
{ 
    KXMFUN_SIGVER, 
    "unary +",
    "Unary plus operator",
    unaryPlus, 
    1, 1, 1, 
    /* this operator takes right value only */
    numArgs + 1, 
    KXMVALTYPE_DOUBLE
};

/*
 *  Unary minus operator implementation
 */
static KXMBOOL unaryMinus( const KXMFUNARGS* args, KXMFUNRES* result )
{
    double value;

    /* get right value */
    if ( !kxmGetDouble( *args->Args, NULL, &value, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: unaryMinus: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value */
    if ( !kxmSetDouble( result->Value, NULL, -value, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: unaryMinus: Unable to set resulting value");
        return FALSE;
    }

    /* success */
    return TRUE;
}

/* Function Definition for unary minus operator */
static KXMFUN minusFunc =
{ 
    KXMFUN_SIGVER, 
    "unary -",
    "Unary minus operator",
    unaryMinus, 
    1, 1, 1, 
    /* this operator takes right value only */
    numArgs + 1, 
    KXMVALTYPE_DOUBLE
};

/*
 *  Bitwise NOT operator implementation
 */
static KXMBOOL bitwiseNot( const KXMFUNARGS* args, KXMFUNRES* result )
{
    KxmIntegralValue ival;

    /* get right value */
    if ( !getIntegralValue( *args->Args, &ival ) )
    {
        SET_ERROR("*** ERROR: bitwiseNot: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value depending on sign of the argument */
    if ( ival.sign )
    {
        /* signed result */
        if ( !kxmSetInt64( result->Value, NULL, ~ival.value.s, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseNot: Unable to set resulting value");
            return FALSE;
        }
    }
    else
    {
        /* unsigned result */
        if ( !kxmSetUInt64( result->Value, NULL, ~ival.value.u, NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseNot: Unable to set resulting value");
            return FALSE;
        }
    }

    /* success */
    return TRUE;
}

/* Function Definition for bitwise NOT operator */
static KXMFUN notFunc =
{ 
    KXMFUN_SIGVER, 
    "~",
    "Bitwise NOT operator",
    bitwiseNot, 
    1, 1, 1, 
    /* this operator takes right value only */
    intArgs + 1, 
    KXMVALTYPE_UINT
};

/*
 *  Multiplication operator implementation
 */
static KXMBOOL multiply( const KXMFUNARGS* args, KXMFUNRES* result )
{
    double lval, rval;

    /* get left value */
    if ( !kxmGetDouble( *args->Args, NULL, &lval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: multiply: Unable to get 'left' value");
        return FALSE;
    }

    /* get right value */
    if ( !kxmGetDouble( args->Args[1], NULL, &rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: multiply: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value */
    if ( !kxmSetDouble( result->Value, NULL, lval * rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: multiply: Unable to set resulting value");
        return FALSE;
    }

    /* success */
    return TRUE;
}

/* Function Definition for multiplication operator */
static KXMFUN mulFunc =
{ 
    KXMFUN_SIGVER, 
    "*",
    "Multiplication operator",
    multiply, 
    2, 2, 2, 
    numArgs, 
    KXMVALTYPE_DOUBLE
};

/*
 *  Division operator implementation
 */
static KXMBOOL divide( const KXMFUNARGS* args, KXMFUNRES* result )
{
    double lval, rval;

    /* get left value */
    if ( !kxmGetDouble( *args->Args, NULL, &lval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: divide: Unable to get 'left' value");
        return FALSE;
    }

    /* get right value */
    if ( !kxmGetDouble( args->Args[1], NULL, &rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: divide: Unable to get 'right' value");
        return FALSE;
    }

    /* prevent division by zero */
    if ( rval == 0.0 )
    {
        SET_ERROR("*** ERROR: divide: Expression attempts to divide by zero");
        return FALSE;
    }

    /* set resulting value */
    if ( !kxmSetDouble( result->Value, NULL, lval / rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: divide: Unable to set resulting value");
        return FALSE;
    }

    /* success */
    return TRUE;
}

/* Function Definition for division operator */
static KXMFUN divFunc =
{ 
    KXMFUN_SIGVER, 
    "/",
    "Division operator",
    divide, 
    2, 2, 2, 
    numArgs, 
    KXMVALTYPE_DOUBLE
};

/*
 *  Remainder operator implementation
 */
static KXMBOOL modulus( const KXMFUNARGS* args, KXMFUNRES* result )
{
    KxmIntegralValue lval, rval;

    /* get left value */
    if ( !getIntegralValue( *args->Args, &lval ) )
    {
        SET_ERROR("*** ERROR: modulus: Unable to get 'left' value");
        return FALSE;
    }

    /* get right value */
    if ( !getIntegralValue( args->Args[1], &rval ) )
    {
        SET_ERROR("*** ERROR: modulus: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value depending on sign of the arguments */
    if ( lval.sign && rval.sign )
    {
        /* prevent division by zero */
        if ( !rval.value.s )
        {
            SET_ERROR("*** ERROR: modulus: Expression attempts to divide by zero");
            return FALSE;
        }

        /* set resulting value */
        if ( !kxmSetUInt64( result->Value, NULL, lval.value.s % rval.value.s, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: modulus: Unable to set resulting value");
            return FALSE;
        }

    }
    else if ( lval.sign && !rval.sign )
    {
        /* prevent division by zero */
        if ( !rval.value.u )
        {
            SET_ERROR("*** ERROR: modulus: Expression attempts to divide by zero");
            return FALSE;
        }

        /* set resulting value */
        if ( !kxmSetInt64( result->Value, NULL, lval.value.s % rval.value.u, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: modulus: Unable to set resulting value");
            return FALSE;
        }
    }
    else if ( !lval.sign && rval.sign )
    {
        /* prevent division by zero */
        if ( !rval.value.s )
        {
            SET_ERROR("*** ERROR: modulus: Expression attempts to divide by zero");
            return FALSE;
        }

        /* set resulting value */
        if ( !kxmSetInt64( result->Value, NULL, lval.value.u % rval.value.s,
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: modulus: Unable to set resulting value");
            return FALSE;
        }
    }
    else
    {
        /* prevent division by zero */
        if ( !rval.value.u )
        {
            SET_ERROR("*** ERROR: modulus: Expression attempts to divide by zero");
            return FALSE;
        }

        /* set resulting value */
        if ( !kxmSetUInt64( result->Value, NULL, lval.value.u % rval.value.u,
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: modulus: Unable to set resulting value");
            return FALSE;
        }
    }

    /* success */
    return TRUE;
}

/* Function Definition for remainder operator */
static KXMFUN modFunc =
{ 
    KXMFUN_SIGVER, 
    "%",
    "Remainder operator",
    modulus, 
    2, 2, 2, 
    intArgs, 
    KXMVALTYPE_INT
};

/*
 *  Bitwise AND operator implementation
 */
static KXMBOOL bitwiseAnd( const KXMFUNARGS* args, KXMFUNRES* result )
{
    const char* ldata;
    const char* rdata;
    KXMULONG leftLength, rightLength;
    KXMBOOL isLeftBin = FALSE, isRightBin = FALSE;
    KxmIntegralValue lval, rval;

    /* attempt to get binary left value */
    if ( kxmGetBinary( *args->Args, NULL, (const void**)&ldata, 
        &leftLength, NULL, NULL ) )
    {
        isLeftBin = TRUE;
    }
    /* attempt to get integral left value */
    else if ( !getIntegralValue( *args->Args, &lval ) )
    {
        SET_ERROR("*** ERROR: bitwiseAnd: Unable to get 'left' value");
        return FALSE;
    }

    /* attempt to get binary right value */
    if ( kxmGetBinary( args->Args[1], NULL, (const void**)&rdata, 
        &rightLength, NULL, NULL ) )
    {
        isRightBin = TRUE;
    }
    /* attempt to get integral right value */
    else if ( !getIntegralValue( args->Args[1], &rval ) )
    {
        SET_ERROR("*** ERROR: bitwiseAnd: Unable to get 'right' value");
        return FALSE;
    }

    /* left and right values are binary */
    if ( isLeftBin && isRightBin )
    {
        KXMULONG idx = 0;
        KXMULONG minLength = MIN( leftLength, rightLength );
        KXMULONG maxLength = MAX( leftLength, rightLength );
        char* outData = (char*)calloc( maxLength, 1 );

        /* sequentially AND binary data */
        for (; idx < minLength; idx++ )
        {
            outData[idx] = ldata[idx] & rdata[idx];
        }   

        /* set resulting value */
        if ( !kxmSetBinary( result->Value, NULL, outData, maxLength, NULL, NULL ) )
        {
            free( outData );
            SET_ERROR("*** ERROR: bitwiseAnd: Unable to set resulting value");
            return FALSE;
        }

        /* success */
        free( outData );
        return TRUE;
    }

    /* just one of values is binary */
    if ( isLeftBin || isRightBin )
    {
        SET_ERROR("*** ERROR: bitwiseAnd: Left and right arguments must have equal data types");
        return FALSE;
    }

    /* set resulting value depending on sign of the arguments */
    if ( !lval.sign && !rval.sign )
    {
        /* unsigned result */
        if ( !kxmSetUInt64( result->Value, NULL, lval.value.u & rval.value.u, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseAnd: Unable to set resulting value");
            return FALSE;
        }
    }
    else
    {
        /* signed result */
        if ( !kxmSetInt64( result->Value, NULL, lval.value.s & rval.value.s, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseAnd: Unable to set resulting value");
            return FALSE;
        }
    }

    /* success */
    return TRUE;
}

/* Function Definition for bitwise AND operator */
static KXMFUN andFunc =
{ 
    KXMFUN_SIGVER, 
    "&",
    "Bitwise AND operator",
    bitwiseAnd, 
    2, 2, 2, 
    intbinArgs, 
    KXMVALTYPE_ANY
};

/*
 *  Bitwise inclusive OR operator implementation
 */
static KXMBOOL bitwiseOr( const KXMFUNARGS* args, KXMFUNRES* result )
{
    KxmIntegralValue lval, rval;
    const char* ldata;
    const char* rdata;
    KXMULONG leftLength, rightLength;
    KXMBOOL isLeftBin = FALSE, isRightBin = FALSE;

    /* attempt to get binary left value */
    if ( kxmGetBinary( *args->Args, NULL, (const void**)&ldata, 
        &leftLength, NULL, NULL ) )
    {
        isLeftBin = TRUE;
    }
    /* attempt to get integral left value */
    else if ( !getIntegralValue( *args->Args, &lval ) )
    {
        SET_ERROR("*** ERROR: bitwiseOr: Unable to get 'left' value");
        return FALSE;
    }

    /* attempt to get binary right value */
    if ( kxmGetBinary( args->Args[1], NULL, (const void**)&rdata, 
        &rightLength, NULL, NULL ) )
    {
        isRightBin = TRUE;
    }
    /* attempt to get integral right value */
    else if ( !getIntegralValue( args->Args[1], &rval ) )
    {
        SET_ERROR("*** ERROR: bitwiseOr: Unable to get 'right' value");
        return FALSE;
    }

    /* left and right values are binary */
    if ( isLeftBin && isRightBin )
    {
        KXMULONG idx = 0;
        KXMULONG minLength = MIN( leftLength, rightLength );
        KXMULONG maxLength = MAX( leftLength, rightLength );
        char* outData = (char*)calloc( maxLength, 1 );

        /* sequentially OR binary data */
        for (; idx < minLength; idx++ )
        {
            outData[idx] = ldata[idx] | rdata[idx];
        }
    
        /* set resulting value */
        if ( !kxmSetBinary( result->Value, NULL, outData, maxLength, 
            NULL, NULL ) )
        {
            free( outData );
            SET_ERROR("*** ERROR: bitwiseOr: Unable to set resulting value");
            return FALSE;
        }

        /* success */
        free( outData );
        return TRUE;
    }

    /* just one of values is binary */
    if ( isLeftBin || isRightBin )
    {
        SET_ERROR("*** ERROR: bitwiseOr: Left and right arguments must have equal data types");
        return FALSE;
    }

    /* set resulting value depending on sign of the arguments */
    if ( !lval.sign && !rval.sign )
    {
        /* unsigned result */
        if ( !kxmSetUInt64( result->Value, NULL, lval.value.u | rval.value.u, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseOr: Unable to set resulting value");
            return FALSE;
        }
    }
    else
    {
        /* signed result */
        if ( !kxmSetInt64( result->Value, NULL, lval.value.s | rval.value.s, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseOr: Unable to set resulting value");
            return FALSE;
        }
    }

    /* success */
    return TRUE;
}

/* Function Definition for bitwise inclusive OR operator */
static KXMFUN orFunc =
{ 
    KXMFUN_SIGVER, 
    "|",
    "Bitwise inclusive OR operator",
    bitwiseOr, 
    2, 2, 2, 
    intbinArgs, 
    KXMVALTYPE_ANY
};

/*
 *  Bitwise exclusive OR operator implementation
 */
static KXMBOOL bitwiseXor( const KXMFUNARGS* args, KXMFUNRES* result )
{
    KxmIntegralValue lval, rval;
    const char* ldata;
    const char* rdata;
    KXMULONG leftLength, rightLength;
    KXMBOOL isLeftBin = FALSE, isRightBin = FALSE;

    /* attempt to get binary left value */
    if ( kxmGetBinary( *args->Args, NULL, (const void**)&ldata, 
        &leftLength, NULL, NULL ) )
    {
        isLeftBin = TRUE;
    }
    /* attempt to get integral left value */
    else if ( !getIntegralValue( *args->Args, &lval ) )
    {
        SET_ERROR("*** ERROR: bitwiseXor: Unable to get 'left' value");
        return FALSE;
    }

    /* attempt to get binary right value */
    if ( kxmGetBinary( args->Args[1], NULL, (const void**)&rdata, 
        &rightLength, NULL, NULL ) )
    {
        isRightBin = TRUE;
    }
    /* attempt to get integral right value */
    else if ( !getIntegralValue( args->Args[1], &rval ) )
    {
        SET_ERROR("*** ERROR: bitwiseXor: Unable to get 'right' value");
        return FALSE;
    }

    /* left and right values are binary */
    if ( isLeftBin && isRightBin )
    {
        KXMULONG idx = 0;
        KXMULONG minLength = MIN( leftLength, rightLength );
        KXMULONG maxLength = MAX( leftLength, rightLength );
        char* outData = (char*)calloc( maxLength, 1 );

        /* sequentially XOR binary data */
        for (; idx < minLength; idx++ )
        {
            outData[idx] = ldata[idx] ^ rdata[idx];
        }

        /* set resulting value */
        if ( !kxmSetBinary( result->Value, NULL, outData, maxLength, NULL, NULL ) )
        {
            free( outData );
            SET_ERROR("*** ERROR: bitwiseXor: Unable to set resulting value");
            return FALSE;
        }

        /* success */
        free( outData );
        return TRUE;
    }

    /* just one of values is binary */
    if ( isLeftBin || isRightBin )
    {
        SET_ERROR("*** ERROR: bitwiseXor: Left and right arguments must have equal data types");
        return FALSE;
    }

    /* set resulting value depending on sign of the arguments */
    if ( !lval.sign && !rval.sign )
    {
        /* unsigned result */
        if ( !kxmSetUInt64( result->Value, NULL, lval.value.u ^ rval.value.u, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseXor: Unable to set resulting value");
            return FALSE;
        }
    }
    else
    {
        /* signed result */
        if ( !kxmSetInt64( result->Value, NULL, lval.value.s ^ rval.value.s, 
            NULL, NULL ) )
        {
            SET_ERROR("*** ERROR: bitwiseXor: Unable to set resulting value");
            return FALSE;
        }
    }

    /* success */
    return TRUE;
}

/* Function Definition for bitwise exclusive OR operator */
static KXMFUN xorFunc =
{ 
    KXMFUN_SIGVER, 
    "^",
    "Bitwise exclusive OR operator",
    bitwiseXor, 
    2, 2, 2, 
    intbinArgs, 
    KXMVALTYPE_ANY
};

/*
 *  Addition operator implementation
 */
static KXMBOOL addition( const KXMFUNARGS* args, KXMFUNRES* result )
{
    double lval, rval;

    /* get left value */
    if ( !kxmGetDouble( *args->Args, NULL, &lval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: addition: Unable to get 'left' value");
        return FALSE;
    }

    /* get right value */
    if ( !kxmGetDouble( args->Args[1], NULL, &rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: addition: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value */
    if ( !kxmSetDouble( result->Value, NULL, lval + rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: addition: Unable to set resulting value");
        return FALSE;
    }

    /* success */
    return TRUE;
}

/* Function Definition for addition operator */
static KXMFUN addFunc =
{ 
    KXMFUN_SIGVER, 
    "+",
    "Addition operator",
    addition, 
    2, 2, 2, 
    numArgs, 
    KXMVALTYPE_DOUBLE
};

/*
 *  Subtraction operator implementation
 */
static KXMBOOL subtraction( const KXMFUNARGS* args, KXMFUNRES* result )
{
    double lval, rval;

    /* get left value */
    if ( !kxmGetDouble( *args->Args, NULL, &lval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: subtraction: Unable to get 'left' value");
        return FALSE;
    }

    /* get right value */
    if ( !kxmGetDouble( args->Args[1], NULL, &rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: subtraction: Unable to get 'right' value");
        return FALSE;
    }

    /* set resulting value */
    if ( !kxmSetDouble( result->Value, NULL, lval - rval, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: subtraction: Unable to set resulting value");
        return FALSE;
    }

    /* success */
    return TRUE;
}

/* Function Definition for subtraction operator */
static KXMFUN subFunc =
{ 
    KXMFUN_SIGVER, 
    "-",
    "Subtraction operator",
    subtraction, 
    2, 2, 2, 
    numArgs, 
    KXMVALTYPE_DOUBLE
};

/* list of functions defined by the library */
static KXMFUN* libFuncs[] = 
{
    &plusFunc,
    &minusFunc,
    &notFunc,
    &mulFunc,
    &divFunc,
    &modFunc,
    &andFunc,
    &orFunc,
    &xorFunc,
    &addFunc,
    &subFunc,
    /* last element must be NULL */
    NULL
};

/* library structure */  
static KXMFUNLIB libDesc = 
{ 
    KXMFUNLIB_SIGVER,  
    KXMFUNAPIVER_CURRENT,
    "Sample arithmetic Operators Library",
    libFuncs,
    NULL,
    NULL
};

/* library entry function */
KXMFUN_DEFINEENTRY( libDesc )

