/********************************************************************
 *                                                                    
 * Module      : kxmufsim.c
 *                                                                    
 * Description : Simple User Defined Function C API Sample
 *
 * This source is a simple example of the map conversion function library 
 * that contains a single user defined function.  The code has been
 * commented to show all aspects required for writing a user defined
 * function.
 *                                                                    
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 * 
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 * 
 ********************************************************************
 * $Id: kxmufsim.c,v 1.12.2.1 2001/09/07 12:19:38 e123479 Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufsim.c,v $
 * $Revision: 1.12.2.1 $                                                   
 ********************************************************************/

#include <stdlib.h>
#include <string.h>
#include <kxmcfun.h>
#include <stdio.h>

/* Utility function to return an error message if there is an error. */
static void setError( const char* errorMsg, const char* file, int line )
{
    KXMERROR error = { KXMERROR_DEFAULT };
    strcpy( error.Primary.ErrorMessage, errorMsg );
    sprintf( error.Primary.Location, "File '%.64s' line %d.", file, line );
    kxmSetLastError( &error ); 
}

#define SET_ERROR( errorMsg )\
    setError( errorMsg, __FILE__, __LINE__ )

/********************************************************************
 * This is the user defined function.  This function will convert
 * all characters in a string to upper case.  The function takes a 
 * single parameter which is a string.  After converting the string,
 * it sets the resulting value.
 *
 * This is a not a complete function.  The string is assumed to be
 * a single byte character set terminating with a NULL character.
 ********************************************************************/
static KXMBOOL simpleToUpper( const KXMFUNARGS* args, KXMFUNRES* result )
{
    const char* userStr;
    char* tempStr;
    char* newStr;
    KXMULONG strLen;

    /* Get the first (and only) parameter value.  Get the value      */
    /* as a string value.  If the string is not set (NULL), then     */
    /* fail the user defined function.                               */
    if( !kxmGetString( args->Args[0], NULL, (const char**)&userStr, &strLen, 
        NULL, NULL ) )
    {
         /* There is an error that we can not recover from.  Set the */
         /* the error and return FALSE from the function.  This will */
         /* cause the message conversion process to fail and the     */
         /* error message to be reported.                            */
         SET_ERROR("*** ERROR: simple_toupper(): Unable to get first string value");
         return FALSE;
    }

    newStr = malloc( strLen + 1 );    
    tempStr = strcpy( newStr, userStr );
    /* While we are not at the end of the string, convert it toupper */
    while( *tempStr )
    {
         *tempStr = toupper(*tempStr);
         ++tempStr;
    }

    /* Set the return value of the function                          */
    if ( !kxmSetString( result->Value, NULL, newStr, strLen, NULL, NULL ) )
    {
        free( newStr );
        SET_ERROR("*** ERROR: simple_toupper(): Unable to set return string value");
        return FALSE;
    }

    /* We successfully complete the function.  Return TRUE to indicate   */
    /* to the conversion engine that the function completed successfully */
    free( newStr );
    return TRUE;
}

/* This is required for each User Defined Map Functon.           */
/* Contains a list of structures.  Each structure provides       */
/* information on a single typed argument that is input to       */
/* the User Defined Map Function.                                */
static KXMFUNARG simple_toupperArgs[] = 
{
    { KXMFUNARG_SIGVER, "string", "String to convert", KXMVALTYPE_STRING }
};            

/* This is required for each User Defined Function.              */
/* This structure provides the information for a single User     */
/* Defined Map Function implemented by the Function Library.     */
static KXMFUN simple_toupperFunc =
{ 
    /* Identification of the signature and version of this   */
        /* structure.                                            */
    KXMFUN_SIGVER, 
    /* Name of the map function.  It is the same name as     */
    /* specified in the function call in the  map action.    */
    "simple_toupper",
        /* Function description.  Describes the functionality    */
        /* of the function for the suers of the maps.            */
    "Converts string characters to upper case",
        /* A pointer to the function callback.  This is the      */
        /* pointer to the callback that will be called when the  */
        /* function is referenced in a map action.               */
    simpleToUpper, 
        /* Minimum number of arguments accepted by the function. */
    1, 
        /* Maximum number of arguments accepted by the function. */
    1, 
        /* Number of typed arguments accepted by the function.   */
        /* If the typed arguments are provided, the Conversion   */
        /* Engine will validate the type of the argument before  */
        /* calling the User Defined Function.                    */
    1, 
        /* The typed arguments for the function.                 */
    simple_toupperArgs, 
        /* The type of the return value from the function.       */
    KXMVALTYPE_STRING 
};

/* This is required for a Function Library.                      */
/* This defines the list of User Defined Functions available in  */
/* This Function Library.  You may define functions in this file */
/* But if they are not listed in this structure, they are not    */
/* available to the Conversion Engine.                           */
static KXMFUN* funcPtrs[] = 
{ 
    &simple_toupperFunc, 
    NULL /* last pointer must be NULL */
};

/* This is required for a Function Library.  It defines the user */
/* defined functions and characteristics of this Function        */
/* Library.  It defines:                                         */
/* a. the API version used by these functions to interface       */
/*    with the Conversion Engine.                                */
/* b. A single line of description for this Function Library.    */
/* c. The list of all the user defined functions in this         */
/*    Function Library.                                          */
/* d. A user defined function to be called when the Function     */
/*    Library is first loaded.                                   */
/* e. A user defined function to be called just before the       */
/*    Function Library is being unloaded.                        */

/* It defines an identification of  */
static KXMFUNLIB libInfo = 
{ 
    /* Identification of the signature and version of this   */
        /* structure.                                            */
    KXMFUNLIB_SIGVER,  
        /* The version of the User Defined Functions API that    */
        /* the Function Library was built for.                   */
    KXMFUNAPIVER_CURRENT,
    /* Description of the Function Library                   */
    "Simple User Defined Map Conversion Function Library",
    /* Each element in this array describes a single User    */
    /* Defined Map Function.                                 */
    funcPtrs,
    /* Optional function to be called just after dynamically */
    /* loading the Function Library                          */
    NULL,
    /* Optional function to be called just before unloading  */
    /* the Function Library                                  */
    NULL
};

/* This is required for a function library.  It exports the entry */
/* point so that the conversion engine can dynamically load the   */
/* function library.                                              */
KXMFUN_DEFINEENTRY( libInfo )

