/********************************************************************
 *                                                                    
 * Module      : kxmufstr.c
 *                                                                    
 * Description : User Defined Functions C API Sample
 *                                                                    
 * This source is example of the map conversion library,
 * which provides four custom conversion functions:
 * "tolower"
 * "toupper" 
 * "substring"
 *                                                                    
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 * 
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 * 
 ********************************************************************
 * $Id: kxmufstr.c,v 1.8.2.2 2002/03/21 13:30:18 e_ab Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufstr.c,v $
 * $Revision: 1.8.2.2 $                                                   
 ********************************************************************/
#include <ctype.h>
#include "kxmufful.h"

#ifdef __cplusplus
extern "C"
{
KXMBOOL expToLower( const KXMFUNARGS* args, KXMFUNRES* result );
KXMBOOL expToUpper( const KXMFUNARGS* args, KXMFUNRES* result );
KXMBOOL expSubstring( const KXMFUNARGS* args, KXMFUNRES* result );
}
#endif

/********************************************************************
 *  tolower( string )
 *
 *  Converts all characters of a string to lower case
 ********************************************************************/
static KXMBOOL expToLower( const KXMFUNARGS* args, KXMFUNRES* result )
{
    const char* userStr;
    char* tempStr;
    char* newStr;
    KXMULONG strLen;

    if( !kxmGetString( args->Args[0], NULL, &userStr, &strLen, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: tolower: Unable to get input value");
        return FALSE;
    }

    newStr = (char *)malloc( strLen + 1 );    
    tempStr = strcpy( newStr, userStr );
    while( *tempStr )
    {
        *tempStr = tolower(*tempStr);
        tempStr++;
    }

    if ( !kxmSetString( result->Value, NULL, newStr, strLen, NULL, NULL ) )
    {
        free( newStr );
      SET_ERROR("*** ERROR: tolower: Unable to set return value");
        return FALSE;
    }

    free( newStr );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for tolower */
static KXMFUNARG tolowerArgs[] = 
{
    { KXMFUNARG_SIGVER, "string", "String to convert", KXMVALTYPE_STRING }
};

/* Function Definition for tolower */
KXMFUN tolowerFunc =
{ 
    KXMFUN_SIGVER, 
    "tolower",
    "Converts string characters to lower case",
    expToLower, 
    1, 1, 1, 
    tolowerArgs, 
    KXMVALTYPE_STRING 
};

/********************************************************************
 *  toupper( string )
 *
 *  Converts all characters of a string to upper case
 ********************************************************************/
static KXMBOOL expToUpper( const KXMFUNARGS* args, KXMFUNRES* result )
{
    const char* userStr;
    char* tempStr;
    char* newStr;
    KXMULONG strLen;

    if( !kxmGetString( args->Args[0], NULL, &userStr, &strLen, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: toupper: Unable to get input value");
        return FALSE;
    }

    newStr = (char *)malloc( strLen + 1 );    
    tempStr = strcpy( newStr, userStr );
    while( *tempStr )
    {
        *tempStr = toupper(*tempStr);
        tempStr++;
    }

    if ( !kxmSetString( result->Value, NULL, newStr, strLen, NULL, NULL ) )
    {
        free( newStr );
      SET_ERROR("*** ERROR: toupper: Unable to set return value");
        return FALSE;
    }

    free( newStr );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for toupper */
static KXMFUNARG toupperArgs[] = 
{
    { KXMFUNARG_SIGVER, "string", "String to convert", KXMVALTYPE_STRING }
};            

/* Function Definition for toupper */
KXMFUN toupperFunc =
{ 
    KXMFUN_SIGVER, 
    "toupper",
    "Converts string characters to upper case",
    expToUpper, 
    1, 1, 1, 
    toupperArgs, 
    KXMVALTYPE_STRING 
};

/********************************************************************
 *  substring( string, start, length )
 *
 *  Returns substring of a string.
 *  If optional substring length parameter
 *  not specified, substring will be extracted
 *  from starting position to end of a string.
 ********************************************************************/
static KXMBOOL expSubstring( const KXMFUNARGS* args, KXMFUNRES* result )
{
    KXMULONG subStart, subLength, strLen;
    const char* userStr;
    char* tempStr;

    /* get string */
    if( !kxmGetString( args->Args[0], NULL, &userStr, &strLen, NULL, NULL )
        /* get starting position */
        || !kxmGetLong( args->Args[1], NULL, (long*)&subStart, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: substring: Unable to get input values");
        return FALSE;
    }

    subLength = strLen;
    /* get optional substring length */
    if ( args->NumOfArgs > 2 &&
        !kxmGetLong( args->Args[2], NULL, (long*)&subLength, NULL, NULL ) )
    {
        SET_ERROR("*** ERROR: substring: Unable to get length value");
        return FALSE;
    }

    if ( strLen <= subStart || !subLength )
    {
        /* return empty string */
        return kxmSetString( result->Value, NULL, "", 0, NULL, NULL );
    }

    strLen -= subStart;
    if ( subLength > strLen ) 
    {
        subLength = strLen;
    }

    /* extract substring */
    tempStr = (char *)malloc( subLength + 1 );
    memcpy( tempStr, userStr + subStart, subLength);
    tempStr[subLength] = '\0';

    /* set result string */
    if ( !kxmSetString( result->Value, NULL, tempStr, subLength, NULL, NULL ) )
    {
        free( tempStr );
        SET_ERROR("*** ERROR: substring: Unable to set return value");
        return FALSE;
    }

    /* success */
    free( tempStr );

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for substring */
static KXMFUNARG substrArgs[] = 
{
    { KXMFUNARG_SIGVER, "string", "Input string", KXMVALTYPE_STRING },
    { KXMFUNARG_SIGVER, "start", "Starting position", KXMVALTYPE_UINT },
    { KXMFUNARG_SIGVER, "length", "Length of substring", KXMVALTYPE_UINT }
};            

/* Function Definition for substring */
KXMFUN substrFunc =
{ 
    KXMFUN_SIGVER, 
    "substring",
    "Extracts substring from a string",
    expSubstring, 
    2, 3, 3, 
    substrArgs, 
    KXMVALTYPE_STRING 
};

