/********************************************************************/
/*                                                                  */
/* Module      : KXCSCLNT.C                                         */
/*                                                                  */
/* Description : Sample Roma Client program                         */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998,2002                           */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Details:                                                         */
/*    Sample Roma Client Program.                                   */
/*                                                                  */
/*    This program sends the user-specified message(s) to an Roma   */
/*    server.  For each message sent (other than datagram messages),*/
/*    a response is retrieved from the server and written to stdout.*/
/*                                                                  */
/*    The messages that are to be sent may be specified in one of   */
/*    three ways, as follows:                                       */
/*                                                                  */
/*    1. This program can prompt the user to enter the text of each */
/*       message that is to be sent.  This is the default if neither*/
/*       of the other options is selected.                          */
/*    2. In a text file (option -f).                                */
/*    3. By specifying an integer count of the number of default    */
/*       messages to be generated and sent to the server (option -l)*/
/*                                                                  */
/*    The format of a message is:                                   */
/*       ServiceName:MessageText                                    */
/*                                                                  */
/*    where:                                                        */
/*       "ServiceName:" -                                           */
/*          is the optional name of the specific business service   */
/*          to use for this message in lieu of the default service. */
/*       "Message text" -                                           */
/*          is a character string token (without blanks or other    */
/*          common delimiters).                                     */
/*                                                                  */
/*    This program terminates upon encountering the string "QUIT"   */
/*    as the text of a message, or when the number of messages      */
/*    specified by the -l option has been sent.                     */
/*                                                                  */
/* Dependencies:                                                    */
/*    Requires the relevant services to be set up in the directory  */
/*    and the underlying transport queues to be configured in       */
/*    support of the routes defined for the directory entry.        */
/*    The server application must also be started prior to running  */
/*    this sample client.                                           */
/*                                                                  */
/* Invocation Syntax:                                               */
/*    c_client  [ -c client_name ]  [ -s service_name  ]            */
/*              [ -m message_format_name ]                          */
/*              [ -l loop_count  ]  [ -f msg_file_name ]            */
/*              [ -?             ]                                  */
/*                                                                  */
/* Input Parameters (all optional):                                 */
/*    -c client_name:                                               */
/*       Specifies the client name to use instead of the default    */
/*       (ClientGroup1).  Substitute a valid client name for        */
/*       "client_name".                                             */
/*                                                                  */
/*    -s service_name:                                              */
/*       Specifies the business service name to use instead of the  */
/*       default (TestService1).  Substitute a valid business       */
/*       service name for "service_name".                           */
/*                                                                  */
/*    -m message_format_name:                                       */
/*       Specifies the message format name that will be set on the  */
/*       request message. There must be a message descriptor in the */
/*       directory with this name and the target must accept it or  */
/*       suitable conversions be defined                            */
/*                                                                  */
/*    -l loop_count:                                                */
/*       Specifies the number of times that a default message is to */
/*       be generated and sent to the Roma server.  This option     */
/*       eliminates the need for user message input.  Substitute a  */
/*       valid integer count for "loop_count".  Ignored if -f is    */
/*       also specified.  If neither -l nor -f is specified, the    */
/*       user will be prompted for message input.                   */
/*                                                                  */
/*    -f msg_file_name:                                             */
/*       Specifies the name of a file from which messages are to be */
/*       read for submission to the Roma server.  Overrides  the -l */
/*       option when both are specified.  Substitute a valid        */
/*       fully-qualified file name for "msg_file_name".  If neither */
/*       -l nor -f is specified, the user will be prompted for      */
/*       message input.                                             */
/*                                                                  */
/*    -? :                                                          */
/*       Requests a help display for this program.  If this option  */
/*       is specified, all others are ignored.                      */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral codes:                          */
/*       EXIT_SUCCESS -- Successful completion.                     */
/*       EXIT_FAILURE -- Unsuccessful completion.                   */
/*                                                                  */
/*                                                                  */
/* Functions in this module:                                        */
/*    main.............. Entry function for program and mainline    */
/*    display_help...... Display help information for this program  */
/*    processError...... Display information for last Roma error    */
/*    endSession........ Check for "end of session" request in msg  */
/*    parse_cmdline..... Validate and return command-line arguments */
/*    split............. Parse caller-specified string into tokens  */
/*    processInput...... Extract message text and service name      */
/*                       from input string.                         */
/*    allCaps........... Convert string to upper-case.              */
/*                                                                  */
/*                                                                  */
/* To build this sample application, make any necessary changes to  */
/* the mksample make file and then issue:                           */
/*                                                                  */
/*             On Unix      -  make -f mksample.mak                 */
/*             On NT/95/98  - nmake -f mksample.mak                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include system header files                                      */
/********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>

/********************************************************************/
/* Include the Roma header file                                     */
/********************************************************************/
#include <kxc.h>


/****
*  Tunable macros -- default Roma names
****/
#define  DEFAULT_COMPONENT_NAME  "ClientGroup1"
#define  DEFAULT_SERVICE_NAME    "TestService1"


/****
*  Message-related symbolic values:
*     QUIT_MSG       -- Message text that requests program termination
*     QML            -- The length of the QUIT_MSG text
*     MAX_MSG_LENGTH -- Maximum length of message text
****/
#define QUIT_MSG                "QUIT"
#define QML                      4
#define MAX_MSG_LENGTH           1024
#define MAX_WAIT_LEN             30000L


/****
*  Standard macros
****/
#define ASSERT(x) assert(x)

#define bool  int
#define false 0
#define true  1


/****
*  Function prototypes
****/
void  processError( char *specific_msg );
void  display_help( void );
bool  endSession( const char *msg );        /* message to end prog  */

int   parse_cmdline( int     argc,
                     char   *argv[],
                     char  **ppClient,
                     char  **ppService,
                     char  **ppLoopcount,
                     char  **ppMsgfilename,
                     char  **ppMsgformatname,
                     int    *pHelp
                   );

int   split( char *pStream, char *pTok[], int token_limit,
             char *pSeparators );

int   processInput( char *pInput, char **ppService, char **ppMessage);
int   getOpt( int  argc,  char  *argv[],  char  **ppOptArg );

char  *allCaps( char *s );                  /* capitalize a string  */

/****
*  Diagnostic tracing macros (disabled by default).
*  To enable the tracing macros:
*     1. ESTREAM must specify an open file descriptor.
*     2. Uncomment the "#define  TRACE" statement.
*
*  Once tracing has been enabled, the tracing macros may be used
*  to conditionally write text to ESTREAM.  There are three such
*  macros:  TRACE0, TRACE1, and TRACE2.  Each requires as its
*  first argument a printf-style format string.  The macro name's
*  numeric suffix indicates the number of replaceable parameters
*  supported.  Examples:
*
*     TRACE0( "Program terminating\n." ) ;
*     TRACE1( "Return code = %i\n", iRC ) ;
*     TRACE2( "Service=%s, Return code = %i\n", Service, iRC ) ;
*
*  If symbolic constant TRACE is undefined, TRACEn macro calls
*  are effectively no-ops.
****/
#define  ESTREAM          stderr
#define  FLUSH            fflush(ESTREAM);

#if !defined(TRACE)
#define TRACE                               /*  turn ON the trace   */
#endif

#ifdef   TRACE
#define  TRACE0(s)        fprintf(ESTREAM,s);       FLUSH
#define  TRACE1(s,p1)     fprintf(ESTREAM,s,p1);    FLUSH
#define  TRACE2(s,p1,p2)  fprintf(ESTREAM,s,p1,p2); FLUSH
#else
#define  TRACE0(s)
#define  TRACE1(s,p1)
#define  TRACE2(s,p1,p2)
#endif


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Description:                                                     */
/*    Entry function for executable module.                         */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments passed.                    */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral codes:                          */
/*       EXIT_SUCCESS -- Successful completion.                     */
/*       EXIT_FAILURE -- Unsuccessful completion.                   */
/*                                                                  */
/********************************************************************/
int  main( int argc, char *argv[] )
{
    KXCAPPDEF   AppDef   = { KXCAPPDEF_DEFAULT };
    HKXC        hKxc     = NULL_HKXC;
    KXCSERVICE  Service;

    KXCMSG      reqMsg   = { KXCMSG_DEFAULT };  /* Request  message  */
    KXCMSG      rspMsg   = { KXCMSG_DEFAULT };  /* Response message  */

    KXCPUTOPT   reqOpt;       /* Options for message(s) to "put"     */
    KXCGETOPT   rspOpt;       /* Options for message(s) to "get"     */

    HKXCTHEME   hTheme   = NULL_HKXCTHEME;
                              /* Theme returned for a "put" message  */

    char   input[MAX_MSG_LENGTH] = {'\0'};   /* Buffer for msg input */

    int    inputlen            ;  /* Length of input message data    */
    int    iMsgNum       = 1   ;  /* Msg no. & loop iteration counter*/
    char  *pMessage      = NULL;  /* Pointer to message text         */
    char  *pService      = NULL;  /* Ptr to overriding service name  */
    int    iLoopCount          ;  /* Loop count specified via -l opt.*/
    int    bMoreToDo     = TRUE;  /* Message-processing loop control */

    FILE  *pMsgFILE      = NULL;  /* Pointer to msg file FILE object */


    /*
    *  Following variables represent command-line parameters:
    */
    char  *pClient_name   = NULL;  /* Pointer to client name         */
    char  *pService_name  = NULL;  /* Pointer to default service name*/
    char  *pLoop_count    = NULL;  /* Pointer to loop count          */
    char  *pMsg_file_name = NULL;  /* Pointer to message file name   */
    char  *pMsg_format_name = NULL;  /* Ptr to message format name   */
    int    help           = FALSE; /* "?" option indicator           */


    /****
    *  Get input parameters.
    ****/
    if  ( !parse_cmdline( argc,
                          argv,
                          &pClient_name  ,
                          &pService_name ,
                          &pLoop_count   ,
                          &pMsg_file_name,
                          &pMsg_format_name,
                          &help             ) )
    {
        return EXIT_FAILURE;
    }


    /****
    *  If a help display was requested, display it and exit.
    ****/
    if  ( help )
    {
        display_help();
        return  EXIT_SUCCESS;
    }


    /****
    *  If the client name and/or service name were not provided via
    *  command-line options, assign the default.  If a loop count
    *  was specified, convert it to binary now.
    ****/
    if  ( !pClient_name  )  pClient_name  = DEFAULT_COMPONENT_NAME ;
    if  ( !pService_name )  pService_name = DEFAULT_SERVICE_NAME   ;
    if  (  pLoop_count   )  iLoopCount    = atoi( pLoop_count )    ;


    /****
    *  If an input message file was provided, open it now and
    *  read the first message.  If the file is empty, terminate
    *  immediately.
    *
    ****/
    if  ( pMsg_file_name )
    {
        pMsgFILE = fopen( pMsg_file_name, "r" ) ;
        if  ( pMsgFILE == NULL )
        {
            processError( "Unable to open message file."
                          "  Terminating.\n" );
            return  EXIT_FAILURE;
        }

        if (  !fgets( input, (sizeof input), pMsgFILE )  )
        {
            processError( "Message file is empty.  Terminating.\n" );
            fclose( pMsgFILE );
            return  EXIT_FAILURE;
        }
    }


    /****
    *  Display initialization messages.
    ****/
    printf( "Roma Client starting...\n" );
    printf( "Using client name: %s\n", pClient_name  );
    printf( "Using service....: %s\n", pService_name );
    if ( pMsg_format_name != NULL )
      printf( "Using message format....: %s\n", pMsg_format_name );

    /****
    *  Set up application definition and use to initialise the Roma
    *  interface.
    ****/
    strncpy ((char *) AppDef.ElementName, 
             (char *) pClient_name, 
             sizeof(KXCOBJNAME));

    if  ( !kxcClientInitialize( &AppDef, &hKxc ) )
    {
        processError( "Failed to initialize Client.  Terminating.\n" );
        if  ( pMsgFILE != NULL )    fclose( pMsgFILE );
        return  EXIT_FAILURE;
    }

    printf( "Roma Client initialized successfully.\n" );


    /****
    *  Set message options for put and get.
    ****/
    kxcSetPutOptDefault( &reqOpt );
#ifdef DATAGRAM_MODEL
    reqOpt.RequestOptions |= KXCPOPT_DATAGRAM;
#endif

    kxcSetGetOptDefault( &rspOpt );
    rspOpt.ReceiveOptions |= (KXCGOPT_WAIT | KXCGOPT_AUTO_ALLOC);
    rspOpt.WaitLen        = MAX_WAIT_LEN;


    /****
    *  Message-processing loop:
    *  -- Generate, read or solicit message depending upon user options
    *  -- Send message to server
    *  -- If not datagram message, retrieve response and display it
    *
    *  Note that for message text read from a file, we must
    *  reset the newline char. to null since fgets retains it.
    ****/
    TRACE0( "Message processing starting.\n" );

    while ( bMoreToDo )
    {
        /****
        *  Get or generate text of next message.
        ****/
        if  ( pMsgFILE )  /* If a message file was supplied: */
        {
            /*  Eliminate newline character from last message read.
            */
            inputlen = strlen( input );
            if  (  (inputlen              >  0   )  &&
                   (input[ inputlen - 1 ] == '\n')
                )
                input[ inputlen - 1 ] = '\0';
        }
        else if  ( pLoop_count )  /* If a loop count was specified: */
        {
            /*  Generate the text of the next message ("QUIT" if we've
                sent the requested no. of msgs, "MESSAGEn" otherwise).
            */
            if  ( iMsgNum  >  iLoopCount )   sprintf( input, "QUIT" );
            else   sprintf( input, "Message%d", iMsgNum );
            ++ iMsgNum;
        }
        else   /* Neither message file nor loop count specified: */
        {
            /*  Prompt user for text of next message.
            */
            printf( "\nEnter data (\"" QUIT_MSG "\" to finish): " );
            fflush( stdout );
            if  (  !gets( input )  )   strcpy( input, QUIT_MSG ) ;
        }

        /****
        *  Extract the text and (if present) the service name
        *  from the input line.  If the message is not valid,
        *  display informational prompt and get next message.
        *  If a service name was not provided, use the default.
        ****/
        if ( !processInput( input, &pService, &pMessage ) )
        {
            printf( "Enter message in format:"
                    " \"<Service>:<Request>\"\n" );
            continue;
        }

        if ( !pService )   pService = pService_name ;


        /****
        *  Set up the service definition and the message
        ****/
        kxcSetServiceDefault( &Service );
        strncpy ((char *) Service.ServiceName, 
                 (char *) pService, 
                 sizeof(KXCOBJNAME));

        reqMsg.MsgLen = strlen(pMessage);
        kxcSetMsgBuffer( &reqMsg, (sizeof(input) - strlen(pService)),
                         pMessage );
        if ( pMsg_format_name != NULL )
        {
           /* Put in the format name */
           kxcSetMsgFormatName( &reqMsg, pMsg_format_name);
        }

        /****
        *  Send message to server
        ****/
        if (  !kxcPutRequest( hKxc, &Service, &reqMsg, &reqOpt, NULL,
                              &hTheme )  )
        {
            processError( "PutRequest failed.\n" );
            break;
        }

#ifndef DATAGRAM_MODEL
        /****
        *  Look for response and display
        ****/
        printf( "GetResponse " );
        if ( !kxcGetResponse( hKxc, &rspMsg, &rspOpt, NULL, &hTheme ) )
        {
            processError( "Error from GetResponse.\n" );
            break;
        }

        printf( "Received: \"%.*s\"\n",
                rspMsg.MsgLen,  (char*)rspMsg.MsgBuffer.pBuffer );
        /***
        * If the message has a Roma Message Format Name - Print it
        ***/
        if ( kxcGetMsgFormat(&rspMsg) == KXCMSGFORMAT_USENAME )
        {
          KXCOBJNAME FrmtNm;
          kxcGetMsgFormatName(&rspMsg, FrmtNm);
          printf("Received with Message Format Name \"%s\"\n", FrmtNm);
        }

        /* Now free any auto-allocated buffer */
        if (rspMsg.MsgBuffer.Status == KXCBUFSTAT_KXC)
        {
           kxcFreeBuffer(hKxc, &rspMsg.MsgBuffer);
        }
#endif

        /****
        *  If the message text requests termination, or if this is
        *  the last in a sequence of messages to be generated,
        *  indicate no more to do.  If we are reading from a message
        *  file, read the next message and, if none left, indicate
        *  no more to do.
        ****/
        if  ( endSession( pMessage ) )    bMoreToDo = FALSE;
        else if  ( pMsgFILE )
        {
            if  (  !fgets( input, (sizeof input), pMsgFILE )  )
                bMoreToDo = FALSE;
        }
        else if  ( pLoop_count )
        {
            if  ( iMsgNum  >  (iLoopCount + 1) )   bMoreToDo = FALSE;
        }

    }  /****  End of message-processing loop  ****/

    TRACE0( "Message processing complete.\n" );


    /****
    *  Terminate Roma interface and return to caller.
    ****/
    if  ( pMsgFILE != NULL )    fclose( pMsgFILE );

    if  ( !kxcTerminate( &hKxc ) )
    {
        processError( "Error occurred terminating Roma.\n" );
        return  EXIT_FAILURE;
    }

    printf( "Roma Client terminated.\n" );
    return  EXIT_SUCCESS;
}


/********************************************************************/
/*                                                                  */
/* Function:  display_help                                          */
/*                                                                  */
/* Description:                                                     */
/*    Display help text for program.                                */
/*                                                                  */
/* Invocation syntax:                                               */
/*    display_help();                                               */
/*                                                                  */
/* Input Parameters:                                                */
/*    None.                                                         */
/*                                                                  */
/* Return Value:                                                    */
/*    void.                                                         */
/*                                                                  */
/********************************************************************/
void  display_help( void )
{
    printf( "\n" );
    printf( "================================"
            "================================\n" );
    printf( "This program sends the user-specified message(s) to\n"  );
    printf( "an Roma server.  For each message sent (other than\n"   );
    printf( "datagram messages), it retrieves and displays a\n"      );
    printf( "response from the server.  It terminates when it\n"     );
    printf( "encounters a \"QUIT\" message or when the number of\n"  );
    printf( "messages specified by -l has been sent.\n"              );
    printf( "\n"                                                     );
    printf( "Invocation Syntax (all parameters optional):\n"         );
    printf( "   c_client  <-c client_name> <-s service_name> \n"     );
    printf( "             <-m msg_format_name> \n"                   );
    printf( "             <-l loop_count> <-f msg_file_name> <-?>\n" );
    printf( "\n"                                                     );
    printf( "\"client_name\" and \"service_name\" specify "
                                                       "overrides\n" );
    printf( "to the default client and service names - \n"           );
    printf( "(ClientGroup1 and TestService1.) \n"                    );
    printf( "\"msg_format_name\" specifies format name of messages\n");
    printf( "to be send to service\n"                                );
    printf( "\"loop_count\" specifies the number of messages to be\n");
    printf( "generated and transmitted.\n"                           );
    printf( "\"msg_file_name\" specifies the name of a file from\n"  );
    printf( "which the messages to be sent should be read "
                                                 "(overrides -l).\n" );
    printf( "-? requests this help display.\n"                       );
    printf( "If neither -l nor -f is specified, the user is\n"       );
    printf( "prompted to enter message input.\n"                     );
    printf( "================================"
            "================================\n\n" );

    return ;
}



/********************************************************************/
/*                                                                  */
/* Function:  processError                                          */
/*                                                                  */
/* Description:                                                     */
/*    Write the caller-specified error message text (if specified)  */
/*    to stdout, then use tracing macros to conditionally display   */
/*    detail error information based on the last Roma return code.  */
/*                                                                  */
/* Invocation syntax:                                               */
/*    processError( char *pSpecific_msg );                          */
/*                                                                  */
/* Input Parameters:                                                */
/*    pSpecific_msg:                                                */
/*       Pointer to text to be displayed before the messages        */
/*       describing the last Roma error.  Optional; specify NULL    */
/*       if no preliminary message is to be displayed.              */
/*                                                                  */
/* Return Value:                                                    */
/*    void.                                                         */
/*                                                                  */
/********************************************************************/
void  processError( char  *pSpecific_msg )
{
    KXCERROR  Err;
    KXCULONG  BufLen=400;
    char      Buffer[400];

    if  ( pSpecific_msg )   printf( pSpecific_msg );

    if  ( kxcGetLastError( &Err ) )
    {
        kxcFormatError(&Err.Result, KXCERROR_FULL,
                       Buffer, &BufLen);
        TRACE1( "Error    = %s\n",  Buffer );

        if  ( Err.Status & KXCERRSTAT_SUB )
        {
            BufLen=400;
            kxcFormatError(&Err.SubResult, KXCERROR_FULL,
                           Buffer, &BufLen);
            TRACE1( "SubError = %s\n",  Buffer );
        }
    }
    else
        TRACE0( "Unable to access error information.\n" );

    return ;
}



/********************************************************************/
/*                                                                  */
/* Function:  endSession                                            */
/*                                                                  */
/* Description:                                                     */
/*    Checks the caller-specified message for "end of session"      */
/*    indicator (QUIT_MSG).                                         */
/*                                                                  */
/* Invocation syntax:                                               */
/*    bool  = endSession( const char *pMsg );                       */
/*                                                                  */
/* Input Parameters:                                                */
/*    pMsg:                                                         */
/*       Pointer to message text to be examined.                    */
/*                                                                  */
/* Return Value:                                                    */
/*    An int value:  1 if the "end of session" indicator was        */
/*    found in the message, 0 otherwise.                            */
/*                                                                  */
/********************************************************************/

bool endSession( const char *pMsg )
{
  char uppercase_msg[ QML + 1 ];            /* initialize buffer    */

  if ( !pMsg )                              /* no message received  */
    return false;                           /*  can not process     */

  allCaps( strncpy( uppercase_msg,          /* capitalize the text  */
            (char * ) pMsg, QML ) );

  return ( !strncmp( uppercase_msg,         /* don't want to quit   */
             QUIT_MSG, QML ) );             /* back to caller       */
 }


/********************************************************************/
/*                                                                  */
/* Function:  parse_cmdline                                         */
/*                                                                  */
/* Description:                                                     */
/*    Validates command-line arguments and returns the arguments    */
/*    in the corresponding caller-specified return areas.           */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int =  parse_cmdline( int     argc,                           */
/*                          char   *argv[],                         */
/*                          char  **ppClient,                       */
/*                          char  **ppService,                      */
/*                          char  **ppLoopcount,                    */
/*                          char  **ppMsgfilename,                  */
/*                          char  **ppMsgformatname,                */
/*                          int    *pHelp                           */
/*                        );                                        */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments.                           */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/*    ppClient:                                                     */
/*       Pointer to area in which the address of the client         */
/*       name argument is to be returned.  The area is set to       */
/*       NULL if the client name was omitted.                       */
/*                                                                  */
/*    ppService:                                                    */
/*       Pointer to area in which the address of the business       */
/*       service name argument is to be returned.  The area is set  */
/*       to NULL if the service name was omitted.                   */
/*                                                                  */
/*    ppLoopcount:                                                  */
/*       Pointer to area in which the address of the loopcount      */
/*       argument is to be returned.  The area is set to NULL if    */
/*       the loopcount was omitted.                                 */
/*                                                                  */
/*    ppMsgfilename:                                                */
/*       Pointer to area in which the address of the message file   */
/*       name argument is to be returned.  The area is set to       */
/*       NULL if the message file name was omitted.                 */
/*                                                                  */
/*    ppMsgformatname:                                              */
/*       Pointer to area in which the address of the message format */
/*       name argument is to be returned.  The area is set to       */
/*       NULL if the message format name was omitted.               */
/*                                                                  */
/*    pHelp:                                                        */
/*       Pointer to help indicator.  If help was requested,         */
/*       this indicator is set to 1; otherwise, it is set to 0.     */
/*                                                                  */
/* Return Value:                                                    */
/*    1 if the command line arguments were successfully parsed,     */
/*    0 otherwise.                                                  */
/*                                                                  */
/********************************************************************/
int   parse_cmdline( int     argc,
                     char   *argv[],
                     char  **ppClient,
                     char  **ppService,
                     char  **ppLoopcount,
                     char  **ppMsgfilename,
                     char  **ppMsgformatname,
                     int    *pHelp  )
{
    int    c      = 0;  /* Work area for option code */

    int    status = 1;  /* Status code (return value), initialized */
                        /*    to indicate successful completion    */

    char  *pOptArg   ;  /* Pointer to option value                 */


    /*
       Validate return area arguments.
    */
    ASSERT( ppClient      != NULL);
    ASSERT( ppService     != NULL);
    ASSERT( ppLoopcount   != NULL);
    ASSERT( ppMsgfilename != NULL);
    ASSERT( ppMsgformatname != NULL);
    ASSERT( pHelp         != NULL);


    /*
       Initialize return areas.
    */
    *ppClient      = NULL ;
    *ppService     = NULL ;
    *ppLoopcount   = NULL ;
    *ppMsgfilename = NULL ;
    *ppMsgformatname = NULL ;
    *pHelp         = 0    ;


    /*
       Return immediately if there are no command-line arguments
       following the program name.
    */
    if  ( argc == 1 )   return 1;


    /*
       Loop through command-line arguments, copying the argument
       pointers to the corresponding return areas.

       Add your own option flags and cases to the loop below.
    */
    while  (  ( c = getOpt( argc, argv, &pOptArg ) )  > 0  )
    {
        switch ( c )
        {
            case 'C':     /* Client name */
            case 'c':
                *ppClient = pOptArg;
                break;

            case 'S':     /* Service name */
            case 's':
                *ppService = pOptArg;
                break;

            case 'L':     /* Loop count */
            case 'l':
                *ppLoopcount = pOptArg;
                break;

            case 'F':     /* Message file name */
            case 'f':
                *ppMsgfilename = pOptArg;
                break;

            case 'M':     /* Message format name */
            case 'm':
                *ppMsgformatname = pOptArg;
                break;

            case '?':     /* Help request */
                *pHelp = 1;
                break;

            default:
                fprintf( stderr, "Unknown option, %c.\n", c );
                status = 0;
                break;
        }
    }


    /*
       If there was an error, display informational message.
    */
    if  ( c < 0 )  status = 0;

    if  ( status == 0 )
        fprintf( stderr,
                 "Usage: %s  -c client_name  -s service_name"
                          "  -m message_format_name  \n",
                          "  -l loop_count  -f msgfile_name  -?\n",
                 argv[0] ) ;

    return  status;   /* Return status code to caller. */
}


/********************************************************************/
/*                                                                  */
/* Function:  split                                                 */
/*                                                                  */
/* Description:                                                     */
/*    Parse the caller-specified string into tokens.                */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int = split( char *pStream,                                   */
/*                 char *pTok[],                                    */
/*                 int   token_limit,                               */
/*                 char *pSeparators );                             */
/*                                                                  */
/* Input Parameters:                                                */
/*    pStream:                                                      */
/*       Pointer to the string to be parsed.                        */
/*                                                                  */
/*    pTok:                                                         */
/*       Pointer to array of token pointers in which the            */
/*       addresses of the tokens found are to be returned.          */
/*                                                                  */
/*    token_limit:                                                  */
/*       The maximum number of tokens expected (the number          */
/*       of elements in the pTok array).                            */
/*                                                                  */
/*    pSeparators:                                                  */
/*       Pointer to null-terminated string contaning a list         */
/*       of characters to be recognized as token delimiters.        */
/*                                                                  */
/* Return Value:                                                    */
/*    The integral count of tokens found.                           */
/*                                                                  */
/********************************************************************/

int  split( char  *pStream,
            char  *pTok[],
            int    token_limit,
            char  *pSeparators )
{
    int    count  = 0     ;  /* Token count   */
    char  *pToken = NULL  ;  /* Token pointer */

    /*
       Return 0 immediately if invalid arguments were passed.
    */
    if  ( (pTok == NULL)  ||  (token_limit < 1) )    return 0;

    /*
       Loop:  Retrieve tokens until none left or limit reached.
    */
    pToken = strtok( pStream, pSeparators );  /* Get first token. */

    while  ( (pToken != NULL)  &&  (count < token_limit) )
    {
        pTok[count ++] = pToken;
        pToken = strtok( NULL, pSeparators );
    }

    return  count;  /* Return count of tokens found to caller. */
}


/********************************************************************/
/*                                                                  */
/* Function:  processInput                                          */
/*                                                                  */
/* Description:                                                     */
/*    Extract the message text and (if present) the service name    */
/*    from the caller-specified input string, and return pointers to*/
/*    the text and service name in the caller-specified areas.      */
/*                                                                  */
/*    If a service name is not present in the caller-specified      */
/*    input string, NULL is returned (this is not considered an     */
/*    error).                                                       */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int = processInput( char   *pInput,                           */
/*                        char  **ppService,                        */
/*                        char  **ppMessage );                      */
/*                                                                  */
/* Input Parameters:                                                */
/*    pInput:                                                       */
/*       Pointer to the text to be parsed.                          */
/*                                                                  */
/*    ppService:                                                    */
/*       Pointer to area in which to return the address of the      */
/*       service name.  If a service name is not present in the     */
/*       text specified by argument input, NULL is returned.        */
/*                                                                  */
/*    ppMessage:                                                    */
/*       Pointer to area in which to return the address of the      */
/*       message text.                                              */
/*                                                                  */
/* Return Value:                                                    */
/*    An int value:  1 for successful completion, 0 if an error     */
/*    was encountered.                                              */
/*                                                                  */
/********************************************************************/
#define  MAX_TOK  2

int  processInput( char *pInput, char **ppService, char **ppMessage )
{
    char    *pTok[MAX_TOK] ;  /* Array of token pointers */
    int      ntoks = 0     ;  /* Count of tokens found   */

    ASSERT( ppService != NULL); /* Ensure service pointer passed. */
    ASSERT( ppMessage != NULL); /* Ensure message pointer passed. */


    /*
       Parse the caller-specified string into tokens representing
       the service name and message text.  If only one token is
       present, treat it as the message text.
    */
    if  (  ( ntoks = split( pInput, pTok, MAX_TOK, ":\n" ) )  == 1 )
    {
        pTok[1] = pTok[0];
        pTok[0] = NULL   ;
    }
    else if ( ntoks != 2 )     return 0;  /* Excessive tokens */


    /*
       pTok[0] and pTok[1] now point to the service name and message,
       respectively.  Copy token pointers to caller's return areas
       and return.
    */
    if  ( pTok[0] )
    {
        TRACE2( "Service:  <%.*s>\n",  strlen( pTok[0] ),  pTok[0] );
    }

    TRACE2( "Message:  \"%.*s\"\n",  strlen( pTok[1] ),  pTok[1] );

    *ppService = pTok[0];
    *ppMessage = pTok[1];

    return 1;
}



/********************************************************************/
/*                                                                  */
/* Function:  allCaps                                               */
/*                                                                  */
/* Description:                                                     */
/*    Convert the lower-case characters in the caller-specified     */
/*    string to upper-case.                                         */
/*                                                                  */
/* Invocation syntax:                                               */
/*    char *allCaps( char *ps ) ;                                   */
/*                                                                  */
/* Input Parameters:                                                */
/*    ps:                                                           */
/*       Pointer to the null-terminated string to be converted.     */
/*                                                                  */
/* Return Value:                                                    */
/*    Pointer to the input argument (the modified string).          */
/*                                                                  */
/********************************************************************/
char  *allCaps( char *ps )
{
    char *cp;

    for  ( cp = ps;  *cp;  cp ++ )
    {
        if  ( islower( *cp ) )      *cp = toupper( *cp );
    }

    return ps;
}


/********************************************************************/
/*                                                                  */
/* Function:  getOpt                                                */
/*                                                                  */
/* Description:                                                     */
/*    Returns the next option name and (if present) option value    */
/*    from the command-line parameters.                             */
/*                                                                  */
/*    This function searches for option specifications observing    */
/*    the following format:                                         */
/*                                                                  */
/*        -o value                                                  */
/*                                                                  */
/*    where "o" is a one-byte option name and "value" is an optional*/
/*    character string value assigned to "o".                       */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int = getOpt( int argc,  char   *argv[], char **ppOptArg );   */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments.                           */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/*    ppOptArg:                                                     */
/*       Pointer to area in which to return the address of the      */
/*       option value if one is present.                            */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral values:                         */
/*       -1    -- Invalid command-line argument detected.           */
/*        0    -- No more named options.                            */
/*       Other -- The name of the next option.  If a value was      */
/*                assigned to this name, a pointer to the value is  */
/*                returned in *ppOptArg; else *ppOptArg set to NULL.*/
/*                                                                  */
/********************************************************************/
int  getOpt( int  argc,  char  *argv[],  char  **ppOptArg )
{
    static int   index = 1 ; /* Index to current entry in argv */
    char        *pc        ; /* Pointer to current argv entry  */
    char         optname   ; /* Name of option                 */

    *ppOptArg = NULL ;  /* Init. return argument value to NULL.*/


    /*
       Increment argument index until we find an argument prefixed by
       a dash or until we run out of arguments.
    */
    for  ( pc = argv[index];
           (index < argc ) && (pc[0] != '-');
           ++ index, pc = argv[index]
         ) ;

    /*
       Return indicating no more options if the current argument
       index points beyond the number of argv entries.  Return
       indicating error if the length of the current argument
       is less than 2.
    */
    if  ( index >= argc )   return  0 ;

    if  ( strlen( argv[index] ) < 2 )
    {
        printf( "Invalid argument, \"%s\".\n", argv[index] );
        ++ index  ;
        return -1 ;
    }


    /*
       Save option name, increment argument index.
    */
    optname = pc[1] ;
    ++ index        ;


    /*
       If the option has trailing text return it as the value
    */
    if (pc[2] != '\0')
    {
      *ppOptArg = &pc[2];
      return optname;
    }

    /*
       If the next entry is prefixed by '-' or '/', or if there
       are no additional arguments, return the current option name
       without an accompanying option value.  Otherwise, return
       the next argument as the option value and increment the index
       again.
    */
    if  ( index >= argc )   return  optname;

    pc = argv[index];
    if  ( pc[0] == '-' )    return optname;

    *ppOptArg = pc ;
    ++ index       ;
    return  optname;
}
