/********************************************************************/
/*                                                                  */
/* Module      : cppClnt.cpp                                        */
/*                                                                  */
/* Description : Sample Roma C++ Client program                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998,99,2000,2002                   */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Description:                                                     */
/*    Sample Roma Client Program.                                   */
/*                                                                  */
/*    This program shows a very simple Roma client in action. It    */
/*    is intended to show representative client usage of the Roma   */
/*    C++ API in a simple situation. The program itself will        */
/*    work with the corresponding server samples provided with the  */
/*    C++, Java and ActiveX Roma language bindings.                 */
/*                                                                  */
/* Details:                                                         */
/*    The program sends a predefined request message to a Roma      */
/*    Business Service, waits for up to 30 seconds for a response   */
/*    and then terminates, displaying the response if received.     */
/*                                                                  */
/*    If no command line flags are specified the program uses the   */
/*    directory definitions for ClientGroup1 and TestService1,      */
/*    which are defined in the Roma directory after installation.   */
/*                                                                  */
/*    These names may be overridden as follows.                     */
/*                                                                  */
/* Invocation Syntax:                                               */
/*    cppclnt   [ -c client_name ]  [ -s service_name  ]            */
/*              [ -?             ]                                  */
/*                                                                  */
/* Input Parameters (all optional):                                 */
/*    -c client_name:                                               */
/*       Specifies the client name to use instead of the default    */
/*       (ClientGroup1).  Substitute a valid client name for        */
/*       "client_name".                                             */
/*                                                                  */
/*    -s service_name:                                              */
/*       Specifies the business service name to use instead of the  */
/*       default (TestService1).  Substitute a valid business       */
/*       service name for "service_name".                           */
/*                                                                  */
/*    -? print a simple usage message                               */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral codes:                          */
/*       EXIT_SUCCESS -- Successful completion.                     */
/*       EXIT_FAILURE -- Unsuccessful completion.                   */
/*                                                                  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include <kxcocl.hpp>


// HELPER FUNCTIONS (defined at the end of this sample)
//
void checkError(RomaClient &cl);
void printn(const void *vp, int n);
bool parse_cmdline(int ac, char* av[], char** client, char** service);


int
main(int argc, char *argv[])
{
  int status = EXIT_SUCCESS;

  char *clientName  = (char *)"ClientGroup1";
  char *serviceName = (char *)"TestService1";

  if (!parse_cmdline(argc, argv, &clientName, &serviceName))
    return EXIT_FAILURE;

  // ------------------------------------------------------------------------
  // ROMA INITIALIZATION
  // ------------------------------------------------------------------------
  printf("Initialize client <%s>: ", clientName);
  RomaClient cl(clientName);

  if (cl.anyError() ) {
    printf("failed\n");
    checkError(cl);
    return EXIT_FAILURE;
  }
  printf("success\n");


  // ------------------------------------------------------------------------
  // SET UP THE REQUEST
  //  Accept defaults for the request options. This includes using
  //  Roma themes. The Theme will be retained for use in the response.
  //
  printf("Send request to service <%s>: ",serviceName);
  RomaClientRequest req(serviceName);
  // ------------------------------------------------------------------------
  // Note: the sample server will stop after QUIT. If you change this
  //       message text, the server will wait for further messages.
  req.setMsgBuffer("QUIT", 4);


  // ------------------------------------------------------------------------
  // SEND THE REQUEST
  // ------------------------------------------------------------------------
  if ( !cl.put(req) ) {
    printf("failed\n");
    checkError(cl);
    return EXIT_FAILURE;
  }
  printf("success\n");


  // ------------------------------------------------------------------------
  // WAIT FOR THE RESPONSE
  //  We accept the default get options built from the request. This
  //  will include selecting the response by Roma theme, which we
  //  then explicity close as we have finished with it and want the
  //  resources freed.
  // ------------------------------------------------------------------------
  RomaGetOptions   go(req);
  go.setWaitInterval(30000);
  go.setCloseTheme(true);

  printf("Waiting for response (timeout is 30 seconds): ");
  RomaClientResponse* rsp = cl.get(go);


  // ------------------------------------------------------------------------
  // CHECK RESULT AND TERMINATE
  // ------------------------------------------------------------------------

  if ( !rsp || cl.anyError() ) {
    printf("timed out\n");
    checkError(cl);
    status = EXIT_FAILURE;
  }
  else {
    printf("success\n");
    printf("Response message is:  ");
    printn(rsp->msgBuffer(),rsp->msgDataLength());

    delete rsp;
  }

  printf("Client finished\n");
  return status;
}


// HELPER FUNCTIONS (IMPLEMENTATION)
// These helper functions are provided so that distracting detail can
// be removed from the main sample code (above).
//
// The functions are not intended to be flexible and robust routines
// for general purpose use.
//
void checkError(RomaClient &cl)
{
  if (cl.anyError()) {
    RomaError e = cl.error();
    printf("Error (Primary):   %s\n",e.errorText());

    bool extraInformation = (e.status() == Roma::STATUS_BOTH);
    printf("Error (Secondary): %s\n",
           extraInformation ? e.errorText(Roma::SECONDARY)
                            : "<None>");
  }
}

void printn(const void *vp, int n)
{
  char *cp = (char *)vp;

  if   (!cp) printf("<nul>\n");
  else {
    printf("<");
    for (int i = 0; i < n ; i++)
      printf("%c",cp[i]);
    printf(">\n");
  }
}

bool parse_cmdline( int     ac,
                    char   *av[],
                    char  **ppClient,
                    char  **ppService)
{
  if (!ppClient || !ppService )
    return false;

  // check if there are any command line arguments to be checked
  if  (ac == 1)
    return true;

  //  Loop through command-line arguments, checking for known flags
  bool status = true;
  int  usage_required = 0;

  for (int p = 1; p < ac; p++) {

    char *flag = av[p];
    bool found = false;

    if (flag[0] != '-')
      found          = false;
    else
    if (flag[1] == '?') {                                  // -?
      usage_required++;
      found          = true;
      status         = false;
    }
    else
    if (flag[1] == 'c') {                                  // -c
      found = (flag[2]  && (*ppClient  = &flag[2]) ||
               ++p < ac && (*ppClient  = av[p] ) );
    }
    if (flag[1] == 's') {                                  // -s
      found = (flag[2]  && (*ppService = &flag[2]) ||
               ++p < ac && (*ppService = av[p] ) );
    }

    if (!found) usage_required++;

    if (usage_required == 1) {
      usage_required++;
      fprintf( stderr,
               "Usage: %s  -c client_name  -s service_name -?\n",
                av[0] ) ;
    }

    if (!found) {
      fprintf( stderr, "Invalid flag or missing value: <%s>\n",flag);
      status = false;
      break;
    }
  }

  return status;
}
