/********************************************************************/
/*                                                                  */
/* Module      : cppsrvr.cpp                                        */
/*                                                                  */
/* Description : Sample Roma C++ Server program                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998,99,2000,2002                   */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Details:                                                         */
/*    Sample Roma Server Program.                                   */
/*                                                                  */
/*    This program waits for up to 2 minutes for a request message  */
/*    from a Roma client. If the request is received, a predefined  */
/*    response message is returned to the client. The program       */
/*    continues to wait for further responses until a QUIT message  */
/*    is received or until the 2 minute timeout is exceeded.        */
/*                                                                  */
/*    If no command line flags are specified the program uses the   */
/*    directory definition for TestElement1 as defined in the Roma  */
/*    directory after installation.                                 */
/*                                                                  */
/*    These names may be overridden as follows.                     */
/*                                                                  */
/* Invocation Syntax:                                               */
/*    cppsrvr   [ -e element_name ] [ -? ]                          */
/*                                                                  */
/* Input Parameters (all optional):                                 */
/*    -e element_name                                               */
/*       Specifies the Roma Business Element name to use instead    */
/*       of the default (ClientGroup1). Substitute a valid element  */
/*       name for "element_name"                                    */
/*                                                                  */
/*    -? print a simple usage message                               */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral codes:                          */
/*       EXIT_SUCCESS -- Successful completion.                     */
/*       EXIT_FAILURE -- Unsuccessful completion.                   */
/*                                                                  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <kxcobe.hpp>


// HELPER FUNCTIONS (defined at the end of this sample)
//
void checkError(RomaBusinessElement &be);
void printn(const void *vp, int n);
bool parse_cmdline(int ac, char* av[], char** element);


int main(int argc, char *argv[])
{
  int status = EXIT_SUCCESS;

  char *elementName  = (char *)"TestElement1";

  if (!parse_cmdline(argc, argv, &elementName))
    return EXIT_FAILURE;

  // ------------------------------------------------------------------------
  // ROMA INITIALIZATION
  // ------------------------------------------------------------------------
  printf("Initialize business element <%s>: ", elementName);
  RomaBusinessElement be(elementName);

  if ( be.anyError() ) {
    printf("failed\n");
    checkError(be);
    return EXIT_FAILURE;
  }
  printf("success\n");


  bool done = false;

  // loop until a message containing QUIT is received, responding to all
  // request messages.
  while (!done) {

    // ---------------------------------------------------------------------
    // ROMA OPTIONS (PRE GETREQUEST)
    // ---------------------------------------------------------------------
    // server will wait for a request for up to 120 seconds ...
    RomaGetOptions go;
    go.setWaitInterval(120000);

    // ---------------------------------------------------------------------
    // ROMA GETREQUEST
    // ---------------------------------------------------------------------
    printf("Waiting for request (timeout is 2 minutes): ");
    RomaBeRequest* req = be.get(go);

    if ( !req || be.anyError() ) {
      printf("timed out\n");
      checkError(be);
      return EXIT_FAILURE;
    }
    printf("success\n");

    // pick up data from request message ...
    printf("Request message is:  ");
    printn(req->msgBuffer(),req->msgDataLength());

    done = !memcmp(req->msgBuffer(), "QUIT", 4);


    // ---------------------------------------------------------------------
    // ROMA OPTIONS (PRE PUTRESPONSE)
    // ---------------------------------------------------------------------
    const int bufferLen = (req->msgDataLength() + 64);
    char * buffer = new char[bufferLen];
    sprintf(buffer, "RESPONSE to Client's <%s>",
            (char *)req->msgBuffer());

    // construct response message, based on request (which can then be deleted)
    RomaBeResponse rsp(req);
    rsp.setMsgBuffer(buffer, strlen(buffer));

    // ---------------------------------------------------------------------
    // ROMA PUTRESPONSE
    // ---------------------------------------------------------------------
    printf("Send response to originating client: ");
    if ( !be.put(rsp) ) {
      printf("failed\n");
      checkError(be);
      delete [] buffer;
      return EXIT_FAILURE;
    }

    printf("success\n");
    delete [] buffer;
    delete req;

  }

  // ----------------------------------------------------------------------
  // ROMA DISCONNECT (automatic on destruction of RomaBusinessElement)
  // ----------------------------------------------------------------------
  printf("Business Element finished\n");
  return EXIT_SUCCESS;
}


// HELPER FUNCTIONS (IMPLEMENTATION)
// These helper functions are provided so that distracting detail can
// be removed from the main sample code (above).
//
// The functions are not intended to be flexible and robust routines
// for general purpose use.
//
void checkError(RomaBusinessElement &be)
{
  if (be.anyError()) {
    RomaError e = be.error();
    printf("Error (Primary):   %s\n",e.errorText());

    bool extraInformation = (e.status() == Roma::STATUS_BOTH);
    printf("Error (Secondary): %s\n",
           extraInformation ? e.errorText(Roma::SECONDARY)
                            : "<None>");
  }
}

void printn(const void *vp, int n)
{
  char *cp = (char *)vp;

  if   (!cp) printf("<nul>\n");
  else {
    printf("<");
    for (int i = 0; i < n ; i++)
      printf("%c",cp[i]);
    printf(">\n");
  }
}

bool parse_cmdline( int     ac,
                    char   *av[],
                    char  **ppElement)
{
  if (!ppElement)
    return false;

  // check if there are any command line arguments to be checked
  if  (ac == 1)
    return true;

  //  Loop through command-line arguments, checking for known flags
  bool status = true;
  int  usage_required = 0;

  for (int p = 1; p < ac; p++) {
    bool found = false;

    char *flag = av[p];

    if (flag[0] != '-')
      found          = false;
    else
    if (flag[1] == '?') {                                  // -?
      usage_required++;
      found          = true;
      status         = false;
    }
    if (flag[1] == 'e') {                                  // -e
      found = (flag[2]  && (*ppElement = &flag[2]) ||
               ++p < ac && (*ppElement = av[p] ) );
    }

    if (!found) usage_required++;

    if (usage_required == 1) {
      usage_required++;
      fprintf( stderr,
               "Usage: %s  -e element_name -?\n",
                av[0] ) ;
    }

    if (!found) {
      fprintf( stderr, "Invalid flag or missing value: <%s>\n",flag);
      status = false;
      break;
    }
  }

  return status;
}
