/********************************************************************/
/*                                                                  */
/* Module      : KXCEMS.C                                           */
/*                                                                  */
/* Description : Sample EMS Server                                  */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998-2004                           */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Details:                                                         */
/*    Sample Roma EMS Server Program.                               */
/*                                                                  */
/*    This program reads EMS messages from its input queue, and     */
/*    processes them according to command-line options.             */
/*    It can browse and display the contents of the EMS messages,   */
/*    or it can modify the user-message data and return the message */
/*    back onto the queue on which the reported operation failed.   */
/*                                                                  */
/*    So, for example, if a Roma server application fails to GET    */
/*    a message from it's input queue (for whatever reason), and    */
/*    Error Profiling is configured to cause an EMS message to be   */
/*    sent to this EMS server, then the EMS server can modify the   */
/*    message data and return the message back to the Roma server's */
/*    input queue to allow it to re-attempt the GET operation.      */
/*                                                                  */
/*    If a PUT operation to a specific queue fails, and an EMS      */
/*    message is set to this EMS server, then it will again replace */
/*    the message data and attempt to PUT the message onto the      */
/*    original target queue again.                                  */
/*                                                                  */
/*                                                                  */
/* Dependencies:                                                    */
/*    Requires the relevant services to be set up in the directory  */
/*    and the underlying transport queues to be configured in       */
/*    support of the routes defined for the directory entry.        */
/*    Error Profiling needs to be configured to have EMS as the     */
/*    required action, and the Busines Service representing this    */
/*    EMS server to be the EMSServer that is used.                  */
/*                                                                  */
/*                                                                  */
/* Invocation Syntax:                                               */
/*    kxcems  [ -e Element_Name ] [ -a Action ] [ -v ]              */
/*            [ -? ]                                                */
/*                                                                  */
/* Input Parameters (all optional):                                 */
/*    -e Element_Name:                                              */
/*       Specifies the Roma Business Element name that this EMS     */
/*       Server will initialise as.  Default is "TestElementEMS".   */
/*                                                                  */
/*    -a Action:                                                    */
/*       Action can be "d" (DISPLAY) or "r" (RETURN).  The default  */
/*       is DISPLAY.  NOTE: when DISPLAY is used, the EMS server    */
/*       will wait if there are no messages on its input queue      */
/*       until a message arrives.                                   */
/*                                                                  */
/*    -v:                                                           */
/*       Turns on "verbose" mode.  Default is off.                  */
/*                                                                  */
/*                                                                  */
/* To build this sample application, modify the appropriate         */
/* makefile for your platform and then issue:                       */
/*                                                                  */
/*   On Unix -   make -f mksample.mak                               */
/*   On NT/95/98 nmake -f mksample.mak                              */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include standard compiler includes                               */
/********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/********************************************************************/
/* Include our own header files                                     */
/********************************************************************/
#include <kxc.h>
#include <kxcutl.h>
#include <kxcxit.h>

/********************************************************************/
/* Function prototypes                                              */
/********************************************************************/
KXCVOID EMSRunServ(KXCULONG    Action,
                   PKXCOBJNAME ElementName);

KXCVOID EMSDisplayMsg(HKXC        hKxc,
                      PKXCMSG     pMsg,
                      PKXCMSGINFO pMsgInfo);

KXCVOID EMSDisplayException(HKXC          hKxc,
                            PKXCEXCEPTION pException);

KXCVOID EMSDisplayWorkFlow(HKXC       hKxc,
                           PKXCBUFFER pBuffer);

KXCVOID EMSDisplayWorkFlowVer4(PKXCWFLOW_4 pWorkflow);

KXCVOID EMSDisplayData(HKXC    hKxc,
                       PKXCMSG pMsg);

KXCVOID EMSHexFormatData(HKXC     hKxc,
                         PKXCVOID pData,
                         KXCULONG DataLen);

KXCVOID EMSReturnMsg(HKXC        hKxc,
                     PKXCMSG     pEMSMsg,
                     PKXCMSGINFO pMsgInfo,
                     PKXCSYNC    pSync);

KXCBOOL EMSRebuildPutTheme(HKXC        hKxc, 
                           PKXCMSGINFO pMsgInfo,
                           PHKXCTHEME  phTheme,
                           PKXCMSGID   pThemeId,
                           PKXCULONG   pRetry);

KXCBOOL EMSRebuildGetTheme(HKXC        hKxc, 
                           PKXCMSGINFO pMsgInfo,
                           PHKXCTHEME  phTheme,
                           PKXCMSGID   pThemeId,
                           PKXCULONG   pRetry);

KXCBOOL EMSRebuildGetWorkflow(HKXC           hKxc,
                              PKXCEXCEPTION  pException,
                              PKXCWFLOW      pWorkflow,
                              PKXCWFLOW     *ppNewFlow);

KXCBOOL EMSRebuildGetWorkflowVer4(HKXC           hKxc,
                                  PKXCEXCEPTION  pException,
                                  PKXCWFLOW_4    pWorkflow,
                                  PKXCWFLOW     *ppNewFlow);

KXCBOOL EMSNormalizeWorkFlow(PKXCWFLOW pWFlow);

KXCBOOL EMSNormalizeWorkFlowVer4(PKXCWFLOW_4 pWFlow);

KXCVOID WriteError(HKXC  hKXC,
                   char *comment);

int   getOpt( int  argc,  
              char  *argv[],  
              char  **ppOptArg );

/********************************************************************/
/* Local data                                                       */
/********************************************************************/
#define BUFFER_SIZE  200
#define RETRY_MAX    5
#define ACT_NONE     0
#define ACT_DISPLAY  1
#define ACT_RETURN   2

static char ModifiedBuffer[BUFFER_SIZE] = "MESSAGE-MODIFIED-BY-EMS";

KXCBOOL bVerbose = FALSE;

/********************************************************************/
/* And now we can define the code                                   */
/********************************************************************/

/********************************************************************/
/* Function:  main                                                  */
/* Purpose:   Entry Point routine for EMS server                    */
/* Input:     argc(int)                                             */
/*            - Number of arguments                                 */
/*            argv(char **)                                         */
/*            - Array of arguments                                  */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
int main(int argc, char **argv)
{
KXCULONG    Action        = ACT_DISPLAY;
KXCOBJNAME  ElementName   = "TestElementEMS";
int         c             = 0;         /* Work area for option code */
char        *pOptArg      = NULL;      /* Pointer to option value   */
int         status        = 1;

  /******************************************************************/
  /* Stop stderr from being buffered                                */
  /******************************************************************/
  setvbuf(stderr, NULL, _IONBF, 0);

  /******************************************************************/
  /* Process the arguments                                          */
  /******************************************************************/

  while (( c = getOpt( argc, argv, &pOptArg ))  > 0)
  {
    switch ( c )
    {
    case 'E':     /* Name to initialise as */
    case 'e':
                if (!pOptArg)
                {
                  status = 0;
                  break;
                }
                strcpy((char *)ElementName,pOptArg);
                break;
    case 'V':
    case 'v':
                bVerbose = TRUE;
                break;
    case 'A':
    case 'a':   if (!pOptArg)
                {
                  status = 0;
                  break;
                }
                  
                switch(*pOptArg)
                {
                  case 'R':
                  case 'r':
                            Action = ACT_RETURN;
                            break;
                  case 'D':
                  case 'd':
                            Action = ACT_DISPLAY;
                            break;
                  default:
                            Action = ACT_NONE;
                            status = 0;
                            break;
                }
                break;

    default:
                status = 0;
                break;
    }
  }
  
  if  ( status == 0 )
  {
    fprintf( stderr,"Sample EMS Server\n\n");
    fprintf( stderr, 
        "Usage: kxcems %s\n\n",
        "[-e element_name ] [ -v ] [-a Action]");

    fprintf( stderr, "  element_name = Name to initialise as\n");
    fprintf( stderr, "  v            = Verbose\n");
    fprintf( stderr, "  Action       = Action to perform\n");
    fprintf( stderr, "                 %s\n", 
                      "d -> Display EMS msg contents");
    fprintf( stderr, "                 %s\n\n", 
                      "r -> Return modified msg to orig queue");

    exit(1);
  }



  /******************************************************************/
  /* Go run the EMS Server if we've got something to do             */
  /******************************************************************/
  if (Action != ACT_NONE)
    EMSRunServ(Action, &ElementName);

  if (bVerbose)
    printf("\nEMS Server ending\n\n");

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return(0);
}

/********************************************************************/
/* Function:  EMSRunServ                                            */
/* Purpose:   Top Level routine for EMS server                      */
/* Input:     Action(KXCULONG)                                      */
/*            - Action to take for the EMS server                   */
/*            ElementName(PKXCOBJNAME)                              */
/*            - Name to initialise EMS server as                    */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSRunServ(KXCULONG    Action,
                   PKXCOBJNAME ElementName)
{
KXCBOOL               bSuccess = TRUE;
KXCBOOL               bEndLoop = FALSE;
KXCULONG              rc;
KXCERROR              Error;
KXCAPPDEF             AppDef;
HKXC                  hKxc     = NULL_HKXC;
KXCSYNC               Sync;
PKXCSYNC              pSync    = NULL;
KXCMSG                Msg;
KXCGETOPT             GetOpts;
KXCMSGINFO            MsgInfo  = { KXCMSGINFO_DEFAULT };

  /******************************************************************/
  /* Trace details on verbose                                       */
  /******************************************************************/
  if (bVerbose)
  {
    printf("\nInitialising EMS Server - [%s]", ElementName);
    printf("\nAction is [%s]\n",
      ((Action == ACT_DISPLAY) ? "DISPLAY" :
       ((Action == ACT_RETURN) ? "RETURN" : "NONE" )) );
  };

  /******************************************************************/
  /* Initialize Application definition                              */
  /******************************************************************/
  kxcSetAppDefDefault(&AppDef);
  strncpy ((char *) AppDef.ElementName, 
           (char *) ElementName,
           sizeof (KXCOBJNAME));

  /******************************************************************/
  /* Initialize the server                                          */
  /******************************************************************/
  bSuccess = kxcServerInitialize(&AppDef, &hKxc);
  if (!bSuccess)
  {
    WriteError(hKxc, "Failed to initialize Server");
  };

  /******************************************************************/
  /* If we initialised OK                                           */
  /******************************************************************/
  if (bSuccess)
  {
    /****************************************************************/
    /* Build Sync options                                           */
    /****************************************************************/
    pSync = &Sync;
    kxcSetSyncDefault(pSync);
    Sync.TransactionType = KXCSYNC_LOCAL;

    /****************************************************************/
    /* Set message definition                                       */
    /****************************************************************/
    kxcSetMsgDefault(&Msg);

    /****************************************************************/
    /* Build Receive options                                        */
    /****************************************************************/
    kxcSetGetOptDefault(&GetOpts);
    GetOpts.ReceiveOptions |= ( KXCGOPT_ALLOWNATIVE |
                                KXCGOPT_CLOSETHEME  |
                                KXCGOPT_AUTO_ALLOC  |
                                KXCGOPT_NOEMS);


    /****************************************************************/
    /* If we're just displaying the message info                    */
    /****************************************************************/
    if (Action == ACT_DISPLAY)
    {
      /**************************************************************/
      /* Set options for a non-destructive BROWSE operation         */
      /**************************************************************/
      GetOpts.ReceiveOptions |= KXCGOPT_PEEK_FIRST;

      /**************************************************************/
      /* Ensure that we don't PEEK in syncpoint                     */
      /**************************************************************/
      pSync = NULL;
    };

    /****************************************************************/
    /* Enter processing loop                                        */
    /****************************************************************/
    while (!bEndLoop)
    {
      /**************************************************************/
      /* Read the next message                                      */
      /**************************************************************/
      bSuccess = kxcGetRequest(hKxc,
                               &Msg,
                               &GetOpts,
                               pSync,
                               NULL);

      /**************************************************************/
      /* If we failed                                               */
      /**************************************************************/
      if (!bSuccess)
      {
        /************************************************************/
        /* Get error information                                    */
        /************************************************************/
        rc = kxcGetLastError(&Error);

        /************************************************************/
        /* If something other than NO_MSG_AVAIL, end loop           */
        /************************************************************/
        if (Error.Result.LastReturnCode != KXCRC_NO_MSG_AVAILABLE)
        { 
          WriteError(hKxc, "Message Get Failed");
          bEndLoop = TRUE;
        }

        /************************************************************/
        /* else if no-msg, and we're displaying, then end loop      */
        /************************************************************/
        else if (Action == ACT_DISPLAY)
        {
          if (bVerbose)
            printf("\nNo more EMS messages");

          bEndLoop = TRUE;
        };
      };

      /**************************************************************/
      /* If we have a message to process                            */
      /**************************************************************/
      if (bSuccess)
      {
        if (bVerbose)
          printf("\nMessage received.  Processing.");

        /************************************************************/
        /* Interpret the exception message                          */
        /************************************************************/
        bSuccess = kxcInterpretMessage(hKxc,
                                       KXCMOPT_HTHEME,
                                       &Msg,
                                       &MsgInfo);

        /************************************************************/
        /* If we failed                                             */
        /************************************************************/
        if (!bSuccess)
        {
          WriteError(hKxc, "Interpret Message Failed");
          bEndLoop = TRUE;
        };

        /************************************************************/
        /* If we have successfully parsed the message               */
        /************************************************************/
        if (bSuccess)
        {
          /**********************************************************/
          /* If we are just displaying                              */
          /**********************************************************/
          if (Action == ACT_DISPLAY)
          { 
            EMSDisplayMsg(hKxc, &Msg, &MsgInfo);
          }

          /**********************************************************/
          /* else if we are repairing and returning                 */
          /**********************************************************/
          else if (Action == ACT_RETURN)
          { 
            EMSReturnMsg(hKxc, &Msg, &MsgInfo, pSync);
          };
        };
      };

      /**************************************************************/
      /* If we're still OK and we're just displaying                */
      /**************************************************************/
      if ((bSuccess) && (Action == ACT_DISPLAY))
      {
        /************************************************************/
        /* Set options for a non-destructive BROWSE operation       */
        /************************************************************/
        GetOpts.ReceiveOptions &= ~KXCGOPT_PEEK_FIRST;
        GetOpts.ReceiveOptions |= KXCGOPT_PEEK_NEXT;
      };
    };

    /****************************************************************/
    /* Finally terminate                                            */
    /****************************************************************/
    bSuccess = kxcTerminate(&hKxc);
    if (!bSuccess)
    {
      WriteError(hKxc, "Failed to terminate");
    };
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSDisplayMsg                                         */
/* Purpose:   Display the contents of the EMS message               */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pMsg(PKXCMSG)                                         */
/*            - Original input message                              */
/*            pMsgInfo(PKXCMSGINFO)                                 */
/*            - MsgInfo details from exception message              */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSDisplayMsg(HKXC        hKxc,
                      PKXCMSG     pMsg,
                      PKXCMSGINFO pMsgInfo)
{

  /******************************************************************/
  /* These are used to get the retry count out of the context       */
  /******************************************************************/
  KXCBOOL          result;
  KXCBUFFER        Buffer = {0, 0, 0};
  KXCULONG         *retry;

  /******************************************************************/
  /* Display main header                                            */
  /******************************************************************/
  printf("\n Roma BSP Exception Message Display ");

  /******************************************************************/
  /* If we have Exception Data                                      */
  /******************************************************************/
  if (pMsgInfo->Status & KXCMSGPART_EXCEPTION)
  {
    /****************************************************************/
    /* Format the Exception Data                                    */
    /****************************************************************/
    EMSDisplayException(hKxc,
                        &(pMsgInfo->Exception));
  };

  /******************************************************************/
  /* If we have WorkFlow information                                */
  /******************************************************************/
  if (pMsgInfo->Status & KXCMSGPART_WORKFLOW)
  {
    /****************************************************************/
    /* Format the WorkFlow Data                                     */
    /****************************************************************/
    EMSDisplayWorkFlow(hKxc,
                       &(pMsgInfo->WorkFlow));
  };

  /******************************************************************/
  /* If we have Message information                                 */
  /******************************************************************/
  if (pMsgInfo->Status & KXCMSGPART_MESSAGE)
  {
    /****************************************************************/
    /* Format the User Data                                         */
    /****************************************************************/
    printf("\n\nKXCEMS: pMsg->Format=%.8x\n", pMsg->Format);
    printf("\n\nKXCEMS: pMsgInfo->Message.Format=%.8x\n", 
        pMsgInfo->Message.Format);
    EMSDisplayData(hKxc,
                   &(pMsgInfo->Message));
  }

  /******************************************************************/
  /* else just dump the message                                     */
  /******************************************************************/
  else
  {
    /****************************************************************/
    /* Format the Original Message                                  */
    /****************************************************************/
    EMSDisplayData(hKxc,
                   pMsg);
  };

  /******************************************************************/
  /* Display the retry count as per IMR 618811.                     */
  /* The retry count is in the theme - it needs to be extracted     */
  /* using QueryTheme.                                              */
  /******************************************************************/
  if (pMsgInfo->hTheme != NULL_HKXCTHEME)
  {
    result = kxcQueryTheme (hKxc, 
                            pMsgInfo->hTheme,
                            KXCTID_RETRY | KXCTID_VERSION_CURR,
                            &Buffer);
    kxcQryLookUp (Buffer.pBuffer, KXCTID_RETRY, &retry);

    printf ("\n Retry Count %d\n", *retry);
  }

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSDisplayException                                   */
/* Purpose:   Display the contents of the EMS Exception data        */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pException(PKXCEXCEPTION)                             */
/*            - Exception structure details                         */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSDisplayException(HKXC          hKxc,
                            PKXCEXCEPTION pException)
{
KXCCHAR        sTimeStamp[16];
char           Buffer[400];
KXCULONG       BufLen = 400;
KXCULONG       Env;
KXCULONG       OS;
char          *Calls[KXCCALLTYPE_NUMCALLS] = 
                          { "Application"
                          , "kxcPutRequest"
                          , "kxcGetRequest"
                          , "kxcPutResponse"
                          , "kxcGetResponse"
                          , "BridgePutMsg"
                          , "BridgeGetMsg"
                          , "kxcPutException"
                          , "kxcSPIInitialize"
                          , "kxcTerminate"
                          , "kxcCommit"
                          , "kxcRollback"
                          , "kxcFreeBuffer"
                          , "kxcCloseHandle"
                          , "kxcInterpretMessage"
                          , "kxcWriteLog"
                          , "kxcSendAlert"
                          , "kxcOpenStorageQueue"
                          , "kxcCloseStorageQueue"
                          , "kxcDeleteStorageQueue"
                          , "kxcPutToStorageQueue"
                          , "kxcGetFromStorageQueue"
                          , "kxcQueryTheme"
                          , "kxcBuildTheme"
                          , "kxcSPIPurgeQToEMS"
                          , "kxcReportError"
                          , "kxcQueryContext"
                          , "kxcSPISetLdapService"
                          };
char          *Elmnt[4] = { "Unknown",
                            "Server",
                            "Bridge",
                            "Client"
                          };

  /******************************************************************/
  /* Print out RomaConf details                                     */
  /******************************************************************/
  if (pException->ExceptionFlags & KXCEXC_ROMADATA)
  {
    printf("\n\n Roma Configuration Data:");
    printf("\n   Application Type .............: [%s]", 
           Elmnt[pException->RomaConfData.ElementType]);
    printf("\n   Application Name .............: [%s]", 
           pException->RomaConfData.ElementName);
    printf("\n   Input Queue Name .............: [%s]", 
           pException->RomaConfData.QueueName);
    printf("\n   Associated Business Component : [%s]",
           pException->RomaConfData.ComponentName);
    printf("\n   Associated Business Service ..: [%s]",
           pException->RomaConfData.ServiceName);
    printf("\n   Node Name ....................: [%s]",
           pException->RomaConfData.NodeName);
  }
  else
  {
    printf("\n\n No Roma Configuration Data present.");
  };

  /******************************************************************/
  /* Print out SystemConf details                                   */
  /******************************************************************/
  if (pException->ExceptionFlags & KXCEXC_SYSTEMDATA)
  {
    Env = pException->SystemConfData.EnvironmentType & 
                                                     KXCENVTYPE_MASK;
    OS  = pException->SystemConfData.EnvironmentType & 
                                                     KXCENVOS_MASK;
    printf("\n\n System Configuration Data:");
    printf("\n   Environment Type : [%s]",
           ((Env & KXCENVTYPE_INTEL) ? "Intel" : 
            ((Env & KXCENVTYPE_UNIX) ? "Unix" : 
             ((Env & KXCENVTYPE_MVS) ? "MVS" : "Unknown"))) );

    printf("\n   Environment OS ..: [%s]",
            (((Env & KXCENVTYPE_INTEL) && 
              (OS & KXCENVOS_WINNT))     ? "Windows NT" : 
            (((Env & KXCENVTYPE_INTEL) && 
              (OS & KXCENVOS_WIN95))     ? "Windows 9x" : 
            (((Env & KXCENVTYPE_UNIX) && 
              (OS & KXCENVOS_AIX))       ? "AIX" :
            (((Env & KXCENVTYPE_UNIX) && 
              (OS & KXCENVOS_HPUX11))    ? "HP/UX 11.0" :
            (((Env & KXCENVTYPE_UNIX) && 
              (OS & KXCENVOS_HPUX102))   ? "HP-UX 10.20" :
            (((Env & KXCENVTYPE_UNIX) && 
              (OS & KXCENVOS_SUN))       ? "Sun Solaris" :
            (((Env & KXCENVTYPE_MVS) && 
              (OS & KXCENVOS_MVSCICS))   ? "MVS CICS" :
            (((Env & KXCENVTYPE_MVS) && 
              (OS & KXCENVOS_MVSBATCH))  ? "MVS BATCH" :
                                           "Unknown")))))))));

    printf("\n   Domain Name .....: [%s]",
           pException->SystemConfData.DomainName);
    printf("\n   User Id .........: [%s]",
           pException->SystemConfData.UserId);
    printf("\n   Program Name ....: [%s]",
           pException->SystemConfData.ProgramName);
    printf("\n   Process Id ......: [0x%x]",
           pException->SystemConfData.ProcessId);
    printf("\n   Thread Id .......: [0x%x]",
           pException->SystemConfData.ThreadId);
  }
  else
  {
    printf("\n\n No System Configuration Data present.");
  };

  /******************************************************************/
  /* Print out TimeStamp details                                    */
  /******************************************************************/
  if (pException->ExceptionFlags & KXCEXC_TIMESTAMP)
  {
    printf("\n\n TimeStamp Information:");
    kxcTimeStampToExternal(sTimeStamp, 
                           &pException->TimeStamp);
    printf("\n   TimeStamp : "
           "[%s%2.2s/%2.2s/%2.2s %2.2s:%2.2s:%2.2s.%3.3s]", 
           (sTimeStamp[0] == '0' ? "19" : "20"),
           sTimeStamp+1, sTimeStamp+3, sTimeStamp+5,
           sTimeStamp+7, sTimeStamp+9, sTimeStamp+11, sTimeStamp+13);
  }
  else
  {
    printf("\n\n No TimeStamp Data present.");
  };

  /******************************************************************/
  /* Print out Error Details                                        */
  /******************************************************************/
  if (pException->ExceptionFlags & KXCEXC_ERRORDATA)
  {
    printf("\n\n Error Data:");
    kxcFormatError(&pException->ErrorData.Error.Result, 
                   KXCERROR_FULL, 
                   Buffer, 
                   &BufLen);
    printf("\n   Error ....: [%s]", Buffer);

    if (pException->ErrorData.Error.Status & KXCERRSTAT_SUB)
    {
      BufLen=400;
      kxcFormatError(&pException->ErrorData.Error.SubResult,
                     KXCERROR_FULL,
                     Buffer, 
                     &BufLen);
      printf("\n   SubError .: [%s]", Buffer);
    };

    printf("\n   Operation : [%s]",
         Calls[pException->ErrorData.CallType]);
    printf("\n   QueueName : [%s]",
         pException->ErrorData.QueueName);
  }
  else
  {
    printf("\n\n No Error Data present.");
  };

  /******************************************************************/
  /* Print out UserData Details                                     */
  /******************************************************************/
  if (pException->ExceptionFlags & KXCEXC_USERDATA)
  {
    printf("\n\nUser Data:-\n");
    EMSHexFormatData(hKxc, 
                     pException->UserData.UserBuf.pBuffer, 
                     pException->UserData.UserBufLen);
  }
  else
  {
    printf("\n\nNo User Data present.");
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSDisplayWorkFlowVer4                                */
/* Purpose:   Display the contents of the Roma 3.0 EMS WorkFlow     */
/* Input:     pWFlow(PKXCWFLOW_4)                                   */
/*            - Pointer to the version 4 workflow                   */
/********************************************************************/
KXCVOID EMSDisplayWorkFlowVer4(PKXCWFLOW_4 pWFlow)
{
PKXCWROUTE_3    pRoute    = pWFlow->Routes;
PKXCWSERVICE_2  pService  = &pWFlow->Service;
KXCULONG        NumRoutes = pWFlow->NumRoutes;
PKXCWROUTE_3    pEnd      = pRoute + NumRoutes;
KXCULONG        i         = 0;

  /******************************************************************/
  /* Print out the service details                                  */
  /******************************************************************/
  printf("\n   Service .......: [%s]", pService->Service);
  printf("\n   Options .......: [%x]", pService->Options);

  /******************************************************************/
  /* Loop through the route entries                                 */
  /******************************************************************/
  for (; pRoute != pEnd; ++pRoute, ++i)
  {
    /****************************************************************/
    /* Print out the route details                                  */
    /****************************************************************/
    printf("\n   %3.3d  QueueName : [%s]",
           i, pRoute->TargetQueue);

    if (pRoute->TargetComponent[0])
    {
      printf("\n        Component : [%s]",
             pRoute->TargetComponent);
    }

    if (pRoute->TargetElement[0])
    {
      printf("\n        Element ..: [%s]",
             pRoute->TargetElement);
    }

    printf("\n        RomaVers .: [%x]", 
             pRoute->NodeVersion);

    printf("\n        CodePage .: [%s]",
             pRoute->TargetCodePage);
  }
}

/********************************************************************/
/* Function:  EMSDisplayWorkFlow                                    */
/* Purpose:   Display the contents of the EMS WorkFlow data         */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pBuffer(PKXCBUFFER)                                   */
/*            - Workflow buffer                                     */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSDisplayWorkFlow(HKXC       hKxc,
                           PKXCBUFFER pBuffer)
{
PKXCWFLOW     pWFlow;
PKXCWROUTE    pRoute;
PKXCWSERVICE  pService;
KXCULONG      NumRoutes;
KXCULONG      i;
PKXCWFLOW     pWFlow4 = NULL;

  printf("\n\n WorkFlow Information:");

  /******************************************************************/
  /* Access Workflow details                                        */
  /******************************************************************/
  pWFlow = (PKXCWFLOW) pBuffer->pBuffer;

  /******************************************************************/
  /* Check the workflow version to be sure we know how to handle it */
  /******************************************************************/
  if (pWFlow->Version < KXCWFLOW_VERSION_4 ||
      pWFlow->Version > KXCWFLOW_VERSION)
  {
      printf("\n\nError: workflow version %d not supported\n",
          pWFlow->Version);
      exit(1);
  }

  /******************************************************************/
  /* If the message was generated by a Roma 3.0 application, handle */
  /* its workflow appropriately                                     */
  /******************************************************************/
  if (pWFlow->Version == KXCWFLOW_VERSION_4)
  {
      EMSDisplayWorkFlowVer4((PKXCWFLOW_4)pWFlow);
      return;
  }

  /******************************************************************/
  /* Access standard route information                              */
  /******************************************************************/
  pRoute    = pWFlow->Routes;
  pService  = &pWFlow->Service;
  NumRoutes = pWFlow->NumRoutes;

  /******************************************************************/
  /* Print out the service details                                  */
  /******************************************************************/
  printf("\n   Service .......: [%s]", pService->Service);
  printf("\n   Options .......: [%x]", pService->Options);

  /******************************************************************/
  /* Loop through the route entries                                 */
  /******************************************************************/
  for (i=0; i < NumRoutes; i++, pRoute++)
  {
    /****************************************************************/
    /* Print out the route details                                  */
    /****************************************************************/
    printf("\n   %3.3d  QueueName : [%s]",
           i, pRoute->TargetQueue);
    if (pRoute->TargetComponent[0])
    {
      printf("\n        Component : [%s]",
             pRoute->TargetComponent);
    };
    if (pRoute->TargetElement[0])
    {
      printf("\n        Element ..: [%s]",
             pRoute->TargetElement);
    };

    /****************************************************************/
    /* Print out some V4 route details                              */
    /****************************************************************/
    if (pWFlow->Version == KXCWFLOW_VERSION_4) 
    {
      printf("\n        RomaVers .: [%x]", 
             pRoute->NodeVersion);

      printf("\n        CodePage .: [%s]",
             pRoute->TargetCodePage);
    };
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSDisplayData                                        */
/* Purpose:   Display the contents of the EMS Message Data          */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pMsg(PKXCMSG)                                         */
/*            - Message buffer                                      */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSDisplayData(HKXC    hKxc,
                       PKXCMSG pMsg)
{
  /******************************************************************/
  /* Print out the details                                          */
  /******************************************************************/
  printf("\n\n Format      %.8x MsgType  %.8x", 
         pMsg->Format, pMsg->MsgType);
  printf("\n Persistence %.8x Priority %.8x", 
         pMsg->Persistence, pMsg->Priority);
  printf("\n Feedback    %.8x Expiry   %.8x", 
         pMsg->Feedback, pMsg->Expiry);
  printf("\n CodePage    %.8x Encoding %.8x", 
         pMsg->CodePage, pMsg->Encoding);
  printf("\n MsgLen      %.8x TagLen   %.8x", 
         pMsg->MsgLen, pMsg->TagLen);

  /******************************************************************/
  /* Format the MsgId                                               */
  /******************************************************************/
  printf("\n\n MsgId:-\n");
  EMSHexFormatData(hKxc, pMsg->MsgId, sizeof(KXCMSGID));

  /******************************************************************/
  /* Format the CorrelId                                            */
  /******************************************************************/
  printf("\n CorrelId:-\n");
  EMSHexFormatData(hKxc, pMsg->CorrelId, sizeof(KXCMSGID));

  /******************************************************************/
  /* Format the Message Data                                        */
  /******************************************************************/
  printf("\n Message Data:-\n"); 
  EMSHexFormatData(hKxc, pMsg->MsgBuffer.pBuffer, pMsg->MsgLen);

  /******************************************************************/
  /* Format the Tag Data                                            */
  /******************************************************************/
  if (pMsg->TagLen > 0)
  {
    printf("\nTag Data:-\n"); 
    EMSHexFormatData(hKxc, pMsg->TagBuffer.pBuffer, pMsg->TagLen);
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSHexFormatData                                      */
/* Purpose:   Hexdump the contents of a specified buffer            */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pData(PKXCVOID)                                       */
/*            - Data buffer                                         */
/*            DataLen(KXCULONG)                                     */
/*            - Data buffer length                                  */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSHexFormatData(HKXC     hKxc,
                         PKXCVOID pData,
                         KXCULONG DataLen)
{
KXCULONG  Column;
KXCULONG  Total; 
char     *pTarget;
PKXCCHAR  pSource;
char      Buffer[100];

  /******************************************************************/
  /* Loop through the message                                       */
  /******************************************************************/
  for (Total=0, pSource = (PKXCCHAR)pData; 
       Total < DataLen; 
       Total += 16)
  {
    /****************************************************************/
    /* Format the offset                                            */
    /****************************************************************/
    sprintf(Buffer, " |%04x| ",Total);
    pTarget = Buffer + strlen(Buffer);

    /****************************************************************/
    /* Format each byte in the line                                 */
    /****************************************************************/
    for(Column=0; Column<16; Column++, pSource++, pTarget+=3)
    { 
      /**************************************************************/
      /* If we still have data to format                            */
      /**************************************************************/
      if (Total + Column < DataLen)
      { 
        /************************************************************/
        /* Format the hex contents                                  */
        /************************************************************/
        sprintf(pTarget, "%02x ", *pSource); 
      }

      /**************************************************************/
      /* else we are on the last line                               */
      /**************************************************************/
      else
      {
        /************************************************************/
        /* Fill with blanks                                         */
        /************************************************************/
        strcat(pTarget, "   ");
      };
    };

    /****************************************************************/
    /* Add a seperator character                                    */
    /****************************************************************/
    strcat(pTarget, "| ");
    pTarget+=2;
    pSource-=16;

    /****************************************************************/
    /* Now loop providing the character versions                    */
    /****************************************************************/
    for (Column=0; Column<16; Column++, pSource++, pTarget++)
    {
      /**************************************************************/
      /* If we still have data to format                            */
      /**************************************************************/
      if (Total + Column < DataLen)
      { 
        /************************************************************/
        /* Format the character                                     */
        /************************************************************/
        *pTarget = isprint(*pSource) ? *pSource : '.'; 
      } 
      else
      {
        /************************************************************/
        /* Fill with blanks                                         */
        /************************************************************/
        *pTarget = ' ';
      };
    };

    /****************************************************************/
    /* Add a seperator character                                    */
    /****************************************************************/
    strcpy(pTarget," | \n");

    /****************************************************************/
    /* print out the line                                           */
    /****************************************************************/
    printf(Buffer);
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSReturnMsg                                          */
/* Purpose:   Modify msg data and put message onto queue around     */
/*            which the original error occurred.                    */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pMsg (PKXCMSG)                                        */
/*            - Message retrived from EMS Q, used for some of the   */
/*            message settings                                      */
/*            pMsgInfo(PKXCMSGINFO)                                 */
/*            - MsgInfo details from exception message              */
/*            pSync(PKXCSYNC)                                       */
/*            - Syncpt structure in same LUW as EMS Get             */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID EMSReturnMsg(HKXC        hKxc,
                     PKXCMSG     pEMSMsg,
                     PKXCMSGINFO pMsgInfo,
                     PKXCSYNC    pSync)
{
KXCBOOL          bSuccess   = TRUE;
KXCULONG         CallType;
PKXCEXCEPTION    pException;
HKXCTHEME        hTheme     = NULL_HKXCTHEME;
KXCMSGID         ThemeId    = { KXCMSGID_NULL };
KXCULONG         RetryCount = 0;
PKXCMSG          pMsg;
PKXCCHAR         pNewMsg    = (PKXCCHAR)ModifiedBuffer;
KXCULONG         BufLen     = BUFFER_SIZE;
KXCULONG         MsgLen     = strlen((char *)pNewMsg);
KXCULONG         PutOptions;
KXCPUTOPT        PutOpts;

  /******************************************************************/
  /* Trace entry on verbose                                         */
  /******************************************************************/
  if (bVerbose)
    printf("\nHandling EMS message");

  /******************************************************************/
  /* If we have Exception data in MsgInfo struct                    */
  /******************************************************************/
  if (pMsgInfo->Status & KXCMSGPART_EXCEPTION)
  {
    /****************************************************************/
    /* Access the Exception structure                               */ 
    /****************************************************************/
    pException = &(pMsgInfo->Exception);
    
    /****************************************************************/
    /* Access the CallType                                          */ 
    /****************************************************************/
    CallType = pException->ErrorData.CallType;

    /****************************************************************/
    /* If the failed call was a PUT                                 */
    /****************************************************************/
    if ((CallType == KXCCALLTYPE_PUTRESPONSE) ||
        (CallType == KXCCALLTYPE_PUTREQUEST))
    {
      /**************************************************************/
      /* Reconstruct theme for a failed PUT call                    */
      /**************************************************************/
      bSuccess = EMSRebuildPutTheme(hKxc, 
                                    pMsgInfo, 
                                    &hTheme,
                                    &ThemeId,
                                    &RetryCount);
    }
   
    /****************************************************************/
    /* else if the failed call was a GET                            */
    /****************************************************************/
    else if ((CallType == KXCCALLTYPE_GETREQUEST) ||
             (CallType == KXCCALLTYPE_GETRESPONSE))
    { 
      /**************************************************************/
      /* Reconstruct theme for a failed GET call                    */
      /**************************************************************/
      bSuccess = EMSRebuildGetTheme(hKxc, 
                                    pMsgInfo, 
                                    &hTheme,
                                    &ThemeId,
                                    &RetryCount);
    };

    /****************************************************************/
    /* If we built workflow OK                                      */
    /****************************************************************/
    if (bSuccess) 
    {
      /**************************************************************/
      /* Check the RetryCount                                       */
      /**************************************************************/
      if (RetryCount >= RETRY_MAX)
      {
        WriteError(hKxc, "Retry count exceeded maximum");
        bSuccess = FALSE;
      }; 
    };

    /****************************************************************/
    /* If we're still OK                                            */
    /****************************************************************/
    if (bSuccess)
    {
      /**************************************************************/
      /* Access the Message structure                               */
      /**************************************************************/
      pMsg = &(pMsgInfo->Message);

      /**************************************************************/
      /* Modify the message data                                    */
      /**************************************************************/
      pMsg->MsgLen = MsgLen;
      kxcSetMsgBuffer(pMsg, BufLen, pNewMsg);

      /**************************************************************/
      /* Build Putopts for returning modified message               */
      /**************************************************************/
      kxcSetPutOptDefault(&PutOpts);
      PutOptions = ( KXCPOPT_USETHEME      |
                     KXCPOPT_CLOSETHEME    |
                     KXCPOPT_ALLOWNOSYNC   |
                     KXCPOPT_INCRETRYCOUNT |
                     KXCPOPT_NOEMS         |
                     KXCPOPT_NOXFORM       |
                     KXCPOPT_NONOTIFY );

      if (memcmp(pMsg->MsgId,
                 KXCMSGID_NULL,
                 KXCMSG_ID_LENGTH))
      {
        PutOptions |= KXCPOPT_USEMSGID;
        memcpy ((char *) PutOpts.MsgId, 
                (char *) pMsg->MsgId,
                KXCMSG_ID_LENGTH);
      };

      if ((memcmp(pMsg->CorrelId,
                  KXCMSGID_NULL,
                  KXCMSG_ID_LENGTH)) &&
          (memcmp(pMsg->CorrelId,
                  &ThemeId,
                  KXCMSG_ID_LENGTH)))
      {
        PutOptions |= KXCPOPT_USECORRELID;
        memcpy ((char *) PutOpts.CorrelId, 
                (char *) pMsg->CorrelId,
                KXCMSG_ID_LENGTH);
      };

      PutOpts.RequestOptions = PutOptions;

      /**************************************************************/
      /* Put "new" message back onto the original queue             */
      /**************************************************************/
      bSuccess = kxcPutResponse(hKxc,
                                pMsg,
                                &PutOpts,
                                pSync,
                                &hTheme); 

      /**************************************************************/
      /* If we failed                                               */
      /**************************************************************/
      if (!bSuccess)
      {
        WriteError(hKxc, "Failed to return modified message");
      }
      else 
      {  
        if (bVerbose)
          printf("\nSent modified msg back to orig queue\n");
      };
    };

    /****************************************************************/
    /* Commit - ensure same LUW                                     */
    /****************************************************************/
    bSuccess = kxcCommit(hKxc, pSync);
    if (!bSuccess)
    {
      WriteError(hKxc, "EMS Commit Failed");
    };
  }

  /******************************************************************/
  /* else report error on absence of Exception data                 */
  /******************************************************************/
  else
  {
    WriteError(hKxc, "No Exception data in EMS msg");
  };
  
  /******************************************************************/
  /* Close the theme handle regardless                              */
  /******************************************************************/
  if (hTheme != NULL_HKXCTHEME)
  {
    kxcCloseHandle(hKxc, &hTheme);
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/* Function:  EMSNormalizeWorkFlowVer4                              */
/* Purpose:   Removes bridges from the route of                     */
/*            the Roma 3.0 workflow                                 */
/* Input:     pWFlow(PKXCWFLOW_4)                                   */
/*            - Pointer to the Roma 3.0 workflow                    */
/* Return:    TRUE if the workflow has been modified                */
/********************************************************************/
KXCBOOL EMSNormalizeWorkFlowVer4(PKXCWFLOW_4 pWFlow)
{
  PKXCWROUTE_3 pRoute = pWFlow->Routes;
  PKXCWROUTE_3 pEnd   = pRoute + pWFlow->NumRoutes;
  KXCBOOL bModified = FALSE;

  /******************************************************************/
  /* Remove bridges from the route                                  */
  /******************************************************************/

  for (; pRoute->RouteType == KXCTYPE_BRIDGE; ++pRoute);

  if (pRoute != pWFlow->Routes)
  {
    /****************************************************************/
    /* Replace bridge route steps with the remaining route          */
    /****************************************************************/
    pWFlow->NumRoutes = pEnd - pRoute;
    memmove(pWFlow->Routes, pRoute, (size_t)pEnd - (size_t)pRoute);
    bModified = TRUE;
  }

  if (bVerbose)
  {
    pRoute = pWFlow->Routes;
    printf("\nReturning msg to application [%s]",
        pRoute->TargetElement);
    printf("\n                     @ queue [%s]",
        pRoute->TargetQueue);
  }

  return bModified;
}

/********************************************************************/
/* Function:  EMSNormalizeWorkFlow                                  */
/* Purpose:   Removes bridges from the workflow route               */
/* Input:     pWFlow(PKXCWFLOW)                                     */
/*            - Pointer to the workflow                             */
/* Return:    TRUE if the workflow has been modified                */
/********************************************************************/
KXCBOOL EMSNormalizeWorkFlow(PKXCWFLOW pWFlow)
{
  PKXCWROUTE pRoute = pWFlow->Routes;
  PKXCWROUTE pEnd   = pRoute + pWFlow->NumRoutes;
  KXCBOOL bModified = FALSE;

  /******************************************************************/
  /* Remove bridges from the route                                  */
  /******************************************************************/

  for (; pRoute->RouteType == KXCTYPE_BRIDGE; ++pRoute);

  if (pRoute != pWFlow->Routes)
  {
    /****************************************************************/
    /* Replace bridge route steps with the remaining route          */
    /****************************************************************/
    pWFlow->NumRoutes = pEnd - pRoute;
    memmove(pWFlow->Routes, pRoute, (size_t)pEnd - (size_t)pRoute);
    bModified = TRUE;
  }

  if (bVerbose)
  {
    pRoute = pWFlow->Routes;
    printf("\nReturning msg to application [%s]",
        pRoute->TargetElement);
    printf("\n                     @ queue [%s]",
        pRoute->TargetQueue);
  }

  return bModified;
}

/********************************************************************/
/* Function:  EMSRebuildPutTheme                                    */
/* Purpose:   Recreate the original theme information used to the   */
/*            failed PUT to the target queue.                       */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pMsgInfo(PKXCMSGINFO)                                 */
/*            - MsgInfo details from exception message              */
/* Output:    phTheme(PHKXCTHEME)                                   */
/*            - Pointer to theme handle                             */
/*            pThemeId(PKXCMSGID)                                   */
/*            - Pointer to themeid details                          */
/*            pRetry(PKXCULONG)                                     */
/*            - Pointer to retry-count value from theme             */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCBOOL EMSRebuildPutTheme(HKXC        hKxc,
                           PKXCMSGINFO pMsgInfo,
                           PHKXCTHEME  phTheme,
                           PKXCMSGID   pThemeId,
                           PKXCULONG   pRetry)
{
KXCBOOL        bSuccess     = TRUE;
HKXCTHEME      hTheme;
KXCBUFFER      QueryBuf     = { KXCBUFFER_DEFAULT };
KXCBUFFER      BuildBuf     = { KXCBUFFER_DEFAULT };
PKXCWFLOW      pWorkflow    = NULL;
PKXCMSGID      pOrigThemeId = NULL;
PKXCULONG      pOrigRetry   = NULL;
KXCBOOL        bRebuild     = FALSE;

  /******************************************************************/
  /* Trace entry on verbose                                         */
  /******************************************************************/
  if (bVerbose)
    printf("\nRebuilding workflow for failed PUT");

  /******************************************************************/
  /* Ensure we have theme info in the MsgInfo struct                */
  /******************************************************************/
  if (pMsgInfo->Status & KXCMSGPART_HTHEME)
  {
    /****************************************************************/
    /* Access theme info                                            */
    /****************************************************************/
    hTheme = pMsgInfo->hTheme;

    /****************************************************************/
    /* Determine the workflow                                       */
    /****************************************************************/
    bSuccess = kxcQueryTheme(hKxc,
                             hTheme,
                             ( KXCTID_THEMEID  |
                               KXCTID_RETRY    |
                               KXCTID_RSPWFLOW |
                               KXCTID_VERSION_CURR ),
                             &QueryBuf);
    if (!bSuccess)
    {
      WriteError(hKxc, "Failed to access Theme information");
    };

    /****************************************************************/
    /* If we've got Theme info                                      */
    /****************************************************************/
    if (bSuccess)
    {
      /**************************************************************/
      /* Pick up the themeid information                            */
      /**************************************************************/
      kxcQryLookUp(QueryBuf.pBuffer,
                   KXCTID_THEMEID, 
                   &pOrigThemeId);

      /**************************************************************/
      /* If found OK, then save as return                           */
      /**************************************************************/
      if (pOrigThemeId)
      {
        memcpy(pThemeId, pOrigThemeId, sizeof(KXCMSGID));
      };

      /**************************************************************/
      /* Pick up the retrycount                                     */
      /**************************************************************/
      kxcQryLookUp(QueryBuf.pBuffer,
                   KXCTID_RETRY, 
                   &pOrigRetry);

      /**************************************************************/
      /* If found OK, then save as return                           */
      /**************************************************************/
      if (pOrigRetry)
      {
        *pRetry = *pOrigRetry; 
      };

      /**************************************************************/
      /* Pick up the workflow information                           */
      /**************************************************************/
      kxcQryLookUp(QueryBuf.pBuffer,
                   KXCTID_RSPWFLOW,
                   &pWorkflow);

      if (pWorkflow == NULL)
      {
          fputs("Error: no workflow information", stderr);
          return FALSE;
      }

      /**************************************************************/
      /* Check the workflow version to be sure we know how          */
      /* to handle it                                               */
      /**************************************************************/
      if (pWorkflow->Version < KXCWFLOW_VERSION_4 ||
          pWorkflow->Version > KXCWFLOW_VERSION)
      {
        fprintf(stderr,
            "\n\nError: workflow version %d is not supported\n",
            pWorkflow->Version);
        exit(1);
      }

      /**************************************************************/
      /* If we have zero entries in the workflow                    */
      /**************************************************************/
      if (pWorkflow->NumRoutes == 0)
      {
        WriteError(hKxc, "No routes in workflow");
        return FALSE;
      }

      /**************************************************************/
      /* Examine the route entries. If the message was sent by      */
      /* a Roma 3.0 application, handle its workflow appropriately  */
      /**************************************************************/
      if (pWorkflow->Version == KXCWFLOW_VERSION_4)
      {
        bRebuild = EMSNormalizeWorkFlowVer4((PKXCWFLOW_4)pWorkflow);
        BuildBuf.BufLen = (sizeof(KXCWFLOW_4) +
                    (pWorkflow->NumRoutes - 1) * sizeof(KXCWROUTE_3));
      }
      else
      {
        bRebuild = EMSNormalizeWorkFlow(pWorkflow);
        BuildBuf.BufLen = (sizeof(KXCWFLOW) +
                    (pWorkflow->NumRoutes - 1) * sizeof(KXCWROUTE));
      }

      if (bRebuild)
      {
        /************************************************************/
        /* Setup the buffer for BuildTheme                          */
        /************************************************************/
        BuildBuf.pBuffer = pWorkflow;
 
        /************************************************************/
        /* Build the new theme with the modified workflow           */
        /************************************************************/
        if (!kxcBuildTheme(hKxc, hTheme, KXCTOPT_SETRSPWFLOW,
                           &BuildBuf, &hTheme))
        {
            return FALSE;
        }
      }

      /**************************************************************/
      /* Save the theme                                             */
      /**************************************************************/
      if (phTheme) 
      {
        *phTheme = hTheme;
      }
    }
  }

  /******************************************************************/
  /* else we have no theme info to work with                        */
  /******************************************************************/
  else
  {
    WriteError(hKxc, "No theme information in MsgInfo struct");
    bSuccess = FALSE;
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return(bSuccess);
}

/********************************************************************/
/* Function:  EMSRebuildGetTheme                                    */
/* Purpose:   Recreate the original theme information used to the   */
/*            failed GET from the target queue.                     */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pMsgInfo(PKXCMSGINFO)                                 */
/*            - MsgInfo details from exception message              */
/* Output:    phTheme(PHKXCTHEME)                                   */
/*            - Pointer to theme handle                             */
/*            pThemeId(PKXCMSGID)                                   */
/*            - Pointer to themeid details                          */
/*            pRetry(PKXCULONG)                                     */
/*            - Pointer to retry-count value from theme             */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCBOOL EMSRebuildGetTheme(HKXC        hKxc,
                           PKXCMSGINFO pMsgInfo,
                           PHKXCTHEME  phTheme,
                           PKXCMSGID   pThemeId,
                           PKXCULONG   pRetry)
{
KXCBOOL        bSuccess     = TRUE;
HKXCTHEME      hTheme;
HKXCTHEME      hNewTheme    = NULL_HKXCTHEME;
KXCBUFFER      QueryBuf     = { KXCBUFFER_DEFAULT };
KXCBUFFER      BuildBuf     = { KXCBUFFER_DEFAULT };
PKXCWFLOW      pWorkflow    = NULL;
PKXCWFLOW      pNewWFlow    = NULL;
PKXCWROUTE     pRoute       = NULL;
KXCULONG       NumRoutes;
PKXCMSGID      pOrigThemeId = NULL;
PKXCULONG      pOrigRetry   = NULL;
PKXCEXCEPTION  pException   = NULL;

  /******************************************************************/
  /* Trace entry on verbose                                         */
  /******************************************************************/
  if (bVerbose)
    printf("\nRebuilding workflow for failed GET");

  /******************************************************************/
  /* Ensure we have theme info in the MsgInfo struct                */
  /******************************************************************/
  if (pMsgInfo->Status & KXCMSGPART_HTHEME)
  {
    /****************************************************************/
    /* Access theme info                                            */
    /****************************************************************/
    hTheme = pMsgInfo->hTheme;

    /****************************************************************/
    /* Determine the workflow                                       */
    /****************************************************************/
    bSuccess = kxcQueryTheme(hKxc,
                             hTheme,
                             ( KXCTID_THEMEID  |
                               KXCTID_RETRY    |
                               KXCTID_RSPWFLOW |
                               KXCTID_VERSION_CURR ),
                             &QueryBuf);
    if (!bSuccess)
    {
      WriteError(hKxc, "Failed to access Theme information");
    };

    /****************************************************************/
    /* If we've got Theme info                                      */
    /****************************************************************/
    if (bSuccess)
    {
      /**************************************************************/
      /* Pick up the themeid information                            */
      /**************************************************************/
      kxcQryLookUp(QueryBuf.pBuffer,
                   KXCTID_THEMEID, 
                   &pOrigThemeId);

      /**************************************************************/
      /* If found OK, then save as return                           */
      /**************************************************************/
      if (pOrigThemeId)
      {
        memcpy(pThemeId, pOrigThemeId, sizeof(KXCMSGID));
      };

      /**************************************************************/
      /* Pick up the retrycount                                     */
      /**************************************************************/
      kxcQryLookUp(QueryBuf.pBuffer,
                   KXCTID_RETRY, 
                   &pOrigRetry);

      /**************************************************************/
      /* If found OK, then save as return                           */
      /**************************************************************/
      if (pOrigRetry)
      {
        *pRetry = *pOrigRetry; 
      };

      /**************************************************************/
      /* Access the Exception information                           */
      /**************************************************************/
      pException = &(pMsgInfo->Exception);

      /**************************************************************/
      /* Pick up the workflow information                           */
      /**************************************************************/
      kxcQryLookUp(QueryBuf.pBuffer,
                   KXCTID_RSPWFLOW,
                   &pWorkflow);

      if (pWorkflow != NULL)
      {
        /************************************************************/
        /* Check the workflow version to be sure we know how        */
        /* to handle it                                             */
        /************************************************************/
        switch (pWorkflow->Version)
        {
            case KXCWFLOW_VERSION_4:
              /******************************************************/
              /* Remove bridges from the route                      */
              /******************************************************/
              EMSNormalizeWorkFlowVer4((PKXCWFLOW_4)pWorkflow);

              /******************************************************/
              /* Rebuild Workflow from Exception data               */
              /******************************************************/
              bSuccess = EMSRebuildGetWorkflowVer4(hKxc, 
                                                   pException,
                                                   (PKXCWFLOW_4)pWorkflow, 
                                                   &pNewWFlow);
              break;

            case KXCWFLOW_VERSION:
              /******************************************************/
              /* Remove bridges from the route                      */
              /******************************************************/
              EMSNormalizeWorkFlow(pWorkflow);

              /******************************************************/
              /* Rebuild Workflow from Exception data               */
              /******************************************************/
              bSuccess = EMSRebuildGetWorkflow(hKxc, 
                                               pException,
                                               pWorkflow, 
                                               &pNewWFlow);
              break;

            default:
                fprintf(stderr,
                    "\nError: workflow version %d is not supported\n",
                    pWorkflow->Version);
                bSuccess = FALSE;
        }
      }
      else
      {
          fputs("\nError: no workflow information", stderr);
          bSuccess = FALSE;
      }

      /**************************************************************/
      /* If we built the workflow info OK                           */
      /**************************************************************/
      if ((bSuccess) && (pNewWFlow != NULL))
      {
        if (bVerbose)
        {
          pRoute = pNewWFlow->Routes;
          printf("\nReturning msg to application [%s]",
                 pRoute->TargetElement);
          printf("\n                   and queue [%s]",
                 pRoute->TargetQueue);
        };

        /************************************************************/
        /* Setup the buffer for BuildTheme                          */
        /************************************************************/
        NumRoutes        = pNewWFlow->NumRoutes;
        BuildBuf.BufLen  = (sizeof(KXCWFLOW) +
                            ((NumRoutes-1) * sizeof(KXCWROUTE)));
        BuildBuf.pBuffer = pNewWFlow;

        /************************************************************/
        /* Build the new theme                                      */
        /************************************************************/
        bSuccess = kxcBuildTheme(hKxc,
                                 hTheme,
                                 KXCTOPT_SETRSPWFLOW,
                                 &BuildBuf,
                                 &hNewTheme);

        /************************************************************/
        /* If we're still OK                                        */
        /************************************************************/
        if (bSuccess)
        {
          /**********************************************************/
          /* Close original theme                                   */
          /**********************************************************/
          kxcCloseHandle(hKxc, &hTheme);

          /**********************************************************/
          /* Reset theme handle                                     */
          /**********************************************************/
          hTheme = hNewTheme;

          /**********************************************************/
          /* Setup the buffer for BuildTheme                        */
          /**********************************************************/
          BuildBuf.BufLen  = sizeof(KXCMSGID);
          BuildBuf.pBuffer = pOrigThemeId;

          /**********************************************************/
          /* Build the new theme                                    */
          /**********************************************************/
          bSuccess = kxcBuildTheme(hKxc,
                                   hTheme,
                                   KXCTOPT_SETTHEMEID, 
                                   &BuildBuf,
                                   &hNewTheme);
        };

        /************************************************************/
        /* If we're still OK                                        */
        /************************************************************/
        if (bSuccess)
        {
          /**********************************************************/
          /* Close original theme                                   */
          /**********************************************************/
          kxcCloseHandle(hKxc, &hTheme);

          /**********************************************************/
          /* Save this new theme                                    */
          /**********************************************************/
          if (phTheme)
          {
            *phTheme = hNewTheme;
          };
        };
      };
    };
  }

  /******************************************************************/
  /* else we have no theme info to work with                        */
  /******************************************************************/
  else
  {
    WriteError(hKxc, "No theme information in MsgInfo struct");
    bSuccess = FALSE;
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return(bSuccess);
}

/********************************************************************/
/* Function:  EMSRebuildGetWorkflowVer4                             */
/* Purpose:   Rebuild the Roma 3.0 workflow for the message to be   */
/*            put back to the applications input queue.             */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pException(PKXCEXCEPTION)                             */
/*            - Exception details from the MsgInfo struct           */
/*            pWorkflow(PKXCWFLOW_4)                                */
/*            - Current workflow of the EMS'd Roma 3.0 message      */
/* Output:    ppNewFlow(PKXCWFLOW *)                                */
/*            - New workflow                                        */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCBOOL EMSRebuildGetWorkflowVer4(HKXC           hKxc,
                                  PKXCEXCEPTION  pException,
                                  PKXCWFLOW_4    pWorkflow,
                                  PKXCWFLOW     *ppNewFlow)
{
KXCBOOL        bSuccess     = TRUE;
PKXCWFLOW      pWFlow       = NULL;
PKXCWROUTE     pRoute       = NULL; 
KXCULONG       ReqdSize     = 0;
KXCULONG       NumRoutes    = pWorkflow->NumRoutes;
PKXCWROUTE_3   pOldRoute    = pWorkflow->Routes; 
PKXCWROUTE_3   pEnd         = pOldRoute + NumRoutes; 

  /******************************************************************/
  /* Check that Roma & Error data are present in Exception struct   */
  /******************************************************************/
  if ((pException) && 
      (pException->ExceptionFlags & KXCEXC_ROMADATA))
  {
    /****************************************************************/
    /* Calculate the Workflow size                                  */
    /****************************************************************/
    ReqdSize = (sizeof(KXCWFLOW) +
                NumRoutes * sizeof(KXCWROUTE));

    /****************************************************************/
    /* Allocate the new buffer                                      */
    /****************************************************************/
    pWFlow = (PKXCWFLOW)malloc(ReqdSize);

    if (pWFlow == NULL)
    {
      fputs("Error: malloc failure. No memory available.", stderr);
      exit(1);
    }

    /****************************************************************/
    /* Initialize the new buffer                                    */
    /****************************************************************/
    pWFlow->Version     = KXCWFLOW_VERSION;
    pWFlow->NumRoutes   = NumRoutes + 1;

    /****************************************************************/
    /* Copy original Service Details                                */
    /****************************************************************/
    memcpy(&pWFlow->Service,
           &pWorkflow->Service,
           sizeof(pWorkflow->Service));

    /****************************************************************/
    /* Ensure we're using the correct CurComponent name             */
    /****************************************************************/
    memcpy(pWFlow->Service.CurComponent,
           pException->RomaConfData.ComponentName,
           sizeof(KXCOBJNAME));

    /****************************************************************/
    /* Make new KXCWSERVICE members empty                           */
    /****************************************************************/
    pWFlow->Service.BusinessService[0] = 0;
    pWFlow->Service.BusinessFlow[0] = 0;
    pWFlow->Service.Operation[0] = 0;

    /****************************************************************/
    /* Access first Route                                           */
    /****************************************************************/
    pRoute = pWFlow->Routes;
      
    /****************************************************************/
    /* Set Type of the first route step                             */
    /****************************************************************/
    pRoute->RouteType = pException->RomaConfData.ElementType;

    /****************************************************************/
    /* Set remaining route details                                  */
    /****************************************************************/
    memcpy(pRoute->TargetQueue,
           pException->RomaConfData.QueueName,
           sizeof(KXCOBJNAME));
    memcpy(pRoute->TargetElement,
           pException->RomaConfData.ElementName,
           sizeof(KXCOBJNAME));
    memcpy(pRoute->TargetComponent,
           pException->RomaConfData.ComponentName,
           sizeof(KXCOBJNAME));

    pRoute->RouteFlags = KXCROUTEFLAGS_NORMAL;
    pRoute->NodeVersion = KXCNODEVER_CURR;
    pRoute->TargetCodePage[0] = 0;
    pRoute->BusinessService[0] = 0;
    pRoute->BusinessFlow[0] = 0;
    pRoute->Operation[0] = 0;
      
    /****************************************************************/
    /* Copy remaining Route details from workflow                   */
    /****************************************************************/
    for (; pOldRoute != pEnd; ++pOldRoute)
    {
      memcpy(++pRoute, pOldRoute, sizeof(*pOldRoute));
      pRoute->BusinessService[0] = 0;
      pRoute->BusinessFlow[0] = 0;
      pRoute->Operation[0] = 0;
    }
  }

  /******************************************************************/
  /* else we can't do anything                                      */
  /******************************************************************/
  else
  {
    WriteError(hKxc, "No RomaConf info in Exception struct");
    bSuccess = FALSE;
  };

  /******************************************************************/
  /* Return any new workflow                                        */
  /******************************************************************/
  *ppNewFlow = pWFlow;

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return bSuccess;
}

/********************************************************************/
/* Function:  EMSRebuildGetWorkflow                                 */
/* Purpose:   Rebuild the workflow for the message to be put back   */
/*            to the applications input queue.                      */
/* Input:     hKxc(HKXC)                                            */
/*            - Context handle                                      */
/*            pException(PKXCEXCEPTION)                             */
/*            - Exception details from the MsgInfo struct           */
/*            pWorkflow(PKXCWFLOW)                                  */
/*            - Current workflow of the EMS'd message               */
/* Output:    ppNewFlow(PKXCWFLOW *)                                */
/*            - New workflow                                        */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCBOOL EMSRebuildGetWorkflow(HKXC           hKxc,
                              PKXCEXCEPTION  pException,
                              PKXCWFLOW      pWorkflow,
                              PKXCWFLOW     *ppNewFlow)
{
KXCBOOL        bSuccess     = TRUE;
PKXCWFLOW      pWFlow       = NULL;
KXCULONG       ReqdSize;
KXCULONG       NumBaseRoutes;
KXCULONG       NumRoutes;
PKXCWROUTE     pRoute       = NULL; 

  /******************************************************************/
  /* Check that Roma & Error data are present in Exception struct   */
  /******************************************************************/
  if ((pException) && 
      (pException->ExceptionFlags & KXCEXC_ROMADATA))
  {
    /****************************************************************/
    /* Store number of original routes                              */
    /****************************************************************/
    NumBaseRoutes = pWorkflow->NumRoutes;

    /****************************************************************/
    /* Calculate number of routes (existing +1)                     */
    /****************************************************************/
    NumRoutes = NumBaseRoutes +1;

    /****************************************************************/
    /* Calculate the Workflow size                                  */
    /****************************************************************/
    ReqdSize = (sizeof(KXCWFLOW) +
                ((NumRoutes-1) * sizeof(KXCWROUTE)));

    /****************************************************************/
    /* Allocate the new buffer                                      */
    /****************************************************************/
    pWFlow = (PKXCWFLOW)malloc(ReqdSize);

    /****************************************************************/
    /* If we're OK                                                  */
    /****************************************************************/
    if (pWFlow != NULL)
    {
      /**************************************************************/
      /* Initialize the new buffer                                  */
      /**************************************************************/
      pWFlow->Version     = KXCWFLOW_VERSION;
      pWFlow->NumRoutes   = NumRoutes;

      /**************************************************************/
      /* Copy original Service Details                              */
      /**************************************************************/
      memcpy(&pWFlow->Service,
             &pWorkflow->Service,
             sizeof(KXCWSERVICE));

      /**************************************************************/
      /* Ensure we're using the correct CurComponent name           */
      /**************************************************************/
      memcpy(pWFlow->Service.CurComponent,
             pException->RomaConfData.ComponentName,
             sizeof(KXCOBJNAME));

      /**************************************************************/
      /* Access first Route                                         */
      /**************************************************************/
      pRoute = pWFlow->Routes;
      
      /**************************************************************/
      /* Set Type of first route entry                              */
      /**************************************************************/
      pRoute->RouteType = pException->RomaConfData.ElementType;

      /**************************************************************/
      /* Set remaining route details                                */
      /**************************************************************/
      memcpy(pRoute->TargetQueue,
             pException->RomaConfData.QueueName,
             sizeof(KXCOBJNAME));
      memcpy(pRoute->TargetElement,
             pException->RomaConfData.ElementName,
             sizeof(KXCOBJNAME));
      memcpy(pRoute->TargetComponent,
             pException->RomaConfData.ComponentName,
             sizeof(KXCOBJNAME));
      pRoute->RouteFlags = KXCROUTEFLAGS_NORMAL;
      pRoute->NodeVersion = KXCNODEVER_CURR;
      pRoute->TargetCodePage[0] = 0;
      pRoute->BusinessService[0] = 0;
      pRoute->BusinessFlow[0] = 0;
      pRoute->Operation[0] = 0;
      
      /**************************************************************/
      /* Copy remaining Route details from workflow                 */
      /**************************************************************/
      if (NumBaseRoutes > 0)
      {
        memcpy(&pWFlow->Routes[1],
               pWorkflow->Routes,
               NumBaseRoutes * sizeof(KXCWROUTE));
      };
    }

    /****************************************************************/
    /* else no memory                                               */
    /****************************************************************/
    else
    {
      WriteError(hKxc, "Malloc failure.  No memory available");
      bSuccess = FALSE;
    };
  }

  /******************************************************************/
  /* else we can't do anything                                      */
  /******************************************************************/
  else
  {
    WriteError(hKxc, "No RomaConf info in Exception struct");
    bSuccess = FALSE;
  };

  /******************************************************************/
  /* Return any new workflow                                        */
  /******************************************************************/
  *ppNewFlow = pWFlow;

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return(bSuccess);
}

/********************************************************************/
/* Function:  WriteError                                            */
/* Purpose:   Report Error Detail                                   */
/* Input:     hKXC(HKXC)                                            */
/*            - Context Handle                                      */
/*            comment(char *)                                       */
/*            - Comment to be displayed                             */
/* Errors:    None                                                  */
/* Notes:     None                                                  */
/********************************************************************/
KXCVOID WriteError(HKXC  hKXC,
                   char *comment)
{
KXCULONG    rc;
KXCERROR    Error;
char        Buffer[400];
KXCULONG    BufLen=400;

  /******************************************************************/
  /* Access last error information                                  */
  /******************************************************************/
  rc = kxcGetLastError(&Error);
  kxcFormatError(&Error.Result, KXCERROR_FULL, Buffer, &BufLen);
  fprintf(stderr,
          "\n%s - \n %s",
          comment, Buffer);
  if (Error.Status & KXCERRSTAT_SUB)
  {
    BufLen=400;
    kxcFormatError(&Error.SubResult, KXCERROR_FULL, Buffer, &BufLen);
    fprintf(stderr, "\n (%s)", Buffer);
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  return;
}

/********************************************************************/
/*                                                                  */
/* Function:  getOpt                                                */
/*                                                                  */
/* Description:                                                     */
/*    Returns the next option name and (if present) option value    */
/*    from the command-line parameters.                             */
/*                                                                  */
/*    This function searches for option specifications observing    */
/*    the following format:                                         */
/*                                                                  */
/*        -o value                                                  */
/*                                                                  */
/*    where "o" is a one-byte option name and "value" is an optional*/
/*    character string value assigned to "o".                       */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int = getOpt( int argc,  char   *argv[], char **ppOptArg );   */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments.                           */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/*    ppOptArg:                                                     */
/*       Pointer to area in which to return the address of the      */
/*       option value if one is present.                            */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral values:                         */
/*       -1    -- Invalid command-line argument detected.           */
/*        0    -- No more named options.                            */
/*       Other -- The name of the next option.  If a value was      */
/*                assigned to this name, a pointer to the value is  */
/*                returned in *ppOptArg; else *ppOptArg set to NULL.*/
/*                                                                  */
/********************************************************************/
int  getOpt( int  argc,  char  *argv[],  char  **ppOptArg )
{
    static int   index = 1 ; /* Index to current entry in argv */
    char        *pc        ; /* Pointer to current argv entry  */
    char         optname   ; /* Name of option                 */

    *ppOptArg = NULL ;  /* Init. return argument value to NULL.*/


    /*
       Increment argument index until we find an argument prefixed by
       a dash or until we run out of arguments.
    */
    for  ( pc = argv[index];
           (index < argc ) && (pc[0] != '-');
           ++ index, pc = argv[index]
         ) ;

    /*
       Return indicating no more options if the current argument
       index points beyond the number of argv entries.  Return
       indicating error if the length of the current argument
       is less than 2.
    */
    if  ( index >= argc )   return  0 ;

    if  ( strlen( argv[index] ) < 2 )
    {
        printf( "Invalid argument, \"%s\".\n", argv[index] );
        ++ index  ;
        return -1 ;
    }


    /*
       Save option name, increment argument index.
    */
    optname = pc[1] ;
    ++ index        ;


    /*
       If the option has trailing text return it as the value
    */
    if (pc[2] != '\0')
    {
      *ppOptArg = &pc[2];
      return optname;
    }

    /*
       If the next entry is prefixed by '-' or '/', or if there
       are no additional arguments, return the current option name
       without an accompanying option value.  Otherwise, return
       the next argument as the option value and increment the index
       again.
    */
    if  ( index >= argc )   return  optname;

    pc = argv[index];
    if  ( pc[0] == '-' )    return optname;

    *ppOptArg = pc ;
    ++ index       ;
    return  optname;
}


/********************************************************************/
/* End of module kxcsemsa.c. Change History follows                 */
/********************************************************************/
/*
 * $Log: kxcsemsa.c,v $
 * Revision 1.1.1.1.10.3  2004/02/13 17:10:34  e_dm
 * Fix for SR 1-42736381 'Roma 3.0 message handling in the EMS Server sample'
 *
 * Revision 1.1.1.1.10.2  2003/10/08 16:29:58  e_dm
 * Use memmove() to copy overlapped memory blocks
 *
 * Revision 1.1.1.1.10.1  2003/08/12 15:27:16  mryan
 * 1-34641704 Transform disalloed on replay of EMS message
 *
 * Revision 1.1.1.1  2001/05/16 09:49:28  pross
 * Created from CTVC xcv310 010511-F1
 *
// 
//    Rev 1.6.2.0.1.0   11 Jan 2001 12:17:06   GCHES
// XCV310: I310.004 Updated with comments from code inspection
// 
//    Rev 1.6.2.0   03 Jan 2001 16:05:24   GCHES
// XC400: I400.004 Updated for support for for extra BSA information, for BSP310.004
// 
//    Rev 1.6   18 Apr 2000 11:00:58   MMALC
// XC300: I1-SK35 Separate the details of HP-UX 10.2 and 11 platforms
// 
//    Rev 1.5   13 Dec 1999 13:49:20   RHEND
// XC300: I804.037 Removed marcos that won't be supported in the end
// 
//    Rev 1.4   10 Dec 1999 14:18:42   RHEND
// XC300: I618811 Added retry count to EMSDisplayMsg
// 
//    Rev 1.3   18 Oct 1999 17:25:30   KOGIL
// XC300: I599700 Change usage message as per code rev
// 
//    Rev 1.2   08 Oct 1999 16:02:02   KOGIL
// XC300: II599700 cmd options use getOpt
// 
//    Rev 1.1   30 Apr 1999 15:49:48   GATAY
// XC300: ID804.001 Display TargetCodePage & NodeVersion fields in Wflow
// 
//    Rev 1.0   19 Feb 1999 18:05:10   GATAY
// XC300: Initial Upload for BSP3.0
 */
/********************************************************************/
