/********************************************************************/
/*                                                                  */
/* Module      : KXCSSERV.C                                         */
/*                                                                  */
/* Description : Sample Roma Server program                         */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998,2002                           */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample program is provided for                 */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This sample program is       */
/*              provided "as is" and Candle expressly disclaims the */
/*              warranties of merchantability and fitness for a     */
/*              particular purpose.                                 */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Details:                                                         */
/*    Sample Roma Server Program.                                   */
/*                                                                  */
/*    This program gets input messages from client applications and */
/*    sends the reply back to the client                            */
/*    (no reply sent on datagram requests.)                         */
/*                                                                  */
/*    This program terminates upon encountering the string "QUIT"   */
/*    as the text of a message.                                     */
/*                                                                  */
/* Dependencies:                                                    */
/*    Requires the relevant services to be set up in the directory  */
/*    and the underlying transport queues to be configured in       */
/*    support of the routes defined for the directory entry.        */
/*                                                                  */
/* Invocation Syntax:                                               */
/*    c_server  [ -e element_name ] [ -m message_format ] [ -? ]    */
/*                                                                  */
/* Input Parameters (all optional):                                 */
/*    -e element_name:                                              */
/*       Specifies the element name to use instead of the default   */
/*       (TestElement1).  Substitute a valid element name for       */
/*       "element_name".                                            */
/*                                                                  */
/*    -m message_format_name:                                       */
/*       Specifies the message format name to use on the response   */
/*       message sent                                               */
/*                                                                  */
/*    -? :                                                          */
/*       Requests a help display for this program.  If this option  */
/*       is specified, all others are ignored.                      */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral codes:                          */
/*       EXIT_SUCCESS -- Successful completion.                     */
/*       EXIT_FAILURE -- Unsuccessful completion.                   */
/*                                                                  */
/* Functions in this module:                                        */
/*    main.............. Entry function for program and mainline    */
/*    display_help...... Display help information for this program  */
/*    processError...... Display information for last Roma error    */
/*    endSession........ Check for "end of session" request in msg  */
/*    parse_cmdline..... Validate and return command-line arguments */
/*    processRequest.... Copy the input buffer to the output buffer */
/*    allCaps........... Convert string to upper-case.              */
/*                                                                  */
/*                                                                  */
/* To build this sample application, make any necessary changes to  */
/* the mksample make file and then issue:                           */
/*                                                                  */
/*             On Unix      -  make -f mksample.mak                 */
/*             On NT/95/98  - nmake -f mksample.mak                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include system header files                                      */
/********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

/********************************************************************/
/* Include the Roma header file                                     */
/********************************************************************/
#include <kxc.h>


/****
*  tunable macros - default business element name
*                   if not provided on command line
****/
#define DEFAULT_ELEMENT_NAME    "TestElement1"

/****
*  QML is the length of the QUIT_MSG ...
****/
#define QUIT_MSG                "QUIT"   /* msg to quit the program */
#define QML                      4       /* length of the quit msg  */

#define MAX_MSG_LENGTH           1024    /* max length of a message */

/****
*  standard macros
****/
#define ASSERT(x) assert(x)

#define bool  int
#define false 0
#define true  1

/****
*  prototypes
****/

int  processRequest( PKXCMSG pReq,          /* msg to be processed  */
                     PKXCMSG pRsp           /* message processed    */
                   );

void display_help( void );                  /* text display routine */

void processError( char *pSpecific_msg );   /* process error message*/

bool endSession( const char *msg );         /* message to end prog  */

int  parse_cmdline( int argc,               /* number of arguments  */
                    char *argv[],           /* array of ptrs to arg */
                    char **ppElement,       /* element to be used   */
                    char **ppMsgFormat,     /* msg format to use    */
                    int  *help              /* help request         */
                  );

char *allCaps( char *s );                   /* capitalize a string  */

int getOpt( int argc, char *argv[],         /* get command line     */
            char **ppOptArg                 /* options if any       */
          );

/****
*  diagnostic trace. (set ESTREAM to an open file descriptor)
*  NOTE:   comment the #define TRACE statement in order to
*          turn the trace off
****/
#if !defined(TRACE)
#define TRACE                               /*  turn ON the trace   */
#endif

#define ESTREAM          stderr
#define FLUSH            fflush( ESTREAM );

#ifdef TRACE

#define TRACE0( s )         fprintf( ESTREAM, s );         FLUSH
#define TRACE1( s, p1 )     fprintf( ESTREAM, s, p1 );     FLUSH
#define TRACE2( s, p1, p2 ) fprintf( ESTREAM, s, p1, p2 ); FLUSH

#else

#define TRACE0( s )
#define TRACE1( s, p1 )
#define TRACE2( s, p1, p2 )

#endif


/********************************************************************/
/*                                                                  */
/* Function:  main                                                  */
/*                                                                  */
/* Description:                                                     */
/*    Entry function for executable module.                         */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments passed.                    */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral codes:                          */
/*       EXIT_SUCCESS -- Successful completion.                     */
/*       EXIT_FAILURE -- Unsuccessful completion.                   */
/*                                                                  */
/********************************************************************/
int main( int argc, char *argv[] )
{
  KXCAPPDEF AppDef = { KXCAPPDEF_DEFAULT }; /* default appl def     */
  HKXC      hKxc   = NULL_HKXC;             /* kxc handle           */
  KXCMSG    reqMsg = { KXCMSG_DEFAULT };    /* default req msg      */
  KXCMSG    rspMsg = { KXCMSG_DEFAULT };    /* default rsp msg      */

  KXCPUTOPT rspOpt;                         /* put option for a rsp */
  KXCGETOPT reqOpt;                         /* get option for a req */

  HKXCTHEME hTheme = NULL_HKXCTHEME;     /* theme handle for put/get*/

  char *element_name = NULL;             /* ptr to business element */
  char *format_name = NULL;              /* ptr to msg format name  */
  int  help = FALSE;                     /* help not requested      */

  if (!parse_cmdline(argc, argv, &element_name, &format_name, &help))
    {                                     /* command line arg error */
      return EXIT_FAILURE;                /* yes, return to caller  */
    }

  if ( help)                              /* syntax help required   */
    {
      display_help();                     /* display brief help     */
      return EXIT_SUCCESS;                /* OK to get out          */
    }

  TRACE0( "Roma Server starting...\n");          /* server is       */
  TRACE1( "Using Business Element:  %s\n",       /* starting        */
              element_name );
  TRACE1( "Using Message Format:  %s\n",
              (format_name == NULL ? "none" : format_name) );


  strncpy ((char *) AppDef.ElementName,          /* setup appl def  */
             (char *) element_name, 
             sizeof(KXCOBJNAME));

  if ( !kxcServerInitialize( &AppDef, &hKxc ) )  /* if initialize   */
    {                                            /* fails           */
      processError( "Failed to initialize Server.\n" );
      return EXIT_FAILURE;
    }                                            /* quit now        */
  TRACE0( "Roma Server initialized successfully.\n" );         


  /****
  *  initialise options structures
  ****/
  kxcSetPutOptDefault( &rspOpt );               /* default put opt  */
  kxcSetGetOptDefault( &reqOpt );               /* default get opt  */
  reqOpt.ReceiveOptions |= ( KXCGOPT_WAIT       /* set up get opt   */
                             | KXCGOPT_AUTO_ALLOC);

  /****
  *  loop:
  *     take text from input queue
  *     process and send response if not datagram
  ****/
  for ( ; ; )                               /* loop thru messages   */
    {
      /****
      *  Get the request
      ****/

      if ( !kxcGetRequest( hKxc,            /* get the message      */
                           &reqMsg,         /* msg to process       */
                           &reqOpt,         /* request option       */
                           NULL,            /* no sync point        */
                           &hTheme ) )      /* theme id             */
        {                                   /* if anything wrong    */
          processError( "GetRequest failed.\n" );
          break;                            /* return quickly       */
        }

      printf( "\n" );
      printf( "Received: \"%.*s\"\n",         /* notify success       */
              reqMsg.MsgLen, (char*)reqMsg.MsgBuffer.pBuffer );
    /***************************************************************/
    /* If the message has a Roma Message Format Name - Print it    */
    /***************************************************************/
    if ( kxcGetMsgFormat(&reqMsg) == KXCMSGFORMAT_USENAME )
    {
      KXCOBJNAME FrmtNm;
      kxcGetMsgFormatName(&reqMsg, FrmtNm);
      printf("Received with Message Format Name \"%s\"\n", FrmtNm);
    }


#ifndef DATAGRAM_MODEL
      /****
      *  Process request and send response as requested
      ****/

      if ( !processRequest( &reqMsg, &rspMsg ) ) /* anything wrong  */
        {
          processError( "Error Processing request"
                         " - No reply sent.\n" );
          break;                                 /* quit            */
        }
      /* Set Message format name */
      if ( format_name  != NULL )
        {
          kxcSetMsgFormatName( &rspMsg, format_name);
        }

      if ( !kxcPutResponse( hKxc,           /* put the message      */
                            &rspMsg,        /* msg processed        */
                            &rspOpt,        /* response option      */
                            NULL,           /* no sync point        */
                            &hTheme ) )     /* theme id             */
        {                                   /* if anything wrong    */
          processError( "PutResponse failed.\n" );
          break;                            /* return quickly       */
        }

      printf( "Response: \"%.*s\"\n",       /* notify success       */
              rspMsg.MsgLen, (char*)rspMsg.MsgBuffer.pBuffer );

#endif

  /****
  *  terminate session if "quit" is entered
  ****/
      if ( endSession( ( char * ) reqMsg.MsgBuffer.pBuffer ) )
        break;                              /* quit if "QUIT"       */

  /****
  *  free buffer if it was auto-allocated
  ****/
      if (reqMsg.MsgBuffer.Status == KXCBUFSTAT_KXC)
      {
         kxcFreeBuffer(hKxc, &reqMsg.MsgBuffer);
      }
    }

  /****
  *  tidy up
  ****/
  if ( !kxcTerminate( &hKxc ) )             /* bad termination?     */
    {
      processError( "Failed to Terminate Roma.\n" );
      return EXIT_FAILURE;                  /* yes, quit            */
    }

  TRACE0( "Roma Server terminated.\n" );    /* termination is OK    */
  return EXIT_SUCCESS;                      /* good termination     */
}


/********************************************************************/
/*                                                                  */
/* Function:  processRequest                                        */
/*                                                                  */
/* Description:                                                     */
/*    Generate a response message from the initial request message. */
/*    This is done by copying the input message into a new buffer   */
/*    This new message is then passed back as the response message. */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int  processRequest( PKXCMSG pReq,                            */
/*                         PKXCMSG pRsp );                          */
/*                                                                  */
/* Input Parameters:                                                */
/*    pReq:                                                         */
/*       Pointer to the input request message.                      */
/*                                                                  */
/*    pRsp:                                                         */
/*       Pointer to the output response message structure, to       */
/*       return the message.                                        */
/*                                                                  */
/* Return Value:                                                    */
/*    An int value:  1 for successful completion, 0 if an error     */
/*    was encountered.                                              */
/*                                                                  */
/********************************************************************/
int processRequest( KXCMSG *pReq, KXCMSG *pRsp )
 {
   static char reply_buffer[ MAX_MSG_LENGTH ]; /* maximize buffer   */

   ASSERT( pReq && pRsp );                  /* ensure working       */
   ASSERT( pReq->MsgLen < MAX_MSG_LENGTH ); /*  area                */

   strncpy( reply_buffer,( char * )         /* copy the string      */
            pReq->MsgBuffer.pBuffer,        /* to response buffer   */
            pReq->MsgLen );

   reply_buffer[ pReq->MsgLen ] = '\0';     /* null terminator      */

   kxcSetMsgBuffer( pRsp, pReq->MsgLen,     /* set message buffer   */
                    reply_buffer );
   pRsp->MsgLen = pReq->MsgLen ;            /* set message length   */

   return TRUE;                             /* return as success    */
 }


/********************************************************************/
/*                                                                  */
/* Function:  processError                                          */
/*                                                                  */
/* Description:                                                     */
/*    Write the caller-specified error message text (if specified)  */
/*    to stdout, then use tracing macros to conditionally display   */
/*    detail error information based on the last Roma return code.  */
/*                                                                  */
/* Invocation syntax:                                               */
/*    processError( char *pSpecific_msg );                          */
/*                                                                  */
/* Input Parameters:                                                */
/*    pSpecific_msg:                                                */
/*       Pointer to text to be displayed before the messages        */
/*       describing the last Roma error.  Optional; specify NULL    */
/*       if no preliminary message is to be displayed.              */
/*                                                                  */
/* Return Value:                                                    */
/*    void.                                                         */
/*                                                                  */
/********************************************************************/
void  processError( char  *pSpecific_msg )
{
    KXCERROR    Err;
    KXCULONG    BufLen=400;
    char        Buffer[400];

    if  ( pSpecific_msg )   printf( pSpecific_msg );

    if  ( kxcGetLastError( &Err ) )
    {
        kxcFormatError(&Err.Result, KXCERROR_FULL,
                       Buffer, &BufLen);
        TRACE1( "Error    = %s\n",  Buffer );

        if  ( Err.Status & KXCERRSTAT_SUB )
        {
            BufLen=400;
            kxcFormatError(&Err.SubResult, KXCERROR_FULL,
                           Buffer, &BufLen);
            TRACE1( "SubError = %s\n",  Buffer );
        }
    }
    else
        TRACE0( "Unable to access error information.\n" );

    return ;
}


/********************************************************************/
/*                                                                  */
/* Function:  display_help                                          */
/*                                                                  */
/* Description:                                                     */
/*    Display help text for program.                                */
/*                                                                  */
/* Invocation syntax:                                               */
/*    display_help();                                               */
/*                                                                  */
/* Input Parameters:                                                */
/*    None.                                                         */
/*                                                                  */
/* Return Value:                                                    */
/*    void.                                                         */
/*                                                                  */
/********************************************************************/
void display_help( void )
 {
   printf( "\n" );
   printf( "================================"
           "================================\n" );
#ifdef DATAGRAM_MODEL
   printf( "This KXC server program expects to receive text\n"   );
   printf( "messages from an Roma client.  The message is\n"     );
   printf( "displayed, but no response is sent.\n"               );
#else
   printf( "This KXC server program expects to receive text\n"   );
   printf( "messages from an Roma client. After display the\n"   );
   printf( "message is returned to the client as a response.\n"  );
#endif
   printf( "It terminates when it encounters a \"QUIT\" message.");
   printf( "\n\n"                                                );
   printf( "Invocation Syntax (all parameters optional):\n"      );
   printf( "   c_server  <-e element_name>      \n"              );
   printf( "             <-m msg_format_name> <-?> \n"           );
   printf( "\n"                                                  );
   printf( "\"element_name\" overrides the default business   \n");
   printf( "element name (TestElement1). \n"                     );
   printf( "\"format_name\" sets the reply message format name\n");
   printf( "-? requests this help display.\n"                    );
   printf( "================================"
           "================================\n\n" );

   return ;
 }


/********************************************************************/
/*                                                                  */
/* Function:  endSession                                            */
/*                                                                  */
/* Description:                                                     */
/*    Checks the caller-specified message for "end of session"      */
/*    indicator (QUIT_MSG).                                         */
/*                                                                  */
/* Invocation syntax:                                               */
/*    bool  = endSession( const char *pMsg );                       */
/*                                                                  */
/* Input Parameters:                                                */
/*    pMsg:                                                         */
/*       Pointer to message text to be examined.                    */
/*                                                                  */
/* Return Value:                                                    */
/*    An int value:  1 if the "end of session" indicator was        */
/*    found in the message, 0 otherwise.                            */
/*                                                                  */
/********************************************************************/

bool endSession( const char *pMsg )
{
  char uppercase_msg[ QML + 1 ];            /* initialize buffer    */

  if ( !pMsg )                              /* no message received  */
    return false;                           /*  can not process     */

  allCaps( strncpy( uppercase_msg,          /* capitalize the text  */
            (char * ) pMsg, QML ) );

  return ( !strncmp( uppercase_msg,         /* don't want to quit   */
             QUIT_MSG, QML ) );             /* back to caller       */
 }


/********************************************************************/
/*                                                                  */
/* Function:  parse_cmdline                                         */
/*                                                                  */
/* Description:                                                     */
/*    Validates command-line arguments and returns the arguments    */
/*    in the corresponding caller-specified return areas.           */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int =  parse_cmdline( int     argc,                           */
/*                          char   *argv[],                         */
/*                          char  **ppElement,                      */
/*                          char  **ppFormat,                       */
/*                          int    *pHelp                           */
/*                        );                                        */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments.                           */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/*    ppElement:                                                    */
/*       Pointer to area in which the address of the element        */
/*       name argument is to be returned.  The area is set to       */
/*       NULL if the element name was omitted.                      */
/*                                                                  */
/*    ppFormat:                                                     */
/*       Pointer to area in which the address of the msg format     */
/*       name argument is to be returned.  The area is set to       */
/*       NULL if the format name was omitted.                       */
/*    pHelp:                                                        */
/*       Pointer to help indicator.  If help was requested,         */
/*       this indicator is set to 1; otherwise, it is set to 0.     */
/*                                                                  */
/* Return Value:                                                    */
/*    1 if the command line arguments were successfully parsed,     */
/*    0 otherwise.                                                  */
/*                                                                  */
/********************************************************************/
int parse_cmdline( int argc, char *argv[],
                   char **ppElement, 
                   char **ppFormat, int *help )
 {
  int  c      = 0;                          /* option code          */
  int  status = TRUE;                       /* completion status    */
  char *pOptArg;                            /* ptr to option value  */

  ASSERT( ppElement != NULL);               /* ensure the work area */
  ASSERT( ppFormat  != NULL);               /* ensure the work area */
  ASSERT( help      != NULL);               /* ensure the work area */

  *ppElement = DEFAULT_ELEMENT_NAME;        /* set to default       */
  *ppFormat = NULL;                         /* set to default       */
  *help = FALSE;                            /* init to no help      */

  if (argc == 1)                            /* if only 1 argument   */
    return TRUE;                            /*  good                */

  /****
  *  TODO: add your own option flags and cases
  *        to the getopt loop below
  ****/
  while ( ( c = getOpt( argc, argv, &pOptArg ) ) > 0 ) /* getting it*/
  switch( c )                               /* what are the options */
  {
    case 'E':                               /* business element name*/
    case 'e':
            *ppElement = pOptArg;           /* OK argument known/set*/
            break;                          /* continue             */
    case 'M':                               /* business element name*/
    case 'm':
            *ppFormat = pOptArg;            /* OK argument known/set*/
            break;                          /* continue             */
    case '?':                               /* commandline help     */
            *help = TRUE;                   /* help is asked        */
            fprintf( stderr, "Usage: %s  -e element_name  -? \n",
                     argv[0] );
            break;                          /* continue             */
    default:                                /* unknown              */
            fprintf( stderr, "unknown option %c.\n", c );
            status = FALSE;                 /* help needed          */
            break;                          /* break out            */
  }

  if ( c < 0 )                              /* nothing received     */
    status = FALSE;                         /* help needed          */

  if ( status == FALSE )                    /* help is here         */
    fprintf( stderr, "Usage: %s  -e element_name -m format  -? \n",
             argv[0] );

  return status;                            /* return to caller     */
 }


/********************************************************************/
/*                                                                  */
/* Function:  allCaps                                               */
/*                                                                  */
/* Description:                                                     */
/*    Convert the lower-case characters in the caller-specified     */
/*    string to upper-case.                                         */
/*                                                                  */
/* Invocation syntax:                                               */
/*    char *allCaps( char *string ) ;                               */
/*                                                                  */
/* Input Parameters:                                                */
/*    string:                                                       */
/*       Pointer to the null-terminated string to be converted.     */
/*                                                                  */
/* Return Value:                                                    */
/*    Pointer to the input argument (the modified string).          */
/*                                                                  */
/********************************************************************/
char *allCaps( char *string )
 {
  char *cp;                                 /* set the pointer      */
  for ( cp = string; *cp; cp++ )            /* start from first char*/
     if ( islower( *cp ) )                  /* if it is in lowercase*/
       *cp = toupper( *cp );                /* turn it to uppercase */

  return string;                            /* done, return it now  */
 }


/********************************************************************/
/*                                                                  */
/* Function:  getOpt                                                */
/*                                                                  */
/* Description:                                                     */
/*    Returns the next option name and (if present) option value    */
/*    from the command-line parameters.                             */
/*                                                                  */
/*    This function searches for option specifications observing    */
/*    the following format:                                         */
/*                                                                  */
/*        -o value                                                  */
/*                                                                  */
/*    where "o" is a one-byte option name and "value" is an optional*/
/*    character string value assigned to "o".                       */
/*                                                                  */
/* Invocation syntax:                                               */
/*    int = getOpt( int argc,  char   *argv[], char **ppOptArg );   */
/*                                                                  */
/* Input Parameters:                                                */
/*    argc:                                                         */
/*       Count of command-line arguments.                           */
/*                                                                  */
/*    argv:                                                         */
/*       Array of pointers to command-line arguments.               */
/*                                                                  */
/*    ppOptArg:                                                     */
/*       Pointer to area in which to return the address of the      */
/*       option value if one is present.                            */
/*                                                                  */
/* Return Value:                                                    */
/*    One of the following integral values:                         */
/*       -1    -- Invalid command-line argument detected.           */
/*        0    -- No more named options.                            */
/*       Other -- The name of the next option.  If a value was      */
/*                assigned to this name, a pointer to the value is  */
/*                returned in *ppOptArg; else *ppOptArg set to NULL.*/
/*                                                                  */
/********************************************************************/
int getOpt( int argc, char *argv[], char **ppOptArg )
 {
   static int index = 1;          /* Index to current entry in argv */
   char       *pc      ;          /* Pointer to current argv entry  */
   char       optname  ;          /* Name of option                 */

   *ppOptArg = NULL ;             /* Init. return arg value to NULL */

   /****
   *  increment argument index until we find an argument prefixed
   *  by a dash or until we run out of arguments.
   ****/
   for  ( pc = argv[index];
          (index < argc ) && (pc[0] != '-');
          ++ index, pc = argv[index]
        );


   /****
   *  return indicating no more options if the current argument
   *  index points beyond the number of argv entries.  Return
   *  indicating error if the length of the current argument
   *  is less than 2.
   ****/
   if ( index >= argc )
     return 0 ;                            /* return if end of list */

   if ( strlen( argv[index] ) < 2 )
     {
       printf( "Invalid argument, \"%s\".", argv[index] );
       ++ index;                            /* point to next one    */
       return -1;                           /* return w/ indicator  */
     }

   /****
   *  Save option name, increment argument index.
   ****/
   optname = pc[1];
   ++ index;

   /****
   *  If the option has trailing text return it as the value
   ****/
   if (pc[2] != '\0')
   {
     *ppOptArg = &pc[2];
     return optname;
   }

   /****
   *  if the next entry is prefixed by '-', or if there are
   *  no additional arguments, return the current option name
   *  without an accompanying option value.  Otherwise, return
   *  the next argument as the option value and increment the
   *  index again.
   ****/
   if ( index >= argc )
     return optname;

   pc = argv[index];
   if ( pc[0] == '-' )
     return optname;

   *ppOptArg = pc;
   ++ index      ;
   return optname;
 }
